@@ -27,10 +27,10 @@
 #include <sys/stat.h> /* O_RDONLY */
 #include <sys/types.h> /* pid_t ssize_t */
 #ifndef _WIN32
-#include <sys/wait.h> /* WEXITSTATUS() WIFEXITED() */
+#include <sys/wait.h> /* waitpid() */
 #endif
-#include <signal.h> /* kill() */
-#include <unistd.h> /* execve() fork() */
+#include <signal.h> /* SIG* kill() */
+#include <unistd.h> /* execve() fork() setsid() */
 
 #include <assert.h> /* assert() */
 #include <errno.h> /* errno */
@@ -80,7 +80,7 @@
  * that has associated external process has the following life cycle:
  *  1. Created by main thread and passed to error thread through new_err_jobs.
  *  2. Either gets marked by signal handler or its stream reaches EOF.
- *  3. Its in_use flag is reset.
+ *  3. Its use_count field is decremented.
  *  4. Main thread frees corresponding entry.
  */
 
@@ -121,14 +121,22 @@ static void free_drained_jobs(bg_job_t **jobs);
 static void import_error_jobs(bg_job_t **jobs);
 static void make_ready_list(const bg_job_t *jobs, selector_t *selector);
 #ifndef _WIN32
+static void rip_children(void);
+static void rip_child(pid_t pid, int status);
 static void report_error_msg(const char title[], const char text[]);
 #endif
+static bg_job_t * launch_external(const char cmd[], BgJobFlags flags,
+		ShellRequester by);
 static void append_error_msg(bg_job_t *job, const char err_msg[]);
+static void place_on_job_bar(bg_job_t *job);
+static void get_off_job_bar(bg_job_t *job);
 static bg_job_t * add_background_job(pid_t pid, const char cmd[],
-		uintptr_t err, uintptr_t data, BgJobType type);
+		uintptr_t err, uintptr_t data, BgJobType type, int with_bg_op);
 static void * background_task_bootstrap(void *arg);
 static void set_current_job(bg_job_t *job);
 static void make_current_job_key(void);
+static int update_job_status(bg_job_t *job);
+static void mark_job_finished(bg_job_t *job, int exit_code);
 static int bg_op_cancel(bg_op_t *bg_op);
 
 bg_job_t *bg_jobs = NULL;
@@ -147,7 +155,7 @@ void
 bg_init(void)
 {
 	pthread_t id;
-	const int err = pthread_create(&id, NULL, &error_thread, NULL);
+	int err = pthread_create(&id, NULL, &error_thread, NULL);
 	assert(err == 0);
 	(void)err;
 
@@ -155,48 +163,24 @@ bg_init(void)
 	set_current_job(NULL);
 }
 
-void
-bg_process_finished_cb(pid_t pid, int exit_code)
-{
-	bg_job_t *job;
-
-	/* Mark any finished jobs. */
-	job = bg_jobs;
-	while(job != NULL)
-	{
-		if(job->pid == pid)
-		{
-			pthread_spin_lock(&job->status_lock);
-			job->running = 0;
-			job->exit_code = exit_code;
-			pthread_spin_unlock(&job->status_lock);
-			break;
-		}
-		job = job->next;
-	}
-}
-
 void
 bg_check(void)
 {
-	bg_job_t *head = bg_jobs;
+#ifndef _WIN32
+	rip_children();
+#endif
 
 	/* Quit if there is no jobs or list is unavailable (e.g. used by another
 	 * invocation of this function). */
-	if(head == NULL)
+	if(bg_jobs == NULL)
 	{
 		set_jobcount_var(0);
 		return;
 	}
 
-	if(bg_jobs_freeze() != 0)
-	{
-		return;
-	}
-
 	int active_jobs = 0;
 
-	head = bg_jobs;
+	bg_job_t *head = bg_jobs;
 	bg_jobs = NULL;
 
 	bg_job_t *p = head;
@@ -206,10 +190,17 @@ bg_check(void)
 		job_check(p);
 
 		pthread_spin_lock(&p->status_lock);
-		int can_remove = (!p->running && !p->in_use);
-		active_jobs += (p->running != 0);
+		int running = p->running;
+		int can_remove = (!p->running && p->use_count == 0);
 		pthread_spin_unlock(&p->status_lock);
 
+		active_jobs += (running != 0 && p->in_menu);
+
+		if(!running && p->on_job_bar)
+		{
+			get_off_job_bar(p);
+		}
+
 		/* Remove job if it is finished now. */
 		if(can_remove)
 		{
@@ -221,11 +212,6 @@ bg_check(void)
 
 			p = p->next;
 
-			if(j->type == BJT_OPERATION)
-			{
-				ui_stat_job_bar_remove(&j->bg_op);
-			}
-
 			job_free(j);
 		}
 		else
@@ -238,8 +224,6 @@ bg_check(void)
 	assert(bg_jobs == NULL && "Job list shouldn't be used by anyone.");
 	bg_jobs = head;
 
-	bg_jobs_unfreeze();
-
 	set_jobcount_var(active_jobs);
 }
 
@@ -284,18 +268,7 @@ job_check(bg_job_t *job)
 	}
 	while(new_errors != NULL);
 
-#ifdef _WIN32
-	DWORD retcode;
-	if(GetExitCodeProcess(job->hprocess, &retcode) != 0)
-	{
-		if(retcode != STILL_ACTIVE)
-		{
-			pthread_spin_lock(&job->status_lock);
-			job->running = 0;
-			pthread_spin_unlock(&job->status_lock);
-		}
-	}
-#endif
+	(void)update_job_status(job);
 }
 
 /* Frees resources allocated by the job as well as the bg_job_t structure
@@ -310,7 +283,7 @@ job_free(bg_job_t *job)
 
 	pthread_spin_destroy(&job->errors_lock);
 	pthread_spin_destroy(&job->status_lock);
-	if(job->type != BJT_COMMAND)
+	if(job->with_bg_op)
 	{
 		pthread_spin_destroy(&job->bg_op_lock);
 	}
@@ -329,7 +302,19 @@ job_free(bg_job_t *job)
 	{
 		CloseHandle(job->hprocess);
 	}
+	if(job->hjob != NO_JOB_ID)
+	{
+		CloseHandle(job->hjob);
+	}
 #endif
+	if(job->input != NULL)
+	{
+		fclose(job->input);
+	}
+	if(job->output != NULL)
+	{
+		fclose(job->output);
+	}
 	free(job->bg_op.descr);
 	free(job->cmd);
 	free(job->new_errors);
@@ -351,17 +336,13 @@ bg_and_wait_for_errors(char cmd[], const struct cancellation_t *cancellation)
 		return -1;
 	}
 
-	(void)set_sigchld(1);
-
 	if((pid = fork()) == -1)
 	{
-		(void)set_sigchld(0);
 		return -1;
 	}
 
 	if(pid == 0)
 	{
-		(void)set_sigchld(0);
 		run_from_fork(error_pipe, 1, 0, cmd, SHELL_BY_APP);
 	}
 	else
@@ -396,16 +377,10 @@ bg_and_wait_for_errors(char cmd[], const struct cancellation_t *cancellation)
 		}
 		else
 		{
-			/* Don't use "const int" variables with WEXITSTATUS() as they cause
-			 * compilation errors in case __USE_BSD is defined.  Anonymous type with
-			 * "const int" is composed via compound literal expression. */
-			int status = get_proc_exit_status(pid);
-			result = (status != -1 && WIFEXITED(status)) ? WEXITSTATUS(status) : -1;
+			result = status_to_exit_code(get_proc_exit_status(pid));
 		}
 	}
 
-	(void)set_sigchld(0);
-
 	return result;
 #else
 	return -1;
@@ -417,6 +392,8 @@ bg_and_wait_for_errors(char cmd[], const struct cancellation_t *cancellation)
 static void *
 error_thread(void *p)
 {
+	enum { ERROR_SELECT_TIMEOUT_MS = 250 };
+
 	bg_job_t *jobs = NULL;
 
 	selector_t *selector = selector_alloc();
@@ -432,7 +409,7 @@ error_thread(void *p)
 	{
 		update_error_jobs(&jobs);
 		make_ready_list(jobs, selector);
-		while(selector_wait(selector, 250))
+		while(selector_wait(selector, ERROR_SELECT_TIMEOUT_MS))
 		{
 			int need_update_list = (jobs == NULL);
 
@@ -469,12 +446,10 @@ error_thread(void *p)
 				if(nread == 0)
 				{
 					/* Reached EOF, exclude corresponding file descriptor from the set,
-					 * cut the job out of our list and allow its deletion. */
+					 * cut the job out of our list and decrement its use counter. */
 					selector_remove(selector, j->err_stream);
 					*job = j->err_next;
-					pthread_spin_lock(&j->status_lock);
-					j->in_use = 0;
-					pthread_spin_unlock(&j->status_lock);
+					bg_job_decref(j);
 					continue;
 				}
 
@@ -522,10 +497,10 @@ free_drained_jobs(bg_job_t **jobs)
 		if(j->drained)
 		{
 			pthread_spin_lock(&j->status_lock);
-			/* If finished, reset in_use mark and drop it from the list. */
+			/* If finished, decrement use_count and drop it from the list. */
 			if(!j->running)
 			{
-				j->in_use = 0;
+				--j->use_count;
 				*job = j->err_next;
 				pthread_spin_unlock(&j->status_lock);
 				continue;
@@ -585,6 +560,39 @@ make_ready_list(const bg_job_t *jobs, selector_t *selector)
 }
 
 #ifndef _WIN32
+
+/* Rips children updating status of jobs in the process. */
+static void
+rip_children(void)
+{
+	int status;
+	pid_t pid;
+
+	/* This needs to be a loop in case of multiple blocked signals. */
+	while((pid = waitpid(-1, &status, WNOHANG)) > 0)
+	{
+		if(WIFEXITED(status) || WIFSIGNALED(status))
+		{
+			rip_child(pid, status);
+		}
+	}
+}
+
+/* Looks up a child in job list and rips it if found. */
+static void
+rip_child(pid_t pid, int status)
+{
+	bg_job_t *job;
+	for(job = bg_jobs; job != NULL; job = job->next)
+	{
+		if(job->pid == pid)
+		{
+			mark_job_finished(job, status_to_exit_code(status));
+			break;
+		}
+	}
+}
+
 /* Either displays error message to the user for foreground operations or saves
  * it for displaying on the next invocation of bg_check(). */
 static void
@@ -602,21 +610,25 @@ report_error_msg(const char title[], const char text[])
 		append_error_msg(job, text);
 	}
 }
+
 #endif
 
 /* Appends message to error-related fields of the job. */
 static void
 append_error_msg(bg_job_t *job, const char err_msg[])
 {
-	pthread_spin_lock(&job->errors_lock);
-	(void)strappend(&job->errors, &job->errors_len, err_msg);
-	(void)strappend(&job->new_errors, &job->new_errors_len, err_msg);
-	pthread_spin_unlock(&job->errors_lock);
+	if(err_msg[0] != '\0')
+	{
+		pthread_spin_lock(&job->errors_lock);
+		(void)strappend(&job->errors, &job->errors_len, err_msg);
+		(void)strappend(&job->new_errors, &job->new_errors_len, err_msg);
+		pthread_spin_unlock(&job->errors_lock);
+	}
 }
 
 #ifndef _WIN32
 pid_t
-bg_run_and_capture(char cmd[], int user_sh, FILE **out, FILE **err)
+bg_run_and_capture(char cmd[], int user_sh, FILE *in, FILE **out, FILE **err)
 {
 	pid_t pid;
 	int out_pipe[2];
@@ -636,6 +648,11 @@ bg_run_and_capture(char cmd[], int user_sh, FILE **out, FILE **err)
 		return (pid_t)-1;
 	}
 
+	if(in != NULL)
+	{
+		fflush(in);
+	}
+
 	if((pid = fork()) == -1)
 	{
 		close(out_pipe[0]);
@@ -661,6 +678,18 @@ bg_run_and_capture(char cmd[], int user_sh, FILE **out, FILE **err)
 			_Exit(EXIT_FAILURE);
 		}
 
+		if(in != NULL)
+		{
+			rewind(in);
+
+			if(dup2(fileno(in), STDIN_FILENO) == -1)
+			{
+				_Exit(EXIT_FAILURE);
+			}
+
+			fclose(in);
+		}
+
 		sh = user_sh ? get_execv_path(cfg.shell) : "/bin/sh";
 		sh_flag = user_sh ? cfg.shell_cmd_flag : "-c";
 		prepare_for_exec();
@@ -677,10 +706,12 @@ bg_run_and_capture(char cmd[], int user_sh, FILE **out, FILE **err)
 }
 #else
 /* Runs command in a background and redirects its stdout and stderr streams to
- * file streams which are set.  Returns (pid_t)0 or (pid_t)-1 on error. */
+ * file streams which are set.  Input is redirected only if in parameter isn't
+ * NULL.  Don't pass pipe for input, it can cause deadlock.  Returns (pid_t)0 or
+ * (pid_t)-1 on error. */
 static pid_t
-background_and_capture_internal(char cmd[], int user_sh, FILE **out, FILE **err,
-		int out_pipe[2], int err_pipe[2])
+background_and_capture_internal(char cmd[], int user_sh, FILE *in, FILE **out,
+		FILE **err, int out_pipe[2], int err_pipe[2])
 {
 	const wchar_t *args[4];
 	char *cwd;
@@ -690,6 +721,15 @@ background_and_capture_internal(char cmd[], int user_sh, FILE **out, FILE **err,
 	wchar_t *wide_sh_flag;
 	const int use_cmd = (!user_sh || curr_stats.shell_type == ST_CMD);
 
+	if(in != NULL)
+	{
+		fflush(in);
+		rewind(in);
+
+		if(_dup2(_fileno(in), _fileno(stdin)) != 0)
+			return (pid_t)-1;
+	}
+
 	if(_dup2(out_pipe[1], _fileno(stdout)) != 0)
 		return (pid_t)-1;
 	if(_dup2(err_pipe[1], _fileno(stderr)) != 0)
@@ -753,8 +793,9 @@ background_and_capture_internal(char cmd[], int user_sh, FILE **out, FILE **err,
 }
 
 pid_t
-bg_run_and_capture(char cmd[], int user_sh, FILE **out, FILE **err)
+bg_run_and_capture(char cmd[], int user_sh, FILE *in, FILE **out, FILE **err)
 {
+	int in_fd;
 	int out_fd, out_pipe[2];
 	int err_fd, err_pipe[2];
 	pid_t pid;
@@ -773,15 +814,17 @@ bg_run_and_capture(char cmd[], int user_sh, FILE **out, FILE **err)
 		return (pid_t)-1;
 	}
 
+	in_fd = dup(_fileno(stdin));
 	out_fd = dup(_fileno(stdout));
 	err_fd = dup(_fileno(stderr));
 
-	pid = background_and_capture_internal(cmd, user_sh, out, err, out_pipe,
+	pid = background_and_capture_internal(cmd, user_sh, in, out, err, out_pipe,
 			err_pipe);
 
 	_close(out_pipe[1]);
 	_close(err_pipe[1]);
 
+	_dup2(in_fd, _fileno(stdin));
 	_dup2(out_fd, _fileno(stdout));
 	_dup2(err_fd, _fileno(stderr));
 
@@ -796,40 +839,136 @@ bg_run_and_capture(char cmd[], int user_sh, FILE **out, FILE **err)
 #endif
 
 int
-bg_run_external(const char cmd[], int skip_errors, ShellRequester by)
+bg_run_external(const char cmd[], int skip_errors, ShellRequester by,
+		FILE **input)
 {
-	bg_job_t *job = NULL;
-#ifndef _WIN32
-	pid_t pid;
-	int error_pipe[2];
-	char *command;
-
-	command = cfg.fast_run ? fast_run_complete(cmd) : strdup(cmd);
+	char *command = (cfg.fast_run ? fast_run_complete(cmd) : strdup(cmd));
 	if(command == NULL)
 	{
-		return -1;
+		return 1;
 	}
 
-	if(pipe(error_pipe) != 0)
+	const BgJobFlags flags = (input == NULL ? BJF_NONE : BJF_SUPPLY_INPUT);
+	bg_job_t *job = launch_external(command, flags, by);
+	free(command);
+	if(job == NULL)
 	{
-		show_error_msg("File pipe error", "Error creating pipe");
-		free(command);
-		return -1;
+		if(input != NULL)
+		{
+			*input = NULL;
+		}
+		return 1;
+	}
+
+	if(input != NULL)
+	{
+		*input = job->input;
+		job->input = NULL;
+	}
+
+	/* It's safe to do this here because bg_check() is executed on the same
+	 * thread as this function. */
+	job->skip_errors = skip_errors;
+	return 0;
+}
+
+bg_job_t *
+bg_run_external_job(const char cmd[], BgJobFlags flags)
+{
+	bg_job_t *job = launch_external(cmd, flags, SHELL_BY_APP);
+	if(job == NULL)
+	{
+		return NULL;
+	}
+
+	/* It's safe to do this here because bg_check() is executed on the same
+	 * thread as this function. */
+	bg_job_incref(job);
+	job->skip_errors = 1;
+
+	if(flags & BJF_JOB_BAR_VISIBLE)
+	{
+		place_on_job_bar(job);
+	}
+
+	job->in_menu = (flags & BJF_MENU_VISIBLE);
+
+	return job;
+}
+
+/* Starts a new external command job.  Returns the new job or NULL on error. */
+static bg_job_t *
+launch_external(const char cmd[], BgJobFlags flags, ShellRequester by)
+{
+	/* TODO: simplify this function (launch_external()) somehow, maybe split in
+	 *       two. */
+	const int jb_visible = (flags & BJF_JOB_BAR_VISIBLE);
+	const int supply_input = (flags & BJF_SUPPLY_INPUT);
+	const int capture_output = (flags & BJF_CAPTURE_OUT);
+	const int merge_streams = (capture_output && (flags & BJF_MERGE_STREAMS));
+
+#ifndef _WIN32
+	pid_t pid;
+	int input_pipe[2];
+	int output_pipe[2];
+
+	/* For the sake of simplicity just use -1, calling close(-1) won't hurt. */
+	int error_pipe[2] = { -1, -1 };
+	if(!merge_streams && pipe(error_pipe) != 0)
+	{
+		show_error_msg("File pipe error", "Error creating error pipe");
+		return NULL;
+	}
+
+	if(supply_input)
+	{
+		if(pipe(input_pipe) != 0)
+		{
+			show_error_msg("File pipe error", "Error creating input pipe");
+			close(error_pipe[0]);
+			close(error_pipe[1]);
+			return NULL;
+		}
+	}
+
+	if(capture_output)
+	{
+		if(pipe(output_pipe) != 0)
+		{
+			show_error_msg("File pipe error", "Error creating output pipe");
+			close(input_pipe[0]);
+			close(input_pipe[1]);
+			close(error_pipe[0]);
+			close(error_pipe[1]);
+			return NULL;
+		}
 	}
 
 	if((pid = fork()) == -1)
 	{
-		free(command);
-		return -1;
+		close(error_pipe[0]);
+		close(error_pipe[1]);
+		if(supply_input)
+		{
+			close(input_pipe[0]);
+			close(input_pipe[1]);
+		}
+		if(capture_output)
+		{
+			close(output_pipe[0]);
+			close(output_pipe[1]);
+		}
+		return NULL;
 	}
 
 	if(pid == 0)
 	{
 		extern char **environ;
 
-		int nullfd;
+		int stderr_pipe = (merge_streams ? output_pipe[1] : error_pipe[1]);
+
 		/* Redirect stderr to write end of pipe. */
-		if(dup2(error_pipe[1], STDERR_FILENO) == -1)
+		if(dup2(stderr_pipe, STDERR_FILENO) == -1)
 		{
 			perror("dup2");
 			_Exit(EXIT_FAILURE);
@@ -837,118 +976,232 @@ bg_run_external(const char cmd[], int skip_errors, ShellRequester by)
 		close(STDIN_FILENO);
 		close(STDOUT_FILENO);
 		/* Close read end of pipe. */
-		close(error_pipe[0]);
+		if(error_pipe[0] != -1)
+		{
+			close(error_pipe[0]);
+		}
 
-		/* Attach stdout, stdin to /dev/null. */
-		nullfd = open("/dev/null", O_RDWR);
+		if(supply_input)
+		{
+			if(dup2(input_pipe[0], STDIN_FILENO) == -1)
+			{
+				perror("dup2");
+				_Exit(EXIT_FAILURE);
+			}
+			/* Close write end of pipe. */
+			close(input_pipe[1]);
+		}
+
+		if(capture_output)
+		{
+			if(dup2(output_pipe[1], STDOUT_FILENO) == -1)
+			{
+				perror("dup2");
+				_Exit(EXIT_FAILURE);
+			}
+			/* Close read end of pipe. */
+			close(output_pipe[0]);
+		}
+
+		/* Attach stdin and optionally stdout to /dev/null. */
+		int nullfd = open("/dev/null", O_RDWR);
 		if(nullfd != -1)
 		{
-			if(dup2(nullfd, STDIN_FILENO) == -1)
+			if(!supply_input && dup2(nullfd, STDIN_FILENO) == -1)
 			{
 				perror("dup2 for stdin");
 				_Exit(EXIT_FAILURE);
 			}
-			if(dup2(nullfd, STDOUT_FILENO) == -1)
+			if(!capture_output && dup2(nullfd, STDOUT_FILENO) == -1)
 			{
 				perror("dup2 for stdout");
 				_Exit(EXIT_FAILURE);
 			}
 		}
 
-		setpgid(0, 0);
+		/* setsid() creates process group as well and doesn't work if current
+		 * process is a group leader, so don't do setpgid(). */
+		if(setsid() == (pid_t)-1)
+		{
+			perror("setsid");
+			_Exit(EXIT_FAILURE);
+		}
 
 		prepare_for_exec();
 		char *sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "-c");
 		execve(get_execv_path(cfg.shell),
-				make_execv_array(cfg.shell, sh_flag, command), environ);
+				make_execv_array(cfg.shell, sh_flag, strdup(cmd)), environ);
 		_Exit(127);
 	}
-	else
-	{
-		/* Close write end of pipe. */
-		close(error_pipe[1]);
 
-		job = add_background_job(pid, command, (uintptr_t)error_pipe[0], 0,
-				BJT_COMMAND);
-		if(job == NULL)
+	/* Close unused ends of pipes. */
+	if(error_pipe[1] != -1)
+	{
+		close(error_pipe[1]);
+	}
+	if(supply_input)
+	{
+		close(input_pipe[0]);
+	}
+	if(capture_output)
+	{
+		close(output_pipe[1]);
+	}
+
+	bg_job_t *job = add_background_job(pid, cmd, (uintptr_t)error_pipe[0], 0,
+			BJT_COMMAND, jb_visible);
+
+	if(supply_input)
+	{
+		job->input = fdopen(input_pipe[1], "w");
+		if(job->input == NULL)
 		{
-			free(command);
-			return -1;
+			close(input_pipe[1]);
 		}
 	}
-	free(command);
+
+	if(capture_output)
+	{
+		job->output = fdopen(output_pipe[0], "r");
+		if(job->output == NULL)
+		{
+			close(output_pipe[0]);
+		}
+	}
+
+	return job;
 #else
-	BOOL ret;
 	STARTUPINFOW startup = { .dwFlags = STARTF_USESTDHANDLES };
 	PROCESS_INFORMATION pinfo;
-	char *command;
 	char *sh_cmd;
 	wchar_t *wide_cmd;
 
-	command = cfg.fast_run ? fast_run_complete(cmd) : strdup(cmd);
-	if(command == NULL)
-	{
-		return -1;
-	}
-
-	SECURITY_ATTRIBUTES sec_attr = {
-		.nLength = sizeof(sec_attr),
-		.lpSecurityDescriptor = NULL,
-		.bInheritHandle = 1,
-	};
-
-	HANDLE hnul = CreateFileA("NUL", GENERIC_READ | GENERIC_WRITE, 0, &sec_attr,
+	HANDLE hnul = CreateFileA("NUL", GENERIC_READ | GENERIC_WRITE, 0, NULL,
 			OPEN_EXISTING, 0, NULL);
 	if(hnul == INVALID_HANDLE_VALUE)
 	{
-		free(command);
-		return -1;
+		return NULL;
 	}
 	startup.hStdInput = hnul;
 	startup.hStdOutput = hnul;
 
-	HANDLE herr;
-	if(!CreatePipe(&herr, &startup.hStdError, &sec_attr, 16*1024))
+	HANDLE herr = INVALID_HANDLE_VALUE;
+	if(!merge_streams && !CreatePipe(&herr, &startup.hStdError, NULL, 16*1024))
 	{
 		CloseHandle(hnul);
-		free(command);
-		return -1;
+		return NULL;
 	}
 
-	sh_cmd = win_make_sh_cmd(command, by);
-	free(command);
-
-	wide_cmd = to_wide(sh_cmd);
-	ret = CreateProcessW(NULL, wide_cmd, NULL, NULL, 1, 0, NULL, NULL, &startup,
-			&pinfo);
-	free(wide_cmd);
-	CloseHandle(hnul);
-	CloseHandle(startup.hStdError);
-
-	if(ret != 0)
+	HANDLE hin = INVALID_HANDLE_VALUE;
+	if(supply_input && !CreatePipe(&startup.hStdInput, &hin, NULL, 16*1024))
 	{
-		CloseHandle(pinfo.hThread);
+		CloseHandle(herr);
+		CloseHandle(hnul);
+		return NULL;
+	}
 
-		job = add_background_job(pinfo.dwProcessId, sh_cmd, (uintptr_t)herr,
-				(uintptr_t)pinfo.hProcess, BJT_COMMAND);
-		if(job == NULL)
+	HANDLE hout = INVALID_HANDLE_VALUE;
+	if(capture_output)
+	{
+		if(!CreatePipe(&hout, &startup.hStdOutput, NULL, 16*1024))
 		{
-			free(sh_cmd);
-			return -1;
+			CloseHandle(herr);
+			CloseHandle(hin);
+			CloseHandle(hnul);
+			return NULL;
+		}
+
+		if(merge_streams)
+		{
+			startup.hStdError = startup.hStdOutput;
 		}
 	}
-	free(sh_cmd);
-	if(ret == 0)
-	{
-		return 1;
-	}
-#endif
 
-	if(job != NULL)
+	SetHandleInformation(startup.hStdInput, HANDLE_FLAG_INHERIT, 1);
+	SetHandleInformation(startup.hStdOutput, HANDLE_FLAG_INHERIT, 1);
+	SetHandleInformation(startup.hStdError, HANDLE_FLAG_INHERIT, 1);
+
+	sh_cmd = win_make_sh_cmd(cmd, by);
+
+	wide_cmd = to_wide(sh_cmd);
+	int started = CreateProcessW(NULL, wide_cmd, NULL, NULL, 1, CREATE_SUSPENDED,
+			NULL, NULL, &startup, &pinfo);
+	free(wide_cmd);
+	CloseHandle(hnul);
+	if(startup.hStdInput != INVALID_HANDLE_VALUE)
 	{
-		job->skip_errors = skip_errors;
+		CloseHandle(startup.hStdInput);
 	}
-	return 0;
+	CloseHandle(startup.hStdOutput);
+	if(startup.hStdError != startup.hStdOutput)
+	{
+		CloseHandle(startup.hStdError);
+	}
+
+	if(!started)
+	{
+		free(sh_cmd);
+		CloseHandle(hout);
+		CloseHandle(hin);
+		return NULL;
+	}
+
+	/* Put the process into its own job object and start its main thread. */
+	HANDLE hjob = CreateJobObject(NULL, NULL);
+	AssignProcessToJobObject(hjob, pinfo.hProcess);
+	ResumeThread(pinfo.hThread);
+	CloseHandle(pinfo.hThread);
+
+	bg_job_t *job = add_background_job(pinfo.dwProcessId, sh_cmd,
+			(uintptr_t)herr, (uintptr_t)pinfo.hProcess, BJT_COMMAND, jb_visible);
+	free(sh_cmd);
+
+	if(job == NULL)
+	{
+		CloseHandle(herr);
+		CloseHandle(hin);
+		CloseHandle(hout);
+		CloseHandle(pinfo.hProcess);
+		CloseHandle(hjob);
+		return NULL;
+	}
+
+	job->hjob = hjob;
+
+	if(supply_input)
+	{
+		const int fd = _open_osfhandle((intptr_t)hin, _O_WRONLY);
+		if(fd == -1)
+		{
+			CloseHandle(hin);
+			return NULL;
+		}
+
+		job->input = fdopen(fd, "w");
+		if(job->input == NULL)
+		{
+			close(fd);
+		}
+	}
+
+	if(capture_output)
+	{
+		const int fd = _open_osfhandle((intptr_t)hout, _O_RDONLY);
+		if(fd == -1)
+		{
+			CloseHandle(hout);
+			return NULL;
+		}
+
+		job->output = fdopen(fd, "r");
+		if(job->output == NULL)
+		{
+			close(fd);
+		}
+	}
+
+	return job;
+#endif
 }
 
 int
@@ -967,7 +1220,7 @@ bg_execute(const char descr[], const char op_descr[], int total, int important,
 	task_args->func = task_func;
 	task_args->args = args;
 	task_args->job = add_background_job(WRONG_PID, descr, (uintptr_t)NO_JOB_ID,
-			(uintptr_t)NO_JOB_ID, important ? BJT_OPERATION : BJT_TASK);
+			(uintptr_t)NO_JOB_ID, important ? BJT_OPERATION : BJT_TASK, 1);
 
 	if(task_args->job == NULL)
 	{
@@ -980,7 +1233,7 @@ bg_execute(const char descr[], const char op_descr[], int total, int important,
 
 	if(task_args->job->type == BJT_OPERATION)
 	{
-		ui_stat_job_bar_add(&task_args->job->bg_op);
+		place_on_job_bar(task_args->job);
 	}
 
 	ret = 0;
@@ -999,11 +1252,31 @@ bg_execute(const char descr[], const char op_descr[], int total, int important,
 	return ret;
 }
 
+/* Makes the job appear on the job bar. */
+static void
+place_on_job_bar(bg_job_t *job)
+{
+	assert(job->with_bg_op && "This function requires bg_op data.");
+	assert(!job->on_job_bar  && "This function requires should be called once.");
+	ui_stat_job_bar_add(&job->bg_op);
+	job->on_job_bar = 1;
+}
+
+/* Removes the job from the job bar. */
+static void
+get_off_job_bar(bg_job_t *job)
+{
+	assert(job->with_bg_op && "This function requires bg_op data.");
+	assert(job->on_job_bar  && "This function requires should be called once.");
+	ui_stat_job_bar_remove(&job->bg_op);
+	job->on_job_bar = 0;
+}
+
 /* Creates structure that describes background job and registers it in the list
  * of jobs. */
 static bg_job_t *
 add_background_job(pid_t pid, const char cmd[], uintptr_t err, uintptr_t data,
-		BgJobType type)
+		BgJobType type, int with_bg_op)
 {
 	bg_job_t *new = malloc(sizeof(*new));
 	if(new == NULL)
@@ -1025,18 +1298,23 @@ add_background_job(pid_t pid, const char cmd[], uintptr_t err, uintptr_t data,
 	pthread_spin_init(&new->errors_lock, PTHREAD_PROCESS_PRIVATE);
 	pthread_spin_init(&new->status_lock, PTHREAD_PROCESS_PRIVATE);
 	new->running = 1;
-	new->in_use = (type == BJT_COMMAND);
+	new->use_count = 0;
 	new->exit_code = -1;
 
+	new->input = NULL;
+	new->output = NULL;
+
 #ifndef _WIN32
 	new->err_stream = (int)err;
 #else
 	new->err_stream = (HANDLE)err;
 	new->hprocess = (HANDLE)data;
+	new->hjob = INVALID_HANDLE_VALUE;
 #endif
 
 	if(new->err_stream != NO_JOB_ID)
 	{
+		++new->use_count;
 		pthread_mutex_lock(&new_err_jobs_lock);
 		new->err_next = new_err_jobs;
 		new_err_jobs = new;
@@ -1044,7 +1322,9 @@ add_background_job(pid_t pid, const char cmd[], uintptr_t err, uintptr_t data,
 		pthread_cond_signal(&new_err_jobs_cond);
 	}
 
-	if(type != BJT_COMMAND)
+	new->with_bg_op = with_bg_op;
+	new->on_job_bar = 0;
+	if(new->with_bg_op)
 	{
 		pthread_spin_init(&new->bg_op_lock, PTHREAD_PROCESS_PRIVATE);
 	}
@@ -1054,6 +1334,8 @@ add_background_job(pid_t pid, const char cmd[], uintptr_t err, uintptr_t data,
 	new->bg_op.descr = NULL;
 	new->bg_op.cancelled = 0;
 
+	new->in_menu = 1;
+
 	bg_jobs = new;
 	return new;
 }
@@ -1073,10 +1355,7 @@ background_task_bootstrap(void *arg)
 	task_args->func(&task_args->job->bg_op, task_args->args);
 
 	/* Mark task as finished normally. */
-	pthread_spin_lock(&task_args->job->status_lock);
-	task_args->job->running = 0;
-	task_args->job->exit_code = 0;
-	pthread_spin_unlock(&task_args->job->status_lock);
+	mark_job_finished(task_args->job, 0);
 
 	free(task_args);
 
@@ -1103,13 +1382,6 @@ make_current_job_key(void)
 int
 bg_has_active_jobs(int important_only)
 {
-	if(bg_jobs_freeze() != 0)
-	{
-		/* Failed to lock jobs list and using safe choice: pretend there are active
-		 * tasks. */
-		return 1;
-	}
-
 	bg_job_t *job;
 	int running = 0;
 	for(job = bg_jobs; job != NULL && !running; job = job->next)
@@ -1121,28 +1393,9 @@ bg_has_active_jobs(int important_only)
 		}
 	}
 
-	bg_jobs_unfreeze();
-
 	return running;
 }
 
-int
-bg_jobs_freeze(void)
-{
-	/* SIGCHLD needs to be blocked anytime the jobs list is accessed from anywhere
-	 * except the received_sigchld(). */
-	return set_sigchld(1);
-}
-
-void
-bg_jobs_unfreeze(void)
-{
-	/* Unblock SIGCHLD signal. */
-	/* FIXME: maybe store previous state of SIGCHLD and don't unblock if it was
-	 *        blocked. */
-	(void)set_sigchld(0);
-}
-
 int
 bg_job_cancel(bg_job_t *job)
 {
@@ -1165,7 +1418,7 @@ bg_job_cancel(bg_job_t *job)
 				(unsigned long long)job->pid);
 	}
 #else
-	if(win_cancel_process(job->pid, job->hprocess) == 0)
+	if(win_cancel_process(job->pid) == 0)
 	{
 		job->cancelled = 1;
 	}
@@ -1188,20 +1441,149 @@ bg_job_cancelled(bg_job_t *job)
 	return job->cancelled;
 }
 
+void
+bg_job_terminate(bg_job_t *job)
+{
+	if(job->type != BJT_COMMAND || !bg_job_is_running(job))
+	{
+		return;
+	}
+
+#ifndef _WIN32
+	if(kill(job->pid, SIGKILL) != 0)
+	{
+		LOG_SERROR_MSG(errno, "Failed to send SIGKILL to %" PRINTF_ULL,
+				(unsigned long long)job->pid);
+	}
+#else
+	if(!TerminateJobObject(job->hjob, 0))
+	{
+		LOG_ERROR_MSG("Failed to terminate job of process %" PRINTF_ULL,
+				(unsigned long long)job->pid);
+		LOG_WERROR(GetLastError());
+	}
+#endif
+}
+
 int
 bg_job_is_running(bg_job_t *job)
 {
-	int running;
 	pthread_spin_lock(&job->status_lock);
-	running = job->running;
+	int running = job->running;
+	pthread_spin_unlock(&job->status_lock);
+	return (running && update_job_status(job));
+}
+
+int
+bg_job_was_killed(bg_job_t *job)
+{
+	pthread_spin_lock(&job->status_lock);
+	int running = job->running;
+	int exit_code = job->exit_code;
+	pthread_spin_unlock(&job->status_lock);
+	return (!running && exit_code >= 0);
+}
+
+int
+bg_job_wait(bg_job_t *job)
+{
+	assert(job->type == BJT_COMMAND &&
+			"Only external commands can be waited for.");
+
+	if(!bg_job_is_running(job))
+	{
+		return 0;
+	}
+
+	/* Close input to avoid situation when the job is blocked on read. */
+	if(job->input != NULL)
+	{
+		fclose(job->input);
+		job->input = NULL;
+	}
+
+	/* Close output to avoid situation when the job is blocked on write. */
+	if(job->output != NULL)
+	{
+		fclose(job->output);
+		job->output = NULL;
+	}
+
+#ifndef _WIN32
+	int status = get_proc_exit_status(job->pid);
+	if(status == -1)
+	{
+		return 1;
+	}
+	mark_job_finished(job, status_to_exit_code(status));
+	return 0;
+#else
+	if(WaitForSingleObject(job->hprocess, INFINITE) != WAIT_OBJECT_0)
+	{
+		return 1;
+	}
+	return update_job_status(job);
+#endif
+}
+
+/* Retrieves exit code of a process associated with the job.  Returns zero on
+ * success (job has just finished), otherwise non-zero is returned. */
+static int
+update_job_status(bg_job_t *job)
+{
+#ifndef _WIN32
+	int status;
+	if(job->pid != -1 && waitpid(job->pid, &status, WNOHANG) == job->pid)
+	{
+		mark_job_finished(job, status_to_exit_code(status));
+		return 0;
+	}
+	return 1;
+#else
+	DWORD retcode;
+	if(GetExitCodeProcess(job->hprocess, &retcode))
+	{
+		if(retcode != STILL_ACTIVE)
+		{
+			mark_job_finished(job, retcode);
+			return 0;
+		}
+	}
+	return 1;
+#endif
+}
+
+/* Marks job as finished with the specified exit code. */
+static void
+mark_job_finished(bg_job_t *job, int exit_code)
+{
+	pthread_spin_lock(&job->status_lock);
+	job->running = 0;
+	job->exit_code = exit_code;
+	pthread_spin_unlock(&job->status_lock);
+}
+
+void
+bg_job_incref(bg_job_t *job)
+{
+	pthread_spin_lock(&job->status_lock);
+	++job->use_count;
+	pthread_spin_unlock(&job->status_lock);
+}
+
+void
+bg_job_decref(bg_job_t *job)
+{
+	pthread_spin_lock(&job->status_lock);
+	--job->use_count;
 	pthread_spin_unlock(&job->status_lock);
-	return running;
 }
 
 void
 bg_op_lock(bg_op_t *bg_op)
 {
 	bg_job_t *const job = STRUCT_FROM_FIELD(bg_job_t, bg_op, bg_op);
+	assert(job->with_bg_op && "This function requires bg_op data.");
 	pthread_spin_lock(&job->bg_op_lock);
 }
 
@@ -1209,6 +1591,7 @@ void
 bg_op_unlock(bg_op_t *bg_op)
 {
 	bg_job_t *const job = STRUCT_FROM_FIELD(bg_job_t, bg_op, bg_op);
+	assert(job->with_bg_op && "This function requires bg_op data.");
 	pthread_spin_unlock(&job->bg_op_lock);
 }
 
