@@ -45,6 +45,18 @@ typedef enum
 }
 BgJobType;
 
+/* Job flags passed to bg_run_external_job(). */
+typedef enum
+{
+	BJF_NONE            = 0,      /* No flags set. */
+	BJF_JOB_BAR_VISIBLE = 1 << 0, /* Makes the job appear on the job bar. */
+	BJF_MENU_VISIBLE    = 1 << 1, /* Makes the job appear in :jobs menu. */
+	BJF_SUPPLY_INPUT    = 1 << 2, /* Open a pipe for standard input stream. */
+	BJF_CAPTURE_OUT     = 1 << 3, /* Capture output stream(s). */
+	BJF_MERGE_STREAMS   = 1 << 4, /* Merge error stream into output stream. */
+}
+BgJobFlags;
+
 /* Auxiliary structure to be updated by background tasks while they progress. */
 typedef struct bg_op_t
 {
@@ -78,40 +90,56 @@ typedef struct bg_job_t
 	/* The lock is meant to guard state-related fields. */
 	pthread_spinlock_t status_lock;
 	int running;   /* Whether this job is still running. */
-	int in_use;    /* Whether this job description is in use by someone. */
-	/* TODO: use or remove this (set to correct value, but not used). */
+	int use_count; /* Count of uses of this job entry. */
 	int exit_code; /* Exit code of external command. */
 
-	/* For background operations and tasks. */
-	pthread_spinlock_t bg_op_lock;
-	bg_op_t bg_op;
+	FILE *input;  /* File stream of standard input or NULL. */
+	FILE *output; /* File stream of standard output or NULL. */
+
+	int with_bg_op;                /* Whether bg_op* fields are active. */
+	int on_job_bar;                /* Whether this task was put on a job bar. */
+	pthread_spinlock_t bg_op_lock; /* Lock for accessing bg_op field. */
+	bg_op_t bg_op;                 /* Progress and cancellation information. */
 
 #ifndef _WIN32
-	int fd;
+	int err_stream;    /* stderr stream of the job or -1. */
 #else
-	HANDLE hprocess;
+	HANDLE err_stream; /* stderr stream of the job or invalid handle. */
+	HANDLE hprocess;   /* Handle to the process of the job or invalid handle. */
+	HANDLE hjob;       /* Handle to the process job object. */
 #endif
 
 	struct bg_job_t *next;     /* Link to the next element in bg_jobs list. */
 
 	/* Used by error thread for BJT_COMMAND jobs. */
-	int drained;               /* Whether error stream of no interest anymore. */
 	struct bg_job_t *err_next; /* Link to the next element in error read list. */
+	int drained;               /* Whether error stream of no interest anymore. */
+
+	int in_menu; /* Whether this task is visible in :jobs menu. */
 }
 bg_job_t;
 
 /* Background task entry point function signature. */
 typedef void (*bg_task_func)(bg_op_t *bg_op, void *arg);
 
-/* List of background jobs.  Use bg_jobs_freeze() before accessing it. */
+/* List of background jobs. */
 extern bg_job_t *bg_jobs;
 
 /* Prepare background unit for the work. */
 void bg_init(void);
 
 /* Creates background job running external command.  Returns zero on success,
- * otherwise non-zero is returned. */
-int bg_run_external(const char cmd[], int skip_errors, ShellRequester by);
+ * otherwise non-zero is returned.  If *input is not NULL, it's set to input
+ * pipe of the background process.  The caller becomes responsible for the
+ * stream. */
+int bg_run_external(const char cmd[], int skip_errors, ShellRequester by,
+		FILE **input);
+
+/* Creates background job running external command which does not interact with
+ * the user and is detached from controlling terminal.  Upon creation the job
+ * has one extra use, which needs to be decremented for it to be freed.  Returns
+ * the job or NULL on error. */
+bg_job_t * bg_run_external_job(const char cmd[], BgJobFlags flags);
 
 struct cancellation_t;
 
@@ -122,13 +150,12 @@ int bg_and_wait_for_errors(char cmd[],
 		const struct cancellation_t *cancellation);
 
 /* Runs command in a background and redirects its stdout and stderr streams to
- * file streams which are set.  Returns id of background process ((pid_t)0 for
- * non-*nix like systems) or (pid_t)-1 on error. */
-pid_t bg_run_and_capture(char cmd[], int user_sh, FILE **out, FILE **err);
-
-/* Callback-like function that marks background job specified by its process id,
- * which is finished with the exit_code. */
-void bg_process_finished_cb(pid_t pid, int exit_code);
+ * file streams which are set.  Input is redirected only if in parameter isn't
+ * NULL.  Don't pass pipe for input, it can cause deadlock.  Returns id of
+ * background process ((pid_t)0 for non-*nix like systems) or (pid_t)-1 on
+ * error. */
+pid_t bg_run_and_capture(char cmd[], int user_sh, FILE *in, FILE **out,
+		FILE **err);
 
 /* Checks status of background jobs (their streams and state).  Removes finished
  * ones from the list, displays any pending error messages, corrects job bar if
@@ -140,17 +167,10 @@ void bg_check(void);
 int bg_execute(const char descr[], const char op_descr[], int total,
 		int important, bg_task_func task_func, void *args);
 
-/* Checks whether there are any internal jobs (not external applications tracked
- * by vifm) running in background. */
-int bg_has_active_jobs(void);
-
-/* Performs preparations necessary for safe access of the bg_jobs list.  Effect
- * of calling this function must be reverted by calling bg_jobs_unfreeze().
- * Returns zero on success, otherwise non-zero is returned. */
-int bg_jobs_freeze(void);
-
-/* Undoes changes made by bg_jobs_freeze(). */
-void bg_jobs_unfreeze(void);
+/* Checks whether there are any internal jobs (important_only is non-zero) or
+ * jobs or tasks (important_only is zero) running in background.  External
+ * applications whose state is tracked are always ignored by this function. */
+int bg_has_active_jobs(int important_only);
 
 /* Cancels the job.  Returns non-zero if job wasn't cancelled before, but is
  * after this call, otherwise zero is returned. */
@@ -160,10 +180,29 @@ int bg_job_cancel(bg_job_t *job);
  * zero is returned. */
 int bg_job_cancelled(bg_job_t *job);
 
+/* Terminates the job in a forceful way leaving it no chance to respond. */
+void bg_job_terminate(bg_job_t *job);
+
 /* Checks whether the job is still running.  Returns non-zero if so, otherwise
  * zero is returned. */
 int bg_job_is_running(bg_job_t *job);
 
+/* Checks whether the job was killed.  Returns non-zero if so, otherwise zero is
+ * returned. */
+int bg_job_was_killed(bg_job_t *job);
+
+/* Waits for external command to finish (don't pass any other kind of job).
+ * Returns zero on success, otherwise non-zero is returned. */
+int bg_job_wait(bg_job_t *job);
+
+/* Increases use counter of the job.  Doing this prevents object deletion while
+ * it's still in use. */
+void bg_job_incref(bg_job_t *job);
+
+/* Decreases use counter of the job.  Assume the object is freed after calling
+ * this functions (unless you've incremented the counter more than once). */
+void bg_job_decref(bg_job_t *job);
+
 /* Temporary locks bg_op_t structure to ensure that it's not modified by
  * anyone during reading/updating its fields.  The structure must be part of
  * bg_job_t. */
