@@ -314,58 +314,58 @@ static key_pair_t key_pairs[] = {
 	{ L"<c-m-y>",    L"\033"L"\x19"       },
 	{ L"<c-m-z>",    L"\033"L"\x1a"       },
 #else
-	{ L"<a-a>",      { ALT_A }            },
-	{ L"<a-b>",      { ALT_B }            },
-	{ L"<a-c>",      { ALT_C }            },
-	{ L"<a-d>",      { ALT_D }            },
-	{ L"<a-e>",      { ALT_E }            },
-	{ L"<a-f>",      { ALT_F }            },
-	{ L"<a-g>",      { ALT_G }            },
-	{ L"<a-h>",      { ALT_H }            },
-	{ L"<a-i>",      { ALT_I }            },
-	{ L"<a-j>",      { ALT_J }            },
-	{ L"<a-k>",      { ALT_K }            },
-	{ L"<a-l>",      { ALT_L }            },
-	{ L"<a-m>",      { ALT_M }            },
-	{ L"<a-n>",      { ALT_N }            },
-	{ L"<a-o>",      { ALT_O }            },
-	{ L"<a-p>",      { ALT_P }            },
-	{ L"<a-q>",      { ALT_Q }            },
-	{ L"<a-r>",      { ALT_R }            },
-	{ L"<a-s>",      { ALT_S }            },
-	{ L"<a-t>",      { ALT_T }            },
-	{ L"<a-u>",      { ALT_U }            },
-	{ L"<a-v>",      { ALT_V }            },
-	{ L"<a-w>",      { ALT_W }            },
-	{ L"<a-x>",      { ALT_X }            },
-	{ L"<a-y>",      { ALT_Y }            },
-	{ L"<a-z>",      { ALT_Z }            },
-	{ L"<m-a>",      { ALT_A }            },
-	{ L"<m-b>",      { ALT_B }            },
-	{ L"<m-c>",      { ALT_C }            },
-	{ L"<m-d>",      { ALT_D }            },
-	{ L"<m-e>",      { ALT_E }            },
-	{ L"<m-f>",      { ALT_F }            },
-	{ L"<m-g>",      { ALT_G }            },
-	{ L"<m-h>",      { ALT_H }            },
-	{ L"<m-i>",      { ALT_I }            },
-	{ L"<m-j>",      { ALT_J }            },
-	{ L"<m-k>",      { ALT_K }            },
-	{ L"<m-l>",      { ALT_L }            },
-	{ L"<m-m>",      { ALT_M }            },
-	{ L"<m-n>",      { ALT_N }            },
-	{ L"<m-o>",      { ALT_O }            },
-	{ L"<m-p>",      { ALT_P }            },
-	{ L"<m-q>",      { ALT_Q }            },
-	{ L"<m-r>",      { ALT_R }            },
-	{ L"<m-s>",      { ALT_S }            },
-	{ L"<m-t>",      { ALT_T }            },
-	{ L"<m-u>",      { ALT_U }            },
-	{ L"<m-v>",      { ALT_V }            },
-	{ L"<m-w>",      { ALT_W }            },
-	{ L"<m-x>",      { ALT_X }            },
-	{ L"<m-y>",      { ALT_Y }            },
-	{ L"<m-z>",      { ALT_Z }            },
+	{ L"<a-a>",      { K(ALT_A) }         },
+	{ L"<a-b>",      { K(ALT_B) }         },
+	{ L"<a-c>",      { K(ALT_C) }         },
+	{ L"<a-d>",      { K(ALT_D) }         },
+	{ L"<a-e>",      { K(ALT_E) }         },
+	{ L"<a-f>",      { K(ALT_F) }         },
+	{ L"<a-g>",      { K(ALT_G) }         },
+	{ L"<a-h>",      { K(ALT_H) }         },
+	{ L"<a-i>",      { K(ALT_I) }         },
+	{ L"<a-j>",      { K(ALT_J) }         },
+	{ L"<a-k>",      { K(ALT_K) }         },
+	{ L"<a-l>",      { K(ALT_L) }         },
+	{ L"<a-m>",      { K(ALT_M) }         },
+	{ L"<a-n>",      { K(ALT_N) }         },
+	{ L"<a-o>",      { K(ALT_O) }         },
+	{ L"<a-p>",      { K(ALT_P) }         },
+	{ L"<a-q>",      { K(ALT_Q) }         },
+	{ L"<a-r>",      { K(ALT_R) }         },
+	{ L"<a-s>",      { K(ALT_S) }         },
+	{ L"<a-t>",      { K(ALT_T) }         },
+	{ L"<a-u>",      { K(ALT_U) }         },
+	{ L"<a-v>",      { K(ALT_V) }         },
+	{ L"<a-w>",      { K(ALT_W) }         },
+	{ L"<a-x>",      { K(ALT_X) }         },
+	{ L"<a-y>",      { K(ALT_Y) }         },
+	{ L"<a-z>",      { K(ALT_Z) }         },
+	{ L"<m-a>",      { K(ALT_A) }         },
+	{ L"<m-b>",      { K(ALT_B) }         },
+	{ L"<m-c>",      { K(ALT_C) }         },
+	{ L"<m-d>",      { K(ALT_D) }         },
+	{ L"<m-e>",      { K(ALT_E) }         },
+	{ L"<m-f>",      { K(ALT_F) }         },
+	{ L"<m-g>",      { K(ALT_G) }         },
+	{ L"<m-h>",      { K(ALT_H) }         },
+	{ L"<m-i>",      { K(ALT_I) }         },
+	{ L"<m-j>",      { K(ALT_J) }         },
+	{ L"<m-k>",      { K(ALT_K) }         },
+	{ L"<m-l>",      { K(ALT_L) }         },
+	{ L"<m-m>",      { K(ALT_M) }         },
+	{ L"<m-n>",      { K(ALT_N) }         },
+	{ L"<m-o>",      { K(ALT_O) }         },
+	{ L"<m-p>",      { K(ALT_P) }         },
+	{ L"<m-q>",      { K(ALT_Q) }         },
+	{ L"<m-r>",      { K(ALT_R) }         },
+	{ L"<m-s>",      { K(ALT_S) }         },
+	{ L"<m-t>",      { K(ALT_T) }         },
+	{ L"<m-u>",      { K(ALT_U) }         },
+	{ L"<m-v>",      { K(ALT_V) }         },
+	{ L"<m-w>",      { K(ALT_W) }         },
+	{ L"<m-x>",      { K(ALT_X) }         },
+	{ L"<m-y>",      { K(ALT_Y) }         },
+	{ L"<m-z>",      { K(ALT_Z) }         },
 #endif
 	{ L"<del>",      L"\177"              },
 #ifdef ENABLE_EXTENDED_KEYS
@@ -583,7 +583,7 @@ find_notation(const wchar_t str[])
 
 	while(l <= u)
 	{
-		const int i = (l + u)/2;
+		const int i = l + (u - l)/2;
 		const key_pair_t *const key_pair = &key_pairs[i];
 		const int comp = wcsncmp(str_lowered, key_pair->notation, key_pair->len);
 		if(comp == 0)
@@ -613,7 +613,7 @@ wstr_to_spec(const wchar_t str[])
 
 	for(i = 0U; i < str_len; i += seq_len)
 	{
-		if(str[i] == L' ')
+		if(str[i] == L' ' && i != 0U && i != str_len - 1)
 		{
 			vle_tb_append(descr, " ");
 			seq_len = 1U;
