@@ -180,6 +180,7 @@ cfg_init(void)
 	cfg.cd_path = strdup(env_get_def("CDPATH", DEFAULT_CD_PATH));
 	replace_char(cfg.cd_path, ':', ',');
 
+	cfg.ellipsis_position = 0;
 	cfg.extra_padding = 1;
 	cfg.side_borders_visible = 1;
 	cfg.use_unicode_characters = 0;
@@ -192,6 +193,10 @@ cfg_init(void)
 
 	cfg.chase_links = 0;
 
+	cfg.graphics_delay = 50000;
+	cfg.hard_graphics_clear = 0;
+	cfg.top_tree_stats = 0;
+
 	cfg.timeout_len = 1000;
 	cfg.min_timeout_len = 150;
 
@@ -700,7 +705,9 @@ cfg_load(void)
 	if(!vifm_testing())
 	{
 		/* Try to load global configuration. */
-		(void)cfg_source_file(PACKAGE_SYSCONF_DIR "/" VIFMRC);
+		char rc_path[PATH_MAX + 1];
+		snprintf(rc_path, sizeof(rc_path), "%s/%s", get_sys_conf_dir(), VIFMRC);
+		(void)cfg_source_file(rc_path);
 	}
 
 	/* Try to load local configuration. */
