@@ -57,6 +57,7 @@ enum
 	COM_LET,
 	COM_MKDIR,
 	COM_MOVE,
+	COM_PLUGIN,
 	COM_PUSHD,
 	COM_RENAME,
 	COM_RLINK,
@@ -69,6 +70,7 @@ enum
 	COM_SYNC,
 	COM_TABNEW,
 	COM_TOUCH,
+	COM_TREE,
 	COM_UNLET,
 	COM_VSPLIT,
 	COM_WINCMD,
@@ -76,6 +78,10 @@ enum
 	COM_WINRUN,
 
 	COM_MENU_WRITE,
+
+	/* Single ID for all foreign commands, which are disambiguated by user
+	 * data. */
+	COM_FOREIGN,
 };
 
 /* Values of type argument for filename_completion() function. */
