@@ -35,6 +35,7 @@
 #include <string.h> /* strcmp() strcpy() strlen() */
 
 #include "cfg/config.h"
+#include "cfg/info.h"
 #include "compat/fs_limits.h"
 #include "compat/os.h"
 #include "engine/autocmds.h"
@@ -49,7 +50,6 @@
 #include "modes/normal.h"
 #include "modes/view.h"
 #include "modes/visual.h"
-#include "ui/color_manager.h"
 #include "ui/color_scheme.h"
 #include "ui/colors.h"
 #include "ui/fileview.h"
@@ -112,6 +112,8 @@ static void save_extcmd(const char command[], CmdInputType type);
 static void post(int id);
 TSTATIC void cmds_select_range(int id, const cmd_info_t *cmd_info);
 static int skip_at_beginning(int id, const char args[]);
+static void create_builtin_vars(void);
+static void session_changed_callback(const char new_session[]);
 static char * pattern_expand_hook(const char pattern[]);
 static int cmd_should_be_processed(int cmd_id);
 TSTATIC char ** break_cmdline(const char cmdline[], int for_menu);
@@ -152,7 +154,7 @@ swap_range(void)
 static int
 resolve_mark(char mark)
 {
-	const int result = check_mark_directory(curr_view, mark);
+	const int result = marks_find_in_view(curr_view, mark);
 	if(result < 0)
 	{
 		ui_sb_errf("Trying to use an invalid mark: '%c", mark);
@@ -246,9 +248,9 @@ prepare_extcmd_file(FILE *fp, const char beginning[], CmdInputType type)
 	int i;
 
 	fprintf(fp, "%s\n", beginning);
-	for(i = 0; i <= hist->pos; i++)
+	for(i = 0; i < hist->size; i++)
 	{
-		fprintf(fp, "%s\n", hist->items[i]);
+		fprintf(fp, "%s\n", hist->items[i].text);
 	}
 
 	if(is_cmd)
@@ -371,20 +373,28 @@ commands_escape_for_insertion(const char cmd_line[], int pos, const char str[])
 static void
 post(int id)
 {
-	if(id != COM_GOTO && curr_view->selected_files > 0 && !keep_view_selection)
+	if(!keep_view_selection)
 	{
-		ui_view_reset_selection_and_reload(curr_view);
+		flist_sel_stash_if_nonempty(curr_view);
 	}
+
+	/* Marking made by this unit is for a single command only and shouldn't be
+	 * processed outside of that command. */
+	curr_view->pending_marking = 0;
 }
 
 TSTATIC void
 cmds_select_range(int id, const cmd_info_t *cmd_info)
 {
-	if(flist_sel_range(curr_view, cmd_info->begin, cmd_info->end,
-				(id != COM_FIND && id != COM_GREP)))
+	if(curr_view->pending_marking)
 	{
-		curr_view->user_selection = 0;
+		/* It must come from parent :command, so keep it. */
+		return;
 	}
+
+	int mark_current = (id != COM_FIND && id != COM_GREP);
+	curr_view->pending_marking = flist_sel_range(curr_view, cmd_info->begin,
+			cmd_info->end, mark_current);
 }
 
 /* Command prefix remover for command parsing unit.  Returns < 0 to do nothing
@@ -426,9 +436,53 @@ init_commands(void)
 	init_bracket_notation();
 	init_variables();
 
+	create_builtin_vars();
+	sessions_set_callback(&session_changed_callback);
+
 	vle_aucmd_set_expand_hook(&pattern_expand_hook);
 }
 
+/* Creates builtin variables with some default values. */
+static void
+create_builtin_vars(void)
+{
+	var_t var = var_from_int(0);
+	setvar("v:count", var);
+	setvar("v:count1", var);
+	var_free(var);
+
+	var = var_from_str("");
+	setvar("v:session", var);
+	var_free(var);
+
+	var = var_from_int(0);
+	setvar("v:jobcount", var);
+	var_free(var);
+}
+
+/* Callback to be invoked when active session has changed. */
+static void
+session_changed_callback(const char new_session[])
+{
+	var_t var = var_from_str(new_session);
+	setvar("v:session", var);
+	var_free(var);
+}
+
+void
+cmds_vars_set_count(int count)
+{
+	var_t var;
+
+	var = var_from_int(count == NO_COUNT_GIVEN ? 0 : count);
+	setvar("v:count", var);
+	var_free(var);
+
+	var = var_from_int(count == NO_COUNT_GIVEN ? 1 : count);
+	setvar("v:count1", var);
+	var_free(var);
+}
+
 /* Performs custom pattern expansion.  Allocate new expanded string. */
 static char *
 pattern_expand_hook(const char pattern[])
@@ -791,7 +845,7 @@ break_cmdline(const char cmdline[], int for_menu)
 
 	if(*cmdline == '\0')
 	{
-		len = add_to_string_array(&cmds, len, 1, cmdline);
+		len = add_to_string_array(&cmds, len, cmdline);
 		goto finish;
 	}
 
@@ -862,7 +916,7 @@ break_cmdline(const char cmdline[], int for_menu)
 				processed = raw;
 			}
 
-			len = add_to_string_array(&cmds, len, 1, cmdline);
+			len = add_to_string_array(&cmds, len, cmdline);
 
 			if(args_kind == CAT_UNTIL_THE_END)
 			{
@@ -900,7 +954,7 @@ is_out_of_arg(const char cmd[], const char pos[])
 			return 1;
 		}
 		if(get_cmd_args_type(cmd) == CAT_EXPR &&
-				pos != cmd && *pos == '|' && pos[-1] != '|' && pos[1] != '|')
+				pos != cmd && pos[0] == '|' && pos[-1] != '|' && pos[1] != '|')
 		{
 			/* For "*[^|]|[^|]*" report that we're out of argument. */
 			return 1;
@@ -1078,15 +1132,15 @@ exec_command(const char cmd[], view_t *view, CmdInputType type)
 	{
 		case CIT_BSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_FSEARCH_PATTERN:
-			return find_npattern(view, cmd, backward, 1);
+			return modnorm_find(view, cmd, backward, 1);
 
 		case CIT_VBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VFSEARCH_PATTERN:
-			return find_vpattern(view, cmd, backward, 1);
+			return modvis_find(view, cmd, backward, 1);
 
 		case CIT_VWBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VWFSEARCH_PATTERN:
-			return view_find_pattern(cmd, backward);
+			return modview_find(cmd, backward);
 
 		case CIT_MENU_COMMAND: menu = 1; /* Fall through. */
 		case CIT_COMMAND:
@@ -1120,15 +1174,15 @@ repeat_command(view_t *view, CmdInputType type)
 	{
 		case CIT_BSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_FSEARCH_PATTERN:
-			return find_npattern(view, hists_search_last(), backward, 1);
+			return modnorm_find(view, hists_search_last(), backward, 1);
 
 		case CIT_VBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VFSEARCH_PATTERN:
-			return find_vpattern(view, hists_search_last(), backward, 1);
+			return modvis_find(view, hists_search_last(), backward, 1);
 
 		case CIT_VWBSEARCH_PATTERN: backward = 1; /* Fall through. */
 		case CIT_VWFSEARCH_PATTERN:
-			return view_find_pattern(NULL, backward);
+			return modview_find(NULL, backward);
 
 		case CIT_COMMAND:
 			return execute_command(view, NULL, 0);
