@@ -19,7 +19,7 @@
 #include "cmds.h"
 
 #include <assert.h> /* assert() */
-#include <ctype.h> /* isalpha() isdigit() isspace() */
+#include <ctype.h> /* isalnum() isalpha() isdigit() isspace() */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h>
 #include <stdlib.h> /* calloc() malloc() free() realloc() */
@@ -43,7 +43,7 @@ typedef struct
 	cmd_t head;
 	cmd_add_t user_cmd_handler;
 	cmd_handler command_handler;
-	int udf_count;
+	int custom_cmd_count;        /* Number of non-builtin commands. */
 }
 inner_t;
 
@@ -61,14 +61,19 @@ static const char * get_cmd_name(const char cmd[], char buf[], size_t buf_len);
 static void init_cmd_info(cmd_info_t *cmd_info);
 static const char * skip_prefix_commands(const char cmd[]);
 static cmd_t * find_cmd(const char name[]);
+static cmd_t * find_cmd_advance(cmd_t *cmd, const char name[]);
+static int find_cmd_match(cmd_t *cmd, const char name[]);
 static const char * parse_range(const char cmd[], cmd_info_t *cmd_info);
 static const char * parse_range_elem(const char cmd[], cmd_info_t *cmd_info,
 		char last_sep);
 static int complete_cmd_args(cmd_t *cur, const char args[],
 		cmd_info_t *cmd_info, void *arg);
 static void complete_cmd_name(const char cmd_name[], int user_only);
-TSTATIC int add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf);
+TSTATIC int add_builtin_cmd(const char name[], CMD_TYPE type,
+		const cmd_add_t *conf);
+static int is_builtin_like_name_ok(const char name[]);
 static int comclear_cmd(const cmd_info_t *cmd_info);
+static void remove_commands(CMD_TYPE type);
 static int command_cmd(const cmd_info_t *cmd_info);
 static void init_command_flags(cmd_t *cmd, int flags);
 static const char * get_user_cmd_name(const char cmd[], char buf[],
@@ -131,6 +136,11 @@ vle_cmds_init(int udf, cmds_conf_t *conf)
 void
 vle_cmds_reset(void)
 {
+	if(inner == NULL)
+	{
+		return;
+	}
+
 	cmd_t *cur = inner->head.next;
 
 	while(cur != NULL)
@@ -146,14 +156,22 @@ vle_cmds_reset(void)
 	inner->user_cmd_handler.handler = NULL;
 
 	free(inner);
+	inner = NULL;
 	cmds_conf->inner = NULL;
 }
 
+void
+vle_cmds_clear(void)
+{
+	remove_commands(USER_CMD);
+	remove_commands(FOREIGN_CMD);
+}
+
 int
 vle_cmds_run(const char cmd[])
 {
 	cmd_info_t cmd_info;
-	char cmd_name[MAX_CMD_NAME_LEN];
+	char cmd_name[MAX_CMD_NAME_LEN + 1];
 	cmd_t *cur;
 	const char *args;
 	int execution_code;
@@ -274,11 +292,13 @@ vle_cmds_run(const char cmd[])
 
 		if(cur->type == USER_CMD)
 		{
-			cmd_info.cmd = cur->cmd;
+			cmd_info.user_cmd = cur->name;
+			cmd_info.user_action = cur->cmd;
 			execution_code = inner->user_cmd_handler.handler(&cmd_info);
 		}
 		else
 		{
+			cmd_info.user_data = cur->user_data;
 			execution_code = cur->handler(&cmd_info);
 		}
 
@@ -432,7 +452,9 @@ init_cmd_info(cmd_info_t *cmd_info)
 	cmd_info->args = NULL;
 	cmd_info->argc = 0;
 	cmd_info->argv = NULL;
-	cmd_info->cmd = NULL;
+	cmd_info->user_cmd = NULL;
+	cmd_info->user_action = NULL;
+	cmd_info->user_data = NULL;
 	cmd_info->sep = ' ';
 	cmd_info->bg = 0;
 	cmd_info->usr1 = 0;
@@ -483,7 +505,7 @@ vle_cmds_complete(const char cmd[], void *arg)
 	cmd_name_pos = parse_range(cmd, &cmd_info);
 	if(cmd_name_pos != NULL)
 	{
-		char cmd_name[MAX_CMD_NAME_LEN];
+		char cmd_name[MAX_CMD_NAME_LEN + 1];
 		const char *args;
 		cmd_t *cur;
 
@@ -498,6 +520,7 @@ vle_cmds_complete(const char cmd[], void *arg)
 		else
 		{
 			prefix_len += args - cmd;
+			cmd_info.user_data = (cur == NULL ? NULL : cur->user_data);
 			prefix_len += complete_cmd_args(cur, args, &cmd_info, arg);
 		}
 	}
@@ -518,7 +541,7 @@ skip_prefix_commands(const char cmd[])
 	cmd_name_pos = parse_range(cmd, &cmd_info);
 	if(cmd_name_pos != NULL)
 	{
-		char cmd_name[MAX_CMD_NAME_LEN];
+		char cmd_name[MAX_CMD_NAME_LEN + 1];
 		const char *args;
 		cmd_t *cur;
 
@@ -549,25 +572,35 @@ skip_prefix_commands(const char cmd[])
 	return cmd;
 }
 
+/* Looks up a command by its name. */
 static cmd_t *
 find_cmd(const char name[])
 {
-	cmd_t *cmd;
+	cmd_t *const cmd = find_cmd_advance(inner->head.next, name);
+	return (find_cmd_match(cmd, name) ? cmd : NULL);
+}
 
-	cmd = inner->head.next;
+/* Advances to the first command whose name is not less than the parameter.
+ * Returns advanced values (could be unchanged or NULL). */
+static cmd_t *
+find_cmd_advance(cmd_t *cmd, const char name[])
+{
 	while(cmd != NULL && strcmp(cmd->name, name) < 0)
 	{
 		cmd = cmd->next;
 	}
-
-	if(cmd != NULL && strncmp(name, cmd->name, strlen(name)) != 0)
-	{
-		cmd = NULL;
-	}
-
 	return cmd;
 }
 
+/* Checks that command search was a success.  Returns non-zero if so, otherwise
+ * zero is returned. */
+static int
+find_cmd_match(cmd_t *cmd, const char name[])
+{
+	return (cmd != NULL)
+	    && (strncmp(name, cmd->name, strlen(name)) == 0);
+}
+
 /* Parses whole command range (e.g. "<val>;+<val>,,-<val>").  Returns advanced
  * value of cmd when parsing is successful, otherwise NULL is returned. */
 static const char *
@@ -686,9 +719,6 @@ parse_range_elem(const char cmd[], cmd_info_t *cmd_info, char last_sep)
 static const char *
 get_cmd_name(const char cmd[], char buf[], size_t buf_len)
 {
-	const char *t;
-	size_t len;
-
 	assert(buf_len != 0 && "The buffer is expected to be of size > 0.");
 
 	if(cmd[0] == '!')
@@ -698,11 +728,28 @@ get_cmd_name(const char cmd[], char buf[], size_t buf_len)
 		return cmd;
 	}
 
-	t = cmd;
-	while(isalpha(*t))
-		t++;
+	cmd_t *c = inner->head.next;
 
-	len = MIN((size_t)(t - cmd), buf_len - 1);
+	const char *t = cmd;
+	if(isalpha(*t))
+		++t;
+	while(isalnum(*t))
+	{
+		if(isdigit(*t))
+		{
+			size_t len = MIN((size_t)(t - cmd), buf_len - 1);
+			copy_str(buf, len + 1, cmd);
+
+			c = find_cmd_advance(c, buf);
+			if(find_cmd_match(c, buf) && c->name[len] != *t)
+			{
+				break;
+			}
+		}
+		++t;
+	}
+
+	size_t len = MIN((size_t)(t - cmd), buf_len - 1);
 	strncpy(buf, cmd, len);
 	buf[len] = '\0';
 	if(*t == '?' || *t == '!')
@@ -814,7 +861,7 @@ complete_cmd_name(const char cmd_name[], int user_only)
 			;
 		else if(cur->name[0] == '\0')
 			;
-		else if(cur->type == USER_CMD)
+		else if(cur->type == USER_CMD && cur->descr == NULL)
 			vle_compl_add_match(cur->name, cur->cmd);
 		else
 			vle_compl_add_match(cur->name, cur->descr);
@@ -834,7 +881,7 @@ vle_cmds_add(const cmd_add_t cmds[], int count)
 		assert(cmds[i].min_args >= 0);
 		assert(cmds[i].max_args == NOT_DEF ||
 				cmds[i].min_args <= cmds[i].max_args);
-		ret_code = add_builtin_cmd(cmds[i].name, 0, &cmds[i]);
+		ret_code = add_builtin_cmd(cmds[i].name, BUILTIN_CMD, &cmds[i]);
 		assert(ret_code == 0);
 		if(cmds[i].abbr != NULL)
 		{
@@ -848,7 +895,7 @@ vle_cmds_add(const cmd_add_t cmds[], int count)
 			while(full_len > short_len)
 			{
 				buf[--full_len] = '\0';
-				ret_code = add_builtin_cmd(buf, 1, &cmds[i]);
+				ret_code = add_builtin_cmd(buf, BUILTIN_ABBR, &cmds[i]);
 				assert(ret_code == 0);
 			}
 		}
@@ -856,9 +903,35 @@ vle_cmds_add(const cmd_add_t cmds[], int count)
 	}
 }
 
+int
+vle_cmds_add_foreign(const cmd_add_t *cmd)
+{
+	if(cmd->min_args < 0)
+	{
+		return 1;
+	}
+
+	if(cmd->max_args != NOT_DEF && cmd->min_args > cmd->max_args)
+	{
+		return 1;
+	}
+
+	if(!is_valid_udc_name(cmd->name))
+	{
+		return CMDS_ERR_INCORRECT_NAME;
+	}
+
+	int failure = (add_builtin_cmd(cmd->name, FOREIGN_CMD, cmd) != 0);
+	if(!failure)
+	{
+		++inner->custom_cmd_count;
+	}
+	return failure;
+}
+
 /* Returns non-zero on error */
 TSTATIC int
-add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf)
+add_builtin_cmd(const char name[], CMD_TYPE type, const cmd_add_t *conf)
 {
 	int cmp;
 	cmd_t *new;
@@ -872,16 +945,9 @@ add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf)
 		return 0;
 	}
 
-	if(strcmp(name, "!") != 0)
+	if(!is_builtin_like_name_ok(name))
 	{
-		unsigned int i;
-		for(i = 0U; name[i] != '\0'; ++i)
-		{
-			if(!isalpha(name[i]))
-			{
-				return -1;
-			}
-		}
+		return -1;
 	}
 
 	cmp = -1;
@@ -910,8 +976,9 @@ add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf)
 	new->name = strdup(name);
 	new->descr = conf->descr;
 	new->id = conf->id;
+	new->user_data = conf->user_data;
 	new->handler = conf->handler;
-	new->type = abbr ? BUILTIN_ABBR : BUILTIN_CMD;
+	new->type = type;
 	new->passed = 0;
 	new->cmd = NULL;
 	new->min_args = conf->min_args;
@@ -922,14 +989,50 @@ add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf)
 	return 0;
 }
 
+/* Checks validity of a name for a builtin-like command.  Returns non-zero if
+ * it's valid and zero otherwise. */
+static int
+is_builtin_like_name_ok(const char name[])
+{
+	if(name[0] == '\0' || strcmp(name, "!") == 0)
+	{
+		return 1;
+	}
+
+	if(!isalpha(name[0]))
+	{
+		return 0;
+	}
+
+	int i;
+	for(i = 1; name[i] != '\0'; ++i)
+	{
+		if(!isalnum(name[i]))
+		{
+			return 0;
+		}
+	}
+
+	return 1;
+}
+
+/* Implements :comclear builtin command provided by this unit. */
 static int
 comclear_cmd(const cmd_info_t *cmd_info)
+{
+	remove_commands(USER_CMD);
+	return 0;
+}
+
+/* Removes commands of the specified type. */
+static void
+remove_commands(CMD_TYPE type)
 {
 	cmd_t *cur = &inner->head;
 
 	while(cur->next != NULL)
 	{
-		if(cur->next->type == USER_CMD)
+		if(cur->next->type == type)
 		{
 			cmd_t *this = cur->next;
 			cur->next = this->next;
@@ -950,51 +1053,60 @@ comclear_cmd(const cmd_info_t *cmd_info)
 			cur = cur->next;
 		}
 	}
-	inner->udf_count = 0;
-	return 0;
+	inner->custom_cmd_count = 0;
 }
 
+/* Implements :command builtin command mostly provided by this unit. */
 static int
 command_cmd(const cmd_info_t *cmd_info)
 {
-	int cmp;
-	char cmd_name[MAX_CMD_NAME_LEN];
-	const char *args;
-	cmd_t *new, *cur;
-	size_t len;
-	int has_emark, has_qmark;
-
 	if(cmd_info->argc < 2)
 	{
-		if(inner->command_handler != NULL)
-			return inner->command_handler(cmd_info);
-		else
+		if(inner->command_handler == NULL)
+		{
 			return CMDS_ERR_TOO_FEW_ARGS;
+		}
+		return inner->command_handler(cmd_info);
 	}
 
-	args = get_user_cmd_name(cmd_info->args, cmd_name, sizeof(cmd_name));
-	args = vle_cmds_at_arg(args);
-	if(args[0] == '\0')
-		return CMDS_ERR_TOO_FEW_ARGS;
-	else if(!is_valid_udc_name(cmd_name))
-		return CMDS_ERR_INCORRECT_NAME;
+	char name[MAX_CMD_NAME_LEN + 1];
+	const char *body = get_user_cmd_name(cmd_info->args, name, sizeof(name));
+	body = vle_cmds_at_arg(body);
 
-	len = strlen(cmd_name);
-	has_emark = (len > 0 && cmd_name[len - 1] == '!');
-	has_qmark = (len > 0 && cmd_name[len - 1] == '?');
+	return vle_cmds_add_user(name, body, NULL, cmd_info->emark);
+}
 
-	cmp = -1;
-	cur = &inner->head;
-	while(cur->next != NULL && (cmp = strcmp(cur->next->name, cmd_name)) < 0)
+int
+vle_cmds_add_user(const char name[], const char body[], const char descr[],
+		int overwrite)
+{
+	if(body[0] == '\0')
 	{
-		if(has_emark && cur->next->type == BUILTIN_CMD && cur->next->emark &&
-				strncmp(cmd_name, cur->next->name, len - 1) == 0)
+		return CMDS_ERR_TOO_FEW_ARGS;
+	}
+	if(!is_valid_udc_name(name))
+	{
+		return CMDS_ERR_INCORRECT_NAME;
+	}
+
+	size_t len = strlen(name);
+	int has_emark = (len > 0 && name[len - 1] == '!');
+	int has_qmark = (len > 0 && name[len - 1] == '?');
+
+	int cmp = -1;
+	cmd_t *cur = &inner->head;
+	while(cur->next != NULL && (cmp = strcmp(cur->next->name, name)) < 0)
+	{
+		int builtin_like = (cur->next->type == BUILTIN_CMD)
+		                || (cur->next->type == FOREIGN_CMD);
+		if(has_emark && builtin_like && cur->next->emark &&
+				strncmp(name, cur->next->name, len - 1) == 0)
 		{
 			cmp = 0;
 			break;
 		}
-		if(has_qmark && cur->next->type == BUILTIN_CMD && cur->next->qmark &&
-				strncmp(cmd_name, cur->next->name, len - 1) == 0)
+		if(has_qmark && builtin_like && cur->next->qmark &&
+				strncmp(name, cur->next->name, len - 1) == 0)
 		{
 			cmp = 0;
 			break;
@@ -1002,12 +1114,13 @@ command_cmd(const cmd_info_t *cmd_info)
 		cur = cur->next;
 	}
 
+	cmd_t *new;
 	if(cmp == 0)
 	{
 		cur = cur->next;
-		if(cur->type == BUILTIN_CMD)
+		if(cur->type == BUILTIN_CMD || cur->type == FOREIGN_CMD)
 			return CMDS_ERR_NO_BUILTIN_REDEFINE;
-		if(!cmd_info->emark)
+		if(!overwrite)
 			return CMDS_ERR_NEED_BANG;
 		free(cur->name);
 		free(cur->cmd);
@@ -1021,18 +1134,18 @@ command_cmd(const cmd_info_t *cmd_info)
 		}
 	}
 
-	new->name = strdup(cmd_name);
-	new->descr = NULL;
+	new->name = strdup(name);
+	new->descr = descr;
 	new->id = USER_CMD_ID;
 	new->type = USER_CMD;
 	new->passed = 0;
-	new->cmd = strdup(args);
+	new->cmd = strdup(body);
 	new->min_args = inner->user_cmd_handler.min_args;
 	new->max_args = inner->user_cmd_handler.max_args;
 	new->deleted = 0;
 	init_command_flags(new, inner->user_cmd_handler.flags);
 
-	++inner->udf_count;
+	++inner->custom_cmd_count;
 	return 0;
 }
 
@@ -1074,7 +1187,7 @@ get_user_cmd_name(const char cmd[], char buf[], size_t buf_len)
 
 	t = vle_cmds_past_arg(cmd);
 
-	len = MIN((size_t)(t - cmd), buf_len);
+	len = MIN((size_t)(t - cmd), buf_len - 1);
 	strncpy(buf, cmd, len);
 	buf[len] = '\0';
 	return t;
@@ -1090,31 +1203,67 @@ is_valid_udc_name(const char name[])
 		return 0;
 	if(strcmp(name, "?") == 0)
 		return 0;
+	if(!isalpha(name[0]))
+		return 0;
+	if(strlen(name) >= MAX_CMD_NAME_LEN)
+		return 0;
 
-	char cmd_name[MAX_CMD_NAME_LEN + 1];
-	copy_str(cmd_name, sizeof(cmd_name), name);
+	char cmd_name[MAX_CMD_NAME_LEN];
+	char *p = cmd_name;
+
+	cmd_t *cmd = inner->head.next;
 
 	while(name[0] != '\0')
 	{
-		if(!isalpha(name[0]))
+		*p++ = *name;
+
+		if(!isalnum(name[0]))
 		{
 			if(name[1] != '\0')
 				return 0;
 			else if(name[0] != '!' && name[0] != '?')
 				return 0;
 		}
-		name++;
+		else if(isdigit(name[1]))
+		{
+			*p = '\0';
+			cmd = find_cmd_advance(cmd, cmd_name);
+
+			if(find_cmd_match(cmd, cmd_name))
+			{
+				if(cmd->type == BUILTIN_CMD || cmd->type == BUILTIN_ABBR)
+				{
+					return 0;
+				}
+
+				const int name_len = p - cmd_name;
+				if(cmd->name[name_len] == '\0' || isalpha(cmd->name[name_len]))
+				{
+					return 0;
+				}
+			}
+		}
+
+		++name;
 	}
 
+	*p = '\0';
+
 	/* Builtins with custom separator have higher priority.  Disallow registering
 	 * user-defined commands which will never be called. */
 	if(name[-1] == '!' || name[-1] == '?')
 	{
 		cmd_name[strlen(cmd_name) - 1] = '\0';
 	}
-	const cmd_t *const c = find_cmd(cmd_name);
-	if(c != NULL && c->cust_sep && strcmp(c->name, cmd_name) == 0)
-		return 0;
+
+	cmd = find_cmd_advance(cmd, cmd_name);
+	if(find_cmd_match(cmd, cmd_name))
+	{
+		if(cmd->cust_sep && strcmp(cmd->name, cmd_name) == 0)
+			return 0;
+		if(isdigit(cmd->name[strlen(cmd_name)]))
+			return 0;
+	}
 
 	return 1;
 }
@@ -1135,8 +1284,15 @@ insert_cmd(cmd_t *after)
 	return new;
 }
 
+/* Implements :command builtin command provided by this unit. */
 static int
 delcommand_cmd(const cmd_info_t *cmd_info)
+{
+	return vle_cmds_del_user(cmd_info->argv[0]);
+}
+
+int
+vle_cmds_del_user(const char name[])
 {
 	int cmp;
 	cmd_t *cur;
@@ -1144,8 +1300,7 @@ delcommand_cmd(const cmd_info_t *cmd_info)
 
 	cmp = -1;
 	cur = &inner->head;
-	while(cur->next != NULL &&
-			(cmp = strcmp(cur->next->name, cmd_info->argv[0])) < 0)
+	while(cur->next != NULL && (cmp = strcmp(cur->next->name, name)) < 0)
 		cur = cur->next;
 
 	if(cur->next == NULL || cmp != 0)
@@ -1157,7 +1312,7 @@ delcommand_cmd(const cmd_info_t *cmd_info)
 	free(cmd->cmd);
 	free(cmd);
 
-	inner->udf_count--;
+	inner->custom_cmd_count--;
 	return 0;
 }
 
@@ -1329,7 +1484,7 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 				argvp[DA_SIZE(argvp) - 1U][1] = end;
 			}
 
-			*count = add_to_string_array(&params, *count, 1, &cmdstr[st]);
+			*count = add_to_string_array(&params, *count, &cmdstr[st]);
 			if(*count == 0)
 			{
 				break;
@@ -1391,7 +1546,8 @@ vle_cmds_list_udcs(void)
 	char **p;
 	cmd_t *cur;
 
-	char **const list = reallocarray(NULL, inner->udf_count*2 + 1, sizeof(*list));
+	char **const list = reallocarray(NULL, inner->custom_cmd_count*2 + 1,
+			sizeof(*list));
 	if(list == NULL)
 	{
 		return NULL;
@@ -1405,7 +1561,12 @@ vle_cmds_list_udcs(void)
 		if(cur->type == USER_CMD)
 		{
 			*p++ = strdup(cur->name);
-			*p++ = strdup(cur->cmd);
+			*p++ = strdup(cur->descr == NULL ? cur->cmd : cur->descr);
+		}
+		else if(cur->type == FOREIGN_CMD)
+		{
+			*p++ = strdup(cur->name);
+			*p++ = strdup(cur->descr);
 		}
 		cur = cur->next;
 	}
