@@ -71,6 +71,7 @@ typedef enum
 {
 	BUILTIN_ABBR, /* Abbreviated version of a builtin command (like `:com`). */
 	BUILTIN_CMD,  /* Builtin command. */
+	FOREIGN_CMD,  /* Externally registered builtin-like command. */
 	USER_CMD,     /* User-defined command. */
 }
 CMD_TYPE;
@@ -93,7 +94,11 @@ typedef struct cmd_info_t
 	char **argv;     /* Values of arguments. */
 	int (*argvp)[2]; /* Start/end positions of arguments in args. */
 
-	const char *cmd; /* For user defined commands. */
+	/* For user defined commands. */
+	const char *user_cmd;    /* Name of user defined command. */
+	const char *user_action; /* Body of user defined command. */
+
+	void *user_data; /* User data associated with the command or NULL. */
 }
 cmd_info_t;
 
@@ -111,6 +116,7 @@ typedef struct cmd_t
 	CMD_TYPE type;     /* Type of command described by this structure. */
 	int passed;        /* Number of times this command was recursively called. */
 
+	void *user_data;     /* User data associated with the command or NULL. */
 	cmd_handler handler; /* Handler for builtin commands. */
 	char *cmd;           /* Command-line for user-defined commands. */
 
@@ -170,6 +176,7 @@ typedef struct
 	const char *name;       /* Full command name. */
 	const char *abbr;       /* Command prefix (can be NULL). */
 	const char *descr;      /* Brief description (stored as a pointer). */
+	void *user_data;        /* User data for the handler or NULL. */
 	int id;                 /* Command id.  Doesn't need to be unique.  Negative
 	                           value means absence of arg completion.  Use, for
 	                           example, -1 for all commands without completion. */
@@ -227,6 +234,9 @@ void vle_cmds_init(int udf, cmds_conf_t *cmds_conf);
 /* Resets state of the unit. */
 void vle_cmds_reset(void);
 
+/* Clears (partially resets) state of the unit. */
+void vle_cmds_clear(void);
+
 /* Executes a command.  Returns one of CMDS_ERR_* codes or code returned by the
  * command handler. */
 int vle_cmds_run(const char cmd[]);
@@ -252,6 +262,20 @@ int vle_cmds_complete(const char cmd[], void *arg);
  * count. */
 void vle_cmds_add(const cmd_add_t cmds[], int count);
 
+/* Registers a foreign builtin-like command.  Returns non-zero on error,
+ * otherwise zero is returned. */
+int vle_cmds_add_foreign(const cmd_add_t *cmd);
+
+/* Adds a new or updates an existing user command.  Non-zero overwrite parameter
+ * enables updating of existing command.  Returns error code or zero on
+ * success. */
+int vle_cmds_add_user(const char name[], const char body[], const char descr[],
+		int overwrite);
+
+/* Removes a user command if one exists.  Returns error code or zero on
+ * success. */
+int vle_cmds_del_user(const char name[]);
+
 /* Finds the first character of the last argument in cmd.  Returns pointer to
  * it. */
 char * vle_cmds_last_arg(const char cmd[], int quotes, size_t *len);
@@ -281,7 +305,7 @@ char * vle_cmds_at_arg(const char args[]);
 char * vle_cmds_next_arg(const char args[]);
 
 TSTATIC_DEFS(
-	int add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf);
+	int add_builtin_cmd(const char name[], CMD_TYPE type, const cmd_add_t *conf);
 	char ** dispatch_line(const char args[], int *count, char sep, int regexp,
 			int quotes, int noescaping, int comments, int *last_arg,
 			int (**positions)[2]);
