@@ -93,7 +93,8 @@ keys_info_t;
 typedef void (*vle_keys_handler)(key_info_t key_info, keys_info_t *keys_info);
 /* Type of function invoked by vle_keys_list() and vle_keys_suggest().  rhs is
  * provided for user-defined keys and is empty otherwise.  Description is empty
- * for user-defined keys or when not set. */
+ * for user-defined keys or when not set.  Extra messages have empty lhs and
+ * rhs, but can have non-empty description. */
 typedef void (*vle_keys_list_cb)(const wchar_t lhs[], const wchar_t rhs[],
 		const char descr[]);
 /* User-provided suggestion callback for multikeys. */
@@ -189,9 +190,10 @@ void vle_keys_list(int mode, vle_keys_list_cb cb, int user_only);
  * difference of returned values.  Returns the number. */
 size_t vle_keys_counter(void);
 
-/* Checks whether a mapping handler is currently been executed.  Returns
- * non-zero if so, otherwise zero is returned. */
-int vle_keys_inside_mapping(void);
+/* Retrieves current mapping state.  Entering a mapping at the top level (so not
+ * a nested mapping) increases state number.  Returns the state number or zero
+ * if no mapping is currently active. */
+int vle_keys_mapping_state(void);
 
 /* Invokes cb for each possible keys continuation.  Intended to be used on
  * KEYS_WAIT and KEYS_WAIT_SHORT returns.  The custom_only flag limits
