@@ -17,7 +17,6 @@
  */
 
 #include "options.h"
-#include "private/options.h"
 
 #include <assert.h> /* assert() */
 #include <ctype.h>
@@ -43,9 +42,10 @@ const char OPT_NAME_CHARS[] = LOWER_CHARS;
 /* List of operations on options of set type for the set_op(...) function. */
 typedef enum
 {
-	SO_SET, /* Set value of the option. */
-	SO_ADD, /* Add item(s) to a set. */
+	SO_SET,    /* Set value of the option. */
+	SO_ADD,    /* Add item(s) to a set. */
 	SO_REMOVE, /* Remove item(s) from a set. */
+	SO_TOGGLE, /* Toggle item(s) in a set. */
 }
 SetOp;
 
@@ -71,8 +71,6 @@ static void enum_options(OPT_SCOPE scope, opt_traverse traverser);
 static void enum_unique_options(opt_traverse traverser);
 static opt_t * get_option(const char option[], OPT_SCOPE scope);
 static opt_t * pick_option(opt_t *opts[2], OPT_SCOPE scope);
-static int set_on(opt_t *opt);
-static int set_off(opt_t *opt);
 static int set_inv(opt_t *opt);
 static int set_reset(opt_t *opt);
 static int uses_str_value(OPT_TYPE type);
@@ -116,7 +114,7 @@ static opt_t *options;
 static size_t option_count;
 
 void
-init_options(int *opts_changed_flag, opt_uni_handler universal_handler)
+vle_opts_init(int *opts_changed_flag, opt_uni_handler universal_handler)
 {
 	assert(opts_changed_flag != NULL);
 
@@ -134,9 +132,9 @@ do_nothing_handler(const char name[], optval_t val, OPT_SCOPE scope)
 }
 
 void
-reset_option_to_default(const char name[], OPT_SCOPE scope)
+vle_opts_restore_default(const char name[], OPT_SCOPE scope)
 {
-	opt_t *opt = find_option(name, scope);
+	opt_t *opt = vle_opts_find(name, scope);
 	if(opt != NULL)
 	{
 		set_reset(opt);
@@ -144,7 +142,7 @@ reset_option_to_default(const char name[], OPT_SCOPE scope)
 }
 
 void
-reset_options_to_default(void)
+vle_opts_restore_defaults(void)
 {
 	reset_options(OPT_ANY);
 }
@@ -163,7 +161,7 @@ reset_options(OPT_SCOPE scope)
 }
 
 void
-clear_options(void)
+vle_opts_reset(void)
 {
 	size_t i;
 	for(i = 0U; i < option_count; ++i)
@@ -185,7 +183,7 @@ clear_options(void)
 }
 
 void
-add_option(const char name[], const char abbr[], const char descr[],
+vle_opts_add(const char name[], const char abbr[], const char descr[],
 		OPT_TYPE type, OPT_SCOPE scope, int val_count, const char *vals[][2],
 		opt_handler handler, optval_t def)
 {
@@ -215,7 +213,7 @@ add_option(const char name[], const char abbr[], const char descr[],
 
 		/* Inserting an option invalidates pointers to other options, so reload
 		 * it. */
-		full = find_option(full_name, scope);
+		full = vle_opts_find(full_name, scope);
 	}
 
 	if(uses_str_value(type))
@@ -260,7 +258,7 @@ add_option_inner(const char name[], const char descr[], OPT_TYPE type,
 	opt_t *p;
 
 	assert(scope != OPT_ANY && "OPT_ANY can't be an option type.");
-	assert(find_option(name, scope) == NULL && "Duplicated option.");
+	assert(vle_opts_find(name, scope) == NULL && "Duplicated option.");
 
 	p = reallocarray(options, option_count + 1, sizeof(*options));
 	if(p == NULL)
@@ -291,9 +289,9 @@ add_option_inner(const char name[], const char descr[], OPT_TYPE type,
 }
 
 void
-set_option(const char name[], optval_t val, OPT_SCOPE scope)
+vle_opts_assign(const char name[], optval_t val, OPT_SCOPE scope)
 {
-	opt_t *opt = find_option(name, scope);
+	opt_t *opt = vle_opts_find(name, scope);
 	assert(opt != NULL && "Wrong option name.");
 
 	if(uses_str_value(opt->type))
@@ -307,7 +305,7 @@ set_option(const char name[], optval_t val, OPT_SCOPE scope)
 }
 
 int
-set_options(const char args[], OPT_SCOPE scope)
+vle_opts_set(const char args[], OPT_SCOPE scope)
 {
 	int errors = 0;
 
@@ -421,11 +419,11 @@ process_option(const char arg[], OPT_SCOPE real_scope, OPT_SCOPE scope,
 	err = 0;
 	if(*suffix == '\0')
 	{
-		opt_t *o = find_option(optname, scope);
+		opt_t *o = vle_opts_find(optname, scope);
 		if(o != NULL)
 		{
 			if(o->type == OPT_BOOL)
-				err = set_on(opt);
+				err = vle_opt_on(opt);
 			else
 			{
 				*print = 1;
@@ -434,7 +432,7 @@ process_option(const char arg[], OPT_SCOPE real_scope, OPT_SCOPE scope,
 		}
 		else if(strncmp(optname, "no", 2) == 0)
 		{
-			err = set_off(opt);
+			err = vle_opt_off(opt);
 		}
 		else if(strncmp(optname, "inv", 3) == 0)
 		{
@@ -461,11 +459,11 @@ process_option(const char arg[], OPT_SCOPE real_scope, OPT_SCOPE scope,
 	}
 	else if(strncmp(suffix, "+=", 2) == 0)
 	{
-		err = set_add(opt, suffix + 2);
+		err = vle_opt_add(opt, suffix + 2);
 	}
 	else if(strncmp(suffix, "-=", 2) == 0)
 	{
-		err = set_remove(opt, suffix + 2);
+		err = vle_opt_remove(opt, suffix + 2);
 	}
 	else if(strncmp(suffix, "^=", 2) == 0)
 	{
@@ -473,7 +471,7 @@ process_option(const char arg[], OPT_SCOPE real_scope, OPT_SCOPE scope,
 	}
 	else if(*suffix == '=' || *suffix == ':')
 	{
-		err = set_set(opt, suffix + 1);
+		err = vle_opt_assign(opt, suffix + 1);
 	}
 	else
 	{
@@ -594,20 +592,20 @@ get_option(const char option[], OPT_SCOPE scope)
 {
 	opt_t *opt;
 
-	opt = find_option(option, scope);
+	opt = vle_opts_find(option, scope);
 	if(opt != NULL)
 		return opt;
 
 	if(strncmp(option, "no", 2) == 0)
-		return find_option(option + 2, scope);
+		return vle_opts_find(option + 2, scope);
 	else if(strncmp(option, "inv", 3) == 0)
-		return find_option(option + 3, scope);
+		return vle_opts_find(option + 3, scope);
 	else
 		return NULL;
 }
 
 opt_t *
-find_option(const char option[], OPT_SCOPE scope)
+vle_opts_find(const char option[], OPT_SCOPE scope)
 {
 	int l = 0, u = option_count - 1;
 	while(l <= u)
@@ -637,7 +635,7 @@ find_option(const char option[], OPT_SCOPE scope)
 			opt = pick_option(opts, scope);
 			if(opt != NULL && opt->full != NULL)
 			{
-				return find_option(opt->full, scope);
+				return vle_opts_find(opt->full, scope);
 			}
 			return opt;
 		}
@@ -689,9 +687,8 @@ pick_option(opt_t *opts[2], OPT_SCOPE scope)
 	}
 }
 
-/* Turns boolean option on.  Returns non-zero in case of error. */
-static int
-set_on(opt_t *opt)
+int
+vle_opt_on(opt_t *opt)
 {
 	if(opt->type != OPT_BOOL)
 		return -1;
@@ -706,9 +703,8 @@ set_on(opt_t *opt)
 	return 0;
 }
 
-/* Turns boolean option off.  Returns non-zero in case of error. */
-static int
-set_off(opt_t *opt)
+int
+vle_opt_off(opt_t *opt)
 {
 	if(opt->type != OPT_BOOL)
 		return -1;
@@ -738,7 +734,7 @@ set_inv(opt_t *opt)
 }
 
 int
-set_set(opt_t *opt, const char value[])
+vle_opt_assign(opt_t *opt, const char value[])
 {
 	if(opt->type == OPT_BOOL)
 		return -1;
@@ -837,7 +833,7 @@ uses_str_value(OPT_TYPE type)
 }
 
 int
-set_add(opt_t *opt, const char value[])
+vle_opt_add(opt_t *opt, const char value[])
 {
 	if(opt->type != OPT_INT && opt->type != OPT_SET && opt->type != OPT_STRLIST &&
 			opt->type != OPT_CHARSET && opt->type != OPT_STR)
@@ -900,7 +896,7 @@ set_add(opt_t *opt, const char value[])
 }
 
 int
-set_remove(opt_t *opt, const char value[])
+vle_opt_remove(opt_t *opt, const char value[])
 {
 	if(opt->type != OPT_INT && opt->type != OPT_SET && opt->type != OPT_STRLIST &&
 			opt->type != OPT_CHARSET)
@@ -946,22 +942,41 @@ set_remove(opt_t *opt, const char value[])
 	return 0;
 }
 
-/* Toggles value(s) from the option (^= operator).  Returns non-zero on
- * success. */
+/* Toggles value(s) from the option (^= operator).  Returns zero on success. */
 static int
 set_hat(opt_t *opt, const char value[])
 {
-	if(opt->type != OPT_CHARSET)
+	if(opt->type != OPT_STRLIST && opt->type != OPT_SET &&
+			opt->type != OPT_CHARSET)
 	{
 		return -1;
 	}
 
-	if(charset_toggle_all(opt, value))
+	if(opt->type == OPT_SET)
 	{
+		if(set_op(opt, value, SO_TOGGLE))
+		{
+			notify_option_update(opt, OP_MODIFIED, opt->val);
+		}
+	}
+	else if(opt->type == OPT_CHARSET)
+	{
+		if(charset_toggle_all(opt, value))
+		{
+			notify_option_update(opt, OP_MODIFIED, opt->val);
+		}
+	}
+	else
+	{
+		if(!str_remove(opt->val.str_val, value))
+		{
+			/* Nothing was removed, so add it. */
+			opt->val.str_val = str_add(opt->val.str_val, value);
+		}
 		notify_option_update(opt, OP_MODIFIED, opt->val);
 	}
-	uni_handler(opt->name, opt->val, opt->scope);
 
+	uni_handler(opt->name, opt->val, opt->scope);
 	return 0;
 }
 
@@ -996,7 +1011,8 @@ set_op(opt_t *opt, const char value[], SetOp op)
 
 		if(i != -1)
 		{
-			if(op == SO_SET || op == SO_ADD)
+			if(op == SO_SET || op == SO_ADD ||
+					(op == SO_TOGGLE && (old_val & (1 << i)) == 0))
 				new_val |= 1 << i;
 			else
 				new_val &= ~(1 << i);
@@ -1144,22 +1160,17 @@ replace_if_changed(char **current, const char new[])
 	return (replace_string(current, new) == 0) ? 1 : 0;
 }
 
-/* And an element to string list.  Reallocates the memory of old and returns
+/* Add an element to string list.  Reallocates the memory of old and returns
  * its new address or NULL if there isn't enough memory. */
 static char *
 str_add(char *old, const char *value)
 {
-	size_t len;
-	char *new;
-
-	len = 0;
-	if(old != NULL)
-		len = strlen(old);
-	new = realloc(old, len + 1 + strlen(value) + 1);
+	const size_t old_len = (old == NULL ? 0 : strlen(old));
+	char *new = realloc(old, old_len + 1 + strlen(value) + 1);
 	if(new == NULL)
 		return old;
 
-	if(old == NULL)
+	if(old_len == 0)
 		strcpy(new, "");
 	else
 		strcat(new, ",");
@@ -1228,26 +1239,26 @@ set_print(const opt_t *opt)
 	}
 	else
 	{
-		vle_tb_append_linef(vle_err, "  %s=%s", opt->name, get_value(opt));
+		vle_tb_append_linef(vle_err, "  %s=%s", opt->name, vle_opt_to_string(opt));
 	}
 	return 0;
 }
 
 const char *
-get_option_value(const char name[], OPT_SCOPE scope)
+vle_opts_get(const char name[], OPT_SCOPE scope)
 {
-	opt_t *opt = find_option(name, scope);
+	opt_t *opt = vle_opts_find(name, scope);
 	assert(opt != NULL && "Wrong option name.");
 	if(opt == NULL)
 	{
 		return NULL;
 	}
 
-	return get_value(opt);
+	return vle_opt_to_string(opt);
 }
 
 const char *
-get_value(const opt_t *opt)
+vle_opt_to_string(const opt_t *opt)
 {
 	static char buf[1024];
 	if(opt->type == OPT_BOOL)
@@ -1290,7 +1301,7 @@ get_value(const opt_t *opt)
 }
 
 void
-complete_options(const char args[], const char **start, OPT_SCOPE scope)
+vle_opts_complete(const char args[], const char **start, OPT_SCOPE scope)
 {
 	int bool_only;
 	int is_value_completion;
@@ -1310,13 +1321,16 @@ complete_options(const char args[], const char **start, OPT_SCOPE scope)
 		if(args == NULL || (*args == '\0' && is_null_or_empty(last_opt)))
 		{
 			/* Just exit on error or reaching a comment. */
-			if(last_opt != NULL)
+			if(is_null_or_empty(last_opt))
 			{
-				vle_compl_put_match(last_opt, "");
+				free(last_opt);
+				/* Need to place original input to avoid its disappearance (if we
+				 * complete with "", it will replace part of the string). */
+				vle_compl_add_match(*start, "");
 			}
 			else
 			{
-				vle_compl_add_match("", "");
+				vle_compl_put_match(last_opt, "");
 			}
 			return;
 		}
@@ -1378,7 +1392,7 @@ complete_options(const char args[], const char **start, OPT_SCOPE scope)
 		}
 		else if(*p == '\0' && opt->type != OPT_BOOL)
 		{
-			vle_compl_put_match(escape_chars(get_value(opt), " |"), "");
+			vle_compl_put_match(escape_chars(vle_opt_to_string(opt), " |"), "");
 		}
 	}
 
@@ -1526,7 +1540,7 @@ skip_alphas(const char str[])
 }
 
 void
-complete_real_option_names(const char beginning[], OPT_SCOPE scope)
+vle_opts_complete_real(const char beginning[], OPT_SCOPE scope)
 {
 	complete_option_name(beginning, 0, 0, scope);
 	vle_compl_finish_group();
