@@ -27,10 +27,11 @@
 #include <stddef.h> /* NULL size_t wchar_t */
 #include <stdlib.h> /* free() */
 #include <string.h> /* memmove() strncpy() */
-#include <wchar.h> /* wint_t wcslen() wcscmp() */
+#include <wchar.h> /* wint_t wcslen() wcscmp() wcsncat() wmemmove() */
 
 #include "cfg/config.h"
 #include "compat/curses.h"
+#include "compat/fs_limits.h"
 #include "engine/completion.h"
 #include "engine/keys.h"
 #include "engine/mode.h"
@@ -50,13 +51,16 @@
 #include "background.h"
 #include "bracket_notation.h"
 #include "filelist.h"
+#include "instance.h"
 #include "ipc.h"
 #include "registers.h"
 #include "status.h"
+#include "vcache.h"
 #include "vifm.h"
 
 static int ensure_term_is_ready(void);
 static int get_char_async_loop(WINDOW *win, wint_t *c, int timeout);
+static int is_previewed(const char path[]);
 static void process_scheduled_updates(void);
 TSTATIC int process_scheduled_updates_of_view(view_t *view);
 static void update_hardware_cursor(void);
@@ -71,6 +75,7 @@ static void draw_suggestion_box(void);
 static WINDOW * prepare_suggestion_box(int *height);
 static void hide_suggestion_box(void);
 static int should_display_suggestion_box(void);
+TSTATIC void feed_keys(const wchar_t input[]);
 
 /* Current input buffer. */
 static const wchar_t *curr_input_buf;
@@ -80,6 +85,9 @@ static const size_t *curr_input_buf_pos;
 /* Whether suggestion box is active. */
 static int suggestions_are_visible;
 
+/* Source of fake input that has priority over real input. */
+static wchar_t input_queue[128];
+
 void
 event_loop(const int *quit)
 {
@@ -113,8 +121,11 @@ event_loop(const int *quit)
 		size_t counter;
 		int got_input;
 
-		lwin.user_selection = 1;
-		rwin.user_selection = 1;
+		/* Reset marking from previous commands.  They are temporary.  Don't really
+		 * like this, any better ideas?  How about post-hook for engine/keys
+		 * unit? */
+		lwin.pending_marking = 0;
+		rwin.pending_marking = 0;
 
 		modes_pre();
 
@@ -186,8 +197,7 @@ event_loop(const int *quit)
 
 			if(c == WC_C_z)
 			{
-				ui_shutdown();
-				stop_process();
+				instance_stop();
 				continue;
 			}
 
@@ -215,22 +225,23 @@ event_loop(const int *quit)
 			assert(counter <= input_buf_pos);
 			if(counter > 0)
 			{
+				curr_stats.save_msg = 0;
 				memmove(input_buf, input_buf + counter,
 						(wcslen(input_buf) - counter + 1)*sizeof(wchar_t));
 			}
 		}
 		else
 		{
-			if(got_input)
-			{
-				curr_stats.save_msg = 0;
-			}
-
 			if(last_result == KEYS_WAIT || last_result == KEYS_WAIT_SHORT)
 			{
 				hide_suggestion_box();
 			}
 
+			if(got_input)
+			{
+				curr_stats.save_msg = 0;
+			}
+
 			last_result = vle_keys_exec(input_buf);
 
 			counter = vle_keys_counter() - counter;
@@ -303,6 +314,11 @@ event_loop(const int *quit)
 static int
 ensure_term_is_ready(void)
 {
+	if(vifm_testing())
+	{
+		return 1;
+	}
+
 	ui_update_term_state();
 
 	update_terminal_settings();
@@ -321,7 +337,7 @@ ensure_term_is_ready(void)
 		modes_redraw();
 
 		curr_stats.save_msg = 0;
-		ui_sb_msg("");
+		ui_sb_clear();
 	}
 
 	return 1;
@@ -359,9 +375,16 @@ get_char_async_loop(WINDOW *win, wint_t *c, int timeout)
 
 		for(i = 0; i < IPC_F && timeout > 0; ++i)
 		{
-			int result;
+			if(curr_stats.ipc != NULL)
+			{
+				ipc_check(curr_stats.ipc);
+			}
+
+			if(vcache_check(&is_previewed))
+			{
+				stats_redraw_later();
+			}
 
-			ipc_check(curr_stats.ipc);
 			wtimeout(win, delay_slice);
 			timeout -= delay_slice;
 
@@ -377,17 +400,49 @@ get_char_async_loop(WINDOW *win, wint_t *c, int timeout)
 			 * them to make it active. */
 			update_hardware_cursor();
 
-			result = compat_wget_wch(win, c);
+			if(input_queue[0] != L'\0')
+			{
+				*c = input_queue[0];
+				wmemmove(input_queue, input_queue + 1, wcslen(input_queue));
+				return OK;
+			}
+
+			int result = compat_wget_wch(win, c);
 			if(result != ERR)
 			{
 				if(result == KEY_CODE_YES)
 				{
-					*c = K(*c);
+#ifdef __PDCURSES__
+					switch(*c)
+					{
+						case PADENTER: *c = WC_CR; result = OK; break;
+						case PADSLASH: *c = '/'; result = OK; break;
+						case PADMINUS: *c = '-'; result = OK; break;
+						case PADSTAR: *c = '*'; result = OK; break;
+						case PADPLUS: *c = '+'; result = OK; break;
+
+						case KEY_A1: *c = KEY_HOME; break;
+						case KEY_A2: *c = KEY_UP; break;
+						case KEY_A3: *c = KEY_PPAGE; break;
+						case KEY_B1: *c = KEY_LEFT; break;
+						case KEY_B3: *c = KEY_RIGHT; break;
+						case KEY_C1: *c = KEY_END; break;
+						case KEY_C2: *c = KEY_DOWN; break;
+						case KEY_C3: *c = KEY_NPAGE; break;
+						case PADSTOP: *c = KEY_DC; break;
+					}
+
+					if(result == KEY_CODE_YES)
+#endif
+					{
+						*c = K(*c);
+					}
 				}
 				else if(*c == L'\0')
 				{
 					*c = WC_C_SPACE;
 				}
+
 				return result;
 			}
 
@@ -399,6 +454,20 @@ get_char_async_loop(WINDOW *win, wint_t *c, int timeout)
 	return ERR;
 }
 
+/* Checks if preview of specified path is visible.  Returns non-zero if so and
+ * zero otherwise. */
+static int
+is_previewed(const char path[])
+{
+	if(curr_stats.preview.on)
+	{
+		dir_entry_t *entry = get_current_entry(curr_view);
+		return fentry_points_to(entry, path);
+	}
+
+	return (fview_previews(curr_view, path) || fview_previews(other_view, path));
+}
+
 /* Updates TUI or its elements if something is scheduled. */
 static void
 process_scheduled_updates(void)
@@ -413,7 +482,7 @@ process_scheduled_updates(void)
 		need_redraw += (process_scheduled_updates_of_view(other_view) != 0);
 	}
 
-	need_redraw += (stats_redraw_fetch() != 0);
+	need_redraw += (stats_update_fetch() != UT_NONE);
 
 	if(need_redraw)
 	{
@@ -635,16 +704,15 @@ prepare_suggestion_box(int *height)
 			ui_stat_job_bar_height() - 2;
 		*height = MIN(count, max_height);
 		wresize(stat_win, *height, getmaxx(stdscr));
-		ui_stat_reposition(getmaxy(status_bar), 1);
+		ui_stat_reposition(getmaxy(status_bar), *height);
 		win = stat_win;
 	}
 
 	/* Clear preview before displaying suggestion for the first time for specific
 	 * input if active preview needs special cleanup. */
-	if(!suggestions_are_visible && curr_stats.preview.on &&
-			curr_stats.preview.cleanup_cmd != NULL)
+	if(!suggestions_are_visible)
 	{
-		qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
+		ui_qv_cleanup_if_needed();
 	}
 
 	ui_set_bg(win, &col, -1);
@@ -683,5 +751,13 @@ should_display_suggestion_box(void)
 	return 0;
 }
 
+/* Adds keys to fake input.  Drops some of them if there isn't enough space. */
+TSTATIC void
+feed_keys(const wchar_t input[])
+{
+	size_t room = ARRAY_LEN(input_queue) - wcslen(input_queue) - 1;
+	wcsncat(input_queue, input, room);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
