@@ -49,10 +49,11 @@ static int apply_r_mod(const char *path, char *buf, size_t buf_len);
 static int apply_e_mod(const char *path, char *buf, size_t buf_len);
 static int apply_s_gs_mod(const char *path, const char *mod,
 		char *buf, size_t buf_len);
+static size_t get_mod_len(const char str[]);
 static const char * find_nth_chr(const char *str, char c, int n);
 
 const char *
-apply_mods(const char path[], const char parent[], const char mod[],
+mods_apply(const char path[], const char parent[], const char mod[],
 		int for_shell)
 {
 	static char buf[PATH_MAX + 1];
@@ -88,7 +89,7 @@ apply_mod(const char *path, const char *parent, const char *mod, int *mod_len,
 		int for_shell)
 {
 	char path_buf[PATH_MAX + 1];
-	static char buf[PATH_MAX + 1];
+	static char buf[PATH_MAX + 16];
 
 	copy_str(path_buf, sizeof(path_buf), path);
 	system_to_internal_slashes(path_buf);
@@ -274,7 +275,25 @@ apply_s_gs_mod(const char *path, const char *mod, char *buf, size_t buf_len)
 }
 
 size_t
-get_mods_len(const char *str)
+mods_length(const char str[])
+{
+	size_t total = 0;
+	while(str[total] != '\0')
+	{
+		size_t len = get_mod_len(&str[total]);
+		if(len == 0)
+		{
+			break;
+		}
+		total += len;
+	}
+	return total;
+}
+
+/* Computes length of filename modifier at the beginning of the passed string.
+ * Returns the length. */
+static size_t
+get_mod_len(const char str[])
 {
 	static const char FIXED_LENGTH_FILEMODS[] = "p~.htre";
 	size_t result = 0;
