@@ -31,6 +31,7 @@
 #include "utils/matchers.h"
 #include "utils/str.h"
 #include "utils/string_array.h"
+#include "utils/path.h"
 #include "utils/utils.h"
 
 static const char * find_existing_cmd(const assoc_list_t *record_list,
@@ -53,15 +54,20 @@ static void reset_list_head(assoc_list_t *assoc_list);
 static void free_assoc_record(assoc_record_t *record);
 static void undouble_commas(char s[]);
 static void free_assoc(assoc_t *assoc);
+static int assoc_records_contains(assoc_records_t *assocs, const char command[],
+		const char description[]);
 static void safe_free(char **adr);
 static int is_assoc_record_empty(const assoc_record_t *record);
 
-/* Predefined builtin command. */
 const assoc_record_t NONE_PSEUDO_PROG = {
 	.command = "",
 	.description = "",
 };
 
+assoc_list_t filetypes;
+assoc_list_t xfiletypes;
+assoc_list_t fileviewers;
+
 /* Internal list that stores only currently active associations.
  * Since it holds only copies of structures from filetype and filextype lists,
  * it doesn't consume much memory, and its items shouldn't be freed */
@@ -89,6 +95,7 @@ ft_exists(const char cmd[])
 
 	char cmd_name[NAME_MAX + 1];
 	(void)extract_cmd_name(cmd, 0, sizeof(cmd_name), cmd_name);
+	system_to_internal_slashes(cmd_name);
 	return external_command_exists_func(cmd_name);
 }
 
@@ -104,8 +111,39 @@ ft_get_viewer(const char file[])
 	return find_existing_cmd(&fileviewers, file);
 }
 
+strlist_t
+ft_get_viewers(const char file[])
+{
+	strlist_t viewers = {};
+
+	int i;
+	for(i = 0; i < fileviewers.count; ++i)
+	{
+		assoc_t *const assoc = &fileviewers.list[i];
+
+		if(!matchers_match(assoc->matchers, file))
+		{
+			continue;
+		}
+
+		int j;
+		for(j = 0; j < assoc->records.count; ++j)
+		{
+			const char *cmd = assoc->records.list[j].command;
+			if(!is_in_string_array(viewers.items, viewers.nitems, cmd) &&
+					ft_exists(cmd))
+			{
+				viewers.nitems = add_to_string_array(&viewers.items, viewers.nitems,
+						cmd);
+			}
+		}
+	}
+
+	return viewers;
+}
+
 /* Finds first existing command which pattern matches given file.  Returns the
- * command (it's lifetime is managed by this unit) or NULL on failure. */
+ * command (its lifetime is managed by this unit) or NULL on failure. */
 static const char *
 find_existing_cmd(const assoc_list_t *record_list, const char file[])
 {
@@ -315,6 +353,12 @@ add_assoc(assoc_list_t *assoc_list, assoc_t assoc)
 ViewerKind
 ft_viewer_kind(const char viewer[])
 {
+	if(is_null_or_empty(viewer))
+	{
+		/* No special viewer implies viewing in text form. */
+		return VK_TEXTUAL;
+	}
+
 	/* %pw and %ph can be useful for text output, but %px and %py are useful
 	 * for graphics only and basically must be used both at the same time. */
 	if(strstr(viewer, "%px") != NULL && strstr(viewer, "%py") != NULL)
@@ -473,6 +517,11 @@ void
 ft_assoc_record_add(assoc_records_t *records, const char *command,
 		const char *description)
 {
+	if(assoc_records_contains(records, command, description))
+	{
+		return;
+	}
+
 	void *p;
 	p = reallocarray(records->list, records->count + 1, sizeof(assoc_record_t));
 	if(p == NULL)
@@ -512,13 +561,35 @@ ft_assoc_record_add_all(assoc_records_t *assocs, const assoc_records_t *src)
 
 	for(i = 0; i < src_count; ++i)
 	{
-		assocs->list[assocs->count + i].command = strdup(src->list[i].command);
-		assocs->list[assocs->count + i].description =
-				strdup(src->list[i].description);
-		assocs->list[assocs->count + i].type = src->list[i].type;
-	}
+		assoc_record_t *record = &src->list[i];
+		if(assoc_records_contains(assocs, record->command, record->description))
+		{
+			continue;
+		}
 
-	assocs->count += src_count;
+		assocs->list[assocs->count].command = strdup(record->command);
+		assocs->list[assocs->count].description = strdup(record->description);
+		assocs->list[assocs->count].type = record->type;
+		++assocs->count;
+	}
+}
+
+/* Checks if list of records contains specified command-description pair.
+ * Returns non-zero if so, otherwise zero is returned. */
+static int
+assoc_records_contains(assoc_records_t *assocs, const char command[],
+		const char description[])
+{
+	int i;
+	for(i = 0; i < assocs->count; ++i)
+	{
+		if(strcmp(assocs->list[i].command, command) == 0
+				&& strcmp(assocs->list[i].description, description) == 0)
+		{
+			return 1;
+		}
+	}
+	return 0;
 }
 
 static void
