@@ -77,11 +77,15 @@ assoc_list_t;
  * name.  Should return non-zero if it exists and zero otherwise. */
 typedef int (*external_command_exists_t)(const char name[]);
 
-const assoc_record_t NONE_PSEUDO_PROG;
+/* Predefined fake builtin command. */
+extern const assoc_record_t NONE_PSEUDO_PROG;
 
-assoc_list_t filetypes;
-assoc_list_t xfiletypes;
-assoc_list_t fileviewers;
+/* All registered non-X filetypes. */
+extern assoc_list_t filetypes;
+/* All registered X filetypes. */
+extern assoc_list_t xfiletypes;
+/* All registered viewers. */
+extern assoc_list_t fileviewers;
 
 /* Unit setup. */
 
@@ -119,6 +123,11 @@ void ft_set_programs(struct matchers_t *matchers, const char programs[],
  * otherwise returns pointer to string stored internally. */
 const char * ft_get_viewer(const char file[]);
 
+struct strlist_t;
+
+/* Gets all existing viewers for file.  Returns the list, which can be empty. */
+struct strlist_t ft_get_viewers(const char file[]);
+
 /* Gets list of programs associated with specified file name.  Returns the list.
  * Caller should free the result by calling ft_assoc_records_free() on it. */
 assoc_records_t ft_get_all_viewers(const char file[]);
@@ -127,7 +136,8 @@ assoc_records_t ft_get_all_viewers(const char file[]);
  * comma separated viewers. */
 void ft_set_viewers(struct matchers_t *matchers, const char viewers[]);
 
-/* Guesses kind of viewer from the invocation command.  Returns the kind. */
+/* Guesses kind of viewer from the invocation command.  The parameter can be
+ * empty or NULL in which case textual kind is implied.  Returns the kind. */
 ViewerKind ft_viewer_kind(const char viewer[]);
 
 /* Records managing. */
