@@ -24,6 +24,7 @@
 
 #include "cfg/config.h"
 #include "compat/reallocarray.h"
+#include "ui/tabs.h"
 #include "ui/ui.h"
 #include "utils/dynarray.h"
 #include "utils/matcher.h"
@@ -37,20 +38,19 @@
 #include "opt_handlers.h"
 
 static void reset_filter(filter_t *filter);
-static int is_newly_filtered(view_t *view, const dir_entry_t *entry,
-		void *arg);
+static int is_newly_filtered(view_t *view, const dir_entry_t *entry, void *arg);
 static void replace_matcher(matcher_t **matcher, const char expr[]);
-static int get_unfiltered_pos(const view_t *const view, int pos);
-static int load_unfiltered_list(view_t *const view);
-static int list_is_incomplete(view_t *const view);
-static void store_local_filter_position(view_t *const view, int pos);
+static int get_unfiltered_pos(const view_t *view, int pos);
+static int load_unfiltered_list(view_t *view);
+static int list_is_incomplete(view_t *view);
+static void store_local_filter_position(view_t *view, int pos);
 static int update_filtering_lists(view_t *view, int add, int clear);
 static void reparent_tree_node(dir_entry_t *original, dir_entry_t *filtered);
 static void ensure_filtered_list_not_empty(view_t *view,
 		dir_entry_t *parent_entry);
-static int extract_previously_selected_pos(view_t *const view);
-static void clear_local_filter_hist_after(view_t *const view, int pos);
-static int find_nearest_neighour(const view_t *const view);
+static int extract_previously_selected_pos(view_t *view);
+static void clear_local_filter_hist_after(view_t *view, int pos);
+static int find_nearest_neighour(const view_t *view);
 static void local_filter_finish(view_t *view);
 static void append_slash(const char name[], char buf[], size_t buf_size);
 
@@ -96,14 +96,22 @@ dot_filter_set(view_t *view, int visible)
 	{
 		load_dot_filter_option(view);
 	}
+
 	if(curr_stats.global_local_settings)
 	{
-		view_t *other = (view == curr_view) ? other_view : curr_view;
-		other->hide_dot_g = other->hide_dot = !visible;
-		ui_view_schedule_reload(other);
-		if(other == curr_view)
+		int i;
+		tab_info_t tab_info;
+		for(i = 0; tabs_enum_all(i, &tab_info); ++i)
 		{
-			load_dot_filter_option(other);
+			if(tab_info.view != view)
+			{
+				tab_info.view->hide_dot_g = tab_info.view->hide_dot = !visible;
+				ui_view_schedule_reload(tab_info.view);
+				if(tab_info.view == curr_view)
+				{
+					load_dot_filter_option(tab_info.view);
+				}
+			}
 		}
 	}
 }
@@ -117,30 +125,38 @@ dot_filter_toggle(view_t *view)
 	{
 		load_dot_filter_option(view);
 	}
+
 	if(curr_stats.global_local_settings)
 	{
-		view_t *other = (view == curr_view) ? other_view : curr_view;
-		other->hide_dot_g = other->hide_dot = !other->hide_dot;
-		ui_view_schedule_reload(other);
-		if(other == curr_view)
+		int i;
+		tab_info_t tab_info;
+		for(i = 0; tabs_enum_all(i, &tab_info); ++i)
 		{
-			load_dot_filter_option(other);
+			if(tab_info.view != view)
+			{
+				tab_info.view->hide_dot = !tab_info.view->hide_dot;
+				tab_info.view->hide_dot_g = tab_info.view->hide_dot;
+				ui_view_schedule_reload(tab_info.view);
+				if(tab_info.view == curr_view)
+				{
+					load_dot_filter_option(tab_info.view);
+				}
+			}
 		}
 	}
 }
 
 void
-name_filters_add_selection(view_t *view)
+name_filters_add_active(view_t *view)
 {
-	dir_entry_t *entry;
 	filter_t filter;
-	int filtered;
-
 	(void)filter_init(&filter, FILTER_DEF_CASE_SENSITIVITY);
 
+	flist_set_marking(view, 0);
+
 	/* Traverse items and update/create filter values. */
-	entry = NULL;
-	while(iter_selection_or_current(view, &entry))
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry))
 	{
 		const char *name = entry->name;
 		char name_with_slash[NAME_MAX + 1 + 1];
@@ -173,21 +189,23 @@ name_filters_add_selection(view_t *view)
 
 	/* Update entry lists to remove entries that must be filtered out now.  No
 	 * view reload is needed. */
-	filtered = zap_entries(view, view->dir_entry, &view->list_rows,
+	int filtered = zap_entries(view, view->dir_entry, &view->list_rows,
 			&is_newly_filtered, &filter, 0, 1);
 	if(flist_custom_active(view))
 	{
-		(void)zap_entries(view, view->local_filter.entries,
-				&view->local_filter.entry_count, &is_newly_filtered, &filter, 1, 1);
+		/* Name exclusion from a custom filter is not reversible. */
+		(void)zap_entries(view, view->custom.full.entries,
+				&view->custom.full.nentries, &is_newly_filtered, &filter, 1, 1);
 	}
-	else
+
+	if(flist_is_fs_backed(view))
 	{
 		view->filtered += filtered;
 	}
 
 	filter_dispose(&filter);
 
-	flist_ensure_pos_is_valid(view);
+	fpos_ensure_valid_pos(view);
 	ui_view_schedule_redraw(view);
 }
 
@@ -277,11 +295,11 @@ filters_invert(view_t *view)
 {
 	view->invert = !view->invert;
 	load_dir_list(view, 1);
-	flist_set_pos(view, 0);
+	fpos_set_pos(view, 0);
 }
 
 int
-filters_file_is_visible(view_t *view, const char dir[], const char name[],
+filters_file_is_visible(const view_t *view, const char dir[], const char name[],
 		int is_dir, int apply_local_filter)
 {
 	/* FIXME: some very long file names won't be matched against some regexps. */
@@ -394,7 +412,7 @@ local_filter_set(view_t *view, const char filter[])
 /* Gets position of an item in dir_entry list at position pos in the unfiltered
  * list.  Returns index on success, otherwise -1 is returned. */
 static int
-get_unfiltered_pos(const view_t *const view, int pos)
+get_unfiltered_pos(const view_t *view, int pos)
 {
 	const int count = (int)view->local_filter.unfiltered_count;
 	const char *const filename = view->dir_entry[pos].name;
@@ -413,7 +431,7 @@ get_unfiltered_pos(const view_t *const view, int pos)
 /* Loads full list of files into unfiltered list of the view.  Returns position
  * of file under cursor in the unfiltered list. */
 static int
-load_unfiltered_list(view_t *const view)
+load_unfiltered_list(view_t *view)
 {
 	int current_file_pos = view->list_pos;
 
@@ -423,7 +441,7 @@ load_unfiltered_list(view_t *const view)
 
 	if(list_is_incomplete(view))
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		dir_entry_t *entry;
 
 		get_current_full_path(view, sizeof(full_path), full_path);
@@ -443,12 +461,8 @@ load_unfiltered_list(view_t *const view)
 			current_file_pos = view->list_rows - 1;
 		}
 	}
-	else
-	{
-		/* Save unfiltered (by local filter) list for further use. */
-		replace_dir_entries(view, &view->local_filter.entries,
-				&view->local_filter.entry_count, view->dir_entry, view->list_rows);
-	}
+
+	flist_custom_save(view);
 
 	view->local_filter.unfiltered = view->dir_entry;
 	view->local_filter.unfiltered_count = view->list_rows;
@@ -461,16 +475,14 @@ load_unfiltered_list(view_t *const view)
 /* Checks whether list of files is incomplete.  Returns non-zero if so,
  * otherwise zero is returned. */
 static int
-list_is_incomplete(view_t *const view)
+list_is_incomplete(view_t *view)
 {
-	int i;
-
-	if(view->filtered > 0)
+	if(view->filtered > 0 && !filter_is_empty(&view->local_filter.filter))
 	{
 		return 1;
 	}
 
-	if(flist_custom_active(view) && view->custom.type != CV_TREE)
+	if(!flist_is_fs_backed(view))
 	{
 		return 0;
 	}
@@ -478,6 +490,7 @@ list_is_incomplete(view_t *const view)
 	/* Check if there are any directories without leaf nodes.  They aren't counted
 	 * as filtered out, so need to check separately (or start counting them in
 	 * some way). */
+	int i;
 	for(i = 0; i < view->list_rows; ++i)
 	{
 		dir_entry_t *const entry = &view->dir_entry[i];
@@ -493,7 +506,7 @@ list_is_incomplete(view_t *const view)
 
 /* Adds local filter position (in unfiltered list) to position history. */
 static void
-store_local_filter_position(view_t *const view, int pos)
+store_local_filter_position(view_t *view, int pos)
 {
 	size_t *const len = &view->local_filter.poshist_len;
 	int *arr = reallocarray(view->local_filter.poshist, *len + 1, sizeof(int));
@@ -697,14 +710,14 @@ local_filter_update_view(view_t *view, int rel_pos)
 /* Finds one of previously selected files and updates list of visited files
  * if needed.  Returns file position in current list of files or -1. */
 static int
-extract_previously_selected_pos(view_t *const view)
+extract_previously_selected_pos(view_t *view)
 {
 	size_t i;
 	for(i = 0; i < view->local_filter.poshist_len; i++)
 	{
 		const int unfiltered_pos = view->local_filter.poshist[i];
 		const dir_entry_t *entry = &view->local_filter.unfiltered[unfiltered_pos];
-		const int filtered_pos = flist_find_entry(view, entry->name, entry->origin);
+		const int filtered_pos = fpos_find_entry(view, entry->name, entry->origin);
 
 		if(filtered_pos >= 0)
 		{
@@ -717,7 +730,7 @@ extract_previously_selected_pos(view_t *const view)
 
 /* Clears all positions after the pos. */
 static void
-clear_local_filter_hist_after(view_t *const view, int pos)
+clear_local_filter_hist_after(view_t *view, int pos)
 {
 	view->local_filter.poshist_len -= view->local_filter.poshist_len - (pos + 1);
 }
@@ -725,7 +738,7 @@ clear_local_filter_hist_after(view_t *const view, int pos)
 /* Find nearest filtered neighbour.  Returns index of nearest unfiltered
  * neighbour of the entry initially pointed to by cursor. */
 static int
-find_nearest_neighour(const view_t *const view)
+find_nearest_neighour(const view_t *view)
 {
 	const int count = view->local_filter.unfiltered_count;
 
@@ -733,9 +746,9 @@ find_nearest_neighour(const view_t *const view)
 	{
 		const int unfiltered_orig_pos = view->local_filter.poshist[0U];
 		int i;
-		for(i = unfiltered_orig_pos; i < count; i++)
+		for(i = unfiltered_orig_pos; i < count; ++i)
 		{
-			const int filtered_pos = find_file_pos_in_list(view,
+			const int filtered_pos = fpos_find_by_name(view,
 					view->local_filter.unfiltered[i].name);
 			if(filtered_pos >= 0)
 			{
@@ -776,16 +789,13 @@ local_filter_apply(view_t *view, const char filter[])
 		return;
 	}
 
-	(void)filter_set(&view->local_filter.filter, filter);
+	int case_sensitive = !regexp_should_ignore_case(filter);
+	(void)filter_change(&view->local_filter.filter, filter, case_sensitive);
 	hists_filter_save(view->local_filter.filter.raw);
 
-	if(flist_custom_active(view) && view->custom.type != CV_TREE &&
-			view->local_filter.entry_count == 0)
-	{
-		/* Save unfiltered (by local filter) list for further use. */
-		replace_dir_entries(view, &view->local_filter.entries,
-				&view->local_filter.entry_count, view->dir_entry, view->list_rows);
-	}
+	flist_custom_save(view);
+
+	ui_view_schedule_reload(view);
 }
 
 void
