@@ -54,11 +54,18 @@ void
 flist_sel_drop(view_t *view)
 {
 	int i;
+	int need_redraw = 0;
 	for(i = 0; i < view->list_rows; ++i)
 	{
+		need_redraw |= view->dir_entry[i].selected;
 		view->dir_entry[i].selected = 0;
 	}
 	view->selected_files = 0;
+
+	if(need_redraw)
+	{
+		ui_view_schedule_redraw(view);
+	}
 }
 
 void
@@ -148,7 +155,6 @@ flist_sel_stash_if_nonempty(view_t *view)
 	if(view->selected_files != 0)
 	{
 		flist_sel_stash(view);
-		ui_view_schedule_redraw(view);
 	}
 }
 
@@ -255,9 +261,10 @@ flist_sel_by_filter(view_t *view, const char cmd[], int erase_old, int select)
 	int nfiles;
 	int i;
 
-	char *const expanded_cmd = ma_expand(cmd, NULL, NULL, 1);
+	MacroFlags flags;
+	char *const expanded_cmd = ma_expand(cmd, NULL, &flags, 1);
 
-	if(run_cmd_for_output(expanded_cmd, &files, &nfiles) != 0)
+	if(rn_for_lines(view, expanded_cmd, &files, &nfiles, flags) != 0)
 	{
 		free(expanded_cmd);
 		ui_sb_err("Failed to start/read output of external command");
@@ -389,55 +396,50 @@ flist_sel_count(view_t *view, int at, int count)
 }
 
 int
-flist_sel_range(view_t *view, int begin, int end, int select_current)
+flist_sel_range(view_t *view, int begin, int end, int mark_current)
 {
-	/* Both starting and ending range positions are given. */
+	/* Range was specified for a command. */
 	if(begin > -1)
 	{
 		int i;
-		flist_sel_stash(view);
-
+		clear_marking(view);
 		for(i = begin; i <= end; ++i)
 		{
 			if(fentry_is_valid(&view->dir_entry[i]))
 			{
-				view->dir_entry[i].selected = 1;
-				++view->selected_files;
+				view->dir_entry[i].marked = 1;
 			}
 		}
-		return view->selected_files > 0;
+		return 1;
 	}
 
+	/* Use user's selection. */
 	if(view->selected_files != 0)
 	{
 		return 0;
 	}
 
+	clear_marking(view);
+
 	/* XXX: is it possible that begin <= -1 and end > -1? */
 	if(end > -1)
 	{
-		flist_sel_stash(view);
-
 		if(fentry_is_valid(&view->dir_entry[end]))
 		{
-			view->dir_entry[end].selected = 1;
-			view->selected_files = 1;
+			view->dir_entry[end].marked = 1;
 		}
 	}
-	else if(select_current)
+	else if(mark_current)
 	{
-		flist_sel_stash(view);
-
 		/* The front check is for tests. */
 		if(view->list_pos < view->list_rows &&
 				fentry_is_valid(&view->dir_entry[view->list_pos]))
 		{
-			view->dir_entry[view->list_pos].selected = 1;
-			view->selected_files = 1;
+			view->dir_entry[view->list_pos].marked = 1;
 		}
 	}
 
-	return view->selected_files > 0;
+	return 1;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
