@@ -21,27 +21,30 @@
 
 #include <regex.h>
 
-#include <fcntl.h>
 #include <sys/stat.h> /* stat umask() */
 #include <sys/types.h> /* mode_t */
+#include <fcntl.h>
+#include <unistd.h> /* unlink() */
+
 #ifdef _WIN32
 #include <windows.h>
 #include <shellapi.h>
 #endif
-#include <unistd.h> /* unlink() */
 
 #include <assert.h> /* assert() */
 #include <errno.h> /* errno */
 #include <stddef.h> /* NULL size_t */
 #include <stdint.h> /* uint64_t */
 #include <stdio.h> /* snprintf() */
-#include <stdlib.h> /* calloc() free() malloc() realloc() strtol() */
-#include <string.h> /* memcmp() strcmp() strdup() strlen() */
+#include <stdlib.h> /* free() malloc() */
+#include <string.h> /* memset() strcat() strcmp() strdup() strlen() */
+#include <time.h> /* clock_gettime() */
 
 #include "cfg/config.h"
 #include "compat/dtype.h"
 #include "compat/fs_limits.h"
 #include "compat/os.h"
+#include "int/ext_edit.h"
 #include "int/vim.h"
 #include "io/ioeta.h"
 #include "io/ionotif.h"
@@ -61,6 +64,7 @@
 #include "utils/path.h"
 #include "utils/str.h"
 #include "utils/string_array.h"
+#include "utils/test_helpers.h"
 #include "utils/utils.h"
 #include "background.h"
 #include "filelist.h"
@@ -77,6 +81,9 @@
  * on progress percentage counting. */
 #define IO_PRECISION 10
 
+/* Maximum value of progress_data_t::last_progress. */
+#define IO_MAX_PROGRESS (100*(IO_PRECISION))
+
 /* Key used to switch to progress dialog. */
 #define IO_DETAILS_KEY 'i'
 
@@ -91,9 +98,20 @@ typedef struct
 		bg_op_t *bg_op; /* Information for background operation. */
 	};
 
-	int last_progress; /* Progress of the operation during previous call. */
+	int last_progress; /* Progress of the operation during previous call.
+	                      Range is [-1; IO_MAX_PROGRESS]. */
 	IoPs last_stage;   /* Stage of the operation during previous call. */
 
+	char *progress_bar;     /* String of progress bar. */
+	int progress_bar_value; /* Value of progress bar during previous call. */
+	int progress_bar_max;   /* Width of progress bar during previous call. */
+
+	/* State of rate calculation. */
+	long long last_calc_time; /* Time of last rate calculation. */
+	uint64_t last_seen_byte;  /* Position at the time of last call. */
+	uint64_t rate;            /* Rate in bytes per millisecond. */
+	char *rate_str;           /* Rate formatted as a string. */
+
 	/* Whether progress is displayed in a dialog, rather than on status bar. */
 	int dialog;
 
@@ -103,6 +121,9 @@ progress_data_t;
 
 static void io_progress_changed(const io_progress_t *state);
 static int calc_io_progress(const io_progress_t *state, int *skip);
+static void update_io_rate(progress_data_t *pdata, const ioeta_estim_t *estim);
+static void update_progress_bar(progress_data_t *pdata,
+		const ioeta_estim_t *estim);
 static void io_progress_fg(const io_progress_t *state, int progress);
 static void io_progress_fg_sb(const io_progress_t *state, int progress);
 static void io_progress_bg(const io_progress_t *state, int progress);
@@ -111,7 +132,10 @@ static void format_pretty_path(const char base_dir[], const char path[],
 		char pretty[], size_t pretty_size);
 static int is_file_name_changed(const char old[], const char new[]);
 static int ui_cancellation_hook(void *arg);
+TSTATIC char ** edit_list(struct ext_edit_t *ext_edit, size_t orig_len,
+		char *orig[], int *edited_len, int load_always);
 static progress_data_t * alloc_progress_data(int bg, void *info);
+static long long time_in_ms(void);
 
 line_prompt_func fops_line_prompt;
 options_prompt_func fops_options_prompt;
@@ -143,7 +167,7 @@ io_progress_changed(const io_progress_t *state)
 	/* Don't query for scheduled redraw or input for background operations. */
 	if(!pdata->bg)
 	{
-		redraw = stats_redraw_fetch();
+		redraw = (stats_update_fetch() != UT_NONE);
 
 		if(!pdata->dialog)
 		{
@@ -187,7 +211,7 @@ io_progress_changed(const io_progress_t *state)
 
 /* Calculates current IO operation progress.  *skip will be set to non-zero
  * value to indicate that progress change is irrelevant.  Returns progress in
- * the range [-1; 100], where -1 means "unknown". */
+ * the range [-1; IO_MAX_PROGRESS], where -1 means "unknown". */
 static int
 calc_io_progress(const io_progress_t *state, int *skip)
 {
@@ -206,9 +230,9 @@ calc_io_progress(const io_progress_t *state, int *skip)
 			return 0;
 		}
 		/* When files are empty, use their number for progress counting. */
-		return (estim->current_item*100*IO_PRECISION)/estim->total_items;
+		return (estim->current_item*IO_MAX_PROGRESS)/estim->total_items;
 	}
-	else if(pdata->last_progress >= 100*IO_PRECISION &&
+	else if(pdata->last_progress >= IO_MAX_PROGRESS &&
 			estim->current_byte == estim->total_bytes)
 	{
 		/* Special handling for unknown total size. */
@@ -221,10 +245,64 @@ calc_io_progress(const io_progress_t *state, int *skip)
 	}
 	else
 	{
-		return (estim->current_byte*100*IO_PRECISION)/estim->total_bytes;
+		return (estim->current_byte*IO_MAX_PROGRESS)/estim->total_bytes;
 	}
 }
 
+/* Updates rate of operation. */
+static void
+update_io_rate(progress_data_t *pdata, const ioeta_estim_t *estim)
+{
+	long long current_time_ms = time_in_ms();
+	long long elapsed_time_ms = current_time_ms - pdata->last_calc_time;
+
+	if(elapsed_time_ms == 0 ||
+			(pdata->last_seen_byte != 0 && elapsed_time_ms < 3000))
+	{
+		/* Rate is updated initially and then once in 3000 milliseconds. */
+		return;
+	}
+
+	uint64_t bytes_difference = estim->current_byte - pdata->last_seen_byte;
+	pdata->rate = bytes_difference/elapsed_time_ms;
+	pdata->last_calc_time = current_time_ms;
+	pdata->last_seen_byte = estim->current_byte;
+
+	char rate_str[64];
+	(void)friendly_size_notation(pdata->rate*1000, sizeof(rate_str) - 8,
+			rate_str);
+	strcat(rate_str, "/s");
+	replace_string(&pdata->rate_str, rate_str);
+}
+
+/* Updates progress bar of operation. */
+static void
+update_progress_bar(progress_data_t *pdata, const ioeta_estim_t *estim)
+{
+	int max_width = pdata->width - 6;
+	if(max_width <= 0)
+	{
+		return;
+	}
+
+	int value = (pdata->last_progress*max_width)/IO_MAX_PROGRESS;
+	if(value == pdata->progress_bar_value && max_width == pdata->progress_bar_max)
+	{
+		return;
+	}
+
+	pdata->progress_bar_value = value;
+	pdata->progress_bar_max = max_width;
+
+	pdata->progress_bar = malloc(max_width + 3);
+
+	pdata->progress_bar[0] = '[';
+	memset(pdata->progress_bar + 1, '=', value);
+	memset(pdata->progress_bar + 1 + value, ' ', max_width - value);
+	pdata->progress_bar[1 + max_width] = ']';
+	pdata->progress_bar[1 + max_width + 1] = '\0';
+}
+
 /* Takes care of progress for foreground operations. */
 static void
 io_progress_fg(const io_progress_t *state, int progress)
@@ -287,34 +365,39 @@ io_progress_fg(const io_progress_t *state, int progress)
 
 	item_num = MIN(estim->current_item + 1, estim->total_items);
 
+	update_io_rate(pdata, estim);
+
 	if(progress < 0)
 	{
 		/* Simplified message for unknown total size. */
 		draw_msgf(title, ctrl_msg, pdata->width,
 				"Location: %s\nItem:     %d of %" PRINTF_ULL "\n"
-				"Overall:  %s\n"
+				"Overall:  %s %s\n"
 				" \n" /* Space is on purpose to preserve empty line. */
 				"file %s\nfrom %s%s",
 				replace_home_part(ops->target_dir), item_num,
-				(unsigned long long)estim->total_items, total_size_str, item_name,
-				src_path, as_part);
+				(unsigned long long)estim->total_items, total_size_str, pdata->rate_str,
+				item_name, src_path, as_part);
 	}
 	else
 	{
 		char *const file_progress = format_file_progress(estim, IO_PRECISION);
+		update_progress_bar(pdata, estim);
 
 		draw_msgf(title, ctrl_msg, pdata->width,
 				"Location: %s\nItem:     %d of %" PRINTF_ULL "\n"
-				"Overall:  %s/%s (%2d%%)\n"
+				"Overall:  %s/%s (%2d%%) %s\n"
+				"%s\n"
 				" \n" /* Space is on purpose to preserve empty line. */
 				"file %s\nfrom %s%s%s",
 				replace_home_part(ops->target_dir), item_num,
 				(unsigned long long)estim->total_items, current_size_str,
-				total_size_str, progress/IO_PRECISION, item_name, src_path, as_part,
-				file_progress);
+				total_size_str, progress/IO_PRECISION, pdata->rate_str,
+				pdata->progress_bar, item_name, src_path, as_part, file_progress);
 
 		free(file_progress);
 	}
+
 	pdata->width = getmaxx(error_win);
 
 	free(as_part);
@@ -431,21 +514,21 @@ format_pretty_path(const char base_dir[], const char path[], char pretty[],
 }
 
 int
-fops_is_name_list_ok(int count, int nlines, char *list[], char *files[])
+fops_is_name_list_ok(int count, int nlines, char *list[], char *files[],
+		char **error)
 {
 	int i;
 
 	if(nlines < count)
 	{
-		ui_sb_errf("Not enough file names (%d/%d)", nlines, count);
-		curr_stats.save_msg = 1;
+		put_string(error, format_str("Not enough file names (%d/%d)", nlines,
+					count));
 		return 0;
 	}
 
 	if(nlines > count)
 	{
-		ui_sb_errf("Too many file names (%d/%d)", nlines, count);
-		curr_stats.save_msg = 1;
+		put_string(error, format_str("Too many file names (%d/%d)", nlines, count));
 		return 0;
 	}
 
@@ -459,14 +542,19 @@ fops_is_name_list_ok(int count, int nlines, char *list[], char *files[])
 			char *list_s = find_slashr(list[i]);
 			if(list_s != NULL || file_s != NULL)
 			{
-				if(list_s - list[i] != file_s - files[i] ||
+				if(list_s == NULL || file_s == NULL ||
+						list_s - list[i] != file_s - files[i] ||
 						strnoscmp(files[i], list[i], list_s - list[i]) != 0)
 				{
 					if(file_s == NULL)
-						ui_sb_errf("Name \"%s\" contains slash", list[i]);
+					{
+						put_string(error, format_str("Name \"%s\" contains slash",
+									list[i]));
+					}
 					else
-						ui_sb_errf("Won't move \"%s\" file", files[i]);
-					curr_stats.save_msg = 1;
+					{
+						put_string(error, format_str("Won't move \"%s\" file", files[i]));
+					}
 					return 0;
 				}
 			}
@@ -474,8 +562,7 @@ fops_is_name_list_ok(int count, int nlines, char *list[], char *files[])
 
 		if(list[i][0] != '\0' && is_in_string_array(list, i, list[i]))
 		{
-			ui_sb_errf("Name \"%s\" duplicates", list[i]);
-			curr_stats.save_msg = 1;
+			put_string(error, format_str("Name \"%s\" duplicates", list[i]));
 			return 0;
 		}
 	}
@@ -484,21 +571,42 @@ fops_is_name_list_ok(int count, int nlines, char *list[], char *files[])
 }
 
 int
-fops_is_rename_list_ok(char *files[], char is_dup[], int len, char *list[])
+fops_is_copy_list_ok(const char dst[], int count, char *list[], int force,
+		char **error)
+{
+	if(force)
+	{
+		return 1;
+	}
+
+	int i;
+	for(i = 0; i < count; ++i)
+	{
+		if(path_exists_at(dst, list[i], NODEREF))
+		{
+			put_string(error, format_str("File \"%s\" already exists", list[i]));
+			return 0;
+		}
+	}
+	return 1;
+}
+
+int
+fops_is_rename_list_ok(char *files[], char is_dup[], int len, char *list[],
+		char **error)
 {
 	int i;
 	const char *const work_dir = flist_get_dir(curr_view);
 	for(i = 0; i < len; ++i)
 	{
-		int j;
-
 		const int check_result =
-			fops_check_file_rename(work_dir, files[i], list[i], ST_NONE);
+			fops_check_file_rename(work_dir, files[i], list[i], error);
 		if(check_result < 0)
 		{
 			continue;
 		}
 
+		int j;
 		for(j = 0; j < len; ++j)
 		{
 			if(strcmp(list[i], files[j]) == 0 && !is_dup[j])
@@ -507,20 +615,19 @@ fops_is_rename_list_ok(char *files[], char is_dup[], int len, char *list[])
 				break;
 			}
 		}
+
 		if(j >= len && check_result == 0)
 		{
-			/* Invoke fops_check_file_rename() again, but this time to produce error
-			 * message. */
-			(void)fops_check_file_rename(work_dir, files[i], list[i], ST_STATUS_BAR);
 			break;
 		}
+		update_string(error, NULL);
 	}
 	return i >= len;
 }
 
 int
 fops_check_file_rename(const char dir[], const char old[], const char new[],
-		SignalType signal_type)
+		char **error)
 {
 	if(!is_file_name_changed(old, new))
 	{
@@ -530,21 +637,7 @@ fops_check_file_rename(const char dir[], const char old[], const char new[],
 	if(path_exists_at(dir, new, NODEREF) && stroscmp(old, new) != 0 &&
 			!is_case_change(old, new))
 	{
-		switch(signal_type)
-		{
-			case ST_STATUS_BAR:
-				ui_sb_errf("File \"%s\" already exists", new);
-				curr_stats.save_msg = 1;
-				break;
-			case ST_DIALOG:
-				show_error_msg("File exists",
-						"That file already exists. Will not overwrite.");
-				break;
-
-			default:
-				assert(signal_type == ST_NONE && "Unhandled signaling type");
-				break;
-		}
+		put_string(error, format_str("File \"%s\" already exists", new));
 		return 0;
 	}
 
@@ -571,7 +664,7 @@ fops_grab_marked_files(view_t *view, size_t *nmarked)
 	*nmarked = 0;
 	while(iter_marked_entries(view, &entry))
 	{
-		*nmarked = add_to_string_array(&marked, *nmarked, 1, entry->name);
+		*nmarked = add_to_string_array(&marked, *nmarked, entry->name);
 	}
 	return marked;
 }
@@ -613,7 +706,7 @@ fops_enqueue_marked_files(ops_t *ops, view_t *view, const char dst_hint[],
 
 		if(to_trash)
 		{
-			char *const trash_dir = pick_trash_dir(entry->origin);
+			char *const trash_dir = trash_pick_dir(entry->origin);
 			ops_enqueue(ops, full_path, trash_dir);
 			free(trash_dir);
 		}
@@ -641,6 +734,7 @@ fops_get_ops(OPS main_op, const char descr[], const char base_dir[],
 		const io_cancellation_t cancellation = { .hook = &ui_cancellation_hook };
 		ops->estim = ioeta_alloc(pdata, cancellation);
 	}
+	ui_cancellation_push_off();
 	return ops;
 }
 
@@ -669,7 +763,7 @@ fops_get_dst_name(const char src_path[], int from_trash)
 {
 	if(from_trash)
 	{
-		return get_real_name_from_trash_name(src_path);
+		return trash_get_real_name_of(src_path);
 	}
 	return get_last_path_component(src_path);
 }
@@ -731,66 +825,105 @@ fops_check_dir_path(const view_t *view, const char path[], char buf[],
 }
 
 char **
-fops_edit_list(size_t count, char *orig[], int *nlines, int load_always)
+fops_query_list(size_t orig_len, char *orig[], int *edited_len, int load_always,
+		fops_query_verify_func verify, void *verify_data)
 {
-	char rename_file[PATH_MAX + 1];
+	ext_edit_t ext_edit = {};
 	char **list = NULL;
-	mode_t saved_umask;
+	int nlines = 0;
+	char *error_str = NULL;
 
-	*nlines = 0;
+	while(1)
+	{
+		list = edit_list(&ext_edit, orig_len, orig, &nlines, 0);
+		if(nlines == 0)
+		{
+			/* Cancelled. */
+			break;
+		}
 
+		if(!verify(orig, orig_len, list, nlines, &error_str, verify_data))
+		{
+			free_string_array(list, nlines);
+			list = NULL;
+			nlines = 0;
+
+			/* Stray space prevents removal of the line. */
+			if(prompt_msgf("Naming error", "%s\n \nRe-edit names?", error_str))
+			{
+				update_string(&ext_edit.last_error, error_str);
+				continue;
+			}
+		}
+
+		break;
+	}
+
+	free(error_str);
+	ext_edit_discard(&ext_edit);
+
+	*edited_len = nlines;
+	return list;
+}
+
+/* Prompts user with a file containing lines from orig array of length orig_len
+ * and returns modified list of strings of length *edited_len or NULL on error
+ * or unchanged list unless load_always is non-zero. */
+TSTATIC char **
+edit_list(ext_edit_t *ext_edit, size_t orig_len, char *orig[], int *edited_len,
+		int load_always)
+{
+	*edited_len = 0;
+
+	char rename_file[PATH_MAX + 1];
 	generate_tmp_file_name("vifm.rename", rename_file, sizeof(rename_file));
 
+	strlist_t prepared = ext_edit_prepare(ext_edit, orig, orig_len);
+
 	/* Allow temporary file to be only readable and writable by current user. */
-	saved_umask = umask(~0600);
-	if(write_file_of_lines(rename_file, orig, count) != 0)
+	mode_t saved_umask = umask(~0600);
+	const int write_error = (write_file_of_lines(rename_file, prepared.items,
+				prepared.nitems) != 0);
+	(void)umask(saved_umask);
+
+	free_string_array(prepared.items, prepared.nitems);
+
+	if(write_error)
 	{
-		(void)umask(saved_umask);
 		show_error_msgf("Error Getting List Of Renames",
 				"Can't create temporary file \"%s\": %s", rename_file, strerror(errno));
 		return NULL;
 	}
-	(void)umask(saved_umask);
 
 	if(vim_view_file(rename_file, -1, -1, 0) != 0)
 	{
+		unlink(rename_file);
 		show_error_msgf("Error Editing File", "Editing of file \"%s\" failed.",
 				rename_file);
-	}
-	else
-	{
-		list = read_file_of_lines(rename_file, nlines);
-		if(list == NULL)
-		{
-			show_error_msgf("Error Getting List Of Renames",
-					"Can't open temporary file \"%s\": %s", rename_file, strerror(errno));
-		}
-
-		if(!load_always)
-		{
-			size_t i = count - 1U;
-			if((size_t)*nlines == count)
-			{
-				for(i = 0U; i < count; ++i)
-				{
-					if(strcmp(list[i], orig[i]) != 0)
-					{
-						break;
-					}
-				}
-			}
-
-			if(i == count)
-			{
-				free_string_array(list, *nlines);
-				list = NULL;
-				*nlines = 0;
-			}
-		}
+		return NULL;
 	}
 
+	int result_len;
+	char **result = read_file_of_lines(rename_file, &result_len);
 	unlink(rename_file);
-	return list;
+	if(result == NULL)
+	{
+		show_error_msgf("Error Getting List Of Renames",
+				"Can't open temporary file \"%s\": %s", rename_file, strerror(errno));
+		return NULL;
+	}
+
+	ext_edit_done(ext_edit, orig, orig_len, result, &result_len);
+
+	if(!load_always && string_array_equal(orig, orig_len, result, result_len))
+	{
+		ext_edit_discard(ext_edit);
+		free_string_array(result, result_len);
+		return NULL;
+	}
+
+	*edited_len = result_len;
+	return result;
 }
 
 void
@@ -829,12 +962,36 @@ alloc_progress_data(int bg, void *info)
 
 	pdata->last_progress = -1;
 	pdata->last_stage = (IoPs)-1;
+
+	pdata->progress_bar = strdup("");
+	pdata->progress_bar_value = 0;
+	pdata->progress_bar_max = 0;
+
+	/* Time of starting the operation to have meaningful first rate. */
+	pdata->last_calc_time = time_in_ms();
+	pdata->last_seen_byte = 0;
+	pdata->rate = 0;
+	pdata->rate_str = strdup("? B/s");
+
 	pdata->dialog = 0;
 	pdata->width = 0;
 
 	return pdata;
 }
 
+/* Retrieves current time in milliseconds. */
+static long long
+time_in_ms(void)
+{
+	struct timespec current_time;
+	if(clock_gettime(CLOCK_MONOTONIC, &current_time) != 0)
+	{
+		return 0;
+	}
+
+	return current_time.tv_sec*1000 + current_time.tv_nsec/1000000;
+}
+
 void
 fops_free_ops(ops_t *ops)
 {
@@ -845,6 +1002,7 @@ fops_free_ops(ops_t *ops)
 
 	if(!ops->bg)
 	{
+		ui_cancellation_pop();
 		ui_drain_input();
 	}
 
@@ -858,7 +1016,10 @@ fops_free_ops(ops_t *ops)
 			free(title);
 		}
 
-		free(ops->estim->param);
+		progress_data_t *pdata = ops->estim->param;
+		free(pdata->progress_bar);
+		free(pdata->rate_str);
+		free(pdata);
 	}
 	ops_free(ops);
 }
@@ -918,7 +1079,7 @@ fops_prepare_for_bg_task(view_t *view, bg_args_t *args)
 
 		get_full_path_of(entry, sizeof(full_path), full_path);
 		args->sel_list_len = add_to_string_array(&args->sel_list,
-				args->sel_list_len, 1, full_path);
+				args->sel_list_len, full_path);
 	}
 
 	ui_view_reset_selection_and_reload(view);
@@ -976,7 +1137,7 @@ fops_view_can_be_changed(const view_t *view)
 int
 fops_view_can_be_extended(const view_t *view, int at)
 {
-	if(flist_custom_active(view) && !cv_tree(view->custom.type))
+	if(!flist_is_fs_backed(view))
 	{
 		show_error_msg("Operation error",
 				"Custom view can't handle this operation.");
