@@ -39,16 +39,27 @@
 #include "trash.h"
 #include "undo.h"
 
+/* Arguments pack for fops_query_list() verification function. */
+typedef struct
+{
+	view_t *view;         /* Source view. */
+	const char *dst_path; /* Destination directory. */
+	int force;            /* Whether operation should be forced. */
+	CopyMoveLikeOp op;    /* Operation. */
+}
+verify_args_t;
+
 static int cp_file(const char src_dir[], const char dst_dir[], const char src[],
 		const char dst[], CopyMoveLikeOp op, int cancellable, ops_t *ops,
 		int force);
+static int is_erroneous(view_t *view, const char dst_dir[], int force);
 static int cpmv_prepare(view_t *view, char ***list, int *nlines,
 		CopyMoveLikeOp op, int force, char undo_msg[], size_t undo_msg_len,
 		char dst_path[], size_t dst_path_len, int *from_file);
-static int is_copy_list_ok(const char dst[], int count, char *list[],
-		int force);
-static int check_for_clashes(view_t *view, CopyMoveLikeOp op,
-		const char dst_path[], char *list[], char *marked[], int nlines);
+static int verify_list(char *files[], int nfiles, char *names[], int nnames,
+		char **error, void *data);
+static int check_for_clashes(verify_args_t *args, char *list[], char *marked[],
+		int nlines, char **error);
 static const char * cmlo_to_str(CopyMoveLikeOp op);
 static void cpmv_files_in_bg(bg_op_t *bg_op, void *arg);
 static void cpmv_file_in_bg(ops_t *ops, const char src[], const char dst[],
@@ -65,7 +76,7 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 	int i;
 	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
 	dir_entry_t *entry;
-	char path[PATH_MAX + 1];
+	char dst_dir[PATH_MAX + 1];
 	int from_file;
 	ops_t *ops;
 
@@ -77,31 +88,28 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 	}
 
 	err = cpmv_prepare(view, &list, &nlines, op, force, undo_msg,
-			sizeof(undo_msg), path, sizeof(path), &from_file);
+			sizeof(undo_msg), dst_dir, sizeof(dst_dir), &from_file);
 	if(err != 0)
 	{
 		return err > 0;
 	}
 
-	if(pane_in_dir(view, path) && force)
+	if(is_erroneous(view, dst_dir, force))
 	{
-		show_error_msg("Operation Error",
-				"Forcing overwrite when destination and source is same directory will "
-				"lead to losing data");
 		return 0;
 	}
 
 	switch(op)
 	{
 		case CMLO_COPY:
-			ops = fops_get_ops(OP_COPY, "Copying", flist_get_dir(view), path);
+			ops = fops_get_ops(OP_COPY, "Copying", flist_get_dir(view), dst_dir);
 			break;
 		case CMLO_MOVE:
-			ops = fops_get_ops(OP_MOVE, "Moving", flist_get_dir(view), path);
+			ops = fops_get_ops(OP_MOVE, "Moving", flist_get_dir(view), dst_dir);
 			break;
 		case CMLO_LINK_REL:
 		case CMLO_LINK_ABS:
-			ops = fops_get_ops(OP_SYMLINK, "Linking", flist_get_dir(view), path);
+			ops = fops_get_ops(OP_SYMLINK, "Linking", flist_get_dir(view), dst_dir);
 			break;
 
 		default:
@@ -109,9 +117,7 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 			return 0;
 	}
 
-	ui_cancellation_reset();
-
-	nmarked_files = fops_enqueue_marked_files(ops, view, path, 0);
+	nmarked_files = fops_enqueue_marked_files(ops, view, dst_dir, 0);
 
 	un_group_open(undo_msg);
 	i = 0;
@@ -122,21 +128,21 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 		/* Must be at this level as dst might point into this buffer. */
 		char src_full[PATH_MAX + 1];
 
-		char dst_full[PATH_MAX + 1];
-		const char *dst = custom_fnames ? list[i] : entry->name;
+		char dst_full[PATH_MAX + 8];
+		const char *dst = (custom_fnames ? list[i] : entry->name);
 		int err, from_trash;
 
 		get_full_path_of(entry, sizeof(src_full), src_full);
-		from_trash = is_under_trash(src_full);
+		from_trash = trash_has_path(src_full);
 
 		if(from_trash && !custom_fnames)
 		{
 			snprintf(src_full, sizeof(src_full), "%s/%s", entry->origin, dst);
 			chosp(src_full);
-			dst = get_real_name_from_trash_name(src_full);
+			dst = trash_get_real_name_of(src_full);
 		}
 
-		snprintf(dst_full, sizeof(dst_full), "%s/%s", path, dst);
+		snprintf(dst_full, sizeof(dst_full), "%s/%s", dst_dir, dst);
 		if(path_exists(dst_full, NODEREF) && !from_trash)
 		{
 			(void)perform_operation(OP_REMOVESL, NULL, NULL, dst_full, NULL);
@@ -153,7 +159,7 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 
 		if(op == CMLO_MOVE)
 		{
-			err = fops_mv_file(entry->name, entry->origin, dst, path, OP_MOVE, 1,
+			err = fops_mv_file(entry->name, entry->origin, dst, dst_dir, OP_MOVE, 1,
 					ops);
 			if(err != 0)
 			{
@@ -162,7 +168,8 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 		}
 		else
 		{
-			err = cp_file(entry->origin, path, entry->name, dst, op, 1, ops, force);
+			err = cp_file(entry->origin, dst_dir, entry->name, dst, op, 1, ops,
+					force);
 		}
 
 		ops_advance(ops, err == 0);
@@ -171,7 +178,12 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 	}
 	un_group_close();
 
-	ui_views_reload_filelists();
+	if(flist_custom_active(view) || pane_in_dir(view, dst_dir))
+	{
+		ui_view_schedule_reload(view);
+	}
+	ui_view_schedule_reload(view == curr_view ? other_view : curr_view);
+
 	if(from_file)
 	{
 		free_string_array(list, nlines);
@@ -188,13 +200,14 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 void
 fops_replace(view_t *view, const char dst[], int force)
 {
-	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
+	char undo_msg[2*PATH_MAX + 32];
 	dir_entry_t *entry;
 	char dst_dir[PATH_MAX + 1];
 	char src_full[PATH_MAX + 1];
 	const char *const fname = get_last_path_component(dst);
 	ops_t *ops;
 	void *cp = (void *)(size_t)1;
+	view_t *const other = (view == curr_view) ? other_view : curr_view;
 
 	copy_str(dst_dir, sizeof(dst_dir), dst);
 	remove_last_path_component(dst_dir);
@@ -215,11 +228,8 @@ fops_replace(view_t *view, const char dst[], int force)
 
 	un_group_open(undo_msg);
 
-	ui_cancellation_reset();
-
 	if(path_exists(dst, NODEREF) && force)
 	{
-		view_t *const other = (view == curr_view) ? other_view : curr_view;
 		(void)fops_delete_current(other, 1, 1);
 	}
 
@@ -239,7 +249,7 @@ fops_replace(view_t *view, const char dst[], int force)
 
 	un_group_close();
 
-	ui_views_reload_filelists();
+	ui_view_schedule_reload(other);
 
 	fops_free_ops(ops);
 }
@@ -279,14 +289,21 @@ fops_cpmv_bg(view_t *view, char *list[], int nlines, int move, int force)
 		return err > 0;
 	}
 
-	args->list = args->from_file ? list : copy_string_array(list, nlines);
+	if(is_erroneous(view, args->path, force))
+	{
+		fops_free_bg_args(args);
+		return 0;
+	}
+
+	args->list = args->from_file ? list :
+	             args->nlines > 0 ? copy_string_array(list, nlines) : NULL;
 
 	fops_prepare_for_bg_task(view, args);
 
 	args->is_in_trash = malloc(args->sel_list_len);
 	for(i = 0U; i < args->sel_list_len; ++i)
 	{
-		args->is_in_trash[i] = is_under_trash(args->sel_list[i]);
+		args->is_in_trash[i] = trash_has_path(args->sel_list[i]);
 	}
 
 	if(args->list == NULL)
@@ -316,6 +333,22 @@ fops_cpmv_bg(view_t *view, char *list[], int nlines, int move, int force)
 	return 0;
 }
 
+/* Checks operation for adequacy.  Displays error message in case of issues.
+ * Returns non-zero if operation must be aborted, otherwise zero is returned. */
+static int
+is_erroneous(view_t *view, const char dst_dir[], int force)
+{
+	const int same_dir = pane_in_dir(view, dst_dir);
+	if(same_dir && force)
+	{
+		show_error_msg("Operation Aborted",
+				"Forcing overwrite when destination and source is the same directory "
+				"will lead to losing data.");
+		return 1;
+	}
+	return 0;
+}
+
 /* Performs general preparations for file copy/move-like operations: resolving
  * destination path, validating names, checking for conflicts, formatting undo
  * message.  Returns zero on success, otherwise positive number for status bar
@@ -327,10 +360,6 @@ cpmv_prepare(view_t *view, char ***list, int *nlines, CopyMoveLikeOp op,
 {
 	view_t *const other = (view == curr_view) ? other_view : curr_view;
 
-	char **marked;
-	size_t nmarked;
-	int error = 0;
-
 	if(op == CMLO_MOVE)
 	{
 		if(!fops_view_can_be_changed(view))
@@ -361,12 +390,36 @@ cpmv_prepare(view_t *view, char ***list, int *nlines, CopyMoveLikeOp op,
 		return -1;
 	}
 
-	marked = fops_grab_marked_files(view, &nmarked);
+	size_t nmarked;
+	char **marked = fops_grab_marked_files(view, &nmarked);
 
-	*from_file = *nlines < 0;
+	/* Custom views can contain several files with the same name. */
+	if(*nlines == 0 && flist_custom_active(view))
+	{
+		size_t i;
+		for(i = 0U; i < nmarked; ++i)
+		{
+			if(is_in_string_array(marked, i, marked[i]))
+			{
+				ui_sb_errf("Source name \"%s\" duplicates", marked[i]);
+				free_string_array(marked, nmarked);
+				return 1;
+			}
+		}
+	}
+
+	verify_args_t verify_args = {
+		.view = view,
+		.dst_path = dst_path,
+		.force = force,
+		.op = op,
+	};
+
+	*from_file = (*nlines < 0);
 	if(*from_file)
 	{
-		*list = fops_edit_list(nmarked, marked, nlines, 1);
+		*list = fops_query_list(nmarked, marked, nlines, 1, &verify_list,
+				&verify_args);
 		if(*list == NULL)
 		{
 			free_string_array(marked, nmarked);
@@ -374,41 +427,20 @@ cpmv_prepare(view_t *view, char ***list, int *nlines, CopyMoveLikeOp op,
 		}
 	}
 
-	if(*nlines > 0 &&
-			(!fops_is_name_list_ok(nmarked, *nlines, *list, NULL) ||
-			!is_copy_list_ok(dst_path, *nlines, *list, force)))
-	{
-		error = 1;
-	}
-	if(*nlines == 0 && !is_copy_list_ok(dst_path, nmarked, marked, force))
-	{
-		error = 1;
-	}
-
-	/* Custom views can contain several files with the same name. */
-	if(flist_custom_active(view))
-	{
-		size_t i;
-		for(i = 0U; i < nmarked && !error; ++i)
-		{
-			if(is_in_string_array(marked, i, marked[i]))
-			{
-				ui_sb_errf("Source name \"%s\" duplicates", marked[i]);
-				curr_stats.save_msg = 1;
-				error = 1;
-			}
-		}
-	}
-
-	if(check_for_clashes(view, op, dst_path, *list, marked, *nlines) != 0)
-	{
-		error = 1;
-	}
+	char *error_str = NULL;
+	int error = !verify_list(marked, nmarked, *list, *nlines, &error_str,
+			&verify_args);
 
 	free_string_array(marked, nmarked);
 
 	if(error)
 	{
+		if(error_str != NULL)
+		{
+			ui_sb_err(error_str);
+			free(error_str);
+		}
+
 		redraw_view(view);
 		if(*from_file)
 		{
@@ -423,47 +455,52 @@ cpmv_prepare(view_t *view, char ***list, int *nlines, CopyMoveLikeOp op,
 			"%s: ", replace_home_part(dst_path));
 	fops_append_marked_files(view, undo_msg, (*nlines > 0) ? *list : NULL);
 
-	if(op == CMLO_MOVE)
-	{
-		fpos_move_out_of(view, FLS_SELECTION);
-	}
-
 	return 0;
 }
 
-/* Checks whether list of files doesn't mention any existing files.  Returns
- * non-zero if everything is fine, otherwise zero is returned. */
+/* Checks that operation can be performed.  Returns non-zero if so, otherwise
+ * zero is returned along with setting *error. */
 static int
-is_copy_list_ok(const char dst[], int count, char *list[], int force)
+verify_list(char *files[], int nfiles, char *names[], int nnames, char **error,
+		void *data)
 {
-	int i;
+	verify_args_t *args = data;
 
-	if(force)
+	int ok = 1;
+
+	if(nnames > 0 &&
+			(!fops_is_name_list_ok(nfiles, nnames, names, NULL, error) ||
+			!fops_is_copy_list_ok(args->dst_path, nnames, names, args->force, error)))
 	{
-		return 1;
+		ok = 0;
+	}
+	else if(nnames == 0 &&
+			!fops_is_copy_list_ok(args->dst_path, nfiles, files, args->force, error))
+	{
+		ok = 0;
 	}
 
-	for(i = 0; i < count; ++i)
+	if(ok && check_for_clashes(args, names, files, nnames, error) != 0)
 	{
-		if(path_exists_at(dst, list[i], DEREF))
-		{
-			ui_sb_errf("File \"%s\" already exists", list[i]);
-			return 0;
-		}
+		ok = 0;
 	}
-	return 1;
+
+	return ok;
 }
 
 /* Checks whether operation is OK from the point of view of losing files due to
- * tree clashes (child move over parent or vice versa).  Returns zero if
- * everything is fine, otherwise non-zero is returned. */
+ * tree clashes (child move over parent or vice versa).  Reallocates *error to
+ * provide error message.  Returns zero if everything is fine, otherwise
+ * non-zero is returned. */
 static int
-check_for_clashes(view_t *view, CopyMoveLikeOp op, const char dst_path[],
-		char *list[], char *marked[], int nlines)
+check_for_clashes(verify_args_t *args, char *list[], char *marked[], int nlines,
+		char **error)
 {
+	const int is_cpmv = ONE_OF(args->op, CMLO_MOVE, CMLO_COPY);
+
 	dir_entry_t *entry = NULL;
 	int i = 0;
-	while(iter_marked_entries(view, &entry))
+	while(iter_marked_entries(args->view, &entry))
 	{
 		char src_full[PATH_MAX + 1], dst_full[PATH_MAX + 1];
 		const char *const dst_name = (nlines > 0) ? list[i] : marked[i];
@@ -471,22 +508,21 @@ check_for_clashes(view_t *view, CopyMoveLikeOp op, const char dst_path[],
 
 		get_full_path_of(entry, sizeof(src_full), src_full);
 
-		snprintf(dst_full, sizeof(dst_full), "%s/%s", dst_path, dst_name);
+		snprintf(dst_full, sizeof(dst_full), "%s/%s", args->dst_path, dst_name);
 		chosp(dst_full);
 
-		if(ONE_OF(op, CMLO_MOVE, CMLO_COPY) && is_in_subtree(dst_full, src_full, 0))
+		if(is_cpmv && is_in_subtree(dst_full, src_full, 0))
 		{
-			ui_sb_errf("Can't move/copy parent inside itself: %s",
-					replace_home_part(src_full));
-			curr_stats.save_msg = 1;
+			put_string(error, format_str("Can't move/copy parent inside itself: %s",
+						replace_home_part(src_full)));
 			return 1;
 		}
 
 		if(is_in_subtree(src_full, dst_full, 0))
 		{
-			ui_sb_errf("Operation would result in loss contents of %s",
-					replace_home_part(src_full));
-			curr_stats.save_msg = 1;
+			put_string(error,
+					format_str("Operation would result in losing contents of %s",
+						replace_home_part(src_full)));
 			return 1;
 		}
 	}
