@@ -19,6 +19,7 @@
 
 #include "fops_misc.h"
 
+#include <sys/stat.h> /* stat */
 #include <sys/types.h> /* gid_t uid_t */
 
 #include <string.h> /* strdup() strlen() */
@@ -54,6 +55,14 @@ typedef struct
 }
 dir_size_args_t;
 
+/* Arguments pack for fops_query_list() verification function. */
+typedef struct
+{
+	const char *dst_path; /* Destination directory. */
+	int force;            /* Whether operation should be forced. */
+}
+verify_args_t;
+
 static int delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash,
 		int nested);
 static const char * get_top_dir(const view_t *view);
@@ -62,13 +71,14 @@ static void delete_file_in_bg(ops_t *ops, const char path[], int use_trash);
 static int prepare_register(int reg);
 static void change_link_cb(const char new_target[]);
 static int complete_filename(const char str[], void *arg);
-static int is_clone_list_ok(int count, char *list[]);
+static int verify_list(char *files[], int nfiles, char *names[], int nnames,
+		char **error, void *data);
 TSTATIC const char * gen_clone_name(const char dir[], const char normal_name[]);
 static int clone_file(const dir_entry_t *entry, const char path[],
 		const char clone[], ops_t *ops);
 static void get_group_file_list(char *list[], int count, char buf[]);
 static void go_to_first_file(view_t *view, char *names[], int count);
-static void update_dir_entry_size(const view_t *view, int index, int force);
+static void update_dir_entry_size(dir_entry_t *entry, int force);
 static void start_dir_size_calc(const char path[], int force);
 static void dir_size_bg(bg_op_t *bg_op, void *arg);
 static void dir_size(bg_op_t *bg_op, char path[], int force);
@@ -107,14 +117,13 @@ fops_delete(view_t *view, int reg, int use_trash)
 		{
 			char name[NAME_MAX + 1];
 			format_entry_name(entry, NF_FULL, sizeof(name), name);
-			marked.nitems = add_to_string_array(&marked.items, marked.nitems, 1,
-					name);
+			marked.nitems = add_to_string_array(&marked.items, marked.nitems, name);
 			continue;
 		}
 
 		char short_path[PATH_MAX + 1];
 		get_short_path_of(view, entry, NF_FULL, 0, sizeof(short_path), short_path);
-		marked.nitems = add_to_string_array(&marked.items, marked.nitems, 1,
+		marked.nitems = add_to_string_array(&marked.items, marked.nitems,
 				short_path);
 	}
 	if(!confirm_deletion(marked.items, marked.nitems, use_trash))
@@ -125,7 +134,7 @@ fops_delete(view_t *view, int reg, int use_trash)
 	free_string_array(marked.items, marked.nitems);
 
 	/* This check for the case when we are for sure in the trash. */
-	if(use_trash && top_dir != NULL && is_under_trash(top_dir))
+	if(use_trash && top_dir != NULL && trash_has_path(top_dir))
 	{
 		show_error_msg("Can't perform deletion",
 				"Current directory is under trash directory");
@@ -145,8 +154,6 @@ fops_delete(view_t *view, int reg, int use_trash)
 	ops = fops_get_ops(OP_REMOVE, use_trash ? "deleting" : "Deleting", curr_dir,
 			curr_dir);
 
-	ui_cancellation_reset();
-
 	nmarked_files = fops_enqueue_marked_files(ops, view, NULL, use_trash);
 
 	entry = NULL;
@@ -197,7 +204,7 @@ fops_delete_current(view_t *view, int use_trash, int nested)
 	use_trash = use_trash && cfg.use_trash;
 
 	/* This check for the case when we are for sure in the trash. */
-	if(use_trash && top_dir != NULL && is_under_trash(top_dir))
+	if(use_trash && top_dir != NULL && trash_has_path(top_dir))
 	{
 		show_error_msg("Can't perform deletion",
 				"Current directory is under trash directory");
@@ -214,8 +221,6 @@ fops_delete_current(view_t *view, int use_trash, int nested)
 	ops = fops_get_ops(OP_REMOVE, use_trash ? "deleting" : "Deleting", curr_dir,
 			curr_dir);
 
-	ui_cancellation_reset();
-
 	entry = &view->dir_entry[view->list_pos];
 
 	fops_progress_msg("Deleting files", 0, 1);
@@ -258,13 +263,13 @@ delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash, int nested)
 			un_group_add_op(OP_REMOVE, NULL, NULL, full_path, "");
 		}
 	}
-	else if(is_trash_directory(full_path))
+	else if(trash_is_at_path(full_path))
 	{
 		show_error_msg("Can't perform deletion",
 				"You cannot delete trash directory to trash");
 		result = -1;
 	}
-	else if(is_under_trash(full_path))
+	else if(trash_has_path(full_path))
 	{
 		show_error_msgf("Skipping file deletion",
 				"File is already in trash: %s", full_path);
@@ -273,7 +278,7 @@ delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash, int nested)
 	else
 	{
 		const OPS op = nested ? OP_MOVETMP4 : OP_MOVE;
-		char *const dest = gen_trash_name(entry->origin, entry->name);
+		char *const dest = trash_gen_path(entry->origin, entry->name);
 		if(dest != NULL)
 		{
 			result = perform_operation(op, ops, NULL, full_path, dest);
@@ -316,7 +321,7 @@ fops_delete_bg(view_t *view, int use_trash)
 
 	use_trash = use_trash && cfg.use_trash;
 
-	if(use_trash && top_dir != NULL && is_under_trash(top_dir))
+	if(use_trash && top_dir != NULL && trash_has_path(top_dir))
 	{
 		show_error_msg("Can't perform deletion",
 				"Current directory is under trash directory");
@@ -340,14 +345,14 @@ fops_delete_bg(view_t *view, int use_trash)
 		for(i = 0U; i < args->sel_list_len; ++i)
 		{
 			const char *const full_file_path = args->sel_list[i];
-			if(is_trash_directory(full_file_path))
+			if(trash_is_at_path(full_file_path))
 			{
 				show_error_msg("Can't perform deletion",
 						"You cannot delete trash directory to trash");
 				fops_free_bg_args(args);
 				return 0;
 			}
-			else if(is_under_trash(full_file_path))
+			else if(trash_has_path(full_file_path))
 			{
 				show_error_msgf("Skipping file deletion",
 						"File is already in trash: %s", full_file_path);
@@ -363,8 +368,6 @@ fops_delete_bg(view_t *view, int use_trash)
 		return 0;
 	}
 
-	fpos_move_out_of(view, FLS_MARKING);
-
 	snprintf(task_desc, sizeof(task_desc), "%celete in %s: ",
 			use_trash ? 'd' : 'D', replace_home_part(curr_dir));
 
@@ -412,7 +415,7 @@ delete_files_in_bg(bg_op_t *bg_op, void *arg)
 		for(i = 0U; i < args->sel_list_len; ++i)
 		{
 			const char *const src = args->sel_list[i];
-			char *trash_dir = args->use_trash ? pick_trash_dir(src) : args->path;
+			char *trash_dir = (args->use_trash ? trash_pick_dir(src) : args->path);
 			ops_enqueue(ops, src, trash_dir);
 			if(trash_dir != args->path)
 			{
@@ -442,10 +445,10 @@ delete_file_in_bg(ops_t *ops, const char path[], int use_trash)
 		return;
 	}
 
-	if(!is_trash_directory(path))
+	if(!trash_is_at_path(path))
 	{
 		const char *const fname = get_last_path_component(path);
-		char *const trash_name = gen_trash_name(path, fname);
+		char *const trash_name = trash_gen_path(path, fname);
 		const char *const dest = (trash_name != NULL) ? trash_name : fname;
 		(void)perform_operation(OP_MOVE, ops, NULL, path, dest);
 		free(trash_name);
@@ -548,7 +551,7 @@ fops_retarget(view_t *view)
 static void
 change_link_cb(const char new_target[])
 {
-	char undo_msg[COMMAND_GROUP_INFO_LEN];
+	char undo_msg[2*PATH_MAX + 32];
 	char full_path[PATH_MAX + 1];
 	char linkto[PATH_MAX + 1];
 	const char *fname;
@@ -601,17 +604,8 @@ complete_filename(const char str[], void *arg)
 int
 fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 {
-	int i;
-	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
 	char dst_path[PATH_MAX + 1];
-	char **marked;
-	size_t nmarked;
-	int custom_fnames;
-	int nmarked_files;
 	int with_dir = 0;
-	int from_file;
-	dir_entry_t *entry;
-	ops_t *ops;
 	const char *const curr_dir = flist_get_dir(view);
 
 	if(!fops_can_read_marked_files(view))
@@ -645,25 +639,36 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 		return 0;
 	}
 
-	marked = fops_grab_marked_files(view, &nmarked);
+	size_t nmarked;
+	char **marked = fops_grab_marked_files(view, &nmarked);
 
-	from_file = nlines < 0;
+	verify_args_t verify_args = { .dst_path = dst_path, .force = force };
+
+	const int from_file = (nlines < 0);
 	if(from_file)
 	{
-		list = fops_edit_list(nmarked, marked, &nlines, 0);
+		list = fops_query_list(nmarked, marked, &nlines, 0, &verify_list,
+				&verify_args);
 		if(list == NULL)
 		{
 			free_string_array(marked, nmarked);
-			return 0;
+			ui_sb_msg("0 files cloned");
+			return 1;
 		}
 	}
 
 	free_string_array(marked, nmarked);
 
+	char *error_str = NULL;
 	if(nlines > 0 &&
-			(!fops_is_name_list_ok(nmarked, nlines, list, NULL) ||
-			(!force && !is_clone_list_ok(nlines, list))))
+			!verify_list(NULL, nmarked, list, nlines, &error_str, &verify_args))
 	{
+		if(error_str != NULL)
+		{
+			ui_sb_err(error_str);
+			free(error_str);
+		}
+
 		redraw_view(view);
 		if(from_file)
 		{
@@ -674,6 +679,7 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 
 	flist_sel_stash(view);
 
+	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
 	if(with_dir)
 	{
 		snprintf(undo_msg, sizeof(undo_msg), "clone in %s to %s: ", curr_dir,
@@ -685,22 +691,18 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 	}
 	fops_append_marked_files(view, undo_msg, list);
 
-	ops = fops_get_ops(OP_COPY, "Cloning", curr_dir,
+	ops_t *ops = fops_get_ops(OP_COPY, "Cloning", curr_dir,
 			with_dir ? list[0] : curr_dir);
 
-	ui_cancellation_reset();
+	int nmarked_files = fops_enqueue_marked_files(ops, view, dst_path, 0);
 
-	nmarked_files = fops_enqueue_marked_files(ops, view, dst_path, 0);
-
-	custom_fnames = (nlines > 0);
+	const int custom_fnames = (nlines > 0);
 
 	un_group_open(undo_msg);
-	entry = NULL;
-	i = 0;
+	dir_entry_t *entry = NULL;
+	int i = 0;
 	while(iter_marked_entries(view, &entry) && !ui_cancellation_requested())
 	{
-		int err;
-		int j;
 		const char *const name = entry->name;
 		const char *const clone_dst = with_dir ? dst_path : entry->origin;
 		const char *clone_name;
@@ -717,7 +719,8 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 
 		fops_progress_msg("Cloning files", i, nmarked_files);
 
-		err = 0;
+		int j;
+		int err = 0;
 		for(j = 0; j < copies; ++j)
 		{
 			if(path_exists_at(clone_dst, clone_name, NODEREF))
@@ -751,20 +754,16 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 	return 1;
 }
 
-/* Checks consistency of user-supplied list of names for clones. */
+/* Checks that cloning can be performed.  Returns non-zero if so, otherwise zero
+ * is returned along with setting *error. */
 static int
-is_clone_list_ok(int count, char *list[])
+verify_list(char *files[], int nfiles, char *names[], int nnames, char **error,
+		void *data)
 {
-	int i;
-	for(i = 0; i < count; ++i)
-	{
-		if(path_exists(list[i], NODEREF))
-		{
-			ui_sb_errf("File \"%s\" already exists", list[i]);
-			return 0;
-		}
-	}
-	return 1;
+	verify_args_t *args = data;
+	return fops_is_name_list_ok(nfiles, nnames, names, NULL, error)
+	    && fops_is_copy_list_ok(args->dst_path, nnames, names, args->force,
+	                            error);
 }
 
 /* Generates name of clone for a file.  Returns pointer to statically allocated
@@ -876,8 +875,6 @@ fops_mkdirs(view_t *view, int at, char **names, int count, int create_parent)
 		}
 	}
 
-	ui_cancellation_reset();
-
 	snprintf(buf, sizeof(buf), "mkdir in %s: ", replace_home_part(dst_dir));
 
 	get_group_file_list(names, count, buf);
@@ -956,8 +953,6 @@ fops_mkfiles(view_t *view, int at, char *names[], int count)
 		}
 	}
 
-	ui_cancellation_reset();
-
 	ops = fops_get_ops(OP_MKFILE, "touching", dst_dir, dst_dir);
 
 	snprintf(buf, sizeof(buf), "touch in %s: ", replace_home_part(dst_dir));
@@ -1011,10 +1006,9 @@ get_group_file_list(char *list[], int count, char buf[])
 static void
 go_to_first_file(view_t *view, char *names[], int count)
 {
+	load_dir_list(view, 1);
+
 	int i;
-
-	load_saving_pos(view);
-
 	for(i = 0; i < view->list_rows; ++i)
 	{
 		if(is_in_string_array(names, count, view->dir_entry[i].name))
@@ -1023,6 +1017,7 @@ go_to_first_file(view_t *view, char *names[], int count)
 			break;
 		}
 	}
+
 	redraw_view(view);
 }
 
@@ -1033,16 +1028,12 @@ fops_restore(view_t *view)
 	dir_entry_t *entry;
 
 	/* This is general check for regular views only. */
-	if(!flist_custom_active(view) && !is_trash_directory(view->curr_dir))
+	if(!flist_custom_active(view) && !trash_is_at_path(view->curr_dir))
 	{
 		show_error_msg("Restore error", "Not a top-level trash directory.");
 		return 0;
 	}
 
-	fpos_move_out_of(view, FLS_SELECTION);
-
-	ui_cancellation_reset();
-
 	un_group_open("restore: ");
 	un_group_close();
 
@@ -1054,7 +1045,7 @@ fops_restore(view_t *view)
 		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
 
-		if(is_trash_directory(entry->origin) && restore_from_trash(full_path) == 0)
+		if(trash_is_at_path(entry->origin) && trash_restore(full_path) == 0)
 		{
 			++m;
 		}
@@ -1068,39 +1059,36 @@ fops_restore(view_t *view)
 }
 
 void
-fops_size_bg(const view_t *view, int force)
+fops_size_bg(view_t *view, int force)
 {
-	int i;
+	/* flist_set_marking(view, 1) isn't helpful here as it won't mark "..". */
+	int user_selection = !view->pending_marking;
+	flist_set_marking(view, 0);
 
-	if(!get_current_entry(view)->selected && view->user_selection)
+	dir_entry_t *curr = get_current_entry(view);
+	if(!curr->marked && user_selection)
 	{
-		update_dir_entry_size(view, view->list_pos, force);
+		update_dir_entry_size(curr, force);
 		return;
 	}
 
-	for(i = 0; i < view->list_rows; ++i)
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry))
 	{
-		const dir_entry_t *const entry = &view->dir_entry[i];
-
-		if(entry->selected && entry->type == FT_DIR)
-		{
-			update_dir_entry_size(view, i, force);
-		}
+		update_dir_entry_size(entry, force);
 	}
 }
 
 /* Initiates background size calculation for view entry. */
 static void
-update_dir_entry_size(const view_t *view, int index, int force)
+update_dir_entry_size(dir_entry_t *entry, int force)
 {
-	char full_path[PATH_MAX + 1];
-	const dir_entry_t *const entry = &view->dir_entry[index];
-
-	if(fentry_is_fake(entry))
+	if(fentry_is_fake(entry) || !fentry_is_dir(entry))
 	{
 		return;
 	}
 
+	char full_path[PATH_MAX + 1];
 	if(is_parent_dir(entry->name))
 	{
 		copy_str(full_path, sizeof(full_path), entry->origin);
@@ -1116,7 +1104,7 @@ update_dir_entry_size(const view_t *view, int index, int force)
 static void
 start_dir_size_calc(const char path[], int force)
 {
-	char task_desc[PATH_MAX + 1];
+	char task_desc[PATH_MAX + 32];
 	dir_size_args_t *args;
 
 	args = malloc(sizeof(*args));
@@ -1191,6 +1179,17 @@ fops_dir_size(const char path[], int force_update,
 	const char *slash;
 	uint64_t size;
 
+	time_t mtime = 0;
+	uint64_t inode = DCACHE_UNKNOWN;
+#ifndef _WIN32
+	struct stat s;
+	if(os_stat(path, &s) == 0)
+	{
+		mtime = s.st_mtime;
+		inode = s.st_ino;
+	}
+#endif
+
 	DIR *dir = os_opendir(path);
 	if(dir == NULL)
 	{
@@ -1213,7 +1212,7 @@ fops_dir_size(const char path[], int force_update,
 		if(fops_is_dir_entry(full_path, dentry))
 		{
 			uint64_t dir_size;
-			dcache_get_at(full_path, &dir_size, NULL);
+			dcache_get_at(full_path, mtime, inode, &dir_size, NULL);
 			if(dir_size == DCACHE_UNKNOWN || force_update)
 			{
 				dir_size = fops_dir_size(full_path, force_update, cancellation);
@@ -1234,7 +1233,7 @@ fops_dir_size(const char path[], int force_update,
 
 	os_closedir(dir);
 
-	(void)dcache_set_at(path, size, DCACHE_UNKNOWN);
+	(void)dcache_set_at(path, inode, size, DCACHE_UNKNOWN);
 	return size;
 }
 
@@ -1252,10 +1251,8 @@ fops_chown(int u, int g, uid_t uid, gid_t gid)
 	dir_entry_t *entry;
 	const char *const curr_dir = flist_get_dir(view);
 
-	ui_cancellation_reset();
-
-	snprintf(undo_msg, sizeof(undo_msg), "ch%s in %s: ",
-			((u && g) || u) ? "own" : "grp", replace_home_part(curr_dir));
+	snprintf(undo_msg, sizeof(undo_msg), "ch%s in %s: ", u ? "own" : "grp",
+			replace_home_part(curr_dir));
 
 	ops = fops_get_ops(OP_CHOWN, "re-owning", curr_dir, curr_dir);
 	(void)fops_enqueue_marked_files(ops, view, NULL, 0);
