@@ -154,9 +154,9 @@ fops_put_bg(view_t *view, int at, int reg_name, int move)
 		task_desc_len = strlen(task_desc);
 
 		args->sel_list_len = add_to_string_array(&args->sel_list,
-				args->sel_list_len, 1, src);
+				args->sel_list_len, src);
 
-		dst_name = fops_get_dst_name(src, is_under_trash(src));
+		dst_name = fops_get_dst_name(src, trash_has_path(src));
 
 		/* Check that no destination files have the same name. */
 		for(j = 0; j < args->nlines; ++j)
@@ -322,15 +322,14 @@ initiate_put_files(view_t *view, int at, CopyMoveLikeOp op, const char descr[],
 		}
 	}
 
-	ui_cancellation_reset();
-	ui_cancellation_enable();
+	ui_cancellation_push_on();
 
 	for(i = 0; i < reg->nfiles && !ui_cancellation_requested(); ++i)
 	{
 		ops_enqueue(put_confirm.ops, reg->files[i], dst_dir);
 	}
 
-	ui_cancellation_disable();
+	ui_cancellation_pop();
 
 	return put_files_i(view, 1);
 }
@@ -407,7 +406,7 @@ is_dir_clash(const char src_path[], const char dst_dir[])
 	char dst_path[PATH_MAX + 1];
 
 	snprintf(dst_path, sizeof(dst_path), "%s/%s", dst_dir,
-			fops_get_dst_name(src_path, is_under_trash(src_path)));
+			fops_get_dst_name(src_path, trash_has_path(src_path)));
 	chosp(dst_path);
 
 	return is_dir(dst_path);
@@ -422,7 +421,7 @@ put_files_i(view_t *view, int start)
 		char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
 		const char *descr;
 		const int from_trash =
-			is_under_trash(put_confirm.reg->files[put_confirm.file_order[0]]);
+			trash_has_path(put_confirm.reg->files[put_confirm.file_order[0]]);
 
 		if(put_confirm.op == CMLO_LINK_ABS)
 		{
@@ -449,8 +448,6 @@ put_files_i(view_t *view, int start)
 		return 1;
 	}
 
-	ui_cancellation_reset();
-
 	while(put_confirm.index < put_confirm.reg->nfiles)
 	{
 		int put_result;
@@ -571,7 +568,7 @@ put_next(int force)
 		return 0;
 	}
 
-	from_trash = is_under_trash(filename);
+	from_trash = trash_has_path(filename);
 	move = from_trash || put_confirm.op == CMLO_MOVE;
 
 	copy_str(src_buf, sizeof(src_buf), filename);
@@ -585,7 +582,7 @@ put_next(int force)
 	snprintf(dst_buf, sizeof(dst_buf), "%s/%s", dst_dir, dst_name);
 	chosp(dst_buf);
 
-	if(!put_confirm.append && path_exists(dst_buf, DEREF))
+	if(!put_confirm.append && path_exists(dst_buf, NODEREF))
 	{
 		if(force)
 		{
@@ -617,6 +614,8 @@ put_next(int force)
 				if(!safe_operation && perform_operation(OP_REMOVESL, put_confirm.ops,
 							NULL, dst_buf, NULL) != 0)
 				{
+					show_error_msgf("Can't replace a file",
+							"Failed to remove a file:\n%s", dst_buf);
 					return 0;
 				}
 				/* Schedule view update to reflect changes in UI. */
@@ -760,7 +759,7 @@ put_next(int force)
 		char dst_path[PATH_MAX + 1];
 		snprintf(dst_path, sizeof(dst_path), "%s/%s", dst_dir, dst_name);
 		put_confirm.put.nitems = add_to_string_array(&put_confirm.put.items,
-				put_confirm.put.nitems, 1, dst_path);
+				put_confirm.put.nitems, dst_path);
 	}
 
 	return 0;
