@@ -21,7 +21,7 @@
 
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
-#include <string.h> /* strcmp() strdup() strlen() */
+#include <string.h> /* memset() strcmp() strdup() strlen() */
 
 #include "compat/os.h"
 #include "modes/dialogs/msg_dialog.h"
@@ -50,6 +50,9 @@ RenameAction;
 
 static void rename_file_cb(const char new_name[]);
 static int complete_filename_only(const char str[], void *arg);
+static char ** list_files_to_rename(view_t *view, int recursive, int *len);
+static int verify_list(char *files[], int nfiles, char *names[], int nnames,
+		char **error, void *data);
 static char ** add_files_to_list(const char base[], const char path[],
 		char *files[], int *len);
 static int perform_renaming(view_t *view, char *files[], char is_dup[], int len,
@@ -128,8 +131,14 @@ rename_file_cb(const char new_name[])
 	snprintf(new, sizeof(new), "%s%s%s", new_name,
 			(rename_file_ext[0] == '\0') ? "" : ".", rename_file_ext);
 
-	if(fops_check_file_rename(forigin, fname, new, ST_DIALOG) <= 0)
+	char *error = NULL;
+	if(fops_check_file_rename(forigin, fname, new, &error) <= 0)
 	{
+		if(error != NULL)
+		{
+			show_error_msg("Rename error", error);
+			free(error);
+		}
 		return;
 	}
 
@@ -162,12 +171,6 @@ complete_filename_only(const char str[], void *arg)
 int
 fops_rename(view_t *view, char *list[], int nlines, int recursive)
 {
-	char **files;
-	int nfiles;
-	dir_entry_t *entry;
-	char *is_dup;
-	int free_list = 0;
-
 	/* Allow list of names in tests. */
 	if(curr_stats.load_stage != 0 && recursive && nlines != 0)
 	{
@@ -179,25 +182,16 @@ fops_rename(view_t *view, char *list[], int nlines, int recursive)
 		return 0;
 	}
 
-	nfiles = 0;
-	files = NULL;
-	entry = NULL;
-	while(iter_marked_entries(view, &entry))
+	int nfiles;
+	char **files = list_files_to_rename(view, recursive, &nfiles);
+	/* No files to process. */
+	if(nfiles == 0)
 	{
-		char path[PATH_MAX + 1];
-		get_short_path_of(view, entry, NF_NONE, 0, sizeof(path), path);
-
-		if(recursive)
-		{
-			files = add_files_to_list(flist_get_dir(view), path, files, &nfiles);
-		}
-		else
-		{
-			nfiles = add_to_string_array(&files, nfiles, 1, path);
-		}
+		ui_sb_msg("0 files renamed");
+		return 1;
 	}
 
-	is_dup = calloc(nfiles, 1);
+	char *is_dup = calloc(nfiles, 1);
 	if(is_dup == NULL)
 	{
 		free_string_array(files, nfiles);
@@ -205,44 +199,97 @@ fops_rename(view_t *view, char *list[], int nlines, int recursive)
 		return 0;
 	}
 
-	/* If we weren't given list of new file names, obtain it from the user. */
+	/* If we weren't given list of new file names, try to obtain it from the
+	 * user. */
+	const int from_file = (nlines == 0);
+	if(from_file)
+	{
+		list = fops_query_list(nfiles, files, &nlines, 0, &verify_list, NULL);
+	}
+
 	if(nlines == 0)
 	{
-		if(nfiles == 0 ||
-				(list = fops_edit_list(nfiles, files, &nlines, 0)) == NULL)
-		{
-			ui_sb_msg("0 files renamed");
-		}
-		else
-		{
-			free_list = 1;
-		}
+		ui_sb_msg("0 files renamed");
 	}
-
-	/* If nlines is 0 here, do nothing. */
-	if(nlines != 0 && fops_is_name_list_ok(nfiles, nlines, list, files) &&
-			fops_is_rename_list_ok(files, is_dup, nfiles, list))
+	else
 	{
-		const int renamed = perform_renaming(view, files, is_dup, nfiles, list);
-		if(renamed >= 0)
+		char *error_str = NULL;
+		if(verify_list(files, nfiles, list, nlines, &error_str, is_dup))
 		{
-			ui_sb_msgf("%d file%s renamed", renamed, (renamed == 1) ? "" : "s");
+			const int renamed = perform_renaming(view, files, is_dup, nfiles, list);
+			if(renamed >= 0)
+			{
+				ui_sb_msgf("%d file%s renamed", renamed, (renamed == 1) ? "" : "s");
+			}
+
+			flist_sel_stash(view);
+			redraw_view(view);
+		}
+		else if(error_str != NULL)
+		{
+			ui_sb_err(error_str);
 		}
 	}
 
-	if(free_list)
+	if(from_file)
 	{
 		free_string_array(list, nlines);
 	}
 	free_string_array(files, nfiles);
 	free(is_dup);
 
-	flist_sel_stash(view);
-	redraw_view(view);
-	curr_stats.save_msg = 1;
 	return 1;
 }
 
+/* Makes list of files to be renamed.  Always sets *len.  Returns list of files,
+ * which might be empty (NULL). */
+static char **
+list_files_to_rename(view_t *view, int recursive, int *len)
+{
+	*len = 0;
+
+	char **files = NULL;
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry))
+	{
+		char path[PATH_MAX + 1];
+		get_short_path_of(view, entry, NF_NONE, 0, sizeof(path), path);
+
+		if(recursive)
+		{
+			files = add_files_to_list(flist_get_dir(view), path, files, len);
+		}
+		else
+		{
+			*len = add_to_string_array(&files, *len, path);
+		}
+	}
+
+	return files;
+}
+
+/* Checks that renaming can be performed.  Returns non-zero if so, otherwise
+ * zero is returned along with setting *error. */
+static int
+verify_list(char *files[], int nfiles, char *names[], int nnames, char **error,
+		void *data)
+{
+	char *is_dup = data;
+	if(is_dup == NULL)
+	{
+		is_dup = calloc(nfiles, 1);
+	}
+
+	int ok = fops_is_name_list_ok(nfiles, nnames, names, files, error)
+	      && fops_is_rename_list_ok(files, is_dup, nfiles, names, error);
+
+	if(data == NULL)
+	{
+		free(is_dup);
+	}
+	return ok;
+}
+
 /* Appends files inside of the specified path to the list of the length *len.
  * Returns new list pointer. */
 static char **
@@ -257,7 +304,7 @@ add_files_to_list(const char base[], const char path[], char *files[], int *len)
 
 	if(!is_dir(full_path))
 	{
-		*len = add_to_string_array(&files, *len, 1, path);
+		*len = add_to_string_array(&files, *len, path);
 		return files;
 	}
 
@@ -425,8 +472,8 @@ fops_incdec(view_t *view, int k)
 
 		get_full_path_of(entry, sizeof(full_path), full_path);
 
-		names_len = add_to_string_array(&names, names_len, 1, full_path);
-		tmp_len = add_to_string_array(&tmp_names, tmp_len, 1,
+		names_len = add_to_string_array(&names, names_len, full_path);
+		tmp_len = add_to_string_array(&tmp_names, tmp_len,
 				make_name_unique(entry->name));
 	}
 
@@ -440,19 +487,22 @@ fops_incdec(view_t *view, int k)
 
 		snprintf(new_path, sizeof(new_path), "%s/%s", entry->origin, new_fname);
 
-		/* Skip fops_check_file_rename() for final name that matches one of original
-		 * names. */
+		/* Skip fops_check_file_rename() for final name that matches one of the
+		 * original names. */
 		if(is_in_string_array_os(names, names_len, new_path))
 		{
 			continue;
 		}
 
+		char *error = NULL;
 		if(fops_check_file_rename(entry->origin, entry->name, new_fname,
-					ST_STATUS_BAR) != 0)
+					&error) != 0)
 		{
 			continue;
 		}
 
+		ui_sb_err(error);
+		free(error);
 		err = -1;
 		break;
 	}
@@ -636,7 +686,7 @@ fops_case(view_t *view, int to_upper)
 			break;
 		}
 
-		ndest = add_to_string_array(&dest, ndest, 1, new_fname);
+		ndest = add_to_string_array(&dest, ndest, new_fname);
 	}
 
 	if(err)
@@ -724,7 +774,7 @@ fops_subst(view_t *view, const char pattern[], const char sub[], int ic,
 				err = 1;
 				break;
 			case RA_RENAME:
-				ndest = add_to_string_array(&dest, ndest, 1, new_fname);
+				ndest = add_to_string_array(&dest, ndest, new_fname);
 				break;
 
 			default:
@@ -785,7 +835,7 @@ fops_tr(view_t *view, const char from[], const char to[])
 				err = 1;
 				break;
 			case RA_RENAME:
-				ndest = add_to_string_array(&dest, ndest, 1, new_fname);
+				ndest = add_to_string_array(&dest, ndest, new_fname);
 				break;
 
 			default:
