@@ -26,7 +26,7 @@
 #ifndef _WIN32
 #include <sys/wait.h> /* WEXITSTATUS() WIFEXITED() waitpid() */
 #endif
-#include <unistd.h> /* execve() fork() rmdir() unlink() */
+#include <unistd.h> /* execve() fork() unlink() */
 
 #include <errno.h> /* errno ENOTDIR */
 #include <stddef.h> /* NULL size_t */
@@ -617,12 +617,9 @@ run_fuse_command(char cmd[], const cancellation_t *cancellation, int *cancelled)
 		return 1;
 	}
 
-	(void)set_sigchld(1);
-
 	pid = fork();
 	if(pid == (pid_t)-1)
 	{
-		(void)set_sigchld(0);
 		LOG_SERROR_MSG(errno, "Forking has failed.");
 		return -1;
 	}
@@ -631,8 +628,6 @@ run_fuse_command(char cmd[], const cancellation_t *cancellation, int *cancelled)
 	{
 		extern char **environ;
 
-		(void)set_sigchld(0);
-
 		prepare_for_exec();
 		(void)execve(get_execv_path(cfg.shell),
 				make_execv_array(cfg.shell, "-c", cmd), environ);
@@ -656,8 +651,6 @@ run_fuse_command(char cmd[], const cancellation_t *cancellation, int *cancelled)
 		*cancelled = 1;
 	}
 
-	(void)set_sigchld(0);
-
 	return status;
 #else
 	return -1;
@@ -675,7 +668,7 @@ kill_mount_point(const char mount_point[])
 	strcpy(no_slash, mount_point);
 	chosp(no_slash);
 
-	if(rmdir(no_slash) != 0 && errno == ENOTDIR)
+	if(os_rmdir(no_slash) != 0 && errno == ENOTDIR)
 	{
 		/* FUSE mounter might replace directory with a symbolic link, account for
 		 * this possibility. */
