@@ -31,8 +31,6 @@
 # include <sys/select.h> /* FD_* select() */
 #else
 # define O_NONBLOCK 0
-# define REQUIRED_WINVER 0x0600 /* To get PIPE_REJECT_REMOTE_CLIENTS. */
-# include "utils/windefs.h"
 # include <windows.h>
 # ifndef PIPE_REJECT_REMOTE_CLIENTS
 #  define PIPE_REJECT_REMOTE_CLIENTS 1
@@ -53,6 +51,7 @@
 #include <stdlib.h> /* free() malloc() snprintf() */
 #include <string.h> /* strcmp() strcpy() strlen() */
 
+#include "compat/os.h"
 #include "utils/fs.h"
 #include "utils/log.h"
 #include "utils/macros.h"
@@ -463,7 +462,7 @@ handle_pkg(ipc_t *ipc, const char pkg[], const char *end)
 	{
 		if(in_body)
 		{
-			len = add_to_string_array(&array, len, 1, pkg);
+			len = add_to_string_array(&array, len, pkg);
 		}
 		else if(starts_with_lit(pkg, "version:"))
 		{
@@ -845,14 +844,14 @@ add_to_list(const char name[], const void *data, void *param)
 		struct stat statbuf;
 		snprintf(path, sizeof(path), "%s/%s", list_data->ipc_dir, name);
 		if(stat(path, &statbuf) != 0 || !S_ISFIFO(statbuf.st_mode) ||
-				!pipe_is_in_use(path))
+				!pipe_is_in_use(path) || os_access(path, R_OK) != 0)
 		{
 			return 0;
 		}
 	}
 #endif
 
-	list_data->len = add_to_string_array(&list_data->lst, list_data->len, 1,
+	list_data->len = add_to_string_array(&list_data->lst, list_data->len,
 			name + strlen(PREFIX));
 	return 0;
 }
