@@ -20,7 +20,7 @@
 #include "macros.h"
 
 #include <assert.h> /* assert() */
-#include <ctype.h> /* tolower() */
+#include <ctype.h> /* isdigit() tolower() */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* snprintf() */
 #include <stdlib.h> /* realloc() free() calloc() */
@@ -29,11 +29,13 @@
 #include "cfg/config.h"
 #include "compat/fs_limits.h"
 #include "modes/dialogs/msg_dialog.h"
+#include "ui/colored_line.h"
 #include "ui/quickview.h"
 #include "ui/ui.h"
 #include "utils/path.h"
 #include "utils/str.h"
 #include "utils/test_helpers.h"
+#include "utils/utf8.h"
 #include "utils/utils.h"
 #include "filelist.h"
 #include "filename_modifiers.h"
@@ -51,13 +53,14 @@ PathType;
 
 /* Should return the same character if processing of the macro is allowed or
  * '\0' if it's not allowed. */
-typedef char (*macro_filter_func)(int *quoted, char c, char data);
+typedef char (*macro_filter_func)(int *quoted, char c, char data,
+		int ncurr, int nother);
 
-static char filter_all(int *quoted, char c, char data);
-static char filter_single(int *quoted, char c, char data);
+static char filter_all(int *quoted, char c, char data, int ncurr, int nother);
+static char filter_single(int *quoted, char c, char data,
+		int ncurr, int nother);
 static char * expand_macros_i(const char command[], const char args[],
 		MacroFlags *flags, int for_shell, macro_filter_func filter);
-static void set_flags(MacroFlags *flags, MacroFlags value);
 TSTATIC char * append_selected_files(view_t *view, char expanded[],
 		int under_cursor, int quotes, const char mod[], int for_shell);
 static char * append_entry(view_t *view, char expanded[], PathType type,
@@ -71,6 +74,8 @@ static preview_area_t get_preview_area(view_t *view);
 static char * append_path_to_expanded(char expanded[], int quotes,
 		const char path[]);
 static char * append_to_expanded(char expanded[], const char str[]);
+static cline_t expand_custom(const char **pattern, size_t nmacros,
+		custom_macro_t macros[], int with_opt, int in_opt);
 static char * add_missing_macros(char expanded[], size_t len, size_t nmacros,
 		custom_macro_t macros[]);
 
@@ -78,13 +83,21 @@ char *
 ma_expand(const char command[], const char args[], MacroFlags *flags,
 		int for_shell)
 {
-	return expand_macros_i(command, args, flags, for_shell, &filter_all);
+	int lpending_marking = lwin.pending_marking;
+	int rpending_marking = rwin.pending_marking;
+
+	char *res = expand_macros_i(command, args, flags, for_shell, &filter_all);
+
+	lwin.pending_marking = lpending_marking;
+	rwin.pending_marking = rpending_marking;
+
+	return res;
 }
 
 /* macro_filter_func instantiation that allows all macros.  Returns the
  * argument. */
 static char
-filter_all(int *quoted, char c, char data)
+filter_all(int *quoted, char c, char data, int ncurr, int nother)
 {
 	return c;
 }
@@ -92,7 +105,14 @@ filter_all(int *quoted, char c, char data)
 char *
 ma_expand_single(const char command[])
 {
+	int lpending_marking = lwin.pending_marking;
+	int rpending_marking = rwin.pending_marking;
+
 	char *const res = expand_macros_i(command, NULL, NULL, 0, &filter_single);
+
+	lwin.pending_marking = lpending_marking;
+	rwin.pending_marking = rpending_marking;
+
 	unescape(res, 0);
 	return res;
 }
@@ -100,7 +120,7 @@ ma_expand_single(const char command[])
 /* macro_filter_func instantiation that filters out non-single element macros.
  * Returns the argument on allowed macro and '\0' otherwise. */
 static char
-filter_single(int *quoted, char c, char data)
+filter_single(int *quoted, char c, char data, int ncurr, int nother)
 {
 	if(strchr("cCdD", c) != NULL)
 	{
@@ -108,12 +128,12 @@ filter_single(int *quoted, char c, char data)
 		return c;
 	}
 
-	if(c == 'f' && curr_view->selected_files <= 1)
+	if(c == 'f' && ncurr <= 1)
 	{
 		return c;
 	}
 
-	if(c == 'F' && other_view->selected_files <= 1)
+	if(c == 'F' && nother <= 1)
 	{
 		return c;
 	}
@@ -147,7 +167,7 @@ expand_macros_i(const char command[], const char args[], MacroFlags *flags,
 	size_t x;
 	int len = 0;
 
-	set_flags(flags, MF_NONE);
+	ma_flags_set(flags, MF_NONE);
 
 	cmd_len = strlen(command);
 
@@ -160,6 +180,13 @@ expand_macros_i(const char command[], const char args[], MacroFlags *flags,
 		return strdup(command);
 	}
 
+	/* Turn selection into marking. */
+	flist_set_marking(&lwin, 0);
+	flist_set_marking(&rwin, 0);
+
+	int ncurr = flist_count_marked(curr_view);
+	int nother = flist_count_marked(other_view);
+
 	if(strstr(command + x, "%r") != NULL)
 	{
 		regs_sync_from_shared_memory();
@@ -182,7 +209,7 @@ expand_macros_i(const char command[], const char args[], MacroFlags *flags,
 			++x;
 		}
 		switch(filter(&quotes, command[x],
-					command[x] == '\0' ? '\0' : command[x + 1]))
+					command[x] == '\0' ? '\0' : command[x + 1], ncurr, nother))
 		{
 			int well_formed;
 			char key;
@@ -233,42 +260,45 @@ expand_macros_i(const char command[], const char args[], MacroFlags *flags,
 				len = strlen(expanded);
 				break;
 			case 'n': /* Forbid using of terminal multiplexer, even if active. */
-				set_flags(flags, MF_NO_TERM_MUX);
+				ma_flags_set(flags, MF_NO_TERM_MUX);
 				break;
 			case 'm': /* Use menu. */
-				set_flags(flags, MF_MENU_OUTPUT);
+				ma_flags_set(flags, MF_MENU_OUTPUT);
 				break;
 			case 'M': /* Use menu like with :locate and :find. */
-				set_flags(flags, MF_MENU_NAV_OUTPUT);
+				ma_flags_set(flags, MF_MENU_NAV_OUTPUT);
 				break;
 			case 'S': /* Show command output in the status bar */
-				set_flags(flags, MF_STATUSBAR_OUTPUT);
+				ma_flags_set(flags, MF_STATUSBAR_OUTPUT);
 				break;
 			case 'q': /* Show command output in the preview */
-				set_flags(flags, MF_PREVIEW_OUTPUT);
+				ma_flags_set(flags, MF_PREVIEW_OUTPUT);
 				break;
-			case 's': /* Split in new screen region and execute command there. */
-				set_flags(flags, MF_SPLIT);
+			case 's': /* Execute command in a new horizontal split. */
+				ma_flags_set(flags, MF_SPLIT);
+				break;
+			case 'v': /* Execute command in a new vertical split. */
+				ma_flags_set(flags, MF_SPLIT_VERT);
 				break;
 			case 'u': /* Parse output as list of files and compose custom view. */
-				set_flags(flags, MF_CUSTOMVIEW_OUTPUT);
+				ma_flags_set(flags, MF_CUSTOMVIEW_OUTPUT);
 				break;
 			case 'U': /* Parse output as list of files and compose unsorted view. */
-				set_flags(flags, MF_VERYCUSTOMVIEW_OUTPUT);
+				ma_flags_set(flags, MF_VERYCUSTOMVIEW_OUTPUT);
 				break;
 			case 'i': /* Ignore output. */
-				set_flags(flags, MF_IGNORE);
+				ma_flags_set(flags, MF_IGNORE);
 				break;
 			case 'I': /* Interactive custom views. */
 				switch(command[x + 1])
 				{
 					case 'u':
 						++x;
-						set_flags(flags, MF_CUSTOMVIEW_IOUTPUT);
+						ma_flags_set(flags, MF_CUSTOMVIEW_IOUTPUT);
 						break;
 					case 'U':
 						++x;
-						set_flags(flags, MF_VERYCUSTOMVIEW_IOUTPUT);
+						ma_flags_set(flags, MF_VERYCUSTOMVIEW_IOUTPUT);
 						break;
 				}
 				break;
@@ -301,6 +331,19 @@ expand_macros_i(const char command[], const char args[], MacroFlags *flags,
 					++x;
 				}
 				break;
+			case 'P': /* Pipe file list. */
+				switch(command[x + 1])
+				{
+					case 'l':
+						++x;
+						ma_flags_set(flags, MF_PIPE_FILE_LIST);
+						break;
+					case 'z':
+						++x;
+						ma_flags_set(flags, MF_PIPE_FILE_LIST_Z);
+						break;
+				}
+				break;
 			case '%':
 				expanded = append_to_expanded(expanded, "%");
 				len = strlen(expanded);
@@ -316,15 +359,7 @@ expand_macros_i(const char command[], const char args[], MacroFlags *flags,
 		if(command[x] != '\0')
 			x++;
 
-		while(x < cmd_len)
-		{
-			size_t len = get_mods_len(command + x);
-			if(len == 0)
-			{
-				break;
-			}
-			x += len;
-		}
+		x += mods_length(command + x);
 
 		y = x;
 
@@ -356,13 +391,25 @@ expand_macros_i(const char command[], const char args[], MacroFlags *flags,
 	return expanded;
 }
 
-/* Sets *flags to the value, if flags isn't NULL. */
-static void
-set_flags(MacroFlags *flags, MacroFlags value)
+void
+ma_flags_set(MacroFlags *flags, MacroFlags flag)
 {
-	if(flags != NULL)
+	if(flags == NULL)
 	{
-		*flags = value;
+		return;
+	}
+
+	if(flag == MF_NONE)
+	{
+		*flags = MF_NONE;
+	}
+	else if(flag < MF_SECOND_SET_)
+	{
+		*flags = (*flags & ~0x0f) | flag;
+	}
+	else
+	{
+		*flags = (*flags & 0x0f) | flag;
 	}
 }
 
@@ -377,19 +424,20 @@ append_selected_files(view_t *view, char expanded[], int under_cursor,
 	size_t old_len = strlen(expanded);
 #endif
 
-	if(view->selected_files && !under_cursor)
+	if(!under_cursor)
 	{
-		int n = 0;
+		int first = 1;
 		dir_entry_t *entry = NULL;
-		while(iter_selected_entries(view, &entry))
+		while(iter_marked_entries(view, &entry))
 		{
-			expanded = append_entry(view, expanded, type, entry, quotes, mod,
-					for_shell);
-
-			if(++n != view->selected_files)
+			if(!first)
 			{
 				expanded = append_to_expanded(expanded, " ");
 			}
+
+			expanded = append_entry(view, expanded, type, entry, quotes, mod,
+					for_shell);
+			first = 0;
 		}
 	}
 	else
@@ -437,7 +485,7 @@ append_entry(view_t *view, char expanded[], PathType type, dir_entry_t *entry,
 			break;
 	}
 
-	modified = apply_mods(path, flist_get_dir(view), mod, for_shell);
+	modified = mods_apply(path, flist_get_dir(view), mod, for_shell);
 	expanded = append_path_to_expanded(expanded, quotes, modified);
 
 	return expanded;
@@ -447,7 +495,7 @@ static char *
 expand_directory_path(view_t *view, char *expanded, int quotes, const char *mod,
 		int for_shell)
 {
-	const char *modified = apply_mods(flist_get_dir(view), "/", mod, for_shell);
+	const char *modified = mods_apply(flist_get_dir(view), "/", mod, for_shell);
 	char *const result = append_path_to_expanded(expanded, quotes, modified);
 
 	if(for_shell && curr_stats.shell_type == ST_CMD)
@@ -481,7 +529,7 @@ expand_register(const char curr_dir[], char expanded[], int quotes,
 
 	for(i = 0; i < reg->nfiles; ++i)
 	{
-		const char *const modified = apply_mods(reg->files[i], curr_dir, mod,
+		const char *const modified = mods_apply(reg->files[i], curr_dir, mod,
 				for_shell);
 		expanded = append_path_to_expanded(expanded, quotes, modified);
 		if(i != reg->nfiles - 1)
@@ -601,8 +649,8 @@ append_to_expanded(char expanded[], const char str[])
 	t = realloc(expanded, len + strlen(str) + 1);
 	if(t == NULL)
 	{
-		show_error_msg("Memory Error", "Unable to allocate enough memory");
 		free(expanded);
+		show_error_msg("Memory Error", "Unable to allocate enough memory");
 		return NULL;
 	}
 	strcpy(t + len, str);
@@ -619,48 +667,125 @@ ma_get_clear_cmd(const char cmd[])
 		return NULL;
 	}
 
-	clear_cmd = break_point + 3;
+	clear_cmd = skip_whitespace(break_point + 3);
 	return is_null_or_empty(clear_cmd) ? NULL : clear_cmd;
 }
 
 char *
-ma_expand_custom(const char pattern[], size_t nmacros, custom_macro_t macros[])
+ma_expand_custom(const char pattern[], size_t nmacros, custom_macro_t macros[],
+		int with_opt)
 {
-	char *expanded = strdup("");
-	size_t len = 0;
-	while(*pattern != '\0')
+	cline_t result = ma_expand_colored_custom(pattern, nmacros, macros, with_opt);
+	free(result.attrs);
+	return result.line;
+}
+
+cline_t
+ma_expand_colored_custom(const char pattern[], size_t nmacros,
+		custom_macro_t macros[], int with_opt)
+{
+	cline_t result = expand_custom(&pattern, nmacros, macros, with_opt, 0);
+	assert(strlen(result.attrs) == utf8_strsw(result.line) && "Broken attrs!");
+	return result;
+}
+
+/* Expands macros of form %x in the pattern (%% is expanded to %) according to
+ * macros specification and accounting for nesting inside %[ and %].  Updates
+ * *pattern pointer.  Returns colored line. */
+static cline_t
+expand_custom(const char **pattern, size_t nmacros, custom_macro_t macros[],
+		int with_opt, int in_opt)
+{
+	cline_t result = cline_make();
+
+	int nexpansions = 0;
+	while(**pattern != '\0')
 	{
-		if(pattern[0] != '%')
+		const char *pat = (*pattern)++;
+		if(pat[0] != '%')
 		{
-			const char single_char[] = { *pattern, '\0' };
-			expanded = extend_string(expanded, single_char, &len);
+			const char single_char[] = { *pat, '\0' };
+			result.line = extend_string(result.line, single_char, &result.line_len);
 		}
-		else if(pattern[1] == '%' || pattern[1] == '\0')
+		else if(pat[1] == '%' || pat[1] == '\0')
 		{
-			expanded = extend_string(expanded, "%", &len);
-			pattern += pattern[1] == '%';
+			result.line = extend_string(result.line, "%", &result.line_len);
+			*pattern += (pat[1] == '%');
+		}
+		else if(pat[1] == '*')
+		{
+			cline_set_attr(&result, '0');
+			++*pattern;
+		}
+		else if(isdigit(pat[1]) && pat[2] == '*')
+		{
+			cline_set_attr(&result, pat[1]);
+			*pattern += 2;
+		}
+		else if(with_opt && pat[1] == '[')
+		{
+			++*pattern;
+			cline_t opt = expand_custom(pattern, nmacros, macros, with_opt, 1);
+			cline_splice_attrs(&result, &opt);
+			result.line = extend_string(result.line, opt.line, &result.line_len);
+			nexpansions += (opt.line[0] != '\0');
+			free(opt.line);
+			continue;
+		}
+		else if(in_opt && pat[1] == ']')
+		{
+			++*pattern;
+			if(nexpansions == 0 && result.line != NULL)
+			{
+				cline_clear(&result);
+			}
+			return result;
 		}
 		else
 		{
 			size_t i = 0U;
-			++pattern;
-			while(i < nmacros && macros[i].letter != *pattern)
+			while(i < nmacros && macros[i].letter != **pattern)
 			{
 				++i;
 			}
+			++*pattern;
 			if(i < nmacros)
 			{
-				expanded = extend_string(expanded, macros[i].value, &len);
+				const char *value = macros[i].value;
+				if(macros[i].expand_mods)
+				{
+					assert(is_path_absolute(macros[i].parent) &&
+							"Invalid parent for mods.");
+					value = mods_apply(value, macros[i].parent, *pattern, 0);
+
+					*pattern += mods_length(*pattern);
+				}
+
+				if(!macros[i].flag)
+				{
+					result.line = extend_string(result.line, value, &result.line_len);
+				}
 				--macros[i].uses_left;
 				macros[i].explicit_use = 1;
+				nexpansions += (value[0] != '\0');
 			}
 		}
-		pattern++;
 	}
 
-	expanded = add_missing_macros(expanded, len, nmacros, macros);
+	if(in_opt)
+	{
+		/* Unmatched %[. */
+		(void)strprepend(&result.line, &result.line_len, "%[");
+		(void)strprepend(&result.attrs, &result.attrs_len, "  ");
+	}
+	else
+	{
+		result.line = add_missing_macros(result.line, result.line_len, nmacros,
+				macros);
+		cline_finish(&result);
+	}
 
-	return expanded;
+	return result;
 }
 
 /* Ensures that the expanded string contains required number of mandatory
@@ -669,7 +794,7 @@ static char *
 add_missing_macros(char expanded[], size_t len, size_t nmacros,
 		custom_macro_t macros[])
 {
-	int groups[nmacros];
+	int groups[nmacros == 0 ? 1 : nmacros];
 	size_t i;
 
 	memset(&groups, 0, sizeof(groups));
@@ -702,11 +827,36 @@ add_missing_macros(char expanded[], size_t len, size_t nmacros,
 	return expanded;
 }
 
+int
+ma_flags_present(MacroFlags flags, MacroFlags flag)
+{
+	if(flag == MF_NONE)
+	{
+		return (flags == MF_NONE);
+	}
+	else if(flag < MF_SECOND_SET_)
+	{
+		return ((flags & 0x0f) == flag);
+	}
+	else
+	{
+		return ((flags & ~0x0f) == flag);
+	}
+}
+
+int
+ma_flags_missing(MacroFlags flags, MacroFlags flag)
+{
+	return !ma_flags_present(flags, flag);
+}
+
 const char *
 ma_flags_to_str(MacroFlags flags)
 {
 	switch(flags)
 	{
+		case MF_FIRST_SET_:
+		case MF_SECOND_SET_:
 		case MF_NONE: return "";
 
 		case MF_MENU_OUTPUT: return "%m";
@@ -719,8 +869,12 @@ ma_flags_to_str(MacroFlags flags)
 		case MF_VERYCUSTOMVIEW_IOUTPUT: return "%IU";
 
 		case MF_SPLIT: return "%s";
+		case MF_SPLIT_VERT: return "%v";
 		case MF_IGNORE: return "%i";
 		case MF_NO_TERM_MUX: return "%n";
+
+		case MF_PIPE_FILE_LIST: return "%Pl";
+		case MF_PIPE_FILE_LIST_Z: return "%Pz";
 	}
 
 	assert(0 && "Unhandled MacroFlags item.");
