@@ -91,7 +91,7 @@ list_dir_history(view_t *view, int *pos)
 			*pos = list.nitems;
 		}
 
-		list.nitems = add_to_string_array(&list.items, list.nitems, 1,
+		list.nitems = add_to_string_array(&list.items, list.nitems,
 				view->history[i].dir);
 	}
 
@@ -115,6 +115,7 @@ list_dir_history(view_t *view, int *pos)
 
 				view->history[i].dir = NULL;
 				view->history[i].file = NULL;
+				view->history[i].timestamp = (time_t)-1;
 				view->history[i].rel_pos = -1;
 			}
 
