@@ -105,15 +105,24 @@ show_find_menu(view_t *view, int with_path, const char args[])
 	m.execute_handler = &execute_find_cb;
 	m.key_handler = &menus_def_khandler;
 
-	cmd = ma_expand_custom(cfg.find_prg, ARRAY_LEN(macros), macros);
+	cmd = ma_expand_custom(cfg.find_prg, ARRAY_LEN(macros), macros, MA_NOOPT);
 
 	free(targets);
 	free(escaped_args);
 	free(custom_args);
 
+	MacroFlags flags = MF_NONE;
+	if(macros[M_u].explicit_use)
+	{
+		ma_flags_set(&flags, MF_CUSTOMVIEW_OUTPUT);
+	}
+	if(macros[M_U].explicit_use)
+	{
+		ma_flags_set(&flags, MF_VERYCUSTOMVIEW_OUTPUT);
+	}
+
 	ui_sb_msg("find...");
-	save_msg = menus_capture(view, cmd, 0, &m, macros[M_u].explicit_use,
-			macros[M_U].explicit_use);
+	save_msg = menus_capture(view, cmd, /*user_sh=*/0, &m, flags);
 	free(cmd);
 
 	return save_msg;
