@@ -56,8 +56,6 @@ show_jobs_menu(view_t *view)
 
 	bg_check();
 
-	bg_jobs_freeze();
-
 	p = bg_jobs;
 
 	i = 0;
@@ -85,7 +83,7 @@ show_jobs_menu(view_t *view)
 
 			snprintf(item_buf, sizeof(item_buf), "%-8s  %s%s", info_buf, p->cmd,
 					bg_job_cancelled(p) ? " (cancelling...)" : "");
-			i = add_to_string_array(&jobs_m.items, i, 1, item_buf);
+			i = add_to_string_array(&jobs_m.items, i, item_buf);
 			jobs_m.void_data = reallocarray(jobs_m.void_data, i,
 					sizeof(*jobs_m.void_data));
 			jobs_m.void_data[i - 1] = p;
@@ -94,8 +92,6 @@ show_jobs_menu(view_t *view)
 		p = p->next;
 	}
 
-	bg_jobs_unfreeze();
-
 	jobs_m.len = i;
 
 	return menus_enter(jobs_m.state, view);
@@ -144,7 +140,6 @@ cancel_job(menu_data_t *m, bg_job_t *job)
 
 	/* We have to make sure the job pointer is still valid and the job is
 	 * running. */
-	bg_jobs_freeze();
 	for(p = bg_jobs; p != NULL; p = p->next)
 	{
 		if(p == job && bg_job_is_running(job))
@@ -158,7 +153,6 @@ cancel_job(menu_data_t *m, bg_job_t *job)
 			break;
 		}
 	}
-	bg_jobs_unfreeze();
 
 	return (p != NULL);
 }
@@ -174,7 +168,6 @@ show_job_errors(view_t *view, menu_data_t *m, bg_job_t *job)
 
 	/* We have to make sure the job pointer is still valid and the job is
 	 * running. */
-	bg_jobs_freeze();
 	for(p = bg_jobs; p != NULL; p = p->next)
 	{
 		if(p == job)
@@ -185,7 +178,6 @@ show_job_errors(view_t *view, menu_data_t *m, bg_job_t *job)
 			break;
 		}
 	}
-	bg_jobs_unfreeze();
 
 	if(p == NULL)
 	{
@@ -203,7 +195,7 @@ show_job_errors(view_t *view, menu_data_t *m, bg_job_t *job)
 		m.key_handler = &errs_khandler;
 		m.items = break_into_lines(errors, errors_len, &m.len, 0);
 
-		menu_reenter_mode(&m);
+		modmenu_reenter(&m);
 	}
 	free(cmd);
 	free(errors);
@@ -216,7 +208,7 @@ errs_khandler(view_t *view, menu_data_t *m, const wchar_t keys[])
 {
 	if(wcscmp(keys, L"h") == 0)
 	{
-		menu_reenter_mode(&jobs_m);
+		modmenu_reenter(&jobs_m);
 		return KHR_REFRESH_WINDOW;
 	}
 	return KHR_UNHANDLED;
