@@ -53,17 +53,25 @@ show_map_menu(view_t *view, const char mode_str[], int mode,
 	const int dialogs = mode == SORT_MODE || mode == ATTR_MODE
 	                 || mode == CHANGE_MODE || mode == FILE_INFO_MODE;
 
+	char *prefix_str = wstr_to_spec(start);
+	const char *fmt = (start[0] == L'\0')
+	                ? "Mappings for %s mode%s"
+	                : "Mappings for %s mode%s with prefix: %s";
+
 	menus_init_data(&m, view,
-			format_str("Mappings for %s mode%s", mode_str, dialogs ? "s" : ""),
+			format_str(fmt, mode_str, dialogs ? "s" : "", prefix_str),
 			strdup("No mappings found"));
 
+	free(prefix_str);
+
 	prefix = start;
 	prefix_len = wcslen(prefix);
 
 	vle_keys_list(mode, &add_mapping_item, dialogs);
 
-	/* If we filtered out all meaningful lines, clear the menu. */
-	if(m.len == 1 && *m.items[0] == '\0')
+	/* If we filtered out all meaningful lines or there were none, clear the
+	 * menu. */
+	if(m.len == 3)
 	{
 		free(m.items[0]);
 		free(m.items);
@@ -80,8 +88,6 @@ add_mapping_item(const wchar_t lhs[], const wchar_t rhs[], const char descr[])
 {
 	enum { MAP_WIDTH = 11 };
 
-	char *mb_lhs;
-
 	const int is_separator = (lhs[0] == L'\0');
 	if(!is_separator)
 	{
@@ -97,13 +103,13 @@ add_mapping_item(const wchar_t lhs[], const wchar_t rhs[], const char descr[])
 		}
 	}
 
-	mb_lhs = wstr_to_spec(lhs);
+	char *mb_lhs = wstr_to_spec(lhs);
 
 	m.items = reallocarray(m.items, m.len + 1, sizeof(char *));
 
 	if(is_separator)
 	{
-		m.items[m.len++] = strdup("");
+		m.items[m.len++] = strdup(descr);
 	}
 	else if(rhs[0] == L'\0')
 	{
