@@ -197,7 +197,8 @@ reload_list(menu_data_t *m)
 	}
 
 	char *cmd = format_str("%s list", cfg.media_prg);
-	if(process_cmd_output("Listing media", cmd, 0, 0, &output_handler, &m) != 0)
+	if(process_cmd_output("Listing media", cmd, NULL, 0, 0, &output_handler,
+				&m) != 0)
 	{
 		free(cmd);
 		show_error_msgf("Listing media devices", "Unable to run: %s", cmd);
@@ -231,7 +232,7 @@ reload_list(menu_data_t *m)
 		if(info->path_count == 0)
 		{
 			put_into_string_array(&m->data, m->len, format_str("m%s", info->device));
-			m->len = add_to_string_array(&m->items, m->len, 1, "`-- (not mounted)");
+			m->len = add_to_string_array(&m->items, m->len, "`-- (not mounted)");
 		}
 		else
 		{
@@ -254,7 +255,7 @@ reload_list(menu_data_t *m)
 		if(i != info_count - 1 && max_rows_with_blanks <= menu_rows)
 		{
 			put_into_string_array(&m->data, m->len, NULL);
-			m->len = add_to_string_array(&m->items, m->len, 1, "");
+			m->len = add_to_string_array(&m->items, m->len, "");
 		}
 	}
 
@@ -269,13 +270,17 @@ output_handler(const char line[], void *arg)
 {
 	if(skip_prefix(&line, "device="))
 	{
-		infos = reallocarray(infos, ++info_count, sizeof(*infos));
-		media_info_t *info = &infos[info_count - 1];
-		info->device = strdup(line);
-		info->text = NULL;
-		info->paths = NULL;
-		info->path_count = 0;
-		info->has_info = 0;
+		media_info_t *new_infos = reallocarray(infos, ++info_count, sizeof(*infos));
+		if(new_infos != NULL)
+		{
+			infos = new_infos;
+			media_info_t *info = &infos[info_count - 1];
+			info->device = strdup(line);
+			info->text = NULL;
+			info->paths = NULL;
+			info->path_count = 0;
+			info->has_info = 0;
+		}
 	}
 	else if(info_count > 0)
 	{
@@ -291,7 +296,7 @@ output_handler(const char line[], void *arg)
 		}
 		else if(skip_prefix(&line, "mount-point="))
 		{
-			info->path_count = add_to_string_array(&info->paths, info->path_count, 1,
+			info->path_count = add_to_string_array(&info->paths, info->path_count,
 					line);
 		}
 	}
@@ -409,7 +414,7 @@ mediaprg_mount(const char data[], menu_data_t *m)
 
 	char *escaped_path = shell_like_escape(path, 0);
 	char *cmd = format_str("%s %s %s", cfg.media_prg, action, escaped_path);
-	if(shellout(cmd, PAUSE_NEVER, 0, SHELL_BY_APP) == 0)
+	if(rn_shell(cmd, PAUSE_NEVER, 0, SHELL_BY_APP) == 0)
 	{
 		reload_list(m);
 	}
