@@ -33,13 +33,13 @@
 #include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../compat/reallocarray.h"
+#include "../int/term_title.h"
 #include "../int/vim.h"
 #include "../modes/dialogs/msg_dialog.h"
 #include "../modes/cmdline.h"
 #include "../modes/menu.h"
 #include "../modes/modes.h"
 #include "../ui/cancellation.h"
-#include "../ui/color_manager.h"
 #include "../ui/color_scheme.h"
 #include "../ui/colors.h"
 #include "../ui/statusbar.h"
@@ -164,11 +164,13 @@ menus_init_data(menu_data_t *m, view_t *view, char title[], char empty_msg[])
 	}
 	menu_state.d = m;
 
+	m->title = escape_unreadable(title);
+	free(title);
+
 	m->top = 0;
 	m->len = 0;
 	m->pos = 0;
 	m->hor_pos = 0;
-	m->title = title;
 	m->items = NULL;
 	m->data = NULL;
 	m->void_data = NULL;
@@ -259,7 +261,7 @@ menus_set_pos(menu_state_t *ms, int pos)
 	normalize_top(ms);
 
 	redraw = 0;
-	if(pos > menu_last_line(m))
+	if(pos > modmenu_last_line(m))
 	{
 		m->top = pos - (ms->win_rows - 2 - 1);
 		redraw = 1;
@@ -279,9 +281,9 @@ menus_set_pos(menu_state_t *ms, int pos)
 			normalize_top(ms);
 			redraw = 1;
 		}
-		if(pos > menu_last_line(m) - s)
+		if(pos > modmenu_last_line(m) - s)
 		{
-			m->top += s - (menu_last_line(m) - pos);
+			m->top += s - (modmenu_last_line(m) - pos);
 			normalize_top(ms);
 			redraw = 1;
 		}
@@ -307,7 +309,7 @@ menus_set_pos(menu_state_t *ms, int pos)
 static void
 show_position_in_menu(const menu_data_t *m)
 {
-	char pos_buf[POS_WIN_MIN_WIDTH + 1];
+	char pos_buf[32];
 	snprintf(pos_buf, sizeof(pos_buf), " %d-%d ", m->pos + 1, m->len);
 
 	ui_ruler_set(pos_buf);
@@ -463,7 +465,7 @@ draw_menu_item(menu_state_t *ms, int pos, int line, int clear)
 	{
 		cs_mix_colors(&col, &cfg.cs.color[CURR_LINE_COLOR]);
 	}
-	int color_pair = colmgr_get_pair(col.fg, col.bg);
+	int color_pair = cs_load_color(&col);
 
 	/* Calculate offset of m->hor_pos's character in item text. */
 	off = 0;
@@ -503,8 +505,7 @@ draw_menu_item(menu_state_t *ms, int pos, int line, int clear)
 
 	if(ms->search_highlight && ms->matches != NULL && ms->matches[pos][0] >= 0)
 	{
-		cchar_t cch;
-		setcchar(&cch, L" ", col.attr, color_pair, NULL);
+		const cchar_t cch = cs_color_to_cchar(&col, color_pair);
 		draw_search_match(item_tail, ms->matches[pos][0] - m->hor_pos,
 				ms->matches[pos][1] - m->hor_pos, line, width, &cch);
 	}
@@ -660,9 +661,10 @@ menus_enter(menu_state_t *m, view_t *view)
 	init_menu_state(m, view);
 
 	ui_setup_for_menu_like();
+	term_title_update(m->d->title);
 	menus_partial_redraw(m);
 	menus_set_pos(m, m->d->pos);
-	menu_enter_mode(m->d, view);
+	modmenu_enter(m->d, view);
 	return 0;
 }
 
@@ -684,7 +686,8 @@ init_menu_state(menu_state_t *ms, view_t *view)
 char *
 menus_get_targets(view_t *view)
 {
-	if(view->selected_files > 0)
+	if(view->selected_files > 0 ||
+			(view->pending_marking && flist_count_marked(view) > 0))
 	{
 		return ma_expand("%f", NULL, NULL, 1);
 	}
@@ -721,12 +724,12 @@ menus_def_khandler(view_t *view, menu_data_t *m, const wchar_t keys[])
 {
 	if(wcscmp(keys, L"gf") == 0)
 	{
-		(void)menus_goto_file(m, curr_view, m->items[m->pos], 0);
+		(void)menus_goto_file(m, view, m->items[m->pos], 0);
 		return KHR_CLOSE_MENU;
 	}
 	else if(wcscmp(keys, L"e") == 0)
 	{
-		(void)menus_goto_file(m, curr_view, m->items[m->pos], 1);
+		(void)menus_goto_file(m, view, m->items[m->pos], 1);
 		return KHR_REFRESH_WINDOW;
 	}
 	else if(wcscmp(keys, L"c") == 0)
@@ -740,7 +743,7 @@ menus_def_khandler(view_t *view, menu_data_t *m, const wchar_t keys[])
 			show_error_msg("Command insertion", "No valid filename found");
 			return KHR_REFRESH_WINDOW;
 		}
-		menu_morph_into_cmdline(CLS_COMMAND, path, 1);
+		modmenu_morph_into_cline(CLS_COMMAND, path, 1);
 		free(path);
 		return KHR_MORPHED_MENU;
 	}
@@ -836,22 +839,30 @@ menu_and_view_are_in_sync(const menu_data_t *m, const view_t *view)
 
 int
 menus_capture(view_t *view, const char cmd[], int user_sh, menu_data_t *m,
-		int custom_view, int very_custom_view)
+		MacroFlags flags)
 {
-	if(custom_view || very_custom_view)
+	if(ma_flags_present(flags, MF_CUSTOMVIEW_OUTPUT) ||
+			ma_flags_present(flags, MF_VERYCUSTOMVIEW_OUTPUT))
 	{
+		rn_for_flist(view, cmd, m->title, flags);
 		menus_reset_data(m);
-		output_to_custom_flist(view, cmd, very_custom_view, 0);
 		return 0;
 	}
 
-	if(process_cmd_output("Loading menu", cmd, user_sh, 0, &output_handler,
-				m) != 0)
+	FILE *input_tmp = make_in_file(view, flags);
+
+	if(process_cmd_output("Loading menu", cmd, input_tmp, user_sh, 0,
+				&output_handler, m) != 0)
 	{
 		show_error_msgf("Trouble running command", "Unable to run: %s", cmd);
 		return 0;
 	}
 
+	if(input_tmp != NULL)
+	{
+		fclose(input_tmp);
+	}
+
 	if(ui_cancellation_requested())
 	{
 		append_to_string(&m->title, "(cancelled)");
@@ -1065,52 +1076,53 @@ navigate_to_match(menu_state_t *m, int pos)
 			menus_erase_current(m);
 			menus_set_pos(m, pos);
 		}
-		menus_search_print_msg(m);
+		menus_search_print_msg(m->d);
 	}
 	else
 	{
 		menus_set_pos(m, m->d->pos);
 		if(cfg.wrap_scan)
 		{
-			menus_search_print_msg(m);
+			menus_search_print_msg(m->d);
 		}
 	}
 	return 1;
 }
 
 void
-menus_search_print_msg(const menu_state_t *m)
+menus_search_print_msg(const menu_data_t *m)
 {
+	const menu_state_t *ms = m->state;
 	int cflags;
 	regex_t re;
 	int err;
 
 	/* Can be NULL after regex compilation failure. */
-	if(m->regexp == NULL)
+	if(ms->regexp == NULL)
 	{
 		return;
 	}
 
-	cflags = get_regexp_cflags(m->regexp);
-	err = regcomp(&re, m->regexp, cflags);
+	cflags = get_regexp_cflags(ms->regexp);
+	err = regcomp(&re, ms->regexp, cflags);
 
 	if(err != 0)
 	{
-		ui_sb_errf("Regexp (%s) error: %s", m->regexp, get_regexp_error(err, &re));
+		ui_sb_errf("Regexp (%s) error: %s", ms->regexp, get_regexp_error(err, &re));
 		regfree(&re);
 		return;
 	}
 
 	regfree(&re);
 
-	if(m->matching_entries > 0)
+	if(ms->matching_entries > 0)
 	{
-		ui_sb_msgf("%d of %d %s", get_match_index(m), m->matching_entries,
-				(m->matching_entries == 1) ? "match" : "matches");
+		ui_sb_msgf("%d of %d %s", get_match_index(ms), ms->matching_entries,
+				(ms->matching_entries == 1) ? "match" : "matches");
 	}
 	else
 	{
-		ui_sb_errf("No matches for: %s", m->regexp);
+		ui_sb_errf("No matches for: %s", ms->regexp);
 	}
 }
 
