@@ -20,10 +20,10 @@
 #ifndef VIFM__MENUS__UNDOLIST_MENU_H__
 #define VIFM__MENUS__UNDOLIST_MENU_H__
 
-#include "../ui/ui.h"
+struct view_t;
 
 /* Returns non-zero if status bar message should be saved. */
-int show_undolist_menu(FileView *view, int with_details);
+int show_undolist_menu(struct view_t *view, int with_details);
 
 #endif /* VIFM__MENUS__UNDOLIST_MENU_H__ */
 
