@@ -31,17 +31,20 @@ static KHandlerResponse users_khandler(view_t *view, menu_data_t *m,
 		const wchar_t keys[]);
 
 int
-show_user_menu(view_t *view, const char command[], int navigate)
+show_user_menu(view_t *view, const char command[], const char title[],
+		MacroFlags flags)
 {
 	static menu_data_t m;
-	menus_init_data(&m, view, strdup(command), strdup("No results found"));
+	menus_init_data(&m, view, strdup(title), strdup("No results found"));
+
+	const int navigate = ma_flags_present(flags, MF_MENU_NAV_OUTPUT);
 
 	m.extra_data = navigate;
 	m.stashable = navigate;
 	m.execute_handler = &execute_users_cb;
 	m.key_handler = &users_khandler;
 
-	return menus_capture(view, command, 1, &m, 0, 0);
+	return menus_capture(view, command, 1, &m, flags);
 }
 
 /* Callback that is called when menu item is selected.  Should return non-zero
@@ -70,7 +73,7 @@ users_khandler(view_t *view, menu_data_t *m, const wchar_t keys[])
 	else if(wcscmp(keys, L"c") == 0)
 	{
 		/* Insert whole line. */
-		menu_morph_into_cmdline(CLS_COMMAND, m->items[m->pos], 1);
+		modmenu_morph_into_cline(CLS_COMMAND, m->items[m->pos], 1);
 		return KHR_MORPHED_MENU;
 	}
 	return KHR_UNHANDLED;
