@@ -24,6 +24,7 @@
 #include <sys/stat.h>
 
 #include <assert.h> /* assert() */
+#include <ctype.h> /* isdigit() */
 #include <inttypes.h> /* PRId64 */
 #include <stddef.h> /* size_t */
 #include <stdint.h> /* uint64_t */
@@ -38,6 +39,7 @@
 #include "../engine/keys.h"
 #include "../engine/mode.h"
 #include "../int/file_magic.h"
+#include "../int/term_title.h"
 #include "../menus/menus.h"
 #include "../ui/ui.h"
 #include "../utils/fs.h"
@@ -57,13 +59,13 @@
 static void leave_file_info_mode(void);
 static int print_item(const char label[], const char path[], int curr_y);
 static int show_file_type(view_t *view, int curr_y);
+static int print_link_info(const dir_entry_t *curr, int curr_y);
 static int show_mime_type(view_t *view, int curr_y);
 static void format_time(time_t t, char buf[], size_t buf_size);
 static void cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info);
 
 static view_t *view;
-static int was_redraw;
 
 static keys_add_info_t builtin_cmds[] = {
 	{WK_C_c,    {{&cmd_ctrl_c}, .descr = "hide file info"}},
@@ -76,7 +78,7 @@ static keys_add_info_t builtin_cmds[] = {
 };
 
 void
-init_file_info_mode(void)
+modfinfo_init(void)
 {
 	int ret_code;
 
@@ -88,7 +90,7 @@ init_file_info_mode(void)
 }
 
 void
-enter_file_info_mode(view_t *v)
+modfinfo_enter(view_t *v)
 {
 	if(fentry_is_fake(get_current_entry(v)))
 	{
@@ -96,31 +98,28 @@ enter_file_info_mode(view_t *v)
 		return;
 	}
 
+	term_title_update("File Information");
 	vle_mode_set(FILE_INFO_MODE, VMT_PRIMARY);
 	view = v;
 	ui_setup_for_menu_like();
-	redraw_file_info_dialog();
+	modfinfo_redraw();
+}
 
-	was_redraw = 0;
+void
+modfinfo_abort(void)
+{
+	leave_file_info_mode();
 }
 
 static void
 leave_file_info_mode(void)
 {
 	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
-
-	if(was_redraw)
-	{
-		update_screen(UT_FULL);
-	}
-	else
-	{
-		update_all_windows();
-	}
+	stats_redraw_later();
 }
 
 void
-redraw_file_info_dialog(void)
+modfinfo_redraw(void)
 {
 	const dir_entry_t *curr;
 	char perm_buf[26];
@@ -173,6 +172,11 @@ redraw_file_info_dialog(void)
 	curr_y += print_item("Attributes: ", perm_buf, curr_y);
 #endif
 
+#ifndef _WIN32
+	snprintf(buf, sizeof(buf), "%d", curr->nlinks);
+	curr_y += print_item("Hard Links: ", buf, curr_y);
+#endif
+
 	format_time(curr->mtime, buf, sizeof(buf));
 	curr_y += print_item("Modified: ", buf, curr_y);
 
@@ -188,10 +192,26 @@ redraw_file_info_dialog(void)
 
 #ifndef _WIN32
 	get_uid_string(curr, 0, sizeof(id_buf), id_buf);
-	curr_y += print_item("Owner: ", id_buf, curr_y);
+	if(isdigit(id_buf[0]))
+	{
+		copy_str(buf, sizeof(buf), id_buf);
+	}
+	else
+	{
+		snprintf(buf, sizeof(buf), "%s (%lu)", id_buf, (unsigned long)curr->uid);
+	}
+	curr_y += print_item("Owner: ", buf, curr_y);
 
 	get_gid_string(curr, 0, sizeof(id_buf), id_buf);
-	curr_y += print_item("Group: ", id_buf, curr_y);
+	if(isdigit(id_buf[0]))
+	{
+		copy_str(buf, sizeof(buf), id_buf);
+	}
+	else
+	{
+		snprintf(buf, sizeof(buf), "%s (%lu)", id_buf, (unsigned long)curr->gid);
+	}
+	curr_y += print_item("Group: ", buf, curr_y);
 #endif
 
 	/* Fake use after last assignment. */
@@ -202,31 +222,34 @@ redraw_file_info_dialog(void)
 	wprint(menu_win, " File Information ");
 	ui_refresh_win(menu_win);
 	checked_wmove(menu_win, 2, 2);
-
-	was_redraw = 1;
 }
 
-/* Prints item prefixed with a label truncating the item if it's too long.
+/* Prints item prefixed with a label wrapping the item if it's too long.
  * Returns increment for curr_y. */
 static int
-print_item(const char label[], const char path[], int curr_y)
+print_item(const char label[], const char text[], int curr_y)
 {
-	const int max_width = getmaxx(menu_win) - strlen(label) - 2;
-	const size_t print_len = utf8_nstrsnlen(path, max_width);
-
 	mvwaddstr(menu_win, curr_y, 2, label);
-	if(path[print_len] == '\0')
-	{
-		wprint(menu_win, path);
-	}
-	else
-	{
-		char path_buf[PATH_MAX + 1];
-		copy_str(path_buf, MIN(sizeof(path_buf), print_len + 1), path);
-		wprint(menu_win, path_buf);
-	}
+	int x = getcurx(menu_win);
+	int max_width = getmaxx(menu_win) - 2 - x;
+	int dy = 0;
 
-	return 2;
+	do
+	{
+		const size_t print_len = utf8_nstrsnlen(text, max_width);
+
+		char part[1000];
+		copy_str(part, MIN(sizeof(part), print_len + 1), text);
+		wprint(menu_win, part);
+
+		text += print_len;
+		++dy;
+		++curr_y;
+		checked_wmove(menu_win, curr_y, x);
+	}
+	while(text[0] != '\0');
+
+	return dy + 1;
 }
 
 /* Returns increment for curr_y. */
@@ -234,37 +257,14 @@ static int
 show_file_type(view_t *view, int curr_y)
 {
 	const dir_entry_t *curr;
-	int x;
 	int old_curr_y = curr_y;
-	x = getmaxx(menu_win);
 
 	curr = get_current_entry(view);
 
 	mvwaddstr(menu_win, curr_y, 2, "Type: ");
-	if(curr->type == FT_LINK)
+	if(curr->type == FT_LINK || is_shortcut(curr->name))
 	{
-		char full_path[PATH_MAX + 1];
-		char linkto[PATH_MAX + NAME_MAX];
-
-		get_current_full_path(view, sizeof(full_path), full_path);
-
-		mvwaddstr(menu_win, curr_y, 8, "Link");
-		curr_y += 2;
-		mvwaddstr(menu_win, curr_y, 2, "Link To: ");
-
-		if(get_link_target(full_path, linkto, sizeof(linkto)) == 0)
-		{
-			mvwaddnstr(menu_win, curr_y, 11, linkto, x - 11);
-
-			if(!path_exists(linkto, DEREF))
-			{
-				mvwaddstr(menu_win, curr_y - 2, 12, " (BROKEN)");
-			}
-		}
-		else
-		{
-			mvwaddstr(menu_win, curr_y, 11, "Couldn't Resolve Link");
-		}
+		curr_y += print_link_info(curr, curr_y);
 	}
 	else if(curr->type == FT_EXEC || curr->type == FT_REG)
 	{
@@ -293,10 +293,11 @@ show_file_type(view_t *view, int curr_y)
 
 		pclose(pipe);
 
-		mvwaddnstr(menu_win, curr_y, 8, buf, x - 9);
-		if(x > 9 && strlen(buf) > (size_t)(x - 9))
+		int max_x = getmaxx(menu_win);
+		mvwaddnstr(menu_win, curr_y, 8, buf, max_x - 9);
+		if(max_x > 9 && strlen(buf) > (size_t)(max_x - 9))
 		{
-			mvwaddnstr(menu_win, curr_y + 1, 8, buf + x - 9, x - 9);
+			mvwaddnstr(menu_win, curr_y + 1, 8, buf + max_x - 9, max_x - 9);
 		}
 #else /* #ifdef HAVE_FILE_PROG */
 		if(curr->type == FT_EXEC)
@@ -354,26 +355,84 @@ show_file_type(view_t *view, int curr_y)
 	return curr_y - old_curr_y;
 }
 
+/* Prints information about a link entry.  Returns increment for curr_y. */
+static int
+print_link_info(const dir_entry_t *curr, int curr_y)
+{
+	int old_curr_y = curr_y;
+	int max_x = getmaxx(menu_win);
+
+	char full_path[PATH_MAX + 1];
+	char linkto[PATH_MAX + NAME_MAX];
+
+	get_full_path_of(curr, sizeof(full_path), full_path);
+
+	int broken_offset;
+	int target_offset;
+	if(curr->type == FT_LINK)
+	{
+		mvwaddstr(menu_win, curr_y, 8, "Link");
+		curr_y += 2;
+		mvwaddstr(menu_win, curr_y, 2, "Link To: ");
+		broken_offset = 12;
+		target_offset = 11;
+	}
+	else
+	{
+		mvwaddstr(menu_win, curr_y, 8, "Shortcut");
+		curr_y += 2;
+		mvwaddstr(menu_win, curr_y, 2, "Shortcut To: ");
+		broken_offset = 16;
+		target_offset = 15;
+	}
+
+	if(get_link_target(full_path, linkto, sizeof(linkto)) == 0)
+	{
+		mvwaddnstr(menu_win, curr_y, target_offset, linkto, max_x - target_offset);
+
+		if(!path_exists(linkto, DEREF))
+		{
+			mvwaddstr(menu_win, curr_y - 2, broken_offset, " (BROKEN)");
+		}
+	}
+	else
+	{
+		mvwaddstr(menu_win, curr_y, target_offset, "Couldn't Resolve Link");
+	}
+
+	if(curr->type == FT_LINK)
+	{
+		curr_y += 2;
+		mvwaddstr(menu_win, curr_y, 2, "Real Path: ");
+
+		char real[PATH_MAX + 1];
+		if(os_realpath(full_path, real) == real)
+		{
+			mvwaddnstr(menu_win, curr_y, 13, real, max_x - 13);
+		}
+		else
+		{
+			waddstr(menu_win, "Couldn't Resolve Path");
+		}
+	}
+
+	return curr_y - old_curr_y;
+}
+
 /* Returns increment for curr_y. */
 static int
 show_mime_type(view_t *view, int curr_y)
 {
 	char full_path[PATH_MAX + 1];
-	const char *mimetype = NULL;
-
 	get_current_full_path(view, sizeof(full_path), full_path);
-	mimetype = get_mimetype(full_path, 0);
-
-	mvwaddstr(menu_win, curr_y, 2, "Mime Type: ");
 
+	const char *mimetype = get_mimetype(full_path, 0);
 	if(mimetype == NULL)
 	{
 		mimetype = "Unknown";
 	}
 
-	mvwaddstr(menu_win, curr_y, 13, mimetype);
-
-	return 2;
+	return print_item("Mime Type: ", mimetype, curr_y);
 }
 
 /* Formats single time field as a string.  Writes empty string on error. */
@@ -399,7 +458,7 @@ cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 {
-	redraw_file_info_dialog();
+	modfinfo_redraw();
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
