@@ -173,6 +173,7 @@ static void cmd_f(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gA(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_ga(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gf(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_gF(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gg(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gh(key_info_t key_info, keys_info_t *keys_info);
 #ifdef _WIN32
@@ -222,6 +223,7 @@ static void cmd_zf(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zm(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zo(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zr(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_zx(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_left_paren(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_right_paren(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_z_k(key_info_t key_info, keys_info_t *keys_info);
@@ -298,13 +300,13 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_C_w WK_x,      {{&cmd_ctrl_wx}, .descr = "exchange panes"}},
 	{WK_C_w WK_C_z,    {{&cmd_ctrl_wz}, .descr = "exit preview/view modes"}},
 	{WK_C_w WK_z,      {{&cmd_ctrl_wz}, .descr = "exit preview/view modes"}},
-	{WK_C_w WK_EQUALS, {{&normal_cmd_ctrl_wequal},   .nim = 1, .descr = "size panes equally"}},
-	{WK_C_w WK_LT,     {{&normal_cmd_ctrl_wless},    .nim = 1, .descr = "decrease pane size by one"}},
-	{WK_C_w WK_GT,     {{&normal_cmd_ctrl_wgreater}, .nim = 1, .descr = "increase pane size by one"}},
-	{WK_C_w WK_PLUS,   {{&normal_cmd_ctrl_wplus},    .nim = 1, .descr = "increase pane size by one"}},
-	{WK_C_w WK_MINUS,  {{&normal_cmd_ctrl_wminus},   .nim = 1, .descr = "decrease pane size by one"}},
-	{WK_C_w WK_BAR,    {{&normal_cmd_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
-	{WK_C_w WK_USCORE, {{&normal_cmd_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
+	{WK_C_w WK_EQUALS, {{&modnorm_ctrl_wequal},   .nim = 1, .descr = "size panes equally"}},
+	{WK_C_w WK_LT,     {{&modnorm_ctrl_wless},    .nim = 1, .descr = "decrease pane size by one"}},
+	{WK_C_w WK_GT,     {{&modnorm_ctrl_wgreater}, .nim = 1, .descr = "increase pane size by one"}},
+	{WK_C_w WK_PLUS,   {{&modnorm_ctrl_wplus},    .nim = 1, .descr = "increase pane size by one"}},
+	{WK_C_w WK_MINUS,  {{&modnorm_ctrl_wminus},   .nim = 1, .descr = "decrease pane size by one"}},
+	{WK_C_w WK_BAR,    {{&modnorm_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
+	{WK_C_w WK_USCORE, {{&modnorm_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
 	{WK_C_x,           {{&cmd_ctrl_x}, .descr = "decrement number in names"}},
 	{WK_C_y,           {{&cmd_ctrl_y}, .descr = "scroll one line up"}},
 	{WK_ESC,           {{&cmd_ctrl_c}, .descr = "reset selection and highlight"}},
@@ -352,6 +354,7 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_g WK_A,        {{&cmd_gA}, .descr = "(re)calculate size"}},
 	{WK_g WK_a,        {{&cmd_ga}, .descr = "calculate size"}},
 	{WK_g WK_f,        {{&cmd_gf}, .descr = "navigate to link target"}},
+	{WK_g WK_F,        {{&cmd_gF}, .descr = "navigate to ultimate link target"}},
 	{WK_g WK_g,        {{&cmd_gg}, .descr = "go to the first item"}},
 	{WK_g WK_h,        {{&cmd_gh}, .descr = "go to parent directory"}},
 	{WK_g WK_j,        {{&cmd_j},  .descr = "go to item below"}},
@@ -392,13 +395,14 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_z WK_R,        {{&cmd_zR}, .descr = "save and reset all filters"}},
 	{WK_z WK_a,        {{&cmd_za}, .descr = "toggle dot files visibility"}},
 	{WK_z WK_d,        {{&cmd_zd}, .descr = "exclude custom view entry"}},
-	{WK_z WK_b,        {{&normal_cmd_zb}, .descr = "push cursor to the bottom"}},
+	{WK_z WK_b,        {{&modnorm_zb}, .descr = "push cursor to the bottom"}},
 	{WK_z WK_f,        {{&cmd_zf}, .descr = "add current file to filter"}},
 	{WK_z WK_m,        {{&cmd_zm}, .descr = "hide dot files"}},
 	{WK_z WK_o,        {{&cmd_zo}, .descr = "show dot files"}},
 	{WK_z WK_r,        {{&cmd_zr}, .descr = "clear local filter"}},
-	{WK_z WK_t,        {{&normal_cmd_zt},   .descr = "push cursor to the top"}},
-	{WK_z WK_z,        {{&normal_cmd_zz},   .descr = "center cursor position"}},
+	{WK_z WK_x,        {{&cmd_zx}, .descr = "toggle directory fold"}},
+	{WK_z WK_t,        {{&modnorm_zt},      .descr = "push cursor to the top"}},
+	{WK_z WK_z,        {{&modnorm_zz},      .descr = "center cursor position"}},
 	{WK_LP,            {{&cmd_left_paren},  .descr = "go to previous group of files"}},
 	{WK_RP,            {{&cmd_right_paren}, .descr = "go to next group of files"}},
 	{WK_z WK_k,        {{&cmd_z_k},  .descr = "go to previous sibling dir"}},
@@ -486,7 +490,7 @@ static keys_add_info_t selectors[] = {
 };
 
 void
-init_normal_mode(void)
+modnorm_init(void)
 {
 	int ret_code;
 
@@ -505,7 +509,7 @@ init_normal_mode(void)
 static void
 cmd_ctrl_a(key_info_t key_info, keys_info_t *keys_info)
 {
-	check_marking(curr_view, 0, NULL);
+	flist_set_marking(curr_view, 0);
 	curr_stats.save_msg = fops_incdec(curr_view, def_count(key_info.count));
 }
 
@@ -582,7 +586,7 @@ page_scroll(int base, int direction)
 static void
 cmd_ctrl_g(key_info_t key_info, keys_info_t *keys_info)
 {
-	enter_file_info_mode(curr_view);
+	modfinfo_enter(curr_view);
 }
 
 static void
@@ -596,7 +600,7 @@ cmd_space(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_emarkemark(key_info_t key_info, keys_info_t *keys_info)
 {
-	char prefix[16];
+	char prefix[32];
 	if(key_info.count != NO_COUNT_GIVEN && key_info.count != 1)
 	{
 		if(curr_view->list_pos + key_info.count >= curr_view->list_rows)
@@ -613,8 +617,8 @@ cmd_emarkemark(key_info_t key_info, keys_info_t *keys_info)
 		strcpy(prefix, ".!");
 	}
 
-	set_count_vars(key_info.count);
-	enter_cmdline_mode(CLS_COMMAND, prefix, NULL);
+	cmds_vars_set_count(key_info.count);
+	modcline_enter(CLS_COMMAND, prefix, NULL);
 }
 
 /* Processes !<selector> normal mode command.  Processes results of applying
@@ -669,7 +673,7 @@ cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_return(key_info_t key_info, keys_info_t *keys_info)
 {
-	open_file(curr_view, FHE_RUN);
+	rn_open(curr_view, FHE_RUN);
 	flist_sel_stash(curr_view);
 	redraw_current_view();
 }
@@ -684,7 +688,7 @@ static void
 cmd_ctrl_r(key_info_t key_info, keys_info_t *keys_info)
 {
 	curr_stats.confirmed = 0;
-	ui_cancellation_reset();
+	ui_cancellation_push_off();
 
 	ui_sb_msg("Redoing...");
 
@@ -723,6 +727,7 @@ cmd_ctrl_r(key_info_t key_info, keys_info_t *keys_info)
 	}
 
 	curr_stats.save_msg = 1;
+	ui_cancellation_pop();
 }
 
 /* Scroll view up by half of its height. */
@@ -832,38 +837,37 @@ cmd_ctrl_wL(key_info_t key_info, keys_info_t *keys_info)
 }
 
 void
-normal_cmd_ctrl_wequal(key_info_t key_info, keys_info_t *keys_info)
+modnorm_ctrl_wequal(key_info_t key_info, keys_info_t *keys_info)
 {
-	curr_stats.splitter_pos = -1;
-	update_screen(UT_REDRAW);
+	stats_set_splitter_pos(-1);
 }
 
 void
-normal_cmd_ctrl_wless(key_info_t key_info, keys_info_t *keys_info)
+modnorm_ctrl_wless(key_info_t key_info, keys_info_t *keys_info)
 {
 	move_splitter(def_count(key_info.count), is_left_or_top() ? -1 : +1);
 }
 
 void
-normal_cmd_ctrl_wgreater(key_info_t key_info, keys_info_t *keys_info)
+modnorm_ctrl_wgreater(key_info_t key_info, keys_info_t *keys_info)
 {
 	move_splitter(def_count(key_info.count), is_left_or_top() ? +1 : -1);
 }
 
 void
-normal_cmd_ctrl_wplus(key_info_t key_info, keys_info_t *keys_info)
+modnorm_ctrl_wplus(key_info_t key_info, keys_info_t *keys_info)
 {
 	move_splitter(def_count(key_info.count), is_left_or_top() ? +1 : -1);
 }
 
 void
-normal_cmd_ctrl_wminus(key_info_t key_info, keys_info_t *keys_info)
+modnorm_ctrl_wminus(key_info_t key_info, keys_info_t *keys_info)
 {
 	move_splitter(def_count(key_info.count), is_left_or_top() ? -1 : +1);
 }
 
 void
-normal_cmd_ctrl_wpipe(key_info_t key_info, keys_info_t *keys_info)
+modnorm_ctrl_wpipe(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(key_info.count == NO_COUNT_GIVEN)
 	{
@@ -919,7 +923,7 @@ cmd_ctrl_wz(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_x(key_info_t key_info, keys_info_t *keys_info)
 {
-	check_marking(curr_view, 0, NULL);
+	flist_set_marking(curr_view, 0);
 	curr_stats.save_msg = fops_incdec(curr_view, -def_count(key_info.count));
 }
 
@@ -960,9 +964,9 @@ go_to_other_window(void)
 static int
 try_switch_into_view_mode(void)
 {
-	if(curr_stats.preview.on)
+	if(curr_stats.preview.on && curr_stats.number_of_windows != 1)
 	{
-		view_enter_mode(other_view, 0);
+		modview_enter(other_view, 0);
 		return 1;
 	}
 	return 0;
@@ -973,7 +977,7 @@ try_switch_into_view_mode(void)
 static void
 cmd_C(key_info_t key_info, keys_info_t *keys_info)
 {
-	check_marking(curr_view, 0, NULL);
+	flist_set_marking(curr_view, 0);
 	curr_stats.save_msg = fops_clone(curr_view, NULL, 0, 0,
 			def_count(key_info.count));
 }
@@ -1041,11 +1045,21 @@ cmd_ga(key_info_t key_info, keys_info_t *keys_info)
 	fops_size_bg(curr_view, 0);
 }
 
+/* Changes current directory to symlink's target (follows just one link). */
 static void
 cmd_gf(key_info_t key_info, keys_info_t *keys_info)
 {
 	flist_sel_stash(curr_view);
-	follow_file(curr_view);
+	rn_follow(curr_view, 0);
+	redraw_current_view();
+}
+
+/* Changes current directory to the ultimate target (follows all links). */
+static void
+cmd_gF(key_info_t key_info, keys_info_t *keys_info)
+{
+	flist_sel_stash(curr_view);
+	rn_follow(curr_view, 1);
 	redraw_current_view();
 }
 
@@ -1060,14 +1074,14 @@ cmd_gg(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_gh(key_info_t key_info, keys_info_t *keys_info)
 {
-	cd_updir(curr_view, def_count(key_info.count));
+	rn_leave(curr_view, def_count(key_info.count));
 }
 
 #ifdef _WIN32
 static void
 cmd_gr(key_info_t key_info, keys_info_t *keys_info)
 {
-	open_file(curr_view, FHE_ELEVATE_AND_RUN);
+	rn_open(curr_view, FHE_ELEVATE_AND_RUN);
 	flist_sel_stash(curr_view);
 	redraw_current_view();
 }
@@ -1172,7 +1186,7 @@ do_gu(key_info_t key_info, keys_info_t *keys_info, int upper)
 static void
 cmd_gv(key_info_t key_info, keys_info_t *keys_info)
 {
-	enter_visual_mode(VS_RESTORE);
+	modvis_enter(VS_RESTORE);
 }
 
 /* Go to the first file in window. */
@@ -1228,7 +1242,7 @@ cmd_P(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_V(key_info_t key_info, keys_info_t *keys_info)
 {
-	enter_visual_mode(VS_NORMAL);
+	modvis_enter(VS_NORMAL);
 }
 
 /* Possibly exits the application without saving vifminfo file or closes a
@@ -1252,7 +1266,7 @@ cmd_quote(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(keys_info->selector)
 	{
-		const int pos = check_mark_directory(curr_view, key_info.multi);
+		const int pos = marks_find_in_view(curr_view, key_info.multi);
 		if(pos >= 0)
 		{
 			pick_files(curr_view, pos, keys_info);
@@ -1260,7 +1274,7 @@ cmd_quote(key_info_t key_info, keys_info_t *keys_info)
 	}
 	else
 	{
-		curr_stats.save_msg = goto_mark(curr_view, key_info.multi);
+		curr_stats.save_msg = marks_goto(curr_view, key_info.multi);
 		if(!cfg.auto_ch_pos)
 		{
 			fpos_set_pos(curr_view, 0);
@@ -1274,7 +1288,7 @@ sug_cmd_quote(vle_keys_list_cb cb)
 {
 	if(cfg.sug.flags & SF_MARKS)
 	{
-		suggest_marks(cb, 0);
+		marks_suggest(curr_view, cb, 0);
 	}
 }
 
@@ -1284,7 +1298,7 @@ sug_sel_quote(vle_keys_list_cb cb)
 {
 	if(cfg.sug.flags & SF_MARKS)
 	{
-		suggest_marks(cb, 1);
+		marks_suggest(curr_view, cb, 1);
 	}
 }
 
@@ -1315,7 +1329,7 @@ cmd_percent(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_equal(key_info_t key_info, keys_info_t *keys_info)
 {
-	enter_cmdline_mode(CLS_FILTER, curr_view->local_filter.filter.raw, NULL);
+	modcline_enter(CLS_FILTER, curr_view->local_filter.filter.raw, NULL);
 }
 
 /* Continues navigation to word which starts with specified character in
@@ -1359,24 +1373,8 @@ cmd_colon(key_info_t key_info, keys_info_t *keys_info)
 		snprintf(prefix, ARRAY_LEN(prefix), ".,.+%d", key_info.count - 1);
 	}
 
-	set_count_vars(key_info.count);
-	enter_cmdline_mode(CLS_COMMAND, prefix, NULL);
-}
-
-void
-set_count_vars(int count)
-{
-	/* TODO: move this to a better place someday, nowhere to place right now. */
-
-	var_t var;
-
-	var = var_from_int(count == NO_COUNT_GIVEN ? 0 : count);
-	setvar("v:count", var);
-	var_free(var);
-
-	var = var_from_int(count == NO_COUNT_GIVEN ? 1 : count);
-	setvar("v:count1", var);
-	var_free(var);
+	cmds_vars_set_count(key_info.count);
+	modcline_enter(CLS_COMMAND, prefix, NULL);
 }
 
 /* Continues navigation to word which starts with specified character in initial
@@ -1416,7 +1414,7 @@ cmd_al(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_av(key_info_t key_info, keys_info_t *keys_info)
 {
-	enter_visual_mode(VS_AMEND);
+	modvis_enter(VS_AMEND);
 }
 
 /* Change word (rename file without extension). */
@@ -1455,23 +1453,21 @@ cmd_co(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_cp(key_info_t key_info, keys_info_t *keys_info)
 {
-	normal_cmd_cp(curr_view, key_info);
+	modnorm_cp(curr_view, key_info);
 }
 
 void
-normal_cmd_cp(view_t *view, key_info_t key_info)
+modnorm_cp(view_t *view, key_info_t key_info)
 {
 #ifndef _WIN32
-	char mode[32];
-	int len;
-
 	if(key_info.count == NO_COUNT_GIVEN)
 	{
 		enter_attr_mode(view);
 		return;
 	}
 
-	len = snprintf(mode, sizeof(mode), "%04d", key_info.count);
+	char mode[32];
+	int len = snprintf(mode, sizeof(mode), "%04d", key_info.count);
 	if(len != 4 || ((int)strspn(mode, "01234567") != len))
 	{
 		/* Update view because there might be not reflected changes of cursor
@@ -1483,6 +1479,7 @@ normal_cmd_cp(view_t *view, key_info_t key_info)
 		return;
 	}
 
+	flist_set_marking(view, 0);
 	files_chmod(view, mode, 0);
 #else
 	enter_attr_mode(view);
@@ -1495,7 +1492,7 @@ cmd_cw(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(curr_view->selected_files > 1)
 	{
-		check_marking(curr_view, 0, NULL);
+		flist_set_marking(curr_view, 0);
 		fops_rename(curr_view, NULL, 0, 0);
 		return;
 	}
@@ -1592,7 +1589,7 @@ cmd_e(key_info_t key_info, keys_info_t *keys_info)
 		curr_stats.save_msg = 1;
 		return;
 	}
-	view_enter_mode(curr_view, 1);
+	modview_enter(curr_view, 1);
 }
 
 /* Navigates to next word which starts with specified character. */
@@ -1638,7 +1635,7 @@ cmd_h(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_i(key_info_t key_info, keys_info_t *keys_info)
 {
-	open_file(curr_view, FHE_NO_RUN);
+	rn_open(curr_view, FHE_NO_RUN);
 	flist_sel_stash(curr_view);
 	redraw_current_view();
 }
@@ -1699,8 +1696,8 @@ cmd_m(key_info_t key_info, keys_info_t *keys_info)
 	const dir_entry_t *const curr = get_current_entry(curr_view);
 	if(!fentry_is_fake(curr))
 	{
-		curr_stats.save_msg = set_user_mark(key_info.multi, curr->origin,
-				curr->name);
+		curr_stats.save_msg = marks_set_user(curr_view, key_info.multi,
+				curr->origin, curr->name);
 	}
 }
 
@@ -1819,7 +1816,7 @@ activate_search(int count, int back, int external)
 	else
 	{
 		const CmdLineSubmode submode = back ? CLS_BSEARCH : CLS_FSEARCH;
-		enter_cmdline_mode(submode, "", NULL);
+		modcline_enter(submode, "", NULL);
 	}
 }
 
@@ -1860,7 +1857,7 @@ static void
 cmd_u(key_info_t key_info, keys_info_t *keys_info)
 {
 	curr_stats.confirmed = 0;
-	ui_cancellation_reset();
+	ui_cancellation_push_off();
 
 	ui_sb_msg("Undoing...");
 
@@ -1899,6 +1896,7 @@ cmd_u(key_info_t key_info, keys_info_t *keys_info)
 	}
 
 	curr_stats.save_msg = 1;
+	ui_cancellation_pop();
 }
 
 /* Yanks files. */
@@ -2007,9 +2005,8 @@ cmd_zd(key_info_t key_info, keys_info_t *keys_info)
 	flist_custom_exclude(curr_view, key_info.count == 1);
 }
 
-/* Redraw with file in bottom of list. */
 void
-normal_cmd_zb(key_info_t key_info, keys_info_t *keys_info)
+modnorm_zb(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(can_scroll_up(curr_view))
 	{
@@ -2019,34 +2016,47 @@ normal_cmd_zb(key_info_t key_info, keys_info_t *keys_info)
 	}
 }
 
-/* Filter selected files. */
+/* Filters active files. */
 static void
 cmd_zf(key_info_t key_info, keys_info_t *keys_info)
 {
-	name_filters_add_selection(curr_view);
+	name_filters_add_active(curr_view);
 }
 
-/* Hide dot files. */
+/* Hides dot files. */
 static void
 cmd_zm(key_info_t key_info, keys_info_t *keys_info)
 {
 	dot_filter_set(curr_view, 0);
 }
 
-/* Show all the dot files. */
+/* Shows all the dot files. */
 static void
 cmd_zo(key_info_t key_info, keys_info_t *keys_info)
 {
 	dot_filter_set(curr_view, 1);
 }
 
-/* Reset local filter. */
+/* Resets local filter. */
 static void
 cmd_zr(key_info_t key_info, keys_info_t *keys_info)
 {
 	local_filter_remove(curr_view);
 }
 
+/* Toggles fold under the cursor if any. */
+static void
+cmd_zx(key_info_t key_info, keys_info_t *keys_info)
+{
+	dir_entry_t *curr = get_current_entry(curr_view);
+	if(curr->type != FT_DIR && curr->child_pos != 0)
+	{
+		fpos_set_pos(curr_view, curr_view->list_pos - curr->child_pos);
+	}
+
+	flist_toggle_fold(curr_view);
+}
+
 /* Moves cursor to the beginning of the previous group of files defined by the
  * primary sorting key. */
 static void
@@ -2189,9 +2199,8 @@ cmd_right_curly_bracket(key_info_t key_info, keys_info_t *keys_info)
 	pick_or_move(keys_info, fpos_find_dir_group(curr_view, 1));
 }
 
-/* Redraw with file in top of list. */
 void
-normal_cmd_zt(key_info_t key_info, keys_info_t *keys_info)
+modnorm_zt(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(can_scroll_down(curr_view))
 	{
@@ -2201,9 +2210,8 @@ normal_cmd_zt(key_info_t key_info, keys_info_t *keys_info)
 	}
 }
 
-/* Redraw with file in center of list. */
 void
-normal_cmd_zz(key_info_t key_info, keys_info_t *keys_info)
+modnorm_zz(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(!fpos_are_all_files_visible(curr_view))
 	{
@@ -2325,8 +2333,7 @@ selector_s(key_info_t key_info, keys_info_t *keys_info)
 }
 
 int
-find_npattern(view_t *view, const char pattern[], int backward,
-		int print_errors)
+modnorm_find(view_t *view, const char pattern[], int backward, int print_errors)
 {
 	const int nrepeats = search_repeat - 1;
 	int i;
