@@ -24,9 +24,10 @@
 #include <unistd.h> /* usleep() */
 
 #include <assert.h> /* assert() */
+#include <limits.h> /* INT_MAX */
 #include <stddef.h> /* ptrdiff_t size_t */
 #include <string.h> /* memset() strdup() */
-#include <stdio.h>  /* fclose() snprintf() */
+#include <stdio.h>  /* snprintf() */
 #include <stdlib.h> /* free() */
 
 #include "../cfg/config.h"
@@ -38,8 +39,6 @@
 #include "../engine/mode.h"
 #include "../int/vim.h"
 #include "../modes/dialogs/msg_dialog.h"
-#include "../ui/cancellation.h"
-#include "../ui/color_manager.h"
 #include "../ui/colors.h"
 #include "../ui/escape.h"
 #include "../ui/fileview.h"
@@ -53,6 +52,7 @@
 #include "../utils/regexp.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
+#include "../utils/test_helpers.h"
 #include "../utils/utf8.h"
 #include "../utils/utils.h"
 #include "../filelist.h"
@@ -60,6 +60,7 @@
 #include "../running.h"
 #include "../status.h"
 #include "../types.h"
+#include "../vcache.h"
 #include "cmdline.h"
 #include "modes.h"
 #include "normal.h"
@@ -73,14 +74,14 @@ enum
 };
 
 /* Describes view state and its properties. */
-struct view_info_t
+struct modview_info_t
 {
 	/* Data of the view. */
-	char **lines;     /* List of real lines. */
+	char **lines;     /* List of real lines (owned by vcache unit). */
 	int (*widths)[2]; /* (virtual line, screen width) pair per real line. */
 	int nlines;       /* Number of real lines. */
 	int nlinesv;      /* Number of virtual (possibly wrapped) lines. */
-	int line;         /* Current real line number. */
+	int line;         /* Current real line number (first visible line). */
 	int linev;        /* Current virtual line number. */
 
 	/* Dimensions, units of actions. */
@@ -97,14 +98,21 @@ struct view_info_t
 	int last_search_backward; /* Value -1 means no search was performed. */
 	int search_repeat;        /* Saved count prefix of search commands. */
 
+	/* Viewers. */
+	strlist_t viewers;       /* List of viewers of current file. */
+	char *ext_viewer;        /* When non-NULL, specifies custom preview command
+	                            (no implicit %c). */
+	const char *curr_viewer; /* Current viewer (pointer to strings of two previous
+	                            fields). */
+
 	/* The rest of the state. */
-	view_t *view;    /* File view association with the view. */
-	char *filename;  /* Full path to the file being viewed. */
-	char *viewer;    /* When non-NULL, specifies custom preview command (no
-	                    implicit %c). */
-	int detached;    /* Whether view mode was detached. */
-	ViewerKind kind; /* Kind of preview. */
-	int wrap;        /* Whether lines are wrapped. */
+	view_t *view;     /* File view association with the view. */
+	char *filename;   /* Full path to the file being viewed. */
+	int detached;     /* Whether view mode was detached. */
+	ViewerKind kind;  /* Kind of preview. */
+	MacroFlags flags; /* Macro flags for ext_viewer. */
+	int wrap;         /* Whether lines are wrapped. */
+	int raw;          /* Forced raw preview. */
 };
 
 /* View information structure indexes and count. */
@@ -117,13 +125,13 @@ enum
 
 static int try_resurrect_detached(const char full_path[], int explore);
 static void try_redraw_explore_view(const view_t *view);
-static void reset_view_info(view_info_t *vi);
-static void init_view_info(view_info_t *vi);
-static void free_view_info(view_info_t *vi);
+static void reset_view_info(modview_info_t *vi);
+static void init_view_info(modview_info_t *vi);
+static void free_view_info(modview_info_t *vi);
 static void redraw(void);
 static void calc_vlines(void);
-static void calc_vlines_wrapped(view_info_t *vi);
-static void calc_vlines_non_wrapped(view_info_t *vi);
+static void calc_vlines_wrapped(modview_info_t *vi);
+static void calc_vlines_non_wrapped(modview_info_t *vi);
 static void draw(void);
 static int get_part(const char line[], int offset, size_t max_len, char part[]);
 static void display_error(const char error_msg[]);
@@ -153,27 +161,33 @@ static void cmd_tab(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_slash(key_info_t key_info, keys_info_t *keys_info);
 static void pick_vi(int explore);
 static void cmd_qmark(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_A(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_F(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_G(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_N(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_R(key_info_t key_info, keys_info_t *keys_info);
-static int load_view_data(view_info_t *vi, const char action[],
+static int load_view_data(modview_info_t *vi, const char action[],
 		const char file_to_view[], int silent);
-static int get_view_data(view_info_t *vi, const char file_to_view[]);
-static void replace_vi(view_info_t *orig, view_info_t *new);
+static const char * get_view_data(modview_info_t *vi,
+		const char file_to_view[]);
+static void pick_current_viewer(modview_info_t *vi);
+static void replace_vi(modview_info_t *orig, modview_info_t *new);
+static void cmd_a(key_info_t key_info, keys_info_t *keys_info);
+static void switch_viewer(modview_info_t *vi, int offset);
 static void cmd_b(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_d(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_f(key_info_t key_info, keys_info_t *keys_info);
 static void check_and_set_from_default_win(key_info_t *key_info);
 static void set_from_default_win(key_info_t *key_info);
 static void cmd_g(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_i(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_j(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_k(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_n(key_info_t key_info, keys_info_t *keys_info);
 static void goto_search_result(int repeat_count, int inverse_direction);
 static void search(int repeat_count, int backward);
-static void find_previous(int vline_offset);
-static void find_next(void);
+static int find_previous(void);
+static int find_next(void);
 static void cmd_q(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_u(key_info_t key_info, keys_info_t *keys_info);
 static void update_with_half_win(key_info_t *key_info);
@@ -183,14 +197,19 @@ static void cmd_z(key_info_t key_info, keys_info_t *keys_info);
 static void update_with_win(key_info_t *key_info);
 static int is_trying_the_same_file(void);
 static int get_file_to_explore(const view_t *view, char buf[], size_t buf_len);
-static int forward_if_changed(view_info_t *vi);
-static int scroll_to_bottom(view_info_t *vi);
-static void reload_view(view_info_t *vi, int silent);
-static view_info_t * view_info_alloc(void);
+static int forward_if_changed(modview_info_t *vi);
+static int scroll_to_bottom(modview_info_t *vi);
+static void reload_view(modview_info_t *vi, int silent);
+static void cleanup(modview_info_t *vi);
+static modview_info_t * view_info_alloc(void);
+TSTATIC int modview_is_raw(modview_info_t *vi);
+TSTATIC const char * modview_current_viewer(modview_info_t *vi);
+TSTATIC int modview_current_line(modview_info_t *vi);
+TSTATIC strlist_t modview_lines(modview_info_t *vi);
 
 /* Points to current (for quick view) or last used (for explore mode)
- * view_info_t structure. */
-static view_info_t *vi;
+ * modview_info_t structure. */
+static modview_info_t *vi;
 
 static keys_add_info_t builtin_cmds[] = {
 	{WK_C_b,           {{&cmd_b},      .descr = "scroll page up"}},
@@ -204,7 +223,7 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_C_n,           {{&cmd_j},      .descr = "scroll one line down"}},
 	{WK_C_p,           {{&cmd_k},      .descr = "scroll one line up"}},
 	{WK_C_r,           {{&cmd_ctrl_l}, .descr = "redraw"}},
-	{WK_C_u,           {{&cmd_u},      .descr = "undo file operation"}},
+	{WK_C_u,           {{&cmd_u},      .descr = "scroll half-page down"}},
 	{WK_C_v,           {{&cmd_f},      .descr = "scroll page down"}},
 	{WK_C_w WK_H,      {{&cmd_ctrl_wH}, .descr = "move window to the left"}},
 	{WK_C_w WK_J,      {{&cmd_ctrl_wJ}, .descr = "move window to the bottom"}},
@@ -235,13 +254,13 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_C_w WK_s,      {{&cmd_ctrl_ws}, .descr = "horizontal split layout"}},
 	{WK_C_w WK_C_v,    {{&cmd_ctrl_wv}, .descr = "vertical split layout"}},
 	{WK_C_w WK_v,      {{&cmd_ctrl_wv}, .descr = "vertical split layout"}},
-	{WK_C_w WK_EQUALS, {{&normal_cmd_ctrl_wequal},   .nim = 1, .descr = "size panes equally"}},
-	{WK_C_w WK_LT,     {{&normal_cmd_ctrl_wless},    .nim = 1, .descr = "decrease pane size by one"}},
-	{WK_C_w WK_GT,     {{&normal_cmd_ctrl_wgreater}, .nim = 1, .descr = "increase pane size by one"}},
-	{WK_C_w WK_PLUS,   {{&normal_cmd_ctrl_wplus},    .nim = 1, .descr = "increase pane size by one"}},
-	{WK_C_w WK_MINUS,  {{&normal_cmd_ctrl_wminus},   .nim = 1, .descr = "decrease pane size by one"}},
-	{WK_C_w WK_BAR,    {{&normal_cmd_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
-	{WK_C_w WK_USCORE, {{&normal_cmd_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
+	{WK_C_w WK_EQUALS, {{&modnorm_ctrl_wequal},   .nim = 1, .descr = "size panes equally"}},
+	{WK_C_w WK_LT,     {{&modnorm_ctrl_wless},    .nim = 1, .descr = "decrease pane size by one"}},
+	{WK_C_w WK_GT,     {{&modnorm_ctrl_wgreater}, .nim = 1, .descr = "increase pane size by one"}},
+	{WK_C_w WK_PLUS,   {{&modnorm_ctrl_wplus},    .nim = 1, .descr = "increase pane size by one"}},
+	{WK_C_w WK_MINUS,  {{&modnorm_ctrl_wminus},   .nim = 1, .descr = "decrease pane size by one"}},
+	{WK_C_w WK_BAR,    {{&modnorm_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
+	{WK_C_w WK_USCORE, {{&modnorm_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
 	{WK_C_y,           {{&cmd_k},     .descr = "scroll one line up"}},
 	{WK_SLASH,         {{&cmd_slash}, .descr = "search forward"}},
 	{WK_QM,            {{&cmd_qmark}, .descr = "search backward"}},
@@ -252,6 +271,7 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_ALT WK_GT,     {{&cmd_G}, .descr = "scroll to the end"}},
 	{WK_ALT WK_SPACE,  {{&cmd_meta_space}, .descr = "scroll down one window"}},
 	{WK_PERCENT,       {{&cmd_percent},    .descr = "to [count]% position"}},
+	{WK_A,             {{&cmd_A}, .descr = "switch to previous viewer alternative"}},
 	{WK_F,             {{&cmd_F}, .descr = "toggle automatic forward scroll"}},
 	{WK_G,             {{&cmd_G}, .descr = "scroll to the end"}},
 	{WK_N,             {{&cmd_N}, .descr = "go to previous search match"}},
@@ -259,18 +279,20 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_R,             {{&cmd_R}, .descr = "reload view contents"}},
 	{WK_Z WK_Q,        {{&cmd_q}, .descr = "leave view mode"}},
 	{WK_Z WK_Z,        {{&cmd_q}, .descr = "leave view mode"}},
+	{WK_a,             {{&cmd_a}, .descr = "switch to next viewer alternative"}},
 	{WK_b,             {{&cmd_b}, .descr = "scroll page up"}},
 	{WK_d,             {{&cmd_d}, .descr = "scroll half-page down"}},
 	{WK_f,             {{&cmd_f}, .descr = "scroll page down"}},
 	{WK_g,             {{&cmd_g}, .descr = "scroll to the beginning"}},
 	{WK_e,             {{&cmd_j}, .descr = "scroll one line down"}},
+	{WK_i,             {{&cmd_i}, .descr = "toggle ignoring viewers (raw mode)"}},
 	{WK_j,             {{&cmd_j}, .descr = "scroll one line down"}},
 	{WK_k,             {{&cmd_k}, .descr = "scroll one line up"}},
 	{WK_n,             {{&cmd_n}, .descr = "go to next search match"}},
 	{WK_p,             {{&cmd_percent}, .descr = "to [count]% position"}},
 	{WK_q,             {{&cmd_q},       .descr = "leave view mode"}},
 	{WK_r,             {{&cmd_ctrl_l},  .descr = "redraw"}},
-	{WK_u,             {{&cmd_u},       .descr = "undo file operation"}},
+	{WK_u,             {{&cmd_u},       .descr = "scroll half-page down"}},
 	{WK_v,             {{&cmd_v},       .descr = "edit file at current line"}},
 	{WK_y,             {{&cmd_k},       .descr = "scroll one line up"}},
 	{WK_w,             {{&cmd_w},       .descr = "scroll backward one window"}},
@@ -278,7 +300,7 @@ static keys_add_info_t builtin_cmds[] = {
 #ifndef __PDCURSES__
 	{WK_ALT WK_v,      {{&cmd_b}, .descr = "scroll page up"}},
 #else
-	{{ALT_V},          {{&cmd_b}, .descr = "scroll page up"}},
+	{{K(ALT_V)},       {{&cmd_b}, .descr = "scroll page up"}},
 #endif
 #ifdef ENABLE_EXTENDED_KEYS
 	{{WC_C_w, K(KEY_BACKSPACE)}, {{&cmd_ctrl_wh}, .descr = "go to left window"}},
@@ -295,7 +317,7 @@ static keys_add_info_t builtin_cmds[] = {
 };
 
 void
-view_init_mode(void)
+modview_init(void)
 {
 	int ret_code;
 
@@ -306,7 +328,7 @@ view_init_mode(void)
 }
 
 void
-view_enter_mode(view_t *view, int explore)
+modview_enter(view_t *view, int explore)
 {
 	char full_path[PATH_MAX + 1];
 
@@ -325,12 +347,14 @@ view_enter_mode(view_t *view, int explore)
 
 	pick_vi(explore);
 
+	vi->filename = is_dir(full_path) ? format_str("%s/", full_path)
+	                                 : strdup(full_path);
+	vi->viewers = ft_get_viewers(vi->filename);
 	vi->view = view;
-	vi->filename = strdup(full_path);
 
 	if(load_view_data(vi, "File exploring", full_path, NOSILENT) != 0)
 	{
-		update_string(&vi->filename, NULL);
+		reset_view_info(vi);
 		return;
 	}
 
@@ -347,11 +371,11 @@ view_enter_mode(view_t *view, int explore)
 	}
 
 	ui_views_update_titles();
-	view_redraw();
+	modview_redraw();
 }
 
 void
-view_detached_make(view_t *view, const char cmd[])
+modview_detached_make(view_t *view, const char cmd[], MacroFlags flags)
 {
 	char full_path[PATH_MAX + 1];
 
@@ -365,7 +389,8 @@ view_detached_make(view_t *view, const char cmd[])
 	reset_view_info(vi);
 
 	vi->view = view;
-	vi->viewer = strdup(cmd);
+	vi->ext_viewer = strdup(cmd);
+	vi->flags = flags;
 	vi->filename = strdup(full_path);
 	vi->detached = 1;
 
@@ -376,7 +401,7 @@ view_detached_make(view_t *view, const char cmd[])
 	}
 
 	ui_views_update_titles();
-	view_redraw();
+	modview_redraw();
 }
 
 /* Either makes use of detached view or prunes it.  Returns zero on success,
@@ -408,7 +433,7 @@ try_resurrect_detached(const char full_path[], int explore)
 }
 
 void
-view_try_activate_mode(void)
+modview_try_activate(void)
 {
 	if(curr_view->explore_mode)
 	{
@@ -418,36 +443,42 @@ view_try_activate_mode(void)
 }
 
 void
-view_pre(void)
+modview_pre(void)
 {
 	if(curr_stats.save_msg == 0)
 	{
-		const char *const suffix = vi->auto_forward ? "(auto forwarding)" : "";
-		ui_sb_msgf("-- VIEW -- %s", suffix);
+		const char *raw_suffix = (vi->raw ? " (raw)" : "");
+		const char *fwd_suffix = (vi->auto_forward ? " (auto forwarding)" : "");
+		ui_sb_msgf("-- VIEW --%s%s", raw_suffix, fwd_suffix);
 		curr_stats.save_msg = 2;
 	}
 }
 
 void
-view_post(void)
+modview_post(void)
 {
-	update_screen(curr_stats.need_update);
-	view_ruler_update();
+	update_screen(stats_update_fetch());
+	modview_ruler_update();
 }
 
 void
-view_ruler_update(void)
+modview_ruler_update(void)
 {
-	char buf[POS_WIN_MIN_WIDTH + 1];
-	snprintf(buf, sizeof(buf), "%d-%d ", vi->line + 1, vi->nlines);
+	char rel_pos[32];
+	format_position(rel_pos, sizeof(rel_pos), vi->line, vi->nlines,
+			vi->view->window_rows);
+
+	char buf[64];
+	int curr_line = vi->line + (vi->nlines > 0 ? 1 : 0);
+	snprintf(buf, sizeof(buf), "%d-%d %s", curr_line, vi->nlines, rel_pos);
 
 	ui_ruler_set(buf);
 }
 
 void
-view_redraw(void)
+modview_redraw(void)
 {
-	view_info_t *saved_vi = vi;
+	modview_info_t *saved_vi = vi;
 
 	try_redraw_explore_view(&lwin);
 	try_redraw_explore_view(&rwin);
@@ -473,13 +504,11 @@ try_redraw_explore_view(const view_t *view)
 }
 
 void
-view_leave_mode(void)
+modview_leave(void)
 {
 	if(vi->kind != VK_TEXTUAL && vi->view->explore_mode)
 	{
-		const char *cmd = qv_get_viewer(vi->filename);
-		cmd = (cmd != NULL) ? ma_get_clear_cmd(cmd) : NULL;
-		qv_cleanup(vi->view, cmd);
+		cleanup(vi);
 	}
 
 	reset_view_info(vi);
@@ -500,12 +529,12 @@ view_leave_mode(void)
 
 	if(curr_view->explore_mode || other_view->explore_mode)
 	{
-		view_redraw();
+		modview_redraw();
 	}
 }
 
 void
-view_quit_explore_mode(view_t *view)
+modview_quit_exploring(view_t *view)
 {
 	assert(!vle_mode_is(VIEW_MODE) && "Unexpected mode.");
 	if(!view->explore_mode)
@@ -521,17 +550,17 @@ view_quit_explore_mode(view_t *view)
 	ui_view_title_update(view);
 }
 
-/* Frees and initializes anew view_info_t structure instance. */
+/* Frees and initializes anew modview_info_t structure instance. */
 static void
-reset_view_info(view_info_t *vi)
+reset_view_info(modview_info_t *vi)
 {
 	free_view_info(vi);
 	init_view_info(vi);
 }
 
-/* Initializes view_info_t structure instance with safe default values. */
+/* Initializes modview_info_t structure instance with safe default values. */
 static void
-init_view_info(view_info_t *vi)
+init_view_info(modview_info_t *vi)
 {
 	memset(vi, '\0', sizeof(*vi));
 	vi->win_size = -1;
@@ -539,25 +568,20 @@ init_view_info(view_info_t *vi)
 	vi->width = -1;
 	vi->last_search_backward = -1;
 	vi->search_repeat = NO_COUNT_GIVEN;
-	vi->nlines = 0;
-	vi->lines = NULL;
-	vi->widths = NULL;
-	vi->filename = NULL;
-	vi->viewer = NULL;
 }
 
-/* Frees all resources allocated by view_info_t structure instance. */
+/* Frees all resources allocated by modview_info_t structure instance. */
 static void
-free_view_info(view_info_t *vi)
+free_view_info(modview_info_t *vi)
 {
-	free_string_array(vi->lines, vi->nlines);
+	free_string_array(vi->viewers.items, vi->viewers.nitems);
 	free(vi->widths);
 	if(vi->last_search_backward != -1)
 	{
 		regfree(&vi->re);
 	}
 	free(vi->filename);
-	free(vi->viewer);
+	free(vi->ext_viewer);
 }
 
 /* Updates line width and redraws the view. */
@@ -594,7 +618,7 @@ calc_vlines(void)
 
 /* Recalculates virtual lines of a view with line wrapping. */
 static void
-calc_vlines_wrapped(view_info_t *vi)
+calc_vlines_wrapped(modview_info_t *vi)
 {
 	int i;
 	vi->nlinesv = 0;
@@ -609,7 +633,7 @@ calc_vlines_wrapped(view_info_t *vi)
 
 /* Recalculates virtual lines of a view without line wrapping. */
 static void
-calc_vlines_non_wrapped(view_info_t *vi)
+calc_vlines_non_wrapped(modview_info_t *vi)
 {
 	int i;
 	vi->nlinesv = vi->nlines;
@@ -624,7 +648,6 @@ static void
 draw(void)
 {
 	int l, vl;
-	const col_scheme_t *cs = ui_view_get_cs(vi->view);
 	const int height = ui_qv_height(vi->view);
 	const int width = ui_qv_width(vi->view);
 	const int max_l = MIN(vi->line + height, vi->nlines);
@@ -633,11 +656,8 @@ draw(void)
 
 	if(vi->kind != VK_TEXTUAL)
 	{
-		const char *cmd = qv_get_viewer(vi->filename);
-		cmd = (cmd != NULL) ? ma_get_clear_cmd(cmd) : NULL;
-		qv_cleanup(vi->view, cmd);
+		cleanup(vi);
 
-		free_string_array(vi->lines, vi->nlines);
 		(void)get_view_data(vi, vi->filename);
 
 		if(vi->kind == VK_PASS_THROUGH)
@@ -653,9 +673,9 @@ draw(void)
 		 * previewer that handles both textual and graphical previews. */
 	}
 
-	esc_state_init(&state, &cs->color[WIN_COLOR], COLORS);
+	esc_state_init(&state, &cfg.cs.color[WIN_COLOR], COLORS);
 
-	ui_view_erase(vi->view);
+	ui_view_erase(vi->view, 1);
 	ui_drop_attr(vi->view->win);
 
 	for(vl = 0, l = vi->line; l < max_l && vl < height; ++l)
@@ -686,7 +706,7 @@ draw(void)
 }
 
 int
-view_find_pattern(const char pattern[], int backward)
+modview_find(const char pattern[], int backward)
 {
 	int err;
 
@@ -714,7 +734,7 @@ view_find_pattern(const char pattern[], int backward)
 static void
 cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 {
-	view_redraw();
+	modview_redraw();
 }
 
 static void
@@ -750,7 +770,7 @@ get_active_view(void)
 }
 
 void
-view_panes_swapped(void)
+modview_panes_swapped(void)
 {
 	if(curr_stats.preview.explore != NULL)
 	{
@@ -831,7 +851,7 @@ static void
 cmd_ctrl_ws(key_info_t key_info, keys_info_t *keys_info)
 {
 	split_view(HSPLIT);
-	view_redraw();
+	modview_redraw();
 }
 
 static void
@@ -864,7 +884,7 @@ static void
 cmd_ctrl_wv(key_info_t key_info, keys_info_t *keys_info)
 {
 	split_view(VSPLIT);
-	view_redraw();
+	modview_redraw();
 }
 
 static void
@@ -901,7 +921,7 @@ cmd_ctrl_wx(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_wz(key_info_t key_info, keys_info_t *keys_info)
 {
-	view_leave_mode();
+	modview_leave();
 	qv_hide();
 }
 
@@ -916,6 +936,11 @@ cmd_meta_space(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_percent(key_info_t key_info, keys_info_t *keys_info)
 {
+	if(vi->nlines == 0)
+	{
+		return;
+	}
+
 	if(key_info.count == NO_COUNT_GIVEN)
 		key_info.count = 0;
 	if(key_info.count > 100)
@@ -933,7 +958,7 @@ cmd_tab(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(!curr_view->explore_mode)
 	{
-		view_leave_mode();
+		modview_leave();
 		return;
 	}
 
@@ -952,7 +977,8 @@ cmd_tab(key_info_t key_info, keys_info_t *keys_info)
 static void
 pick_vi(int explore)
 {
-	view_info_t **ptr = (explore ? &curr_view->vi : &curr_stats.preview.explore);
+	modview_info_t **ptr = explore ? &curr_view->vi
+	                               : &curr_stats.preview.explore;
 
 	if(*ptr == NULL)
 	{
@@ -966,14 +992,21 @@ static void
 cmd_slash(key_info_t key_info, keys_info_t *keys_info)
 {
 	vi->search_repeat = key_info.count;
-	enter_cmdline_mode(CLS_VWFSEARCH, "", NULL);
+	modcline_enter(CLS_VWFSEARCH, "", NULL);
 }
 
 static void
 cmd_qmark(key_info_t key_info, keys_info_t *keys_info)
 {
 	vi->search_repeat = key_info.count;
-	enter_cmdline_mode(CLS_VWBSEARCH, "", NULL);
+	modcline_enter(CLS_VWBSEARCH, "", NULL);
+}
+
+/* Switches to the previous viewer. */
+static void
+cmd_A(key_info_t key_info, keys_info_t *keys_info)
+{
+	switch_viewer(vi, -1);
 }
 
 /* Toggles automatic forwarding of file. */
@@ -1020,46 +1053,35 @@ cmd_R(key_info_t key_info, keys_info_t *keys_info)
 	reload_view(vi, NOSILENT);
 }
 
-/* Loads list of strings and related data into view_info_t structure from
+/* Loads list of strings and related data into modview_info_t structure from
  * specified file.  The action parameter is a title to be used for error
  * messages.  Returns non-zero on error, otherwise zero is returned. */
 static int
-load_view_data(view_info_t *vi, const char action[], const char file_to_view[],
-		int silent)
+load_view_data(modview_info_t *vi, const char action[],
+		const char file_to_view[], int silent)
 {
-	const int error = get_view_data(vi, file_to_view);
+	const char *error = get_view_data(vi, file_to_view);
 
-	if(error != 0 && silent)
+	if(error != NULL && silent)
 	{
 		return 1;
 	}
 
-	switch(error)
+	if(error != NULL)
 	{
-		case 0:
-			break;
-
-		case 2:
-			show_error_msg(action, "Can't open file for reading");
-			return 1;
-		case 3:
-			show_error_msg(action, "Can't get data from viewer");
-			return 1;
-		case 4:
-			show_error_msg(action, "Nothing to explore");
-			return 1;
-
-		default:
-			assert(0 && "Unhandled error code.");
-			return 1;
+		show_error_msg(action, error);
+		return 1;
 	}
 
-	if(vi->nlines != 0)
+	if(vi->nlines == 0)
+	{
+		vi->widths = NULL;
+	}
+	else
 	{
 		vi->widths = reallocarray(NULL, vi->nlines, sizeof(*vi->widths));
 		if(vi->widths == NULL)
 		{
-			free_string_array(vi->lines, vi->nlines);
 			vi->lines = NULL;
 			vi->nlines = 0;
 			show_error_msg(action, "Not enough memory");
@@ -1070,113 +1092,99 @@ load_view_data(view_info_t *vi, const char action[], const char file_to_view[],
 	return 0;
 }
 
-/* Reads data to be displayed handling error cases.  Returns zero on success, 2
- * on file reading error, 3 on issues with viewer or 4 on empty input. */
-static int
-get_view_data(view_info_t *vi, const char file_to_view[])
+/* Reads data to be displayed handling error cases.  Returns error message on
+ * failure or NULL on success. */
+static const char *
+get_view_data(modview_info_t *vi, const char file_to_view[])
 {
-	FILE *fp;
-	const char *const viewer = qv_get_viewer(file_to_view);
+	pick_current_viewer(vi);
 
-	if(vi->viewer == NULL && is_null_or_empty(viewer))
+	const ViewerKind kind = ft_viewer_kind(vi->curr_viewer);
+	view_t *const curr = curr_view;
+	curr_view = curr_stats.preview.on ? curr_view
+						: (vi->view != NULL) ? vi->view : curr_view;
+
+	const preview_area_t parea = {
+		.source = vi->view,
+		.view = vi->view,
+		.x = ui_qv_left(vi->view),
+		.y = ui_qv_top(vi->view),
+		.w = ui_qv_width(vi->view),
+		.h = ui_qv_height(vi->view),
+	};
+	curr_stats.preview_hint = &parea;
+
+	if(kind != VK_TEXTUAL)
 	{
-		if(is_dir(file_to_view))
-		{
-			ui_cancellation_reset();
-			ui_cancellation_enable();
-			fp = qv_view_dir(file_to_view);
-			ui_cancellation_disable();
-		}
-		else
-		{
-			fp = os_fopen(file_to_view, "rb");
-		}
+		/* Wait a bit to let terminal emulator do actual refresh (at least some of
+		 * them need this). */
+		usleep(cfg.graphics_delay);
+	}
 
-		if(fp == NULL)
-		{
-			return 2;
-		}
+	const char *error;
+	const char *viewer = (vi->raw ? NULL : vi->curr_viewer);
 
-		vi->lines = read_file_lines(fp, &vi->nlines);
+	strlist_t lines;
+	if(vi->curr_viewer == vi->ext_viewer)
+	{
+		/* No macros in this viewer. */
+		lines = vcache_lookup(file_to_view, vi->ext_viewer, vi->flags, kind,
+				INT_MAX, VC_SYNC, &error);
 	}
 	else
 	{
-		const char *const v = (vi->viewer != NULL) ? vi->viewer : viewer;
-		const ViewerKind kind = ft_viewer_kind(v);
-		view_t *const curr = curr_view;
-		curr_view = curr_stats.preview.on ? curr_view
-		          : (vi->view != NULL) ? vi->view : curr_view;
-
-		const preview_area_t parea = {
-			.source = vi->view,
-			.view = vi->view,
-			.x = ui_qv_left(vi->view),
-			.y = ui_qv_top(vi->view),
-			.w = ui_qv_width(vi->view),
-			.h = ui_qv_height(vi->view),
-		};
-		curr_stats.preview_hint = &parea;
-
-		if(kind != VK_TEXTUAL)
-		{
-			/* Wait a bit to let terminal emulator do actual refresh (at least some
-			 * of them need this). */
-			usleep(50000);
-		}
-		if(vi->viewer == NULL)
-		{
-			fp = qv_execute_viewer(viewer);
-		}
-		else
-		{
-			/* Don't add implicit %c to a command with %e macro. */
-			char *const cmd = ma_expand(vi->viewer, NULL, NULL, 1);
-			fp = read_cmd_output(cmd, 0);
-			free(cmd);
-		}
-
-		curr_view = curr;
-		curr_stats.preview_hint = NULL;
-
-		if(fp == NULL)
-		{
-			return 3;
-		}
-
-		vi->kind = kind;
-
-		ui_cancellation_reset();
-		ui_cancellation_enable();
-		vi->lines = read_stream_lines(fp, &vi->nlines, 0, NULL, NULL);
-		ui_cancellation_disable();
+		char *expanded = (viewer == NULL ? NULL : qv_expand_viewer(viewer));
+		lines = vcache_lookup(file_to_view, expanded, MF_NONE, kind, INT_MAX,
+				VC_SYNC, &error);
+		free(expanded);
 	}
 
-	fclose(fp);
+	curr_view = curr;
+	curr_stats.preview_hint = NULL;
 
-	if(vi->kind != VK_TEXTUAL && vi->nlines == 0)
-	{
-		/* Exploring absent output gives error, add an empty line to allow empty
-		 * output for graphical previewers. */
-		vi->nlines = add_to_string_array(&vi->lines, vi->nlines, 1, "");
-	}
-	if(vi->lines == NULL || vi->nlines == 0)
-	{
-		return 4;
-	}
+	vi->lines = lines.items;
+	vi->nlines = lines.nitems;
 
-	return 0;
+	vi->kind = kind;
+
+	return error;
 }
 
-/* Replaces view_info_t structure with another one preserving as much as
+/* Makes sure that vi->curr_viewer field has a sensible value. */
+static void
+pick_current_viewer(modview_info_t *vi)
+{
+	if(vi->curr_viewer != NULL)
+	{
+		return;
+	}
+
+	if(vi->ext_viewer != NULL)
+	{
+		vi->curr_viewer = vi->ext_viewer;
+	}
+	else if(vi->viewers.nitems != 0)
+	{
+		vi->curr_viewer = vi->viewers.items[0];
+	}
+}
+
+/* Replaces modview_info_t structure with another one preserving as much as
  * possible. */
 static void
-replace_vi(view_info_t *orig, view_info_t *new)
+replace_vi(modview_info_t *orig, modview_info_t *new)
 {
 	new->filename = orig->filename;
 	orig->filename = NULL;
 
-	new->viewer = orig->viewer;
-	orig->viewer = NULL;
+	new->viewers = orig->viewers;
+	orig->viewers.items = NULL;
+	orig->viewers.nitems = 0;
+
+	new->curr_viewer = orig->curr_viewer;
+
+	new->ext_viewer = orig->ext_viewer;
+	orig->ext_viewer = NULL;
 
 	if(orig->last_search_backward != -1)
 	{
@@ -1191,12 +1199,48 @@ replace_vi(view_info_t *orig, view_info_t *new)
 	new->linev = orig->linev;
 	new->view = orig->view;
 	new->auto_forward = orig->auto_forward;
-	filemon_assign(&new->file_mon, &orig->file_mon);
+	new->file_mon = orig->file_mon;
 
 	free_view_info(orig);
 	*orig = *new;
 }
 
+/* Switches to the next viewer. */
+static void
+cmd_a(key_info_t key_info, keys_info_t *keys_info)
+{
+	switch_viewer(vi, 1);
+}
+
+/* Switches to the next or previous viewer. */
+static void
+switch_viewer(modview_info_t *vi, int offset)
+{
+	assert((offset == 1 || offset == -1) && "Invalid offset.");
+
+	if(vi->raw || vi->viewers.nitems <= 1)
+	{
+		return;
+	}
+
+	int i;
+	for(i = 0; i < vi->viewers.nitems; ++i)
+	{
+		if(vi->viewers.items[i] == vi->curr_viewer)
+		{
+			break;
+		}
+	}
+	if(i == vi->viewers.nitems)
+	{
+		return;
+	}
+
+	i = (i + vi->viewers.nitems + offset)%vi->viewers.nitems;
+	vi->curr_viewer = vi->viewers.items[i];
+	reload_view(vi, NOSILENT);
+}
+
 static void
 cmd_b(key_info_t key_info, keys_info_t *keys_info)
 {
@@ -1237,13 +1281,27 @@ cmd_g(key_info_t key_info, keys_info_t *keys_info)
 	key_info.count = MIN(vi->nlinesv - ui_qv_height(vi->view), key_info.count);
 	key_info.count = MAX(1, key_info.count);
 
-	if(vi->linev == vi->widths[key_info.count - 1][0])
+	if(vi->nlines == 0 || vi->linev == vi->widths[key_info.count - 1][0])
+	{
 		return;
+	}
+
 	vi->line = key_info.count - 1;
 	vi->linev = vi->widths[vi->line][0];
 	draw();
 }
 
+/* Toggles raw mode (ignoring of defined viewers). */
+static void
+cmd_i(key_info_t key_info, keys_info_t *keys_info)
+{
+	if(vi->curr_viewer != vi->ext_viewer)
+	{
+		vi->raw = !vi->raw;
+		reload_view(vi, NOSILENT);
+	}
+}
+
 static void
 cmd_j(key_info_t key_info, keys_info_t *keys_info)
 {
@@ -1328,35 +1386,43 @@ search(int repeat_count, int backward)
 		repeat_count = 1;
 	}
 
-	while(repeat_count-- > 0 && curr_stats.save_msg == 0)
+	while(repeat_count-- > 0)
 	{
-		if(backward)
+		if(backward ? find_previous() : find_next())
 		{
-			find_previous(1);
-		}
-		else
-		{
-			find_next();
+			break;
 		}
 	}
 }
 
-static void
-find_previous(int vline_offset)
+/* Scrolls to the previous search match.  Returns zero on success and non-zero
+ * if pattern wasn't found.  Prints a message on search failure. */
+static int
+find_previous(void)
 {
-	int i;
-	int offset = 0;
-	char buf[ui_qv_width(vi->view)*4];
-	int vl, l;
+	if(vi->linev == 0)
+	{
+		draw();
+		display_error("Nothing to search");
+		return 1;
+	}
 
-	vl = vi->linev - vline_offset;
-	l = vi->line;
+	char buf[ui_qv_width(vi->view)*4];
+
+	int vl = vi->linev - 1;
+	int l = vi->line;
 
 	if(l > 0 && vl < vi->widths[l][0])
-		l--;
+	{
+		--l;
+	}
 
-	for(i = 0; i <= vl - vi->widths[l][0]; i++)
+	int i;
+	int offset = 0;
+	for(i = 0; l < vi->nlines && i <= vl - vi->widths[l][0]; ++i)
+	{
 		offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
+	}
 
 	/* Don't stop until we go above first virtual line of the first line. */
 	while(l >= 0 && vl >= 0)
@@ -1369,38 +1435,47 @@ find_previous(int vline_offset)
 		}
 		if(l > 0 && vl - 1 < vi->widths[l][0])
 		{
-			l--;
+			--l;
 			offset = 0;
 			for(i = 0; i <= vl - 1 - vi->widths[l][0]; i++)
 				offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
 		}
 		else
 			offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
-		vl--;
+		--vl;
 	}
+
 	draw();
-	if(vi->line != l)
+
+	if(vi->linev != vl || vi->nlines == 0)
 	{
 		display_error("Pattern not found");
+		return 1;
 	}
+	return 0;
 }
 
-static void
+/* Scrolls to the next search match.  Returns zero on success and non-zero if
+ * pattern wasn't found.  Prints a message on search failure. */
+static int
 find_next(void)
 {
-	int i;
-	int offset = 0;
 	char buf[ui_qv_width(vi->view)*4];
-	int vl, l;
 
-	vl = vi->linev + 1;
-	l = vi->line;
+	int vl = vi->linev + 1;
+	int l = vi->line;
 
 	if(l < vi->nlines - 1 && vl == vi->widths[l + 1][0])
-		l++;
+	{
+		++l;
+	}
 
-	for(i = 0; i <= vl - vi->widths[l][0]; i++)
+	int i;
+	int offset = 0;
+	for(i = 0; l < vi->nlines && i <= vl - vi->widths[l][0]; ++i)
+	{
 		offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
+	}
 
 	while(l < vi->nlines)
 	{
@@ -1415,17 +1490,21 @@ find_next(void)
 		{
 			if(l == vi->nlines - 1)
 				break;
-			l++;
+			++l;
 			offset = 0;
 		}
 		offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
-		vl++;
+		++vl;
 	}
+
 	draw();
-	if(vi->line != l)
+
+	if(vi->linev != vl || vi->nlines == 0)
 	{
 		display_error("Pattern not found");
+		return 1;
 	}
+	return 0;
 }
 
 /* Extracts part of the line replacing all occurrences of horizontal tabulation
@@ -1455,7 +1534,7 @@ display_error(const char error_msg[])
 static void
 cmd_q(key_info_t key_info, keys_info_t *keys_info)
 {
-	view_leave_mode();
+	modview_leave();
 }
 
 static void
@@ -1472,9 +1551,10 @@ update_with_half_win(key_info_t *key_info)
 {
 	if(key_info->count == NO_COUNT_GIVEN)
 	{
+		int height = ui_qv_height(vi->view);
 		key_info->count = (vi->half_win > 0)
 			? vi->half_win
-			: ui_qv_height(vi->view)/2;
+			: (height == 1 ? 1 : height/2);
 	}
 	else
 	{
@@ -1494,7 +1574,7 @@ cmd_v(key_info_t key_info, keys_info_t *keys_info)
 	/* In some cases two redraw operations are needed, otherwise TUI is not fully
 	 * redrawn. */
 	update_screen(UT_REDRAW);
-	stats_redraw_schedule();
+	stats_redraw_later();
 }
 
 static void
@@ -1530,13 +1610,19 @@ update_with_win(key_info_t *key_info)
 static void
 set_from_default_win(key_info_t *key_info)
 {
-	key_info->count = (vi->win_size > 0)
-		? vi->win_size
-		: (ui_qv_height(vi->view) - 1);
+	if(vi->win_size > 0)
+	{
+		key_info->count = vi->win_size;
+	}
+	else
+	{
+		int height = ui_qv_height(vi->view);
+		key_info->count = (height == 1 ? 1 : height - 1);
+	}
 }
 
 int
-view_detached_draw(void)
+modview_detached_draw(void)
 {
 	pick_vi(0);
 
@@ -1608,7 +1694,7 @@ get_file_to_explore(const view_t *view, char buf[], size_t buf_len)
 }
 
 void
-view_check_for_updates(void)
+modview_check_for_updates(void)
 {
 	int need_redraw = 0;
 
@@ -1618,14 +1704,14 @@ view_check_for_updates(void)
 
 	if(need_redraw)
 	{
-		stats_redraw_schedule();
+		stats_redraw_later();
 	}
 }
 
 /* Forwards the view if underlying file changed.  Returns non-zero if reload
  * occurred, otherwise zero is returned. */
 static int
-forward_if_changed(view_info_t *vi)
+forward_if_changed(modview_info_t *vi)
 {
 	filemon_t mon;
 
@@ -1644,7 +1730,7 @@ forward_if_changed(view_info_t *vi)
 		return 0;
 	}
 
-	filemon_assign(&vi->file_mon, &mon);
+	vi->file_mon = mon;
 	reload_view(vi, SILENT);
 	return scroll_to_bottom(vi);
 }
@@ -1652,7 +1738,7 @@ forward_if_changed(view_info_t *vi)
 /* Scrolls view to the bottom if there is any room for that.  Returns non-zero
  * if position was changed, otherwise zero is returned. */
 static int
-scroll_to_bottom(view_info_t *vi)
+scroll_to_bottom(modview_info_t *vi)
 {
 	if(vi->linev + 1 + ui_qv_height(vi->view) > vi->nlinesv)
 	{
@@ -1674,40 +1760,65 @@ scroll_to_bottom(view_info_t *vi)
 /* Reloads contents of the specified view by rerunning corresponding viewer or
  * just rereading a file. */
 static void
-reload_view(view_info_t *vi, int silent)
+reload_view(modview_info_t *vi, int silent)
 {
-	view_info_t new_vi;
+	modview_info_t new_vi;
 
 	init_view_info(&new_vi);
 	/* These fields are used in get_view_data(). */
 	new_vi.view = vi->view;
-	new_vi.viewer = vi->viewer;
+	new_vi.curr_viewer = vi->curr_viewer;
+	new_vi.ext_viewer = vi->ext_viewer;
+	new_vi.viewers = vi->viewers;
+	new_vi.raw = vi->raw;
 
 	if(load_view_data(&new_vi, "File exploring reload", vi->filename, silent)
 			== 0)
 	{
 		replace_vi(vi, &new_vi);
-		view_redraw();
+		modview_redraw();
 	}
 }
 
-const char *
-view_detached_get_viewer(void)
+void
+modview_hide_graphics(void)
 {
-	return (vi == NULL ? NULL : vi->viewer);
+	if(lwin.vi != NULL && lwin.vi->kind != VK_TEXTUAL)
+	{
+		cleanup(lwin.vi);
+	}
+	if(rwin.vi != NULL && rwin.vi->kind != VK_TEXTUAL)
+	{
+		cleanup(rwin.vi);
+	}
+}
+
+/* Cleans view area, possibly using an external application. */
+static void
+cleanup(modview_info_t *vi)
+{
+	const char *viewer = vi->curr_viewer;
+	const char *cmd = (viewer != NULL ? ma_get_clear_cmd(viewer) : NULL);
+	qv_cleanup(vi->view, cmd);
+}
+
+const char *
+modview_detached_get_viewer(void)
+{
+	return (vi == NULL ? NULL : vi->ext_viewer);
 }
 
 /* Allocates and initializes view mode information.  Returns pointer to it. */
-static view_info_t *
+static modview_info_t *
 view_info_alloc(void)
 {
-	view_info_t *const vi = malloc(sizeof(*vi));
+	modview_info_t *const vi = malloc(sizeof(*vi));
 	init_view_info(vi);
 	return vi;
 }
 
 void
-view_info_free(view_info_t *info)
+modview_info_free(modview_info_t *info)
 {
 	if(info != NULL)
 	{
@@ -1720,5 +1831,30 @@ view_info_free(view_info_t *info)
 	}
 }
 
+TSTATIC int
+modview_is_raw(modview_info_t *vi)
+{
+	return vi->raw;
+}
+
+TSTATIC const char *
+modview_current_viewer(modview_info_t *vi)
+{
+	return vi->curr_viewer;
+}
+
+TSTATIC int
+modview_current_line(modview_info_t *vi)
+{
+	return vi->line;
+}
+
+TSTATIC strlist_t
+modview_lines(modview_info_t *vi)
+{
+	strlist_t lines = { .items = vi->lines, .nitems = vi->nlines };
+	return lines;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
