@@ -19,6 +19,9 @@
 #ifndef VIFM__MODES__VIEW_H__
 #define VIFM__MODES__VIEW_H__
 
+#include "../utils/test_helpers.h"
+#include "../macros.h"
+
 struct view_t;
 
 /* Holds state of a single view mode window. */
@@ -74,8 +77,10 @@ void modview_hide_graphics(void);
  * maintained. */
 
 /* Creates detached view in the view using output of passed command as a
- * source, but doesn't enter the view mode. */
-void modview_detached_make(struct view_t *view, const char cmd[]);
+ * source, but doesn't enter the view mode.  The command is assumed to not have
+ * any unexpanded macros. */
+void modview_detached_make(struct view_t *view, const char cmd[],
+		MacroFlags flags);
 
 /* Tries to draw an detached view mode and updates internal state if needed.
  * Returns non-zero on success, otherwise zero is returned. */
@@ -89,6 +94,14 @@ const char * modview_detached_get_viewer(void);
 /* Frees view info.  The parameter can be NULL. */
 void modview_info_free(modview_info_t *vi);
 
+TSTATIC_DEFS(
+	int modview_is_raw(modview_info_t *vi);
+	const char * modview_current_viewer(modview_info_t *vi);
+	int modview_current_line(modview_info_t *vi);
+	struct strlist_t;
+	struct strlist_t modview_lines(modview_info_t *vi);
+)
+
 #endif /* VIFM__MODES__VIEW_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
