@@ -246,11 +246,11 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_u,          {{&cmd_gu}, .descr = "convert to lowercase"}},
 	{WK_v,          {{&cmd_v},  .descr = "leave/switch to regular visual mode"}},
 	{WK_y,          {{&cmd_y},  .descr = "yank files"}},
-	{WK_z WK_b,     {{&normal_cmd_zb}, .descr = "push cursor to the bottom"}},
+	{WK_z WK_b,     {{&modnorm_zb}, .descr = "push cursor to the bottom"}},
 	{WK_z WK_d,     {{&cmd_zd}, .descr = "exclude custom view entry"}},
 	{WK_z WK_f,     {{&cmd_zf}, .descr = "add selection to filter"}},
-	{WK_z WK_t,     {{&normal_cmd_zt},   .descr = "push cursor to the top"}},
-	{WK_z WK_z,     {{&normal_cmd_zz},   .descr = "center cursor position"}},
+	{WK_z WK_t,     {{&modnorm_zt},      .descr = "push cursor to the top"}},
+	{WK_z WK_z,     {{&modnorm_zz},      .descr = "center cursor position"}},
 	{WK_LP,         {{&cmd_left_paren},  .descr = "go to previous group of files"}},
 	{WK_RP,         {{&cmd_right_paren}, .descr = "go to next group of files"}},
 	{WK_z WK_k,     {{&cmd_z_k},  .descr = "go to previous sibling dir"}},
@@ -275,7 +275,7 @@ static keys_add_info_t builtin_cmds[] = {
 };
 
 void
-init_visual_mode(void)
+modvis_init(void)
 {
 	int ret_code;
 
@@ -286,10 +286,10 @@ init_visual_mode(void)
 }
 
 void
-enter_visual_mode(VisualSubmodes sub_mode)
+modvis_enter(VisualSubmodes sub_mode)
 {
-	const int ub = check_mark_directory(curr_view, '<');
-	const int lb = check_mark_directory(curr_view, '>');
+	const int ub = marks_find_in_view(curr_view, '<');
+	const int lb = marks_find_in_view(curr_view, '>');
 
 	if(sub_mode == VS_RESTORE && (ub < 0 || lb < 0))
 	{
@@ -325,11 +325,11 @@ enter_visual_mode(VisualSubmodes sub_mode)
 }
 
 void
-leave_visual_mode(int save_msg, int goto_top, int clear_selection)
+modvis_leave(int save_msg, int goto_top, int clear_selection)
 {
 	if(goto_top)
 	{
-		int ub = check_mark_directory(view, '<');
+		int ub = marks_find_in_view(view, '<');
 		if(ub != -1)
 			view->list_pos = ub;
 	}
@@ -496,9 +496,8 @@ cmd_ctrl_x(key_info_t key_info, keys_info_t *keys_info)
 static void
 call_incdec(int count)
 {
-	int save_msg;
-	check_marking(view, 0, NULL);
-	save_msg = fops_incdec(view, count);
+	flist_set_marking(view, 0);
+	int save_msg = fops_incdec(view, count);
 	accept_and_leave(save_msg);
 }
 
@@ -518,9 +517,8 @@ cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_C(key_info_t key_info, keys_info_t *keys_info)
 {
-	int save_msg;
-	check_marking(view, 0, NULL);
-	save_msg = fops_clone(view, NULL, 0, 0, def_count(key_info.count));
+	flist_set_marking(view, 0);
+	int save_msg = fops_clone(view, NULL, 0, 0, def_count(key_info.count));
 	accept_and_leave(save_msg);
 }
 
@@ -589,7 +587,7 @@ cmd_O(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_quote(key_info_t key_info, keys_info_t *keys_info)
 {
-	const int pos = check_mark_directory(view, key_info.multi);
+	const int pos = marks_find_in_view(view, key_info.multi);
 	if(pos >= 0)
 	{
 		goto_pos(pos);
@@ -602,7 +600,7 @@ sug_quote(vle_keys_list_cb cb)
 {
 	if(cfg.sug.flags & SF_MARKS)
 	{
-		suggest_marks(cb, 1);
+		marks_suggest(view, cb, 1);
 	}
 }
 
@@ -657,8 +655,8 @@ static void
 cmd_colon(key_info_t key_info, keys_info_t *keys_info)
 {
 	update_marks(view);
-	set_count_vars(key_info.count);
-	enter_cmdline_mode(CLS_COMMAND, "", NULL);
+	cmds_vars_set_count(key_info.count);
+	modcline_enter(CLS_COMMAND, "", NULL);
 }
 
 /* Continues navigation to word which starts with specified character in initial
@@ -712,7 +710,7 @@ cmd_d(key_info_t key_info, keys_info_t *keys_info)
 static void
 delete(key_info_t key_info, int use_trash)
 {
-	check_marking(view, 0, NULL);
+	flist_set_marking(view, 0);
 	if(fops_delete(view, def_reg(key_info.reg), use_trash))
 	{
 		accept_and_leave(1);
@@ -753,13 +751,13 @@ cmd_cp(key_info_t key_info, keys_info_t *keys_info)
 
 	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
 	update_marks(view);
-	ub = check_mark_directory(view, '<');
+	ub = marks_find_in_view(view, '<');
 	if(ub != -1)
 	{
 		view->list_pos = ub;
 	}
 
-	normal_cmd_cp(view, key_info);
+	modnorm_cp(view, key_info);
 }
 
 /* Renames selected files of the view. */
@@ -767,9 +765,9 @@ static void
 cmd_cw(key_info_t key_info, keys_info_t *keys_info)
 {
 	update_marks(view);
-	leave_visual_mode(0, 1, 0);
+	modvis_leave(0, 1, 0);
 
-	check_marking(view, 0, NULL);
+	flist_set_marking(view, 0);
 	(void)fops_rename(view, NULL, 0, 0);
 }
 
@@ -783,8 +781,8 @@ static void
 cmd_gl(key_info_t key_info, keys_info_t *keys_info)
 {
 	update_marks(view);
-	leave_visual_mode(curr_stats.save_msg, 1, 0);
-	open_file(view, FHE_RUN);
+	modvis_leave(curr_stats.save_msg, 1, 0);
+	rn_open(view, FHE_RUN);
 	flist_sel_stash(view);
 	redraw_view(view);
 }
@@ -807,10 +805,8 @@ cmd_gu(key_info_t key_info, keys_info_t *keys_info)
 static void
 do_gu(int upper)
 {
-	int save_msg;
-
-	check_marking(view, 0, NULL);
-	save_msg = fops_case(view, upper);
+	flist_set_marking(view, 0);
+	int save_msg = fops_case(view, upper);
 	accept_and_leave(save_msg);
 }
 
@@ -826,8 +822,8 @@ cmd_gv(key_info_t key_info, keys_info_t *keys_info)
 static void
 restore_previous_selection(void)
 {
-	int ub = check_mark_directory(view, '<');
-	int lb = check_mark_directory(view, '>');
+	int ub = marks_find_in_view(view, '<');
+	int lb = marks_find_in_view(view, '>');
 
 	if(ub < 0 || lb < 0)
 		return;
@@ -889,7 +885,7 @@ cmd_h(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_i(key_info_t key_info, keys_info_t *keys_info)
 {
-	open_file(view, FHE_NO_RUN);
+	rn_open(view, FHE_NO_RUN);
 	accept_and_leave(curr_stats.save_msg);
 }
 
@@ -950,7 +946,7 @@ cmd_m(key_info_t key_info, keys_info_t *keys_info)
 	const dir_entry_t *const curr = get_current_entry(view);
 	if(!fentry_is_fake(curr))
 	{
-		set_user_mark(key_info.multi, curr->origin, curr->name);
+		marks_set_user(view, key_info.multi, curr->origin, curr->name);
 	}
 }
 
@@ -975,7 +971,7 @@ search(key_info_t key_info, int backward, int interactive)
 	if(view->matches == 0)
 	{
 		const char *const pattern = hists_search_last();
-		curr_stats.save_msg = find_vpattern(view, pattern, backward, interactive);
+		curr_stats.save_msg = modvis_find(view, pattern, backward, interactive);
 		return;
 	}
 
@@ -1034,7 +1030,7 @@ activate_search(int count, int back, int external)
 	else
 	{
 		const CmdLineSubmode submode = back ? CLS_VBSEARCH : CLS_VFSEARCH;
-		enter_cmdline_mode(submode, "", NULL);
+		modcline_enter(submode, "", NULL);
 	}
 }
 
@@ -1080,11 +1076,8 @@ change_amend_type(AmendType new_amend_type)
 static void
 cmd_y(key_info_t key_info, keys_info_t *keys_info)
 {
-	int save_msg;
-
-	check_marking(view, 0, NULL);
-	save_msg = fops_yank(view, def_reg(key_info.reg));
-
+	flist_set_marking(view, 0);
+	int save_msg = fops_yank(view, def_reg(key_info.reg));
 	accept_and_leave(save_msg);
 }
 
@@ -1115,7 +1108,7 @@ static void
 leave_clearing_selection(int go_to_top, int save_msg)
 {
 	update_marks(view);
-	leave_visual_mode(save_msg, go_to_top, 1);
+	modvis_leave(save_msg, go_to_top, 1);
 }
 
 static void
@@ -1148,8 +1141,8 @@ update_marks(view_t *view)
 		end_entry -= delta;
 	}
 
-	set_spec_mark(start_mark, start_entry->origin, start_entry->name);
-	set_spec_mark(end_mark, end_entry->origin, end_entry->name);
+	marks_set_special(view, start_mark, start_entry->origin, start_entry->name);
+	marks_set_special(view, end_mark, end_entry->origin, end_entry->name);
 }
 
 /* Excludes entries from custom view. */
@@ -1160,10 +1153,11 @@ cmd_zd(key_info_t key_info, keys_info_t *keys_info)
 	accept_and_leave(0);
 }
 
+/* Filters selected files. */
 static void
 cmd_zf(key_info_t key_info, keys_info_t *keys_info)
 {
-	name_filters_add_selection(view);
+	name_filters_add_active(view);
 	accept_and_leave(0);
 }
 
@@ -1421,7 +1415,7 @@ revert_selection(int pos)
 }
 
 void
-update_visual_mode(void)
+modvis_update(void)
 {
 	const int pos = view->list_pos;
 
@@ -1436,8 +1430,7 @@ update_visual_mode(void)
 }
 
 int
-find_vpattern(view_t *view, const char pattern[], int backward,
-		int print_errors)
+modvis_find(view_t *view, const char pattern[], int backward, int print_errors)
 {
 	int i;
 	int result;
@@ -1525,7 +1518,7 @@ move_pos(int pos)
 }
 
 const char *
-describe_visual_mode(void)
+modvis_describe(void)
 {
 	static const char *descriptions[] = {
 		[AT_NONE]   = "VISUAL",
