@@ -361,7 +361,15 @@ op_removesl(ops_t *ops, void *data, const char *src, const char *dst)
 
 			/* SHFileOperationW requires pFrom to be double-nul terminated. */
 			const size_t len = wcslen(utf16_path);
-			utf16_path = reallocarray(utf16_path, len + 1U + 1U, sizeof(*utf16_path));
+			wchar_t *new_utf16_path = reallocarray(utf16_path, len + 1U + 1U,
+					sizeof(*utf16_path));
+			if(new_utf16_path == NULL)
+			{
+				free(utf16_path);
+				return -1;
+			}
+
+			utf16_path = new_utf16_path;
 			utf16_path[len + 1U] = L'\0';
 
 			SHFILEOPSTRUCTW fo = {
@@ -759,12 +767,15 @@ op_symlink(ops_t *ops, void *data, const char *src, const char *dst)
 		char *escaped_src, *escaped_dst;
 		char cmd[6 + PATH_MAX*2 + 1];
 		int result;
-#ifdef _WIN32
-		char exe_dir[PATH_MAX + 2];
-#endif
 
+#ifndef _WIN32
 		escaped_src = shell_like_escape(src, 0);
 		escaped_dst = shell_like_escape(dst, 0);
+#else
+		escaped_src = strdup(enclose_in_dquotes(src));
+		escaped_dst = strdup(enclose_in_dquotes(dst));
+#endif
+
 		if(escaped_src == NULL || escaped_dst == NULL)
 		{
 			free(escaped_dst);
@@ -777,6 +788,7 @@ op_symlink(ops_t *ops, void *data, const char *src, const char *dst)
 		LOG_INFO_MSG("Running ln command: \"%s\"", cmd);
 		result = run_operation_command(ops, cmd, 1);
 #else
+		char exe_dir[PATH_MAX + 2];
 		if(get_exe_dir(exe_dir, ARRAY_LEN(exe_dir)) != 0)
 		{
 			free(escaped_dst);
@@ -962,8 +974,8 @@ exec_io_op(ops_t *ops, int (*func)(io_args_t *), io_args_t *args,
 		}
 		else
 		{
-			/* ui_cancellation_reset() should be called outside this unit to allow
-			 * bulking several operations together. */
+			/* ui_cancellation_push_off() should be called outside of this unit to
+			 * allow bulking several operations together. */
 			ui_cancellation_enable();
 			args->cancellation.hook = &ui_cancellation_hook;
 		}
@@ -1124,9 +1136,9 @@ prompt_user(const io_args_t *args, const char title[], const char msg[],
 {
 	/* Active cancellation conflicts with input processing by putting terminal in
 	 * a cooked mode. */
-	const int cancellation_state = ui_cancellation_pause();
+	ui_cancellation_push_off();
 	const char response = prompt_msg_custom(title, msg, variants);
-	ui_cancellation_resume(cancellation_state);
+	ui_cancellation_pop();
 
 	return response;
 }
@@ -1162,7 +1174,7 @@ run_operation_command(ops_t *ops, char cmd[], int cancellable)
 	else
 	{
 		int result;
-		/* ui_cancellation_reset() should be called outside this unit to allow
+		/* ui_cancellation_push_off() should be called outside this unit to allow
 		 * bulking several operations together. */
 		ui_cancellation_enable();
 		result = bg_and_wait_for_errors(cmd, &ui_cancellation_info);
