@@ -34,6 +34,7 @@
 #include "engine/options.h"
 #include "engine/text_buffer.h"
 #include "int/term_title.h"
+#include "lua/vlua.h"
 #include "ui/fileview.h"
 #include "ui/quickview.h"
 #include "ui/statusbar.h"
@@ -83,7 +84,6 @@ optinit_t;
 
 static void uni_handler(const char name[], optval_t val, OPT_SCOPE scope);
 static void init_classify(optval_t *val);
-static char * double_commas(const char str[]);
 static void init_cpoptions(optval_t *val);
 static void init_dirsize(optval_t *val);
 static const char * to_endpoint(int i, char buffer[]);
@@ -93,6 +93,7 @@ static void init_lsoptions(optval_t *val);
 static void init_lsview(optval_t *val);
 static void init_milleroptions(optval_t *val);
 static void init_millerview(optval_t *val);
+static void init_previewoptions(optval_t *val);
 static void init_quickview(optval_t *val);
 static void init_shortmess(optval_t *val);
 static void init_sizefmt(optval_t *val);
@@ -106,7 +107,7 @@ static void init_tuioptions(optval_t *val);
 static void init_wordchars(optval_t *val);
 static void load_options_defaults(void);
 static void add_options(void);
-static void load_sort_option_inner(view_t *view, char sort_keys[]);
+static void load_sort_option_inner(view_t *view, signed char sort_keys[]);
 static void aproposprg_handler(OPT_OP op, optval_t val);
 static void autochpos_handler(OPT_OP op, optval_t val);
 static void caseoptions_handler(OPT_OP op, optval_t val);
@@ -148,6 +149,7 @@ static void mediaprg_handler(OPT_OP op, optval_t val);
 #endif
 static void mintimeoutlen_handler(OPT_OP op, optval_t val);
 static void scroll_line_down(view_t *view);
+static void previewoptions_handler(OPT_OP op, optval_t val);
 static void quickview_handler(OPT_OP op, optval_t val);
 static void rulerformat_handler(OPT_OP op, optval_t val);
 static void runexec_handler(OPT_OP op, optval_t val);
@@ -192,14 +194,14 @@ static void update_num_type(view_t *view, NumberingType *num_type,
 		NumberingType type, int enable);
 static void sort_global(OPT_OP op, optval_t val);
 static void sort_local(OPT_OP op, optval_t val);
-static void set_sort(view_t *view, char sort_keys[], char order[]);
+static void set_sort(view_t *view, signed char sort_keys[], char order[]);
 static void sortgroups_global(OPT_OP op, optval_t val);
 static void sortgroups_local(OPT_OP op, optval_t val);
 static void set_sortgroups(view_t *view, char **opt, char value[]);
 static void sorting_changed(view_t *view, int defer_slow);
 static void sortorder_global(OPT_OP op, optval_t val);
 static void sortorder_local(OPT_OP op, optval_t val);
-static void set_sortorder(view_t *view, int ascending, char sort_keys[]);
+static void set_sortorder(view_t *view, int ascending, signed char sort_keys[]);
 static void viewcolumns_global(OPT_OP op, optval_t val);
 static void viewcolumns_local(OPT_OP op, optval_t val);
 static void set_viewcolumns(view_t *view, const char view_columns[]);
@@ -208,13 +210,17 @@ static void set_view_columns_option(view_t *view, const char value[],
 static void add_column(columns_t *columns, column_info_t column_info);
 static int map_name(const char name[], void *arg);
 static void resort_view(view_t * view);
+static void sessionoptions_handler(OPT_OP op, optval_t val);
 static void statusline_handler(OPT_OP op, optval_t val);
 static void suggestoptions_handler(OPT_OP op, optval_t val);
 static void reset_suggestoptions(void);
 static void syncregs_handler(OPT_OP op, optval_t val);
 static void syscalls_handler(OPT_OP op, optval_t val);
+static void tablabel_handler(OPT_OP op, optval_t val);
+static void tabprefix_handler(OPT_OP op, optval_t val);
 static void tabscope_handler(OPT_OP op, optval_t val);
 static void tabstop_handler(OPT_OP op, optval_t val);
+static void tabsuffix_handler(OPT_OP op, optval_t val);
 static void timefmt_handler(OPT_OP op, optval_t val);
 static void timeoutlen_handler(OPT_OP op, optval_t val);
 static void title_handler(OPT_OP op, optval_t val);
@@ -303,10 +309,12 @@ static const char *confirm_vals[][2] = {
 
 /* Possible values of 'dotdirs'. */
 static const char *dotdirs_vals[][2] = {
-	[BIT(DD_ROOT_PARENT)]    = { "rootparent",
-	                             "show .. in FS root" },
-	[BIT(DD_NONROOT_PARENT)] = { "nonrootparent",
-	                             "show .. in non-root directories" },
+	[BIT(DD_ROOT_PARENT)]       = { "rootparent",
+	                                "show .. in FS root" },
+	[BIT(DD_NONROOT_PARENT)]    = { "nonrootparent",
+	                                "show .. in non-root directories" },
+	[BIT(DD_TREE_LEAFS_PARENT)] = { "treeleafsparent",
+	                                "show .. in empty directories of :tree" },
 };
 ARRAY_GUARD(dotdirs_vals, NUM_DOT_DIRS);
 
@@ -323,6 +331,13 @@ static const char *lsoptions_enum[][2] = {
 	{ "transposed", "fill grid by column instead of by line" },
 };
 
+/* Possible values of 'previewoptions'. */
+static const char *previewoptions_vals[][2] = {
+	{ "graphicsdelay:",    "delay before drawing graphics" },
+	{ "hardgraphicsclear", "redraw screen to get rid of graphics" },
+	{ "toptreestats",      "show file counts on top of the tree" },
+};
+
 /* Possible values of 'suggestoptions'. */
 static const char *suggestoptions_vals[][2] = {
 	[BIT(SF_NORMAL)]      = { "normal",      "display in normal mode" },
@@ -375,10 +390,13 @@ static const char *tabscope_vals[][2] = {
 
 /* Possible flags of 'tuioptions' and their count. */
 static const char *tuioptions_vals[][2] = {
-	{ "psu", "all tuioptions values" },
-	{ "p",   "use padding in views and preview" },
-	{ "s",   "display side borders" },
-	{ "u",   "use Unicode characters in the TUI" },
+	{ "lprsuv", "all tuioptions values" },
+	{ "l",      "always show pane title ellipsis on the left" },
+	{ "p",      "use padding in views and preview" },
+	{ "r",      "always show pane title ellipsis on the right" },
+	{ "s",      "display side borders" },
+	{ "u",      "use Unicode characters in the TUI" },
+	{ "v",      "vary width of middle border to equalize view sizes" },
 };
 
 /* Possible values of 'dirsize' option. */
@@ -528,6 +546,7 @@ static const char *vifminfo_set[][2] = {
 	[BIT(VINFO_REGISTERS)] = { "registers", "contents of registers" },
 	[BIT(VINFO_PHISTORY)]  = { "phistory",  "prompt history" },
 	[BIT(VINFO_FHISTORY)]  = { "fhistory",  "local filter history" },
+	[BIT(VINFO_TABS)]      = { "tabs",      "global or pane tabs" },
 };
 ARRAY_GUARD(vifminfo_set, NUM_VINFO);
 
@@ -694,6 +713,11 @@ options[] = {
 	  OPT_INT, 0, NULL, &mintimeoutlen_handler, NULL,
 	  { .ref.int_val = &cfg.min_timeout_len },
 	},
+	{ "previewoptions", "", "tweaks for how preview is done",
+	  OPT_STRLIST, ARRAY_LEN(previewoptions_vals), previewoptions_vals,
+		&previewoptions_handler, NULL,
+	  { .init = &init_previewoptions },
+	},
 	{ "quickview", "", "whether quick view is active",
 	  OPT_BOOL, 0, NULL, &quickview_handler, NULL,
 	  { .init = &init_quickview },
@@ -714,6 +738,11 @@ options[] = {
 	  OPT_INT, 0, NULL, &scrolloff_handler, NULL,
 	  { .ref.int_val = &cfg.scroll_off },
 	},
+	{ "sessionoptions", "ssop", "what to store in a session file",
+	  OPT_SET, ARRAY_LEN(vifminfo_set), vifminfo_set, &sessionoptions_handler,
+	  NULL,
+	  { .ref.set_items = &cfg.session_options },
+	},
 	{ "shell", "sh", "shell to run external commands",
 	  OPT_STR, 0, NULL, &shell_handler, NULL,
 	  { .ref.str_val = &cfg.shell },
@@ -754,7 +783,7 @@ options[] = {
 	  OPT_STR, 0, NULL, &statusline_handler, NULL,
 	  { .ref.str_val = &cfg.status_line },
 	},
-	{ "suggestoptions", "", "when to display key suggestions",
+	{ "suggestoptions", "", "when and how to display key suggestions",
 	  OPT_STRLIST, ARRAY_LEN(suggestoptions_vals), suggestoptions_vals,
 		&suggestoptions_handler, NULL,
 	  { .ref.str_val = &empty },
@@ -766,6 +795,14 @@ options[] = {
 	  OPT_BOOL, 0, NULL, &syscalls_handler, NULL,
 	  { .ref.bool_val = &cfg.use_system_calls },
 	},
+	{ "tablabel", "", "format of main part of a single tab's label",
+	  OPT_STR, 0, NULL, &tablabel_handler, NULL,
+	  { .ref.str_val = &cfg.tab_label },
+	},
+	{ "tabprefix", "", "format of prefix of a tab's label",
+	  OPT_STR, 0, NULL, &tabprefix_handler, NULL,
+	  { .ref.str_val = &cfg.tab_prefix },
+	},
 	{ "tabscope", "", "level at which tabs operate",
 	  OPT_ENUM, ARRAY_LEN(tabscope_vals), tabscope_vals, &tabscope_handler, NULL,
 	  { .ref.bool_val = &cfg.pane_tabs },
@@ -774,6 +811,10 @@ options[] = {
 	  OPT_INT, 0, NULL, &tabstop_handler, NULL,
 	  { .ref.int_val = &cfg.tab_stop },
 	},
+	{ "tabsuffix", "", "format of suffix of a tab's label",
+	  OPT_STR, 0, NULL, &tabsuffix_handler, NULL,
+	  { .ref.str_val = &cfg.tab_suffix },
+	},
 	{ "timefmt", "", "time format",
 	  OPT_STR, 0, NULL, &timefmt_handler, NULL,
 	  { .ref.str_val = &cfg.time_format },
@@ -911,13 +952,13 @@ init_option_handlers(void)
 }
 
 /* Additional handler for processing of global-local options, namely for
- * updating the other view. */
+ * updating inactive view(s). */
 static void
 uni_handler(const char name[], optval_t val, OPT_SCOPE scope)
 {
-	/* += and similar operators act in their way only for the current view, the
-	 * other view just gets value resulted from the operation.  The behaviour is
-	 * fine and doesn't seem to be a bug. */
+	/* += and similar operators act in their way only for the current view, other
+	 * views just get value resulted from the operation.  The behaviour is fine
+	 * and doesn't seem to be a bug. */
 
 	static size_t first_local = (size_t)-1;
 
@@ -943,19 +984,33 @@ uni_handler(const char name[], optval_t val, OPT_SCOPE scope)
 	/* Look up option name and update it in the other view if found. */
 	for(i = first_local; i < ARRAY_LEN(options); ++i)
 	{
-		if(strcmp(options[i].name, name) == 0)
+		if(strcmp(options[i].name, name) != 0)
 		{
-			view_t *const tmp_view = curr_view;
-			curr_view = other_view;
+			continue;
+		}
 
-			/* Make sure option value remains valid even if updated in the handler.
-			 * Do this before calling load_view_options(), because it can change the
-			 * value. */
-			if(ONE_OF(options[i].type, OPT_STR, OPT_STRLIST, OPT_CHARSET))
+		view_t *const tmp_view = curr_view;
+
+		/* Make sure option value remains valid even if updated in the handler.
+		 * Do this before calling load_view_options(), because it can change the
+		 * value. */
+		if(ONE_OF(options[i].type, OPT_STR, OPT_STRLIST, OPT_CHARSET))
+		{
+			val.str_val = strdup(val.str_val);
+		}
+
+		int j;
+		tab_info_t tab_info;
+		for(j = 0; tabs_enum_all(j, &tab_info); ++j)
+		{
+			if(tab_info.view == curr_view)
 			{
-				val.str_val = strdup(val.str_val);
+				continue;
 			}
 
+			curr_view = tab_info.view;
+			/* XXX: do we actually need to load options?  We're calling handlers
+			 *      directly anyway.  */
 			load_view_options(curr_view);
 
 			if(scope == OPT_LOCAL)
@@ -966,16 +1021,16 @@ uni_handler(const char name[], optval_t val, OPT_SCOPE scope)
 			{
 				options[i].global_handler(OP_SET, val);
 			}
-
-			if(ONE_OF(options[i].type, OPT_STR, OPT_STRLIST, OPT_CHARSET))
-			{
-				free(val.str_val);
-			}
-
-			curr_view = tmp_view;
-			load_view_options(curr_view);
-			break;
 		}
+
+		if(ONE_OF(options[i].type, OPT_STR, OPT_STRLIST, OPT_CHARSET))
+		{
+			free(val.str_val);
+		}
+
+		curr_view = tmp_view;
+		load_view_options(curr_view);
+		break;
 	}
 }
 
@@ -1009,15 +1064,20 @@ classify_to_str(void)
 	for(i = 0; i < cfg.name_dec_count && !memerr; ++i)
 	{
 		const file_dec_t *const name_dec = &cfg.name_decs[i];
-		char *const doubled = double_commas(matchers_get_expr(name_dec->matchers));
-		char *const addition = format_str("%s%s::%s::%s",
-				classify[0] != '\0' ? "," : "", name_dec->prefix, doubled,
-				name_dec->suffix);
+		char *const doubled_commas_pat =
+			double_char(matchers_get_expr(name_dec->matchers), ',');
 
-		memerr |= (addition == NULL || strappend(&classify, &len, addition) != 0);
+		if(classify[0] != '\0')
+		{
+			memerr |= strappendch(&classify, &len, ',');
+		}
+		memerr |= strappend(&classify, &len, name_dec->prefix);
+		memerr |= strappend(&classify, &len, "::");
+		memerr |= strappend(&classify, &len, doubled_commas_pat);
+		memerr |= strappend(&classify, &len, "::");
+		memerr |= strappend(&classify, &len, name_dec->suffix);
 
-		free(addition);
-		free(doubled);
+		free(doubled_commas_pat);
 	}
 
 	/* Type-dependent decorations. */
@@ -1027,39 +1087,21 @@ classify_to_str(void)
 		const char *const suffix = cfg.type_decs[ft][DECORATION_SUFFIX];
 		if(prefix[0] != '\0' || suffix[0] != '\0')
 		{
-			char item[64];
 			if(classify[0] != '\0')
 			{
 				memerr |= strappendch(&classify, &len, ',');
 			}
-			(void)snprintf(item, sizeof(item), "%s:%s:%s", prefix,
-					get_type_str(ft), suffix);
-			memerr |= strappend(&classify, &len, item);
+			memerr |= strappend(&classify, &len, prefix);
+			memerr |= strappend(&classify, &len, ":");
+			memerr |= strappend(&classify, &len, get_type_str(ft));
+			memerr |= strappend(&classify, &len, ":");
+			memerr |= strappend(&classify, &len, suffix);
 		}
 	}
 
 	return memerr ? NULL : classify;
 }
 
-/* Clones string passed as the argument doubling commas in the process.  Returns
- * cloned value. */
-static char *
-double_commas(const char str[])
-{
-	char *const doubled = malloc(strlen(str)*2 + 1);
-	char *p = doubled;
-	while(*str != '\0')
-	{
-		*p++ = *str++;
-		if(p[-1] == ',')
-		{
-			*p++ = ',';
-		}
-	}
-	*p = '\0';
-	return doubled;
-}
-
 /* Composes initial value for 'cpoptions' option from a set of configuration
  * flags. */
 static void
@@ -1133,6 +1175,32 @@ init_millerview(optval_t *val)
 	val->bool_val = curr_view->miller_view_g;
 }
 
+/* Initializes 'previewoptions' option from global state. */
+static void
+init_previewoptions(optval_t *val)
+{
+	static char buf[128];
+
+	size_t len = 0U;
+	buf[0] = '\0';
+
+	if(cfg.hard_graphics_clear)
+	{
+		(void)sstrappend(buf, &len, sizeof(buf), "hardgraphicsclear,");
+	}
+	if(cfg.top_tree_stats)
+	{
+		(void)sstrappend(buf, &len, sizeof(buf), "toptreestats,");
+	}
+	if(cfg.graphics_delay != 0)
+	{
+		snprintf(buf + len, sizeof(buf) - len, "graphicsdelay:%d,",
+				cfg.graphics_delay);
+	}
+
+	val->str_val = buf;
+}
+
 /* Initializes 'quickview' option from global state. */
 static void
 init_quickview(optval_t *val)
@@ -1206,10 +1274,13 @@ static void
 init_tuioptions(optval_t *val)
 {
 	static char buf[32];
-	snprintf(buf, sizeof(buf), "%s%s%s",
+	snprintf(buf, sizeof(buf), "%s%s%s%s%s%s",
+			cfg.ellipsis_position < 0 ? "l" : "",
 			cfg.extra_padding ? "p" : "",
+			cfg.ellipsis_position > 0 ? "r" : "",
 			cfg.side_borders_visible ? "s" : "",
-			cfg.use_unicode_characters ? "u" : "");
+			cfg.use_unicode_characters ? "u" : "",
+			cfg.flexible_splitter ? "v" : "");
 	val->str_val = buf;
 }
 
@@ -1430,8 +1501,6 @@ load_view_options(view_t *view)
 void
 clone_local_options(const view_t *from, view_t *to, int defer_slow)
 {
-	const char *sort;
-
 	replace_string(&to->view_columns, from->view_columns);
 	replace_string(&to->view_columns_g, from->view_columns_g);
 
@@ -1447,9 +1516,9 @@ clone_local_options(const view_t *from, view_t *to, int defer_slow)
 	replace_string(&to->sort_groups, from->sort_groups);
 	replace_string(&to->sort_groups_g, from->sort_groups_g);
 
-	sort = (flist_custom_active(from) && ui_view_unsorted(from))
-	     ? (char *)from->custom.sort
-	     : (char *)from->sort;
+	const signed char *sort =
+	  (flist_custom_active(from) && ui_view_unsorted(from)) ? from->custom.sort
+	                                                        : from->sort;
 	memcpy(to->sort, sort, sizeof(to->sort));
 	memcpy(to->sort_g, from->sort_g, sizeof(to->sort_g));
 	sorting_changed(to, defer_slow);
@@ -1488,7 +1557,7 @@ load_sort_option(view_t *view)
 
 /* Loads sorting related options ("sort" and "sortorder"). */
 static void
-load_sort_option_inner(view_t *view, char sort_keys[])
+load_sort_option_inner(view_t *view, signed char sort_keys[])
 {
 	/* This approximate maximum length also includes "+" or "-" sign and a
 	 * comma (",") between items. */
@@ -1665,13 +1734,14 @@ classify_handler(OPT_OP op, optval_t val)
 		assert(sizeof(cfg.type_decs) == sizeof(type_decs) && "Arrays diverged");
 		memcpy(&cfg.type_decs, &type_decs, sizeof(cfg.type_decs));
 
-		ui_view_reset_decor_cache(&lwin);
-		ui_view_reset_decor_cache(&rwin);
-
-		/* 'classify' option affects columns layout, hence views must be reloaded as
-		 * loading list of files performs calculation of filename properties. */
-		ui_view_schedule_reload(curr_view);
-		ui_view_schedule_reload(other_view);
+		int i;
+		tab_info_t tab_info;
+		for(i = 0; tabs_enum_all(i, &tab_info); ++i)
+		{
+			ui_view_reset_decor_cache(tab_info.view);
+			fview_decors_updated(tab_info.view);
+			ui_view_schedule_redraw(tab_info.view);
+		}
 	}
 	else
 	{
@@ -1790,14 +1860,20 @@ pick_out_decoration(char classify_item[], FileType *type, const char **expr)
 	int filetype;
 	for(filetype = 0; filetype < FT_COUNT; ++filetype)
 	{
+		/* Below is a faster version of (pick_out_decoration can be invoked a lot):
+		 * len = snprintf(name, sizeof(name), ":%s:", get_type_str(filetype)); */
 		char name[16];
-		char *item_name;
-		(void)snprintf(name, sizeof(name), ":%s:", get_type_str(filetype));
-		if((item_name = strstr(classify_item, name)) != NULL)
+		name[0] = ':';
+		int len = copy_str(name + 1, sizeof(name) - 1, get_type_str(filetype));
+		name[len++] = ':';
+		name[len] = '\0';
+
+		char *item_name = strstr(classify_item, name);
+		if(item_name != NULL)
 		{
 			*type = filetype;
 			item_name[0] = '\0';
-			return &item_name[strlen(name)];
+			return &item_name[len];
 		}
 	}
 
@@ -2000,7 +2076,7 @@ fillchars_handler(OPT_OP op, optval_t val)
 	if(part == NULL)
 	{
 		/* Schedule a redraw if option was set successfully. */
-		curr_stats.need_update = UT_REDRAW;
+		stats_redraw_later();
 	}
 }
 
@@ -2124,7 +2200,7 @@ laststatus_handler(OPT_OP op, optval_t val)
 		scroll_line_down(&rwin);
 	}
 
-	curr_stats.need_update = UT_REDRAW;
+	stats_redraw_later();
 }
 
 /* Handles updates of the global 'lines' option, which reflects height of
@@ -2209,12 +2285,76 @@ scroll_line_down(view_t *view)
 	wresize(view->win, view->window_rows, view->window_cols);
 }
 
+/* Handles updates of the 'previewoptions' option. */
+static void
+previewoptions_handler(OPT_OP op, optval_t val)
+{
+	char *new_val = strdup(val.str_val);
+	char *part = new_val, *state = NULL;
+
+	int graphics_delay = 0;
+	int hard_graphics_clear = 0;
+	int top_tree_stats = 0;
+
+	while((part = split_and_get(part, ',', &state)) != NULL)
+	{
+		if(starts_with_lit(part, "graphicsdelay:"))
+		{
+			const char *const num = after_first(part, ':');
+			if(!read_int(num, &graphics_delay))
+			{
+				vle_tb_append_linef(vle_err,
+						"Failed to parse \"graphicsdelay\" value: %s", num);
+				break;
+			}
+			if(graphics_delay < 0)
+			{
+				vle_tb_append_linef(vle_err,
+						"\"graphicsdelay\" can't be negative, got: %s", num);
+				break;
+			}
+		}
+		else if(strcmp(part, "hardgraphicsclear") == 0)
+		{
+			hard_graphics_clear = 1;
+		}
+		else if(strcmp(part, "toptreestats") == 0)
+		{
+			top_tree_stats = 1;
+		}
+		else
+		{
+			break_at(part, ':');
+			vle_tb_append_linef(vle_err,
+					"Unknown key for 'previewoptions' option: %s", part);
+			break;
+		}
+	}
+	free(new_val);
+
+	if(part == NULL)
+	{
+		cfg.graphics_delay = graphics_delay;
+		cfg.hard_graphics_clear = hard_graphics_clear;
+		if(top_tree_stats != cfg.top_tree_stats)
+		{
+			text_option_changed();
+		}
+		cfg.top_tree_stats = top_tree_stats;
+	}
+
+	/* In case of error, restore previous value, otherwise reload it anyway to
+	 * remove any duplicates. */
+	init_previewoptions(&val);
+	vle_opts_assign("previewoptions", val, OPT_GLOBAL);
+}
+
 /* Handles switch that controls visibility of quick view. */
 static void
 quickview_handler(OPT_OP op, optval_t val)
 {
 	curr_stats.preview.on = val.bool_val;
-	stats_redraw_schedule();
+	stats_redraw_later();
 }
 
 static void
@@ -2303,7 +2443,7 @@ static void
 showtabline_handler(OPT_OP op, optval_t val)
 {
 	cfg.show_tab_line = (val.enum_item > 2 ? val.enum_item - 3 : val.enum_item);
-	stats_redraw_schedule();
+	stats_redraw_later();
 }
 
 /* Handles changes of 'sizefmt' option by parsing string value and updating
@@ -2368,7 +2508,7 @@ sizefmt_handler(OPT_OP op, optval_t val)
 		cfg.sizefmt.precision = precision;
 		cfg.sizefmt.space = space;
 
-		curr_stats.need_update = UT_REDRAW;
+		stats_redraw_later();
 	}
 	else if(base == -1)
 	{
@@ -2705,7 +2845,7 @@ static void
 previewprg_local(OPT_OP op, optval_t val)
 {
 	replace_string(&curr_view->preview_prg, val.str_val);
-	curr_stats.need_update = UT_REDRAW;
+	stats_redraw_later();
 }
 
 /* Handles relative file numbers displaying toggle in global option. */
@@ -2755,16 +2895,16 @@ sort_local(OPT_OP op, optval_t val)
 {
 	/* Make sure we don't sort unsorted custom view on :restart or when it's a
 	 * compare view. */
-	char *const sort = (curr_stats.restart_in_progress ||
-	                    cv_compare(curr_view->custom.type))
-	                 ? ui_view_sort_list_get(curr_view, curr_view->sort)
-	                 : curr_view->sort;
+	signed char *const sort = (curr_stats.restart_in_progress ||
+	                           cv_compare(curr_view->custom.type))
+	                        ? ui_view_sort_list_get(curr_view, curr_view->sort)
+	                        : curr_view->sort;
 	set_sort(curr_view, sort, val.str_val);
 }
 
 /* Sets sorting value for the view. */
 static void
-set_sort(view_t *view, char sort_keys[], char order[])
+set_sort(view_t *view, signed char sort_keys[], char order[])
 {
 	char *part = order, *state = NULL;
 	int key_count = 0;
@@ -2922,7 +3062,7 @@ sortorder_local(OPT_OP op, optval_t val)
 
 /* Updates sorting order for the view. */
 static void
-set_sortorder(view_t *view, int ascending, char sort_keys[])
+set_sortorder(view_t *view, int ascending, signed char sort_keys[])
 {
 	if((ascending ? +1 : -1)*sort_keys[0] < 0)
 	{
@@ -3028,23 +3168,26 @@ add_column(columns_t *columns, column_info_t column_info)
 	}
 }
 
-/* Maps column name to column id.  Returns column id. */
+/* Maps column name to column id.  Returns column id or -1 on error. */
 static int
 map_name(const char name[], void *arg)
 {
-	int pos;
-
 	/* Handle secondary key (designated by {}). */
 	if(*name == '\0')
 	{
 		const view_t *const view = arg;
-		const char *const sort = ui_view_sort_list_get(view, view->sort);
+		const signed char *const sort = ui_view_sort_list_get(view, view->sort);
 		return (int)get_secondary_key((SortingKey)abs(sort[0]));
 	}
 
-	pos = string_array_pos((char **)&sort_enum[1], ARRAY_LEN(sort_enum) - 1,
+	int pos = string_array_pos((char **)&sort_enum[1], ARRAY_LEN(sort_enum) - 1,
 			name);
-	return (pos >= 0) ? (pos + 1) : -1;
+	if(pos >= 0)
+	{
+		return (pos + 1);
+	}
+
+	return vlua_viewcolumn_map(curr_stats.vlua, name);
 }
 
 void
@@ -3055,6 +3198,13 @@ load_quickview_option(void)
 	vle_opts_assign("quickview", val, OPT_GLOBAL);
 }
 
+void
+load_tabscope_option(void)
+{
+	optval_t val = { .enum_item = (cfg.pane_tabs ? 1 : 0) };
+	vle_opts_assign("tabscope", val, OPT_GLOBAL);
+}
+
 void
 load_geometry(void)
 {
@@ -3072,10 +3222,18 @@ resort_view(view_t * view)
 	ui_view_schedule_redraw(curr_view);
 }
 
+/* Configures what is stored as part of a session file. */
+static void
+sessionoptions_handler(OPT_OP op, optval_t val)
+{
+	cfg.session_options = val.set_items;
+}
+
 static void
 statusline_handler(OPT_OP op, optval_t val)
 {
 	(void)replace_string(&cfg.status_line, val.str_val);
+	stats_redraw_later();
 }
 
 /* Sets when to display key suggestions. */
@@ -3227,6 +3385,22 @@ syscalls_handler(OPT_OP op, optval_t val)
 	cfg.use_system_calls = val.bool_val;
 }
 
+/* Sets format string for main part of tab label. */
+static void
+tablabel_handler(OPT_OP op, optval_t val)
+{
+	replace_string(&cfg.tab_label, val.str_val);
+	stats_redraw_later();
+}
+
+/* Sets format string for tab label's prefix. */
+static void
+tabprefix_handler(OPT_OP op, optval_t val)
+{
+	replace_string(&cfg.tab_prefix, val.str_val);
+	stats_redraw_later();
+}
+
 /* Sets scope of a single tab. */
 static void
 tabscope_handler(OPT_OP op, optval_t val)
@@ -3238,10 +3412,11 @@ tabscope_handler(OPT_OP op, optval_t val)
 
 	/* Erase tabs on current scope before switching it. */
 	tabs_only(curr_view);
+	tabs_only(other_view);
 	tabs_rename(curr_view, NULL);
 
 	cfg.pane_tabs = val.bool_val;
-	curr_stats.need_update = UT_REDRAW;
+	stats_redraw_later();
 }
 
 static void
@@ -3259,6 +3434,14 @@ tabstop_handler(OPT_OP op, optval_t val)
 	text_option_changed();
 }
 
+/* Sets format string for tab label's suffix. */
+static void
+tabsuffix_handler(OPT_OP op, optval_t val)
+{
+	replace_string(&cfg.tab_suffix, val.str_val);
+	stats_redraw_later();
+}
+
 /* Sets format according to which time is displayed. */
 static void
 timefmt_handler(OPT_OP op, optval_t val)
@@ -3309,7 +3492,7 @@ static void
 trashdir_handler(OPT_OP op, optval_t val)
 {
 	char *const expanded_path = expand_path(val.str_val);
-	if(set_trash_dir(expanded_path) != 0)
+	if(trash_set_specs(expanded_path) != 0)
 	{
 		/* Reset the 'trashdir' option to its previous value. */
 		val.str_val = cfg.trash_dir;
@@ -3328,6 +3511,8 @@ tuioptions_handler(OPT_OP op, optval_t val)
 	cfg.extra_padding = 0;
 	cfg.side_borders_visible = 0;
 	cfg.use_unicode_characters = 0;
+	cfg.flexible_splitter = 0;
+	cfg.ellipsis_position = 0;
 
 	/* And set the ones present in the value. */
 	p = val.str_val;
@@ -3335,15 +3520,24 @@ tuioptions_handler(OPT_OP op, optval_t val)
 	{
 		switch(*p)
 		{
+			case 'l':
+				cfg.ellipsis_position = -1;
+				break;
 			case 'p':
 				cfg.extra_padding = 1;
 				break;
+			case 'r':
+				cfg.ellipsis_position = 1;
+				break;
 			case 's':
 				cfg.side_borders_visible = 1;
 				break;
 			case 'u':
 				cfg.use_unicode_characters = 1;
 				break;
+			case 'v':
+				cfg.flexible_splitter = 1;
+				break;
 
 			default:
 				assert(0 && "Unhandled tuioptions flag.");
@@ -3352,10 +3546,14 @@ tuioptions_handler(OPT_OP op, optval_t val)
 		++p;
 	}
 
+	/* Make sure "r" and "l" flags don't appear at the same time. */
+	init_tuioptions(&val);
+	vle_opts_assign("tuioptions", val, OPT_GLOBAL);
+
 	curr_stats.ellipsis = (cfg.use_unicode_characters ? "…" : "...");
 	columns_set_ellipsis(curr_stats.ellipsis);
 
-	curr_stats.need_update = UT_REDRAW;
+	stats_redraw_later();
 }
 
 static void
@@ -3378,6 +3576,7 @@ vixcmd_handler(OPT_OP op, optval_t val)
 	cfg.vi_x_cmd_bg = cut_suffix(cfg.vi_x_command, "&");
 }
 
+/* Configures what is stored between sessions. */
 static void
 vifminfo_handler(OPT_OP op, optval_t val)
 {
@@ -3519,7 +3718,7 @@ wrap_handler(OPT_OP op, optval_t val)
 static void
 text_option_changed(void)
 {
-	stats_redraw_schedule();
+	stats_redraw_later();
 }
 
 static void
