@@ -55,6 +55,9 @@ void load_view_columns_option(struct view_t *view, const char value[]);
 /* Loads current state of quick view into corresponding option. */
 void load_quickview_option(void);
 
+/* Loads current state of tab scopr into corresponding option. */
+void load_tabscope_option(void);
+
 /* Updates geometry related options. */
 void load_geometry(void);
 
