@@ -48,7 +48,9 @@
 #include "compat/os.h"
 #include "int/file_magic.h"
 #include "int/fuse.h"
+#include "int/path_env.h"
 #include "int/vim.h"
+#include "lua/vlua.h"
 #include "menus/users_menu.h"
 #include "modes/dialogs/msg_dialog.h"
 #include "modes/view.h"
@@ -61,7 +63,6 @@
 #include "utils/path.h"
 #include "utils/str.h"
 #include "utils/string_array.h"
-#include "utils/test_helpers.h"
 #include "utils/utils.h"
 #include "utils/utf8.h"
 #include "background.h"
@@ -79,18 +80,18 @@
 /* Kinds of symbolic link file treatment on file handling. */
 typedef enum
 {
-	FHL_NO_FOLLOW, /* Don't follow (navigate to instead of navigation inside). */
-	FHL_FOLLOW,    /* Follow (end up on the link target, not inside it). */
+	FHL_NO_FOLLOW,  /* Don't follow (navigate to instead of navigation inside). */
+	FHL_FOLLOW,     /* Follow (end up on the link target, not inside it). */
+	FHL_FOLLOW_ALL, /* Follow all the way (end up on final link's target). */
 }
 FileHandleLink;
 
 static void handle_file(view_t *view, FileHandleExec exec,
 		FileHandleLink follow);
-static int is_runnable(const view_t *view, const char full_path[], int type,
+static int is_runnable(view_t *view, const char full_path[], int type,
 		int force_follow);
 static int is_executable(const char full_path[], const dir_entry_t *curr,
 		int dont_execute, int runnable);
-static int is_dir_entry(const char full_path[], int type);
 #ifdef _WIN32
 static void run_win_executable(char full_path[], int elevate);
 static int run_win_executable_as_evaluated(const char full_path[]);
@@ -101,13 +102,19 @@ static void run_selection(view_t *view, int dont_execute);
 static void run_with_defaults(view_t *view);
 static void run_selection_separately(view_t *view, int dont_execute);
 static int is_multi_run_compat(view_t *view, const char prog_cmd[]);
-static void run_explicit_prog(const char prog_spec[], int pause, int force_bg);
+static void run_explicit_prog(view_t *view, const char prog_spec[], int pause,
+		int force_bg);
 static void run_implicit_prog(view_t *view, const char prog_spec[], int pause,
 		int force_bg);
 static void view_current_file(const view_t *view);
-static void follow_link(view_t *view, int follow_dirs);
+static void follow_link(view_t *view, int follow_dirs, int ultimate);
+static void enter_dir(struct view_t *view);
 static int cd_to_parent_dir(view_t *view);
 static void extract_last_path_component(const char path[], char buf[]);
+static char * run_shell_prepare(const char command[], ShellPause pause,
+		int use_term_multiplexer, ShellRequester by);
+static int run_shell_finish(const char cmd[], const char final_cmd[],
+		ShellPause pause, int status);
 static void setup_shellout_env(void);
 static void cleanup_shellout_env(void);
 static char * gen_shell_cmd(const char cmd[], int pause,
@@ -116,14 +123,14 @@ static char * gen_term_multiplexer_cmd(const char cmd[], int pause,
 		ShellRequester by);
 static char * gen_term_multiplexer_title_arg(const char cmd[]);
 static char * gen_normal_cmd(const char cmd[], int pause);
-static char * gen_term_multiplexer_run_cmd(void);
 static void set_pwd_in_screen(const char path[]);
 static int try_run_with_filetype(view_t *view, const assoc_records_t assocs,
 		const char start[], int background);
-static void output_to_statusbar(const char cmd[]);
-static int output_to_preview(const char cmd[]);
-static void output_to_nowhere(const char cmd[]);
-static void run_in_split(const view_t *view, const char cmd[]);
+static void output_to_statusbar(const char cmd[], view_t *view,
+		MacroFlags flags);
+static int output_to_preview(view_t *view, const char cmd[], MacroFlags flags);
+static void output_to_nowhere(const char cmd[], view_t *view, MacroFlags flags);
+static void run_in_split(const view_t *view, const char cmd[], int vert_split);
 static void path_handler(const char line[], void *arg);
 static void line_handler(const char line[], void *arg);
 
@@ -132,13 +139,13 @@ static void line_handler(const char line[], void *arg);
 static const char *const FUSE_FILE_ENVVAR = "VIFM_FUSE_FILE";
 
 void
-open_file(view_t *view, FileHandleExec exec)
+rn_open(view_t *view, FileHandleExec exec)
 {
 	handle_file(view, exec, FHL_NO_FOLLOW);
 }
 
 void
-follow_file(view_t *view)
+rn_follow(view_t *view, int ultimate)
 {
 	if(flist_custom_active(view))
 	{
@@ -156,17 +163,18 @@ follow_file(view_t *view)
 		return;
 	}
 
-	handle_file(view, FHE_RUN, FHL_FOLLOW);
+	handle_file(view, FHE_RUN, ultimate ? FHL_FOLLOW_ALL : FHL_FOLLOW);
 }
 
 static void
 handle_file(view_t *view, FileHandleExec exec, FileHandleLink follow)
 {
 	char full_path[PATH_MAX + 1];
-	int executable;
-	int runnable;
 	const dir_entry_t *const curr = get_current_entry(view);
 
+	int user_selection = !view->pending_marking;
+	flist_set_marking(view, 1);
+
 	if(fentry_is_fake(curr))
 	{
 		return;
@@ -174,25 +182,31 @@ handle_file(view_t *view, FileHandleExec exec, FileHandleLink follow)
 
 	get_full_path_of(curr, sizeof(full_path), full_path);
 
-	if(is_dir(full_path) || is_unc_root(view->curr_dir))
+	int could_enter_entry = (curr->type != FT_LINK || follow == FHL_NO_FOLLOW);
+	int selected_entry = (curr->marked && (!user_selection || curr->selected));
+	if(!selected_entry && could_enter_entry)
 	{
-		if(!curr->selected && (curr->type != FT_LINK || follow == FHL_NO_FOLLOW))
+		int dir_like_entry = (is_dir(full_path) || is_unc_root(view->curr_dir));
+		if(dir_like_entry)
 		{
-			open_dir(view);
+			enter_dir(view);
 			return;
 		}
 	}
 
-	runnable = is_runnable(view, full_path, curr->type, follow == FHL_FOLLOW);
-	executable = is_executable(full_path, curr, exec == FHE_NO_RUN, runnable);
+	int runnable = is_runnable(view, full_path, curr->type,
+			follow != FHL_NO_FOLLOW);
+	int executable = is_executable(full_path, curr, exec == FHE_NO_RUN, runnable);
 
 	if(stats_file_choose_action_set() && (executable || runnable))
 	{
+		/* Reuse marking second time. */
+		view->pending_marking = 1;
 		/* The call below does not return. */
 		vifm_choose_files(view, 0, NULL);
 	}
 
-	if(executable && !is_dir_entry(full_path, curr->type))
+	if(executable && !fentry_is_dir(curr))
 	{
 		execute_file(full_path, exec == FHE_ELEVATE_AND_RUN);
 	}
@@ -200,33 +214,39 @@ handle_file(view_t *view, FileHandleExec exec, FileHandleLink follow)
 	{
 		run_selection(view, exec == FHE_NO_RUN);
 	}
-	else if(curr->type == FT_LINK)
+	else if(curr->type == FT_LINK || is_shortcut(curr->name))
 	{
-		follow_link(view, follow == FHL_FOLLOW);
+		follow_link(view, follow != FHL_NO_FOLLOW, follow == FHL_FOLLOW_ALL);
 	}
 }
 
-/* Returns non-zero if file can be executed or it's link to a directory (it can
- * be entered), otherwise zero is returned. */
+/* Returns non-zero if file can be executed or it's a link to a directory (it
+ * can be entered), otherwise zero is returned. */
 static int
-is_runnable(const view_t *view, const char full_path[], int type,
-		int force_follow)
+is_runnable(view_t *view, const char full_path[], int type, int force_follow)
 {
-	int runnable = !cfg.follow_links && type == FT_LINK &&
-		get_symlink_type(full_path) != SLT_DIR;
-	if(runnable && force_follow)
+	int count = 0;
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry))
 	{
-		runnable = 0;
+		if(++count > 1)
+		{
+			return 1;
+		}
 	}
-	if(view->selected_files > 0)
+
+	if(!force_follow && !cfg.follow_links && type == FT_LINK &&
+			get_symlink_type(full_path) != SLT_DIR)
 	{
-		runnable = 1;
+		return 1;
 	}
-	if(!runnable)
+
+	if(type == FT_REG)
 	{
-		runnable = type == FT_REG || type == FT_EXEC || type == FT_DIR;
+		return (!force_follow || !is_shortcut(full_path));
 	}
-	return runnable;
+
+	return (type == FT_EXEC || type == FT_DIR);
 }
 
 /* Returns non-zero if file can be executed, otherwise zero is returned. */
@@ -244,14 +264,6 @@ is_executable(const char full_path[], const dir_entry_t *curr, int dont_execute,
 	return executable && !dont_execute && cfg.auto_execute;
 }
 
-/* Returns non-zero if entry is directory or link to a directory, otherwise zero
- * is returned. */
-static int
-is_dir_entry(const char full_path[], int type)
-{
-	return type == FT_DIR || (type == FT_LINK && is_dir(full_path));
-}
-
 #ifdef _WIN32
 
 /* Runs a Windows executable handling errors and rights elevation. */
@@ -341,19 +353,10 @@ static int
 selection_is_consistent(view_t *view)
 {
 	int files = 0, dirs = 0;
-	dir_entry_t *entry;
-
-	if(view->selected_files < 2)
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry))
 	{
-		return 1;
-	}
-
-	entry = NULL;
-	while(iter_selected_entries(view, &entry))
-	{
-		char full[PATH_MAX + 1];
-		get_full_path_of(entry, sizeof(full), full);
-		if(is_dir_entry(full, entry->type))
+		if(fentry_is_dir(entry))
 		{
 			++dirs;
 		}
@@ -372,7 +375,7 @@ execute_file(const char full_path[], int elevate)
 {
 #ifndef _WIN32
 	char *const escaped = shell_like_escape(full_path, 0);
-	shellout(escaped, PAUSE_ALWAYS, 1, SHELL_BY_APP);
+	rn_shell(escaped, PAUSE_ALWAYS, 1, SHELL_BY_APP);
 	free(escaped);
 #else
 	char *const dquoted_full_path = strdup(enclose_in_dquotes(full_path));
@@ -389,15 +392,6 @@ execute_file(const char full_path[], int elevate)
 static void
 run_selection(view_t *view, int dont_execute)
 {
-	/* TODO: refactor this function run_selection() */
-
-	char *typed_fname;
-	const char *multi_prog_cmd;
-	int undef;
-	int same;
-	dir_entry_t *entry;
-	int no_multi_run;
-
 	if(!selection_is_consistent(view))
 	{
 		show_error_msg("Selection error",
@@ -405,24 +399,19 @@ run_selection(view_t *view, int dont_execute)
 		return;
 	}
 
-	if(!get_current_entry(view)->selected)
-	{
-		flist_sel_stash(view);
-	}
-
-	typed_fname = get_typed_entry_fpath(get_current_entry(view));
-	multi_prog_cmd = ft_get_program(typed_fname);
+	char *typed_fname = get_typed_entry_fpath(get_current_entry(view));
+	const char *common_prog_cmd = ft_get_program(typed_fname);
 	free(typed_fname);
 
-	no_multi_run = !is_multi_run_compat(view, multi_prog_cmd);
-	undef = 0;
-	same = 1;
+	int can_multi_run = (is_multi_run_compat(view, common_prog_cmd) != 0);
+	int files_without_handler = (common_prog_cmd == NULL);
+	int identical_handlers = 1;
+	int nentries = 0;
 
-	entry = NULL;
-	while(iter_selected_entries(view, &entry))
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry))
 	{
-		char *typed_fname;
-		const char *entry_prog_cmd;
+		++nentries;
 
 		if(!path_exists_at(entry->origin, entry->name, DEREF))
 		{
@@ -431,52 +420,45 @@ run_selection(view_t *view, int dont_execute)
 			return;
 		}
 
-		typed_fname = get_typed_entry_fpath(entry);
-		entry_prog_cmd = ft_get_program(typed_fname);
+		char *typed_fname = get_typed_entry_fpath(entry);
+		const char *entry_prog_cmd = ft_get_program(typed_fname);
 		free(typed_fname);
 
 		if(entry_prog_cmd == NULL)
 		{
-			++undef;
+			++files_without_handler;
 			continue;
 		}
 
-		no_multi_run += !is_multi_run_compat(view, entry_prog_cmd);
-		if(multi_prog_cmd == NULL)
+		can_multi_run &= (is_multi_run_compat(view, entry_prog_cmd) != 0);
+		if(common_prog_cmd == NULL)
 		{
-			multi_prog_cmd = entry_prog_cmd;
+			common_prog_cmd = entry_prog_cmd;
 		}
-		else if(strcmp(entry_prog_cmd, multi_prog_cmd) != 0)
+		else if(strcmp(entry_prog_cmd, common_prog_cmd) != 0)
 		{
-			same = 0;
+			identical_handlers = 0;
 		}
 	}
 
-	if(!same && undef == 0 && no_multi_run)
-	{
-		show_error_msg("Run error", "Handlers of selected files are incompatible.");
-		return;
-	}
-	if(undef > 0)
-	{
-		multi_prog_cmd = NULL;
-	}
+	can_multi_run &= (nentries > 1);
 
-	/* Check for a filetype */
-	/* vi is set as the default for any extension without a program */
-	if(multi_prog_cmd == NULL)
+	if(files_without_handler > 0)
 	{
 		run_with_defaults(view);
-		return;
 	}
-
-	if(no_multi_run)
+	else if(can_multi_run)
 	{
-		run_using_prog(view, multi_prog_cmd, dont_execute, 0);
+		run_selection_separately(view, dont_execute);
+	}
+	else if(identical_handlers)
+	{
+		rn_open_with(view, common_prog_cmd, dont_execute, 0);
 	}
 	else
 	{
-		run_selection_separately(view, dont_execute);
+		show_error_msg("Run error", "Handlers of selected files are "
+		                            "incompatible.");
 	}
 }
 
@@ -487,13 +469,13 @@ run_with_defaults(view_t *view)
 {
 	if(get_current_entry(view)->type == FT_DIR)
 	{
-		open_dir(view);
+		enter_dir(view);
+		return;
 	}
-	else if(view->selected_files <= 1)
-	{
-		view_current_file(view);
-	}
-	else if(vim_edit_selection() != 0)
+
+	/* Reuse marking second time. */
+	view->pending_marking = 1;
+	if(vim_edit_marking() != 0)
 	{
 		show_error_msg("Running error", "Can't edit selection");
 	}
@@ -503,12 +485,10 @@ run_with_defaults(view_t *view)
 static void
 run_selection_separately(view_t *view, int dont_execute)
 {
-	dir_entry_t *entry;
-
 	const int pos = view->list_pos;
 
-	entry = NULL;
-	while(iter_selected_entries(view, &entry))
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry))
 	{
 		char *typed_fname;
 		const char *entry_prog_cmd;
@@ -518,7 +498,7 @@ run_selection_separately(view_t *view, int dont_execute)
 		free(typed_fname);
 
 		view->list_pos = entry_to_pos(view, entry);
-		run_using_prog(view, entry_prog_cmd, dont_execute, 0);
+		rn_open_with(view, entry_prog_cmd, dont_execute, 0);
 	}
 
 	view->list_pos = pos;
@@ -533,10 +513,9 @@ is_multi_run_compat(view_t *view, const char prog_cmd[])
 	size_t len;
 	if(prog_cmd == NULL)
 		return 0;
-	if(view->selected_files <= 1)
-		return 0;
 	if((len = strlen(prog_cmd)) == 0)
 		return 0;
+	/* XXX: should the check be for " &" and not just "&"? */
 	if(prog_cmd[len - 1] != '&')
 		return 0;
 	if(strstr(prog_cmd, "%f") != NULL || strstr(prog_cmd, "%F") != NULL)
@@ -547,7 +526,7 @@ is_multi_run_compat(view_t *view, const char prog_cmd[])
 }
 
 void
-run_using_prog(view_t *view, const char prog_spec[], int dont_execute,
+rn_open_with(view_t *view, const char prog_spec[], int dont_execute,
 		int force_bg)
 {
 	const dir_entry_t *const curr = get_current_entry(view);
@@ -559,6 +538,12 @@ run_using_prog(view_t *view, const char prog_spec[], int dont_execute,
 		return;
 	}
 
+	if(vlua_handler_cmd(curr_stats.vlua, prog_spec))
+	{
+		vlua_open_file(curr_stats.vlua, prog_spec, curr);
+		return;
+	}
+
 	if(fuse_is_mount_string(prog_spec))
 	{
 		if(dont_execute)
@@ -572,11 +557,11 @@ run_using_prog(view_t *view, const char prog_spec[], int dont_execute,
 	}
 	else if(strcmp(prog_spec, VIFM_PSEUDO_CMD) == 0)
 	{
-		open_dir(view);
+		enter_dir(view);
 	}
 	else if(strchr(prog_spec, '%') != NULL)
 	{
-		run_explicit_prog(prog_spec, pause, force_bg);
+		run_explicit_prog(view, prog_spec, pause, force_bg);
 	}
 	else
 	{
@@ -587,18 +572,18 @@ run_using_prog(view_t *view, const char prog_spec[], int dont_execute,
 /* Executes current file of the current view by program specification that
  * includes at least one macro. */
 static void
-run_explicit_prog(const char prog_spec[], int pause, int force_bg)
+run_explicit_prog(view_t *view, const char prog_spec[], int pause, int force_bg)
 {
-	int bg;
 	MacroFlags flags;
-	int save_msg;
 	char *const cmd = ma_expand(prog_spec, NULL, &flags, 1);
 
-	bg = cut_suffix(cmd, " &");
+	const ShellPause pause_shell = (pause ? PAUSE_ALWAYS : PAUSE_ON_ERROR);
+
+	int bg = cut_suffix(cmd, " &");
 	bg = !pause && (bg || force_bg);
 
-	save_msg = 0;
-	if(run_ext_command(cmd, flags, bg, &save_msg) != 0)
+	int save_msg = 0;
+	if(rn_ext(view, cmd, prog_spec, flags, bg, &save_msg) != 0)
 	{
 		if(save_msg)
 		{
@@ -607,13 +592,18 @@ run_explicit_prog(const char prog_spec[], int pause, int force_bg)
 	}
 	else if(bg)
 	{
-		assert(flags != MF_IGNORE && "This case is for run_ext_command()");
-		(void)bg_run_external(cmd, flags == MF_IGNORE, SHELL_BY_USER);
+		assert(ma_flags_missing(flags, MF_IGNORE) && "This case is for rn_ext()");
+		rn_start_bg_command(view, cmd, flags);
+	}
+	else if(ma_flags_present(flags, MF_PIPE_FILE_LIST) ||
+			ma_flags_present(flags, MF_PIPE_FILE_LIST_Z))
+	{
+		rn_pipe(cmd, view, flags, pause_shell);
 	}
 	else
 	{
-		(void)shellout(cmd, pause ? PAUSE_ALWAYS : PAUSE_ON_ERROR,
-				flags != MF_NO_TERM_MUX, SHELL_BY_USER);
+		const int use_term_multiplexer = ma_flags_missing(flags, MF_NO_TERM_MUX);
+		(void)rn_shell(cmd, pause_shell, use_term_multiplexer, SHELL_BY_USER);
 	}
 
 	free(cmd);
@@ -648,11 +638,11 @@ run_implicit_prog(view_t *view, const char prog_spec[], int pause, int force_bg)
 
 	if(bg)
 	{
-		(void)bg_run_external(cmd, 0, SHELL_BY_USER);
+		(void)bg_run_external(cmd, 0, SHELL_BY_USER, NULL);
 	}
 	else
 	{
-		(void)shellout(cmd, pause ? PAUSE_ALWAYS : PAUSE_ON_ERROR, 1,
+		(void)rn_shell(cmd, pause ? PAUSE_ALWAYS : PAUSE_ON_ERROR, 1,
 				SHELL_BY_USER);
 	}
 }
@@ -666,26 +656,58 @@ view_current_file(const view_t *view)
 	(void)vim_view_file(full_path, -1, -1, 1);
 }
 
-/* Resolve link target and either navigate inside directory link points to or
- * navigate to directory where target is located pointing cursor on
- * it (the follow_dirs flag controls behaviour). */
+/* Resolves link target and either navigates inside directory the link points to
+ * or navigates to directory where target is located placing cursor at it (the
+ * follow_dirs flag controls the behaviour). */
 static void
-follow_link(view_t *view, int follow_dirs)
+follow_link(view_t *view, int follow_dirs, int ultimate)
 {
 	char *dir, *file;
-	char full_path[PATH_MAX + 1];
-	char linkto[PATH_MAX + NAME_MAX];
-	const dir_entry_t *const curr = get_current_entry(curr_view);
+	char linkto[PATH_MAX + NAME_MAX + 1];
+	const dir_entry_t *const curr = get_current_entry(view);
 
+	char full_path[PATH_MAX + 1];
 	get_full_path_of(curr, sizeof(full_path), full_path);
 
-	if(get_link_target_abs(full_path, curr->origin, linkto, sizeof(linkto)) != 0)
+	int resolve_one_level = !ultimate;
+
+	if(ultimate)
 	{
-		show_error_msg("Error", "Can't read link.");
-		return;
+		if(os_realpath(full_path, linkto) != linkto)
+		{
+			show_error_msg("Error", "Can't resolve the link.");
+			return;
+		}
+
+#ifdef _WIN32
+		/* This might be a Windows shortcut which aren't resolved by
+		 * os_realpath(). */
+		resolve_one_level = paths_are_equal(full_path, linkto);
+#endif
 	}
 
-	if(!path_exists(linkto, DEREF))
+	if(resolve_one_level)
+	{
+		/* We resolve origin to a real path because relative symbolic links are
+		 * relative to real location of the link.  Can't simply do realpath() on
+		 * full path as it would resolve symbolic links recursively, while we want
+		 * to follow only one level deep. */
+
+		char origin_real[PATH_MAX + 1];
+		if(os_realpath(curr->origin, origin_real) != origin_real)
+		{
+			show_error_msg("Error", "Can't resolve origin of the link.");
+			return;
+		}
+
+		if(get_link_target_abs(full_path, origin_real, linkto, sizeof(linkto)) != 0)
+		{
+			show_error_msg("Error", "Can't read link.");
+			return;
+		}
+	}
+
+	if(!path_exists(linkto, NODEREF))
 	{
 		show_error_msg("Broken Link",
 				"Can't access link destination.  It might be broken.");
@@ -707,35 +729,36 @@ follow_link(view_t *view, int follow_dirs)
 		file = get_last_path_component(linkto);
 	}
 
-	if(dir[0] != '\0')
+	if(dir[0] != '\0' && file != NULL)
+	{
+		navigate_to_file(view, dir, file, 1);
+	}
+	else if(dir[0] != '\0')
 	{
 		navigate_to(view, dir);
 	}
-
-	if(file != NULL)
+	else if(file != NULL)
 	{
-		const int pos = fpos_find_by_name(view, file);
-		if(pos >= 0)
-		{
-			fpos_set_pos(view, pos);
-		}
+		fpos_ensure_selected(view, file);
 	}
 
 	free(dir);
 }
 
-void
-open_dir(view_t *view)
+/* Handles opening of current entry of the view as a directory. */
+static void
+enter_dir(view_t *view)
 {
-	char full_path[PATH_MAX + 1];
+	dir_entry_t *const curr = get_current_entry(view);
 
-	if(is_parent_dir(get_current_file_name(view)))
+	if(is_parent_dir(curr->name) && !curr->owns_origin)
 	{
-		cd_updir(view, 1);
+		rn_leave(view, 1);
 		return;
 	}
 
-	get_current_full_path(view, sizeof(full_path), full_path);
+	char full_path[PATH_MAX + 1];
+	get_full_path_of(curr, sizeof(full_path), full_path);
 
 	if(cd_is_possible(full_path))
 	{
@@ -746,7 +769,7 @@ open_dir(view_t *view)
 }
 
 void
-cd_updir(view_t *view, int levels)
+rn_leave(view_t *view, int levels)
 {
 	/* Do not save intermediate directories in directory history. */
 	curr_stats.drop_new_dir_hist = 1;
@@ -760,7 +783,7 @@ cd_updir(view_t *view, int levels)
 	}
 
 	curr_stats.drop_new_dir_hist = 0;
-	flist_hist_save(view, NULL, NULL, -1);
+	flist_hist_save(view);
 }
 
 /* Goes one directory up from current location.  Returns zero unless it won't
@@ -813,46 +836,86 @@ extract_last_path_component(const char path[], char buf[])
 }
 
 int
-shellout(const char command[], ShellPause pause, int use_term_multiplexer,
+rn_shell(const char command[], ShellPause pause, int use_term_multiplexer,
 		ShellRequester by)
 {
-	char *cmd;
-	int result;
-	int ec;
+	char *cmd = run_shell_prepare(command, pause, use_term_multiplexer, by);
+	int status = vifm_system(cmd, by);
+	int exit_code = run_shell_finish(command, cmd, pause, status);
 
-	if(pause == PAUSE_ALWAYS && command != NULL && ends_with(command, "&"))
+	free(cmd);
+	return exit_code;
+}
+
+int
+rn_pipe(const char command[], view_t *view, MacroFlags flags, ShellPause pause)
+{
+	assert((ma_flags_present(flags, MF_PIPE_FILE_LIST)
+			|| ma_flags_present(flags, MF_PIPE_FILE_LIST_Z))
+			&& "rn_pipe() must be called only when piping is requested");
+
+	FILE *input_tmp = make_in_file(view, flags);
+	char *cmd = run_shell_prepare(command, pause, 0, SHELL_BY_USER);
+
+	int status = vifm_system_input(cmd, input_tmp, SHELL_BY_USER);
+	int exit_code = run_shell_finish(command, cmd, pause, status);
+
+	free(cmd);
+	fclose(input_tmp);
+
+	return exit_code;
+}
+
+/* Prepares to run a command.  Returns actual command to execute, which should
+ * be freed by the caller. */
+static char *
+run_shell_prepare(const char command[], ShellPause pause,
+		int use_term_multiplexer, ShellRequester by)
+{
+	/* Shutdown UI at this point, where $PATH isn't cleared. */
+	ui_shutdown();
+
+	if(command == NULL)
 	{
-		pause = PAUSE_ON_ERROR;
+		command = env_get_def("SHELL", cfg.shell);
+
+		/* Run shell with clean $PATH environment variable. */
+		load_clean_path_env();
 	}
 
 	setup_shellout_env();
 
-	cmd = gen_shell_cmd(command, pause == PAUSE_ALWAYS, use_term_multiplexer,
+	return gen_shell_cmd(command, pause == PAUSE_ALWAYS, use_term_multiplexer,
 			&by);
+}
 
-	ui_shutdown();
-	ec = vifm_system(cmd, by);
-	/* No WIFEXITED(ec) check here, since vifm_system(...) shouldn't return until
-	 * subprocess exited. */
-	result = WEXITSTATUS(ec);
-
+/* Handles command running result.  Returns exit code. */
+static int
+run_shell_finish(const char cmd[], const char final_cmd[], ShellPause pause,
+		int status)
+{
 	cleanup_shellout_env();
 
-	if(result != 0 && pause == PAUSE_ON_ERROR)
+	int exit_code = WEXITSTATUS(status);
+
+	if(pause == PAUSE_ALWAYS && final_cmd != NULL && ends_with(final_cmd, "&"))
 	{
-		LOG_ERROR_MSG("Subprocess (%s) exit code: %d (0x%x); status = 0x%x", cmd,
-				result, result, ec);
+		pause = PAUSE_ON_ERROR;
+	}
+
+	if(exit_code != 0 && pause == PAUSE_ON_ERROR)
+	{
+		LOG_ERROR_MSG("Subprocess (%s) exit code: %d (0x%x); status = 0x%x",
+				final_cmd, exit_code, exit_code, status);
 		pause_shell();
 	}
 
-	free(cmd);
-
 	/* Force updates of views that don't have associated watchers. */
-	if(flist_custom_active(&lwin) && lwin.custom.type != CV_TREE)
+	if(!flist_is_fs_backed(&lwin))
 	{
 		ui_view_schedule_reload(&lwin);
 	}
-	if(flist_custom_active(&rwin) && rwin.custom.type != CV_TREE)
+	if(!flist_is_fs_backed(&rwin))
 	{
 		ui_view_schedule_reload(&rwin);
 	}
@@ -862,7 +925,7 @@ shellout(const char command[], ShellPause pause, int use_term_multiplexer,
 	if(!curr_stats.skip_shellout_redraw)
 	{
 		/* Redraw to handle resizing of terminal that we could have missed. */
-		curr_stats.need_update = UT_REDRAW;
+		stats_redraw_later();
 	}
 
 	if(curr_stats.load_stage > 0)
@@ -870,7 +933,12 @@ shellout(const char command[], ShellPause pause, int use_term_multiplexer,
 		curs_set(0);
 	}
 
-	return result;
+	if(cmd == NULL)
+	{
+		load_real_path_env();
+	}
+
+	return exit_code;
 }
 
 /* Configures environment variables before shellout.  Should be used in pair
@@ -933,37 +1001,24 @@ cleanup_shellout_env(void)
 	free(cmd);
 }
 
-/* Composes shell command to run basing on parameters for execution.  NULL cmd
- * parameter opens shell.  Returns a newly allocated string, which should be
- * freed by the caller. */
+/* Composes shell command to run based on parameters for execution.  Returns a
+ * newly allocated string, which should be freed by the caller. */
 static char *
 gen_shell_cmd(const char cmd[], int pause, int use_term_multiplexer,
 		ShellRequester *by)
 {
-	char *shell_cmd = NULL;
+	char *shell_cmd;
 
-	if(cmd != NULL)
+	if(use_term_multiplexer && curr_stats.term_multiplexer != TM_NONE)
 	{
-		if(use_term_multiplexer && curr_stats.term_multiplexer != TM_NONE)
-		{
-			shell_cmd = gen_term_multiplexer_cmd(cmd, pause, *by);
-			/* User shell settings were taken into account in command for
-			 * multiplexer, don't use them to invoke the multiplexer itself. */
-			*by = SHELL_BY_APP;
-		}
-		else
-		{
-			shell_cmd = gen_normal_cmd(cmd, pause);
-		}
+		shell_cmd = gen_term_multiplexer_cmd(cmd, pause, *by);
+		/* User shell settings were taken into account in command for multiplexer,
+		 * don't use them to invoke the multiplexer itself. */
+		*by = SHELL_BY_APP;
 	}
-	else if(use_term_multiplexer)
+	else
 	{
-		shell_cmd = gen_term_multiplexer_run_cmd();
-	}
-
-	if(shell_cmd == NULL)
-	{
-		shell_cmd = strdup(cfg.shell);
+		shell_cmd = gen_normal_cmd(cmd, pause);
 	}
 
 	return shell_cmd;
@@ -1085,6 +1140,11 @@ gen_normal_cmd(const char cmd[], int pause)
 		{
 			cmd_with_pause_fmt = "%s" PAUSE_STR;
 		}
+		else if(curr_stats.shell_type == ST_PS)
+		{
+			/* TODO: make pausing work. */
+			cmd_with_pause_fmt = "%s";
+		}
 		else
 		{
 			cmd_with_pause_fmt = "%s; " PAUSE_CMD;
@@ -1098,31 +1158,6 @@ gen_normal_cmd(const char cmd[], int pause)
 	}
 }
 
-/* Composes shell command to run active terminal multiplexer.  Returns a newly
- * allocated string, which should be freed by the caller. */
-static char *
-gen_term_multiplexer_run_cmd(void)
-{
-	char *shell_cmd = NULL;
-
-	if(curr_stats.term_multiplexer == TM_SCREEN)
-	{
-		set_pwd_in_screen(curr_view->curr_dir);
-
-		shell_cmd = strdup("screen");
-	}
-	else if(curr_stats.term_multiplexer == TM_TMUX)
-	{
-		shell_cmd = strdup("tmux new-window");
-	}
-	else
-	{
-		assert(0 && "Unexpected active terminal multiplexer value.");
-	}
-
-	return shell_cmd;
-}
-
 /* Changes $PWD in running GNU/screen session to the specified path.  Needed for
  * symlink directories and sshfs mounts. */
 static void
@@ -1138,7 +1173,7 @@ set_pwd_in_screen(const char path[])
 }
 
 int
-run_with_filetype(view_t *view, const char beginning[], int background)
+rn_open_with_match(view_t *view, const char beginning[], int background)
 {
 	dir_entry_t *const curr = get_current_entry(view);
 	assoc_records_t ft, magic;
@@ -1176,7 +1211,7 @@ try_run_with_filetype(view_t *view, const assoc_records_t assocs,
 	{
 		if(strncmp(assocs.list[i].command, start, len) == 0)
 		{
-			run_using_prog(view, assocs.list[i].command, 0, background);
+			rn_open_with(view, assocs.list[i].command, 0, background);
 			return 1;
 		}
 	}
@@ -1184,9 +1219,14 @@ try_run_with_filetype(view_t *view, const assoc_records_t assocs,
 }
 
 int
-run_ext_command(const char cmd[], MacroFlags flags, int bg, int *save_msg)
+rn_ext(view_t *view, const char cmd[], const char title[], MacroFlags flags,
+		int bg, int *save_msg)
 {
-	if(bg && flags != MF_NONE && flags != MF_NO_TERM_MUX && flags != MF_IGNORE)
+	if(bg && (ma_flags_missing(flags, MF_NONE) &&
+	          ma_flags_missing(flags, MF_NO_TERM_MUX) &&
+	          ma_flags_missing(flags, MF_IGNORE) &&
+	          ma_flags_missing(flags, MF_PIPE_FILE_LIST) &&
+	          ma_flags_missing(flags, MF_PIPE_FILE_LIST_Z)))
 	{
 		ui_sb_errf("\"%s\" macro can't be combined with \" &\"",
 				ma_flags_to_str(flags));
@@ -1194,18 +1234,19 @@ run_ext_command(const char cmd[], MacroFlags flags, int bg, int *save_msg)
 		return -1;
 	}
 
-	if(flags == MF_STATUSBAR_OUTPUT)
+	if(ma_flags_present(flags, MF_STATUSBAR_OUTPUT))
 	{
-		output_to_statusbar(cmd);
+		output_to_statusbar(cmd, view, flags);
 		*save_msg = 1;
 		return -1;
 	}
-	else if(flags == MF_PREVIEW_OUTPUT)
+	else if(ma_flags_present(flags, MF_PREVIEW_OUTPUT))
 	{
-		*save_msg = output_to_preview(cmd);
+		view_t *other = (view == curr_view ? other_view : curr_view);
+		*save_msg = output_to_preview(other, cmd, flags);
 		return -1;
 	}
-	else if(flags == MF_IGNORE)
+	else if(ma_flags_present(flags, MF_IGNORE))
 	{
 		*save_msg = 0;
 		if(bg)
@@ -1213,7 +1254,7 @@ run_ext_command(const char cmd[], MacroFlags flags, int bg, int *save_msg)
 			int error;
 
 			setup_shellout_env();
-			error = (bg_run_external(cmd, 1, SHELL_BY_USER) != 0);
+			error = (bg_run_external(cmd, 1, SHELL_BY_USER, NULL) != 0);
 			cleanup_shellout_env();
 
 			if(error)
@@ -1224,29 +1265,30 @@ run_ext_command(const char cmd[], MacroFlags flags, int bg, int *save_msg)
 		}
 		else
 		{
-			output_to_nowhere(cmd);
+			output_to_nowhere(cmd, view, flags);
 		}
 		return -1;
 	}
-	else if(flags == MF_MENU_OUTPUT || flags == MF_MENU_NAV_OUTPUT)
+	else if(ma_flags_present(flags, MF_MENU_OUTPUT) ||
+			ma_flags_present(flags, MF_MENU_NAV_OUTPUT))
 	{
-		const int navigate = flags == MF_MENU_NAV_OUTPUT;
 		setup_shellout_env();
-		*save_msg = show_user_menu(curr_view, cmd, navigate) != 0;
+		*save_msg = show_user_menu(view, cmd, title, flags) != 0;
 		cleanup_shellout_env();
 	}
-	else if(flags == MF_SPLIT && curr_stats.term_multiplexer != TM_NONE)
+	else if((ma_flags_present(flags, MF_SPLIT) ||
+	         ma_flags_present(flags, MF_SPLIT_VERT)) &&
+	        curr_stats.term_multiplexer != TM_NONE)
 	{
-		run_in_split(curr_view, cmd);
+		const int vert_split = ma_flags_present(flags, MF_SPLIT_VERT);
+		run_in_split(view, cmd, vert_split);
 	}
-	else if(ONE_OF(flags, MF_CUSTOMVIEW_OUTPUT, MF_VERYCUSTOMVIEW_OUTPUT,
-				MF_CUSTOMVIEW_IOUTPUT, MF_VERYCUSTOMVIEW_IOUTPUT))
+	else if(ma_flags_present(flags, MF_CUSTOMVIEW_OUTPUT) ||
+	        ma_flags_present(flags, MF_VERYCUSTOMVIEW_OUTPUT) ||
+	        ma_flags_present(flags, MF_CUSTOMVIEW_IOUTPUT) ||
+	        ma_flags_present(flags, MF_VERYCUSTOMVIEW_IOUTPUT))
 	{
-		const int very =
-			ONE_OF(flags, MF_VERYCUSTOMVIEW_OUTPUT, MF_VERYCUSTOMVIEW_IOUTPUT);
-		const int interactive =
-			ONE_OF(flags, MF_CUSTOMVIEW_IOUTPUT, MF_VERYCUSTOMVIEW_IOUTPUT);
-		output_to_custom_flist(curr_view, cmd, very, interactive);
+		rn_for_flist(view, cmd, title, flags);
 	}
 	else
 	{
@@ -1257,17 +1299,27 @@ run_ext_command(const char cmd[], MacroFlags flags, int bg, int *save_msg)
 
 /* Executes the cmd and displays its output on the status bar. */
 static void
-output_to_statusbar(const char cmd[])
+output_to_statusbar(const char cmd[], view_t *view, MacroFlags flags)
 {
 	FILE *file, *err;
 	char buf[2048];
 	char *lines;
 	size_t len;
-	int error;
+	FILE *input_tmp = make_in_file(view, flags);
 
 	setup_shellout_env();
-	error = (bg_run_and_capture((char *)cmd, 1, &file, &err) == (pid_t)-1);
+	int error = 0;
+	if(bg_run_and_capture((char *)cmd, 1, input_tmp, &file, &err) == (pid_t)-1)
+	{
+		error = 1;
+	}
 	cleanup_shellout_env();
+
+	if(input_tmp != NULL)
+	{
+		fclose(input_tmp);
+	}
+
 	if(error)
 	{
 		show_error_msgf("Trouble running command", "Unable to run: %s", cmd);
@@ -1299,26 +1351,37 @@ output_to_statusbar(const char cmd[])
 /* Runs the command and captures its output into detached preview.  Returns new
  * value for the save_msg flag. */
 static int
-output_to_preview(const char cmd[])
+output_to_preview(view_t *view, const char cmd[], MacroFlags flags)
 {
 	if(qv_ensure_is_shown() != 0)
 	{
 		return 1;
 	}
-	view_detached_make(other_view, cmd);
+
+	modview_detached_make(view, cmd, flags);
 	return 0;
 }
 
 /* Executes the cmd ignoring its output. */
 static void
-output_to_nowhere(const char cmd[])
+output_to_nowhere(const char cmd[], view_t *view, MacroFlags flags)
 {
 	FILE *file, *err;
-	int error;
+	FILE *input_tmp = make_in_file(view, flags);
 
 	setup_shellout_env();
-	error = (bg_run_and_capture((char *)cmd, 1, &file, &err) == (pid_t)-1);
+	int error = 0;
+	if(bg_run_and_capture((char *)cmd, 1, input_tmp, &file, &err) == (pid_t)-1)
+	{
+		error = 1;
+	}
 	cleanup_shellout_env();
+
+	if(input_tmp != NULL)
+	{
+		fclose(input_tmp);
+	}
+
 	if(error)
 	{
 		show_error_msgf("Trouble running command", "Unable to run: %s", cmd);
@@ -1335,7 +1398,7 @@ output_to_nowhere(const char cmd[])
 /* Runs the cmd in a split window of terminal multiplexer.  Runs shell, if cmd
  * is NULL. */
 static void
-run_in_split(const view_t *view, const char cmd[])
+run_in_split(const view_t *view, const char cmd[], int vert_split)
 {
 	char *const escaped_cmd = (cmd == NULL)
 	                        ? strdup(cfg.shell)
@@ -1346,7 +1409,8 @@ run_in_split(const view_t *view, const char cmd[])
 	if(curr_stats.term_multiplexer == TM_TMUX)
 	{
 		char cmd[1024];
-		snprintf(cmd, sizeof(cmd), "tmux split-window %s", escaped_cmd);
+		snprintf(cmd, sizeof(cmd), "tmux split-window %s %s",
+			vert_split ? "-h" : "", escaped_cmd);
 		(void)vifm_system(cmd, SHELL_BY_APP);
 	}
 	else if(curr_stats.term_multiplexer == TM_SCREEN)
@@ -1356,8 +1420,16 @@ run_in_split(const view_t *view, const char cmd[])
 		/* "eval" executes each argument as a separate argument, but escaping rules
 		 * are not exactly like in shell, so last command is run separately. */
 		char *const escaped_dir = shell_like_escape(flist_get_dir(view), 0);
-		snprintf(cmd, sizeof(cmd), "screen -X eval chdir\\ %s 'focus bottom' "
-				"split 'focus bottom'", escaped_dir);
+		if(vert_split)
+		{
+			snprintf(cmd, sizeof(cmd), "screen -X eval chdir\\ %s 'focus right' "
+					"split\\ -v 'focus right'", escaped_dir);
+		}
+		else
+		{
+			snprintf(cmd, sizeof(cmd), "screen -X eval chdir\\ %s 'focus bottom' "
+					"split 'focus bottom'", escaped_dir);
+		}
 		free(escaped_dir);
 		(void)vifm_system(cmd, SHELL_BY_APP);
 
@@ -1375,27 +1447,62 @@ run_in_split(const view_t *view, const char cmd[])
 	free(escaped_cmd);
 }
 
-int
-output_to_custom_flist(view_t *view, const char cmd[], int very,
-		int interactive)
+void
+rn_start_bg_command(view_t *view, const char cmd[], MacroFlags flags)
 {
-	char *title;
-	int error;
+	const int supply_input = ma_flags_present(flags, MF_PIPE_FILE_LIST)
+	                      || ma_flags_present(flags, MF_PIPE_FILE_LIST_Z);
+	FILE *input = NULL;
 
-	title = format_str("!%s", cmd);
-	flist_custom_start(view, title);
-	free(title);
+	bg_run_external(cmd, ma_flags_present(flags, MF_IGNORE), SHELL_BY_USER,
+			supply_input ? &input : NULL);
+
+	if(input != NULL)
+	{
+		const int null_sep = ma_flags_present(flags, MF_PIPE_FILE_LIST_Z);
+		write_marked_paths(input, view, null_sep);
+		fclose(input);
+	}
+}
+
+int
+rn_for_flist(view_t *view, const char cmd[], const char title[],
+		MacroFlags flags)
+{
+	enum { MAX_TITLE_WIDTH = 80 };
+
+	const int very = ma_flags_present(flags, MF_VERYCUSTOMVIEW_OUTPUT)
+	              || ma_flags_present(flags, MF_VERYCUSTOMVIEW_IOUTPUT);
+	const int interactive = ma_flags_present(flags, MF_CUSTOMVIEW_IOUTPUT)
+	                     || ma_flags_present(flags, MF_VERYCUSTOMVIEW_IOUTPUT);
+
+	/* It makes sense to do escaping before adding ellipses to get a predictable
+	 * result. */
+	char *escaped_title = escape_unreadable(title);
+	char *final_title = right_ellipsis(escaped_title, MAX_TITLE_WIDTH,
+			curr_stats.ellipsis);
+	free(escaped_title);
+
+	flist_custom_start(view, final_title);
+	free(final_title);
 
 	if(interactive && curr_stats.load_stage != 0)
 	{
 		ui_shutdown();
 	}
 
+	FILE *input_tmp = make_in_file(view, flags);
+
 	setup_shellout_env();
-	error = (process_cmd_output("Loading custom view", cmd, 1, interactive,
-				&path_handler, view) != 0);
+	int error = (process_cmd_output("Loading custom view", cmd, input_tmp, 1,
+				interactive, &path_handler, view) != 0);
 	cleanup_shellout_env();
 
+	if(input_tmp != NULL)
+	{
+		fclose(input_tmp);
+	}
+
 	if(error)
 	{
 		show_error_msgf("Trouble running command", "Unable to run: %s", cmd);
@@ -1416,24 +1523,31 @@ path_handler(const char line[], void *arg)
 }
 
 int
-run_cmd_for_output(const char cmd[], char ***files, int *nfiles)
+rn_for_lines(view_t *view, const char cmd[], char ***lines, int *nlines,
+		MacroFlags flags)
 {
 	int error;
 	strlist_t list = {};
+	FILE *input_tmp = make_in_file(view, flags);
 
 	setup_shellout_env();
-	error = (process_cmd_output("Loading list", cmd, 1, 0, &line_handler,
-				&list) != 0);
+	error = (process_cmd_output("Loading list", cmd, input_tmp, 1, 0,
+				&line_handler, &list) != 0);
 	cleanup_shellout_env();
 
+	if(input_tmp != NULL)
+	{
+		fclose(input_tmp);
+	}
+
 	if(error)
 	{
 		free_string_array(list.items, list.nitems);
 		return 1;
 	}
 
-	*files = list.items;
-	*nfiles = list.nitems;
+	*lines = list.items;
+	*nlines = list.nitems;
 	return 0;
 }
 
@@ -1442,7 +1556,7 @@ static void
 line_handler(const char line[], void *arg)
 {
 	strlist_t *const list = arg;
-	list->nitems = add_to_string_array(&list->items, list->nitems, 1, line);
+	list->nitems = add_to_string_array(&list->items, list->nitems, line);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
