@@ -37,14 +37,12 @@
 #include "ui/ui.h"
 #include "utils/log.h"
 #include "utils/macros.h"
+#include "vcache.h"
 
 static void _gnuc_noreturn shutdown_nicely(int sig, const char descr[]);
 
 #ifndef _WIN32
 
-#include <sys/types.h> /* pid_t */
-#include <sys/wait.h> /* WEXITSTATUS() WIFEXITED() waitpid() */
-
 #include <stdlib.h> /* EXIT_FAILURE _Exit() */
 
 #include "utils/macros.h"
@@ -60,11 +58,11 @@ received_sigwinch(void)
 
 	if(!isendwin())
 	{
-		stats_redraw_schedule();
+		stats_redraw_later();
 	}
 	else
 	{
-		curr_stats.need_update = UT_FULL;
+		stats_refresh_later();
 	}
 }
 
@@ -72,28 +70,7 @@ static void
 received_sigcont(void)
 {
 	reset_prog_mode();
-	stats_redraw_schedule();
-}
-
-static void
-received_sigchld(void)
-{
-	int status;
-	pid_t pid;
-
-	/* This needs to be a loop in case of multiple blocked signals. */
-	while((pid = waitpid(-1, &status, WNOHANG)) > 0)
-	{
-		if(WIFEXITED(status))
-		{
-			bg_process_finished_cb(pid, WEXITSTATUS(status));
-		}
-		else if(WIFSIGNALED(status))
-		{
-			/* Child was terminated by a signal. */
-			bg_process_finished_cb(pid, -1);
-		}
-	}
+	stats_redraw_later();
 }
 
 static void
@@ -107,9 +84,6 @@ handle_signal(int sig)
 		case SIGINT:
 			ui_cancellation_request();
 			break;
-		case SIGCHLD:
-			received_sigchld();
-			break;
 		case SIGWINCH:
 			received_sigwinch();
 			break;
@@ -162,7 +136,8 @@ shutdown_nicely(int sig, const char descr[])
 	ui_shutdown();
 	term_title_update(NULL);
 	fuse_unmount_all();
-	write_info_file();
+	state_store();
+	vcache_finish();
 	fprintf(stdout, "Vifm killed by signal: %d (%s).\n", sig, descr);
 
 	/* Alternatively we could do this sequence:
@@ -186,10 +161,9 @@ setup_signals(void)
 
 	/* Assumption: we work under shell with job control support.  If it's not the
 	 * case, this code enables handling of terminal related signals the shell
-	 * wanted us to have disabled (e.g. the app will catch Ctrl-C send to another
+	 * wanted us to have disabled (e.g., the app will catch Ctrl-C sent to another
 	 * process). */
 
-	sigaction(SIGCHLD, &handle_signal_action, NULL);
 	sigaction(SIGHUP, &handle_signal_action, NULL);
 	sigaction(SIGINT, &handle_signal_action, NULL);
 	sigaction(SIGQUIT, &handle_signal_action, NULL);
