@@ -46,8 +46,9 @@
 static void sort_tree_slice(dir_entry_t *entries, const dir_entry_t *children,
 		size_t nchildren, int root);
 static void sort_sequence(dir_entry_t *entries, size_t nentries);
-static void sort_by_groups(dir_entry_t *entries, size_t nentries);
-static void sort_by_key(dir_entry_t *entries, size_t nentries, char key,
+static void sort_by_groups(dir_entry_t *entries, signed char key,
+		size_t nentries);
+static void sort_by_key(dir_entry_t *entries, size_t nentries, signed char key,
 		void *data);
 static int sort_dir_list(const void *one, const void *two);
 TSTATIC int strnumcmp(const char s[], const char t[]);
@@ -68,9 +69,9 @@ static int compare_group(const char f[], const char s[], regex_t *regex);
 static int compare_targets(const dir_entry_t *f, const dir_entry_t *s);
 
 /* View which is being sorted. */
-static view_t* view;
+static view_t *view;
 /* Picked sort array of the view. */
-static const char *view_sort;
+static const signed char *view_sort;
 /* Picked sort groups setting of the view. */
 static const char *view_sort_groups;
 /* Whether the view displays custom file list. */
@@ -198,16 +199,17 @@ sort_sequence(dir_entry_t *entries, size_t nentries)
 	int i = SK_COUNT;
 	while(--i >= 0)
 	{
-		const char sorting_key = view_sort[i];
+		const signed char sorting_key = view_sort[i];
+		const int sorting_type = abs(sorting_key);
 
-		if(abs(sorting_key) > SK_LAST)
+		if(sorting_type > SK_LAST)
 		{
 			continue;
 		}
 
-		if(sorting_key == SK_BY_GROUPS)
+		if(sorting_type == SK_BY_GROUPS)
 		{
-			sort_by_groups(entries, nentries);
+			sort_by_groups(entries, sorting_key, nentries);
 			continue;
 		}
 
@@ -222,7 +224,7 @@ sort_sequence(dir_entry_t *entries, size_t nentries)
 
 /* Sorts specified range of entries according to sorting groups option. */
 static void
-sort_by_groups(dir_entry_t *entries, size_t nentries)
+sort_by_groups(dir_entry_t *entries, signed char key, size_t nentries)
 {
 	char **groups = NULL;
 	int ngroups = 0;
@@ -233,7 +235,7 @@ sort_by_groups(dir_entry_t *entries, size_t nentries)
 	char *group = copy, *state = NULL;
 	while((group = split_and_get(group, ',', &state)) != NULL)
 	{
-		ngroups = add_to_string_array(&groups, ngroups, 1, group);
+		ngroups = add_to_string_array(&groups, ngroups, group);
 	}
 	free(copy);
 
@@ -241,12 +243,12 @@ sort_by_groups(dir_entry_t *entries, size_t nentries)
 	{
 		regex_t regex;
 		(void)regcomp(&regex, groups[i], REG_EXTENDED | REG_ICASE);
-		sort_by_key(entries, nentries, SK_BY_GROUPS, &regex);
+		sort_by_key(entries, nentries, key, &regex);
 		regfree(&regex);
 	}
 	if(optimize && ngroups != 0)
 	{
-		sort_by_key(entries, nentries, SK_BY_GROUPS, &view->primary_group);
+		sort_by_key(entries, nentries, key, &view->primary_group);
 	}
 
 	free_string_array(groups, ngroups);
@@ -254,7 +256,7 @@ sort_by_groups(dir_entry_t *entries, size_t nentries)
 
 /* Sorts specified range of entries by the key in a stable way. */
 static void
-sort_by_key(dir_entry_t *entries, size_t nentries, char key, void *data)
+sort_by_key(dir_entry_t *entries, size_t nentries, signed char key, void *data)
 {
 	sort_descending = (key < 0);
 	sort_type = (SortingKey)abs(key);
