@@ -26,6 +26,8 @@
 #undef MIN
 #endif
 
+#include <sys/types.h> /* ino_t */
+
 #include <assert.h> /* assert() */
 #include <limits.h> /* INT_MIN */
 #include <stddef.h> /* NULL */
@@ -61,6 +63,9 @@
 typedef struct
 {
 	uint64_t value;   /* Stored value. */
+#ifndef _WIN32
+	ino_t inode;      /* Inode number. */
+#endif
 	time_t timestamp; /* When the value was set. */
 }
 dcache_data_t;
@@ -70,12 +75,17 @@ static void determine_fuse_umount_cmd(status_t *stats);
 static void set_gtk_available(status_t *stats);
 static int reset_dircache(void);
 static void set_last_cmdline_command(const char cmd[]);
-static void save_into_history(const char item[], hist_t *hist, int len);
+static void dcache_get(const char path[], time_t mtime, uint64_t inode,
+		dcache_result_t *size, dcache_result_t *nitems);
 static void size_updater(void *data, void *arg);
 
 status_t curr_stats;
 
+/* Whether redraw operation is scheduled. */
 static int pending_redraw;
+/* Whether reload operation is scheduled.  Redrawing is then assumed to be
+ * scheduled too, as it's part of reloading. */
+static int pending_refresh;
 static int inside_screen;
 static int inside_tmux;
 
@@ -105,6 +115,8 @@ stats_init(config_t *config)
 	curr_stats.exec_env_type = get_exec_env_type();
 	stats_update_shell_type(config->shell);
 
+	update_string(&curr_stats.term_name, env_get("TERM"));
+
 	(void)hist_init(&curr_stats.cmd_hist, config->history_len);
 	(void)hist_init(&curr_stats.search_hist, config->history_len);
 	(void)hist_init(&curr_stats.prompt_hist, config->history_len);
@@ -120,11 +132,9 @@ stats_init(config_t *config)
 static void
 load_def_values(status_t *stats, config_t *config)
 {
-	size_t i;
-
 	pending_redraw = 0;
+	pending_refresh = 0;
 
-	stats->need_update = UT_NONE;
 	stats->last_char = 0;
 	stats->save_msg = 0;
 	stats->use_register = 0;
@@ -134,6 +144,7 @@ load_def_values(status_t *stats, config_t *config)
 	stats->use_input_bar = 1;
 	stats->drop_new_dir_hist = 0;
 	stats->load_stage = 0;
+	update_string(&stats->term_name, NULL);
 	stats->term_state = TS_NORMAL;
 	stats->ch_pos = 1;
 	stats->confirmed = 0;
@@ -146,10 +157,13 @@ load_def_values(status_t *stats, config_t *config)
 	update_string(&stats->preview.cleanup_cmd, NULL);
 	stats->preview.clearing = 0;
 
+	stats->direct_color = 0;
+
 	stats->msg_head = 0;
 	stats->msg_tail = 0;
 	stats->save_msg_in_list = 1;
 	stats->allow_sb_msg_truncation = 1;
+	size_t i;
 	for(i = 0U; i < ARRAY_LEN(stats->msgs); ++i)
 	{
 		update_string(&stats->msgs[i], NULL);
@@ -158,11 +172,14 @@ load_def_values(status_t *stats, config_t *config)
 	stats->scroll_bind_off = 0;
 	stats->split = VSPLIT;
 	stats->splitter_pos = -1;
+	stats->splitter_ratio = 0.5;
 
 	stats->sourcing_state = SOURCING_NONE;
 
 	stats->restart_in_progress = 0;
 
+	stats->reusing_statusline = 0;
+
 	stats->exec_env_type = EET_EMULATOR;
 
 	stats->term_multiplexer = TM_NONE;
@@ -191,6 +208,8 @@ load_def_values(status_t *stats, config_t *config)
 	stats->history_size = 0;
 
 	stats->ipc = NULL;
+	stats->vlua = NULL;
+	stats->plugs = NULL;
 
 #ifdef HAVE_LIBGTK
 	stats->gtk_available = 0;
@@ -251,23 +270,6 @@ reset_dircache(void)
 	return (dcache_size == NULL || dcache_nitems == NULL);
 }
 
-void
-stats_redraw_schedule(void)
-{
-	pending_redraw = 1;
-}
-
-int
-stats_redraw_fetch(void)
-{
-	if(pending_redraw)
-	{
-		pending_redraw = 0;
-		return 1;
-	}
-	return 0;
-}
-
 void
 stats_set_use_multiplexer(int use_term_multiplexer)
 {
@@ -371,6 +373,82 @@ stats_set_quickview(int on)
 	load_quickview_option();
 }
 
+void
+stats_set_splitter_pos(int position)
+{
+	double max = (curr_stats.split == HSPLIT ? cfg.lines : cfg.columns);
+	double ratio = (position < 0 ? 0.5 : (max == INT_MIN ? -1 : position/max));
+
+	curr_stats.splitter_ratio = ratio;
+	if(curr_stats.splitter_pos != position)
+	{
+		curr_stats.splitter_pos = position;
+		stats_redraw_later();
+	}
+}
+
+void
+stats_set_splitter_ratio(double ratio)
+{
+	if(ratio == -1)
+	{
+		stats_set_splitter_pos(curr_stats.splitter_pos);
+		return;
+	}
+
+	curr_stats.splitter_ratio = ratio;
+
+	int max = (curr_stats.split == HSPLIT ? cfg.lines : cfg.columns);
+	if(max == INT_MIN)
+	{
+		/* Can't compute position from ratio, so leave it as is. */
+		return;
+	}
+
+	int position = (ratio == 0.5 ? -1 : max*ratio + 0.5);
+	if(curr_stats.splitter_pos != position)
+	{
+		curr_stats.splitter_pos = position;
+		stats_redraw_later();
+	}
+}
+
+void
+stats_redraw_later(void)
+{
+	pending_redraw = 1;
+}
+
+void
+stats_refresh_later(void)
+{
+	pending_refresh = 1;
+}
+
+UpdateType
+stats_update_fetch(void)
+{
+	if(pending_refresh)
+	{
+		pending_refresh = 0;
+		pending_redraw = 0;
+		return UT_FULL;
+	}
+	if(pending_redraw)
+	{
+		pending_redraw = 0;
+		return UT_REDRAW;
+	}
+	return UT_NONE;
+}
+
+int
+stats_redraw_planned(void)
+{
+	return pending_refresh != 0
+	    || pending_redraw != 0;
+}
+
 void
 stats_silence_ui(int more)
 {
@@ -380,15 +458,12 @@ stats_silence_ui(int more)
 	}
 	else if(--silent_ui == 0)
 	{
-		if(silence_skipped_updates)
-		{
-			silence_skipped_updates = 0;
-			pending_redraw = 0;
-			modes_redraw();
-		}
+		stats_unsilence_ui();
+	}
+	else if(silent_ui < 0)
+	{
+		silent_ui = 0;
 	}
-
-	assert(silent_ui >= 0 && "Unbalanced calls to stats_silence_ui()");
 }
 
 int
@@ -402,50 +477,27 @@ stats_silenced_ui(void)
 	return 0;
 }
 
+void
+stats_unsilence_ui(void)
+{
+	silent_ui = 0;
+	if(silence_skipped_updates)
+	{
+		silence_skipped_updates = 0;
+		pending_redraw = 0;
+		modes_redraw();
+	}
+}
+
 void
 hists_resize(int new_size)
 {
-	const int old_size = curr_stats.history_size;
-	const int delta = new_size - old_size;
-
-	if(new_size <= 0)
-	{
-		hist_reset(&curr_stats.search_hist, old_size);
-		hist_reset(&curr_stats.cmd_hist, old_size);
-		hist_reset(&curr_stats.prompt_hist, old_size);
-		hist_reset(&curr_stats.filter_hist, old_size);
-		curr_stats.history_size = 0;
-		return;
-	}
-
 	curr_stats.history_size = new_size;
 
-	if(delta < 0)
-	{
-		hist_trunc(&curr_stats.search_hist, new_size, -delta);
-		hist_trunc(&curr_stats.cmd_hist, new_size, -delta);
-		hist_trunc(&curr_stats.prompt_hist, new_size, -delta);
-		hist_trunc(&curr_stats.filter_hist, new_size, -delta);
-	}
-
-	curr_stats.cmd_hist.items = reallocarray(curr_stats.cmd_hist.items, new_size,
-			sizeof(char *));
-	curr_stats.search_hist.items = reallocarray(curr_stats.search_hist.items,
-			new_size, sizeof(char *));
-	curr_stats.prompt_hist.items = reallocarray(curr_stats.prompt_hist.items,
-			new_size, sizeof(char *));
-	curr_stats.filter_hist.items = reallocarray(curr_stats.filter_hist.items,
-			new_size, sizeof(char *));
-
-	if(delta > 0)
-	{
-		const size_t str_item_len = sizeof(char *)*delta;
-
-		memset(curr_stats.cmd_hist.items + old_size, 0, str_item_len);
-		memset(curr_stats.search_hist.items + old_size, 0, str_item_len);
-		memset(curr_stats.prompt_hist.items + old_size, 0, str_item_len);
-		memset(curr_stats.filter_hist.items + old_size, 0, str_item_len);
-	}
+	hist_resize(&curr_stats.search_hist, new_size);
+	hist_resize(&curr_stats.cmd_hist, new_size);
+	hist_resize(&curr_stats.prompt_hist, new_size);
+	hist_resize(&curr_stats.filter_hist, new_size);
 }
 
 void
@@ -457,7 +509,7 @@ hists_commands_save(const char command[])
 		{
 			set_last_cmdline_command(command);
 		}
-		save_into_history(command, &curr_stats.cmd_hist, curr_stats.history_size);
+		hist_add(&curr_stats.cmd_hist, command, -1);
 	}
 }
 
@@ -477,29 +529,19 @@ set_last_cmdline_command(const char cmd[])
 void
 hists_search_save(const char pattern[])
 {
-	save_into_history(pattern, &curr_stats.search_hist, curr_stats.history_size);
+	hist_add(&curr_stats.search_hist, pattern, -1);
 }
 
 void
 hists_prompt_save(const char input[])
 {
-	save_into_history(input, &curr_stats.prompt_hist, curr_stats.history_size);
+	hist_add(&curr_stats.prompt_hist, input, -1);
 }
 
 void
 hists_filter_save(const char input[])
 {
-	save_into_history(input, &curr_stats.filter_hist, curr_stats.history_size);
-}
-
-/* Adaptor for the hist_add() function, which handles signed history length. */
-static void
-save_into_history(const char item[], hist_t *hist, int len)
-{
-	if(len >= 0)
-	{
-		hist_add(hist, item, len);
-	}
+	hist_add(&curr_stats.filter_hist, input, -1);
 }
 
 const char *
@@ -507,36 +549,24 @@ hists_search_last(void)
 {
 	return hist_is_empty(&curr_stats.search_hist)
 	     ? ""
-	     : curr_stats.search_hist.items[0];
+	     : curr_stats.search_hist.items[0].text;
 }
 
 void
-dcache_get_at(const char path[], uint64_t *size, uint64_t *nitems)
+dcache_get_at(const char path[], time_t mtime, uint64_t inode, uint64_t *size,
+		uint64_t *nitems)
 {
-	/* Initialization to make condition false by default. */
-	dcache_data_t size_data, nitems_data;
-
-	pthread_mutex_lock(&dcache_size_mutex);
-	if(fsdata_get(dcache_size, path, &size_data, sizeof(size_data)) != 0)
-	{
-		size_data.value = DCACHE_UNKNOWN;
-	}
-	pthread_mutex_unlock(&dcache_size_mutex);
-
-	pthread_mutex_lock(&dcache_nitems_mutex);
-	if(fsdata_get(dcache_nitems, path, &nitems_data, sizeof(nitems_data)) != 0)
-	{
-		nitems_data.value = DCACHE_UNKNOWN;
-	}
-	pthread_mutex_unlock(&dcache_nitems_mutex);
+	dcache_result_t size_res, nitems_res;
+	dcache_get(path, mtime, inode, (size == NULL ? NULL : &size_res),
+			(nitems == NULL ? NULL : &nitems_res));
 
 	if(size != NULL)
 	{
-		*size = size_data.value;
+		*size = (size_res.is_valid ? size_res.value : DCACHE_UNKNOWN);
 	}
 	if(nitems != NULL)
 	{
-		*nitems = nitems_data.value;
+		*nitems = (nitems_res.is_valid ? nitems_res.value : DCACHE_UNKNOWN);
 	}
 }
 
@@ -544,37 +574,63 @@ void
 dcache_get_of(const dir_entry_t *entry, dcache_result_t *size,
 		dcache_result_t *nitems)
 {
-	dcache_data_t size_data, nitems_data;
+	if(size == NULL && nitems == NULL)
+	{
+		return;
+	}
 
 	char full_path[PATH_MAX + 1];
 	get_full_path_of(entry, sizeof(full_path), full_path);
 
-	size->value = DCACHE_UNKNOWN;
-	size->is_valid = 0;
+	uint64_t inode = get_true_inode(entry);
+	dcache_get(full_path, entry->mtime, inode, size, nitems);
+}
 
-	nitems->value = DCACHE_UNKNOWN;
-	nitems->is_valid = 0;
-
-	pthread_mutex_lock(&dcache_size_mutex);
-	if(fsdata_get(dcache_size, full_path, &size_data, sizeof(size_data)) == 0)
+/* Retrieves information about the path checking whether it's outdated.  size
+ * and/or nitems can be NULL. */
+static void
+dcache_get(const char path[], time_t mtime, uint64_t inode,
+		dcache_result_t *size, dcache_result_t *nitems)
+{
+	if(size != NULL)
 	{
-		size->value = size_data.value;
-		/* We check strictly for less than to handle scenario when multiple changes
-		 * occurred during the same second. */
-		size->is_valid = (entry->mtime < size_data.timestamp);
-	}
-	pthread_mutex_unlock(&dcache_size_mutex);
+		size->value = DCACHE_UNKNOWN;
+		size->is_valid = 0;
 
-	pthread_mutex_lock(&dcache_nitems_mutex);
-	if(fsdata_get(dcache_nitems, full_path, &nitems_data,
-				sizeof(nitems_data)) == 0)
+		pthread_mutex_lock(&dcache_size_mutex);
+		dcache_data_t size_data;
+		if(fsdata_get(dcache_size, path, &size_data, sizeof(size_data)) == 0)
+		{
+			size->value = size_data.value;
+			/* We check strictly for less than to handle scenario when multiple
+			 * changes occurred during the same second. */
+			size->is_valid = (mtime < size_data.timestamp);
+#ifndef _WIN32
+			size->is_valid &= (inode == size_data.inode);
+#endif
+		}
+		pthread_mutex_unlock(&dcache_size_mutex);
+	}
+
+	if(nitems != NULL)
 	{
-		nitems->value = nitems_data.value;
-		/* We check strictly for less than to handle scenario when multiple changes
-		 * occurred during the same second. */
-		nitems->is_valid = (entry->mtime < nitems_data.timestamp);
+		nitems->value = DCACHE_UNKNOWN;
+		nitems->is_valid = 0;
+
+		pthread_mutex_lock(&dcache_nitems_mutex);
+		dcache_data_t nitems_data;
+		if(fsdata_get(dcache_nitems, path, &nitems_data, sizeof(nitems_data)) == 0)
+		{
+			nitems->value = nitems_data.value;
+			/* We check strictly for less than to handle scenario when multiple
+			 * changes occurred during the same second. */
+			nitems->is_valid = (mtime < nitems_data.timestamp);
+#ifndef _WIN32
+			nitems->is_valid &= (inode == nitems_data.inode);
+#endif
+		}
+		pthread_mutex_unlock(&dcache_nitems_mutex);
 	}
-	pthread_mutex_unlock(&dcache_nitems_mutex);
 }
 
 void
@@ -596,14 +652,17 @@ size_updater(void *data, void *arg)
 }
 
 int
-dcache_set_at(const char path[], uint64_t size, uint64_t nitems)
+dcache_set_at(const char path[], uint64_t inode, uint64_t size, uint64_t nitems)
 {
 	int ret = 0;
 	const time_t ts = time(NULL);
 
 	if(size != DCACHE_UNKNOWN)
 	{
-		const dcache_data_t data = { .value = size, .timestamp = ts };
+		dcache_data_t data = { .value = size, .timestamp = ts };
+#ifndef _WIN32
+		data.inode = (ino_t)inode;
+#endif
 
 		pthread_mutex_lock(&dcache_size_mutex);
 		ret |= fsdata_set(dcache_size, path, &data, sizeof(data));
@@ -612,7 +671,10 @@ dcache_set_at(const char path[], uint64_t size, uint64_t nitems)
 
 	if(nitems != DCACHE_UNKNOWN)
 	{
-		const dcache_data_t data = { .value = nitems, .timestamp = ts };
+		dcache_data_t data = { .value = nitems, .timestamp = ts };
+#ifndef _WIN32
+		data.inode = (ino_t)inode;
+#endif
 
 		pthread_mutex_lock(&dcache_nitems_mutex);
 		ret |= fsdata_set(dcache_nitems, path, &data, sizeof(data));
