@@ -23,6 +23,8 @@
 #include <regex.h> /* regcomp() regexec() */
 
 #include <assert.h> /* assert() */
+#include <limits.h> /* INT_MAX */
+#include <math.h> /* abs() */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* snprintf() */
 #include <stdlib.h> /* free() */
@@ -52,6 +54,7 @@ char *HI_GROUPS[] = {
 	[DIRECTORY_COLOR]    = "Directory",
 	[LINK_COLOR]         = "Link",
 	[BROKEN_LINK_COLOR]  = "BrokenLink",
+	[HARD_LINK_COLOR]    = "HardLink",
 	[SOCKET_COLOR]       = "Socket",
 	[DEVICE_COLOR]       = "Device",
 	[FIFO_COLOR]         = "Fifo",
@@ -81,6 +84,9 @@ char *HI_GROUPS[] = {
 	[USER7_COLOR]        = "User7",
 	[USER8_COLOR]        = "User8",
 	[USER9_COLOR]        = "User9",
+	[OTHER_WIN_COLOR]    = "OtherWin",
+	[LINE_NUM_COLOR]     = "LineNr",
+	[ODD_LINE_COLOR]     = "OddLine",
 };
 ARRAY_GUARD(HI_GROUPS, MAXNUM_COLOR);
 
@@ -89,6 +95,7 @@ const char *HI_GROUPS_DESCR[] = {
 	[DIRECTORY_COLOR]    = "directories and links to directories",
 	[LINK_COLOR]         = "symbolic links",
 	[BROKEN_LINK_COLOR]  = "dangling symbolic links",
+	[HARD_LINK_COLOR]    = "regular files with more than one hard link",
 	[SOCKET_COLOR]       = "sockets",
 	[DEVICE_COLOR]       = "device file",
 	[FIFO_COLOR]         = "pipe",
@@ -118,6 +125,9 @@ const char *HI_GROUPS_DESCR[] = {
 	[USER7_COLOR]        = "user color #7",
 	[USER8_COLOR]        = "user color #8",
 	[USER9_COLOR]        = "user color #9",
+	[OTHER_WIN_COLOR]    = "additional highlighting of inactive pane",
+	[LINE_NUM_COLOR]     = "color of line number column in panes",
+	[ODD_LINE_COLOR]     = "color of every second entry line in a pane",
 };
 ARRAY_GUARD(HI_GROUPS_DESCR, ARRAY_LEN(HI_GROUPS));
 
@@ -391,6 +401,50 @@ char *XTERM256_COLOR_NAMES[256] = {
 	[255] = "Grey93",
 };
 
+/* Maps 256 colors into corresponding #RRGGBB values.  The array is offset by
+ * one to allow handling of default color (-1). */
+static const int XTERM256_PALETTE[] = {
+	-1,
+	0, 1, 2, 3, 4, 5, 6, 7,
+	0x808080, 0xff0000, 0x00ff00, 0xffff00, 0x0000ff, 0xff00ff, 0x00ffff,
+	0xffffff, 0x000000, 0x00005f, 0x000087, 0x0000af, 0x0000d7, 0x0000ff,
+	0x005f00, 0x005f5f, 0x005f87, 0x005faf, 0x005fd7, 0x005fff, 0x008700,
+	0x00875f, 0x008787, 0x0087af, 0x0087d7, 0x0087ff, 0x00af00, 0x00af5f,
+	0x00af87, 0x00afaf, 0x00afd7, 0x00afff, 0x00d700, 0x00d75f, 0x00d787,
+	0x00d7af, 0x00d7d7, 0x00d7ff, 0x00ff00, 0x00ff5f, 0x00ff87, 0x00ffaf,
+	0x00ffd7, 0x00ffff, 0x5f0000, 0x5f005f, 0x5f0087, 0x5f00af, 0x5f00d7,
+	0x5f00ff, 0x5f5f00, 0x5f5f5f, 0x5f5f87, 0x5f5faf, 0x5f5fd7, 0x5f5fff,
+	0x5f8700, 0x5f875f, 0x5f8787, 0x5f87af, 0x5f87d7, 0x5f87ff, 0x5faf00,
+	0x5faf5f, 0x5faf87, 0x5fafaf, 0x5fafd7, 0x5fafff, 0x5fd700, 0x5fd75f,
+	0x5fd787, 0x5fd7af, 0x5fd7d7, 0x5fd7ff, 0x5fff00, 0x5fff5f, 0x5fff87,
+	0x5fffaf, 0x5fffd7, 0x5fffff, 0x870000, 0x87005f, 0x870087, 0x8700af,
+	0x8700d7, 0x8700ff, 0x875f00, 0x875f5f, 0x875f87, 0x875faf, 0x875fd7,
+	0x875fff, 0x878700, 0x87875f, 0x878787, 0x8787af, 0x8787d7, 0x8787ff,
+	0x87af00, 0x87af5f, 0x87af87, 0x87afaf, 0x87afd7, 0x87afff, 0x87d700,
+	0x87d75f, 0x87d787, 0x87d7af, 0x87d7d7, 0x87d7ff, 0x87ff00, 0x87ff5f,
+	0x87ff87, 0x87ffaf, 0x87ffd7, 0x87ffff, 0xaf0000, 0xaf005f, 0xaf0087,
+	0xaf00af, 0xaf00d7, 0xaf00ff, 0xaf5f00, 0xaf5f5f, 0xaf5f87, 0xaf5faf,
+	0xaf5fd7, 0xaf5fff, 0xaf8700, 0xaf875f, 0xaf8787, 0xaf87af, 0xaf87d7,
+	0xaf87ff, 0xafaf00, 0xafaf5f, 0xafaf87, 0xafafaf, 0xafafd7, 0xafafff,
+	0xafd700, 0xafd75f, 0xafd787, 0xafd7af, 0xafd7d7, 0xafd7ff, 0xafff00,
+	0xafff5f, 0xafff87, 0xafffaf, 0xafffd7, 0xafffff, 0xd70000, 0xd7005f,
+	0xd70087, 0xd700af, 0xd700d7, 0xd700ff, 0xd75f00, 0xd75f5f, 0xd75f87,
+	0xd75faf, 0xd75fd7, 0xd75fff, 0xd78700, 0xd7875f, 0xd78787, 0xd787af,
+	0xd787d7, 0xd787ff, 0xdfaf00, 0xdfaf5f, 0xdfaf87, 0xdfafaf, 0xdfafdf,
+	0xdfafff, 0xdfdf00, 0xdfdf5f, 0xdfdf87, 0xdfdfaf, 0xdfdfdf, 0xdfdfff,
+	0xdfff00, 0xdfff5f, 0xdfff87, 0xdfffaf, 0xdfffdf, 0xdfffff, 0xff0000,
+	0xff005f, 0xff0087, 0xff00af, 0xff00df, 0xff00ff, 0xff5f00, 0xff5f5f,
+	0xff5f87, 0xff5faf, 0xff5fdf, 0xff5fff, 0xff8700, 0xff875f, 0xff8787,
+	0xff87af, 0xff87df, 0xff87ff, 0xffaf00, 0xffaf5f, 0xffaf87, 0xffafaf,
+	0xffafdf, 0xffafff, 0xffdf00, 0xffdf5f, 0xffdf87, 0xffdfaf, 0xffdfdf,
+	0xffdfff, 0xffff00, 0xffff5f, 0xffff87, 0xffffaf, 0xffffdf, 0xffffff,
+	0x080808, 0x121212, 0x1c1c1c, 0x262626, 0x303030, 0x3a3a3a, 0x444444,
+	0x4e4e4e, 0x585858, 0x626262, 0x6c6c6c, 0x767676, 0x808080, 0x8a8a8a,
+	0x949494, 0x9e9e9e, 0xa8a8a8, 0xb2b2b2, 0xbcbcbc, 0xc6c6c6, 0xd0d0d0,
+	0xdadada, 0xe4e4e4, 0xeeeeee,
+};
+ARRAY_GUARD(XTERM256_PALETTE, 1 + 256);
+
 /* Default color scheme definition. */
 static const col_attr_t default_cs[] = {
 	                      /* fg             bg           attr */
@@ -398,6 +452,7 @@ static const col_attr_t default_cs[] = {
 	[DIRECTORY_COLOR]    = { COLOR_CYAN,    -1,          A_BOLD                  },
 	[LINK_COLOR]         = { COLOR_YELLOW,  -1,          A_BOLD                  },
 	[BROKEN_LINK_COLOR]  = { COLOR_RED,     -1,          A_BOLD                  },
+	[HARD_LINK_COLOR]    = { COLOR_YELLOW,  -1,          0                       },
 	[SOCKET_COLOR]       = { COLOR_MAGENTA, -1,          A_BOLD                  },
 	[DEVICE_COLOR]       = { COLOR_RED,     -1,          A_BOLD                  },
 	[FIFO_COLOR]         = { COLOR_CYAN,    -1,          A_BOLD                  },
@@ -427,6 +482,9 @@ static const col_attr_t default_cs[] = {
 	[USER7_COLOR]        = { -1,            -1,          -1                      },
 	[USER8_COLOR]        = { -1,            -1,          -1                      },
 	[USER9_COLOR]        = { -1,            -1,          -1                      },
+	[OTHER_WIN_COLOR]    = { -1,            -1,          -1                      },
+	[LINE_NUM_COLOR]     = { -1,            -1,          -1                      },
+	[ODD_LINE_COLOR]     = { -1,            -1,          -1                      },
 };
 ARRAY_GUARD(default_cs, MAXNUM_COLOR);
 
@@ -442,6 +500,11 @@ static const char * get_global_colors_dir(void);
 static void check_cs(col_scheme_t *cs);
 static void load_color_pairs(col_scheme_t *cs);
 static void ensure_dir_map_exists(void);
+static void overlap_cterm_colors(col_attr_t *color,
+		const col_attr_t *admixture);
+static void overlap_gui_colors(col_attr_t *color, const col_attr_t *admixture);
+static col_attr_t convert_to_gui(const col_attr_t *color);
+static int color_diff(int a, int b);
 
 /* Mapping of color schemes associations onto file system tree. */
 static fsddata_t *dir_map;
@@ -483,8 +546,8 @@ cs_rename_all(void)
 #ifndef _WIN32
 		unsigned char type;
 #endif
-		char full_old_path[PATH_MAX + 16 + NAME_MAX];
-		char full_new_path[PATH_MAX + 16 + NAME_MAX];
+		char full_old_path[PATH_MAX + 32 + NAME_MAX];
+		char full_new_path[PATH_MAX + 32 + NAME_MAX];
 		snprintf(full_old_path, sizeof(full_old_path), "%s/%s", cfg.colors_dir,
 				d->d_name);
 		snprintf(full_new_path, sizeof(full_new_path), "%s/%s.vifm", cfg.colors_dir,
@@ -557,7 +620,7 @@ list_cs_files(int *len)
 int
 cs_exists(const char name[])
 {
-	char cs_path[PATH_MAX + 1];
+	char cs_path[PATH_MAX + 32];
 	get_cs_path(name, cs_path, sizeof(cs_path));
 	return is_regular_file(cs_path);
 }
@@ -603,7 +666,8 @@ cs_write(void)
 	fprintf(fp, "\" Cyan = 6\n");
 	fprintf(fp, "\" White = 7\n\n");
 
-	fprintf(fp, "\" Light versions of colors are also available (set bold attribute):\n");
+	fprintf(fp, "\" Light versions of colors are also available (they set bold\n"
+	            "\" attribute in terminals with less than 16 colors):\n");
 	fprintf(fp, "\" LightBlack\n");
 	fprintf(fp, "\" LightRed\n");
 	fprintf(fp, "\" LightGreen\n");
@@ -619,6 +683,7 @@ cs_write(void)
 	fprintf(fp, "\" reverse or inverse\n");
 	fprintf(fp, "\" standout\n");
 	fprintf(fp, "\" italic (on unsupported systems becomes reverse)\n");
+	fprintf(fp, "\" combine\n");
 	fprintf(fp, "\" none\n\n");
 
 	fprintf(fp, "\" Vifm supports 256 colors you can use color numbers 0-255\n");
@@ -634,49 +699,56 @@ cs_write(void)
 	for(i = 0U; i < ARRAY_LEN(default_cs); ++i)
 	{
 		char fg_buf[16], bg_buf[16];
+		cs_color_to_str(default_cs[i].fg, sizeof(fg_buf), fg_buf, /*is_gui=*/0);
+		cs_color_to_str(default_cs[i].bg, sizeof(bg_buf), bg_buf, /*is_gui=*/0);
 
-		if(i == OTHER_LINE_COLOR)
+		if(default_cs[i].attr == -1)
 		{
-			/* Skip OtherLine as there is no way to express defaults. */
-			continue;
+			fprintf(fp, "highlight %s ctermfg=%s ctermbg=%s\n", HI_GROUPS[i], fg_buf,
+					bg_buf);
+		}
+		else
+		{
+			fprintf(fp, "highlight %s cterm=%s ctermfg=%s ctermbg=%s\n", HI_GROUPS[i],
+					cs_attrs_to_str(&default_cs[i], /*gui_part=*/0), fg_buf, bg_buf);
 		}
-
-		cs_color_to_str(default_cs[i].fg, sizeof(fg_buf), fg_buf);
-		cs_color_to_str(default_cs[i].bg, sizeof(bg_buf), bg_buf);
-
-		fprintf(fp, "highlight %s cterm=%s ctermfg=%s ctermbg=%s\n", HI_GROUPS[i],
-				cs_attrs_to_str(default_cs[i].attr), fg_buf, bg_buf);
 	}
 
 	fclose(fp);
 }
 
 void
-cs_color_to_str(int color, size_t buf_len, char str_buf[])
+cs_color_to_str(int color, size_t buf_len, char str_buf[], int is_gui)
 {
+	int color_limit = (is_gui ? 8 : ARRAY_LEN(XTERM256_COLOR_NAMES));
+
 	if(color == -1)
 	{
 		copy_str(str_buf, buf_len, "default");
 	}
-	else if(color >= 0 && color < (int)ARRAY_LEN(XTERM256_COLOR_NAMES))
+	else if(color >= 0 && color < color_limit)
 	{
 		copy_str(str_buf, buf_len, XTERM256_COLOR_NAMES[color]);
 	}
+	else if(is_gui)
+	{
+		snprintf(str_buf, buf_len, "#%06x", color);
+	}
 	else
 	{
 		snprintf(str_buf, buf_len, "%d", color);
 	}
 }
 
-int
+void
 cs_load_primary(const char name[])
 {
 	col_scheme_t prev_cs = {};
 
 	if(!cs_exists(name))
 	{
-		show_error_msgf("Color Scheme", "Invalid color scheme name: \"%s\"", name);
-		return 0;
+		show_error_msgf("Color Scheme", "No such color scheme: \"%s\"", name);
+		return;
 	}
 
 	cs_assign(&prev_cs, &cfg.cs);
@@ -687,27 +759,69 @@ cs_load_primary(const char name[])
 	{
 		restore_primary_cs(&prev_cs);
 		show_error_msgf("Color Scheme Sourcing",
-				"An error occurred on loading color scheme: \"%s\"", name);
-		cfg.cs.state = CSS_NORMAL;
-		return 0;
+				"An error occurred on sourcing color scheme: \"%s\"", name);
+		return;
 	}
-	copy_str(cfg.cs.name, sizeof(cfg.cs.name), name);
+
 	check_cs(&cfg.cs);
-
-	update_attributes();
-
 	if(cfg.cs.state == CSS_DEFAULTED)
 	{
 		restore_primary_cs(&prev_cs);
 		show_error_msgf("Color Scheme Error",
 				"\"%s\" color scheme is not supported by the terminal, restored \"%s\"",
 				name, prev_cs.name);
-		return 0;
+		return;
 	}
 
+	copy_str(cfg.cs.name, sizeof(cfg.cs.name), name);
+	update_attributes();
+
 	free_cs_highlights(&prev_cs);
 	cfg.cs.state = CSS_NORMAL;
-	return 0;
+}
+
+void
+cs_load_primary_list(char *names[], int count)
+{
+	col_scheme_t prev_cs = {};
+	cs_assign(&prev_cs, &cfg.cs);
+
+	while(count-- > 0)
+	{
+		const char *name = *names++;
+
+		cs_assign(&cfg.cs, &prev_cs);
+		curr_stats.cs = &cfg.cs;
+		cfg.cs.state = CSS_LOADING;
+
+		if(!cs_exists(name))
+		{
+			continue;
+		}
+
+		if(source_cs(name) != 0)
+		{
+			show_error_msgf("Color Scheme Sourcing",
+					"An error occurred on sourcing color scheme: \"%s\"", name);
+			continue;
+		}
+
+		check_cs(&cfg.cs);
+		if(cfg.cs.state == CSS_DEFAULTED)
+		{
+			continue;
+		}
+
+		copy_str(cfg.cs.name, sizeof(cfg.cs.name), name);
+		update_attributes();
+
+		free_cs_highlights(&prev_cs);
+		cfg.cs.state = CSS_NORMAL;
+
+		return;
+	}
+
+	restore_primary_cs(&prev_cs);
 }
 
 /* Restore previous state of primary color scheme. */
@@ -756,7 +870,6 @@ reset_to_default_cs(col_scheme_t *cs)
 	reset_cs_colors(cs);
 
 	copy_str(cs->name, sizeof(cs->name), DEF_CS_NAME);
-	copy_str(cs->dir, sizeof(cs->dir), "/");
 
 	cs->state = CSS_NORMAL;
 }
@@ -828,12 +941,10 @@ clone_cs_highlights(const col_scheme_t *from)
 int
 cs_load_local(int left, const char dir[])
 {
-	char *p;
-	char t;
-	int altered;
-
-	if(dir_map == NULL)
+	if(dir_map == NULL || curr_stats.cs != &cfg.cs)
 	{
+		/* Either have nothing to do, or we're already doing it (sourcing a file can
+		 * trigger view redraws). */
 		return 0;
 	}
 
@@ -841,8 +952,10 @@ cs_load_local(int left, const char dir[])
 	cs_assign(curr_stats.cs, &cfg.cs);
 
 	/* TODO: maybe use split_and_get() here as in io/iop:iop_mkdir(). */
-	p = (char *)dir;
-	altered = 0;
+	char *const dir_copy = strdup(dir);
+	char *p = dir_copy;
+	int altered = 0;
+	char t;
 	do
 	{
 		void *name;
@@ -850,16 +963,17 @@ cs_load_local(int left, const char dir[])
 		t = *p;
 		*p = '\0';
 
-		if(fsddata_get(dir_map, dir, &name) == 0 && cs_exists(name))
+		if(fsddata_get(dir_map, dir_copy, &name) == 0 && cs_exists(name))
 		{
 			(void)source_cs(name);
 			altered = 1;
 		}
 
 		*p = t;
-		p = until_first(p + 1, '/');
+		p = (t == '\0' ? NULL : until_first(p + 1, '/'));
 	}
 	while(t != '\0');
+	free(dir_copy);
 
 	curr_stats.cs = &cfg.cs;
 
@@ -916,16 +1030,12 @@ get_cs_path(const char name[], char buf[], size_t buf_size)
 static const char *
 get_global_colors_dir(void)
 {
-#ifndef _WIN32
-	return GLOBAL_COLORS_DIR;
-#else
 	static char dir_path[PATH_MAX + 1];
 	if(dir_path[0] == '\0')
 	{
-		snprintf(dir_path, sizeof(dir_path), "%s/colors", get_installed_data_dir());
+		snprintf(dir_path, sizeof(dir_path), "%s/colors", get_sys_conf_dir());
 	}
 	return dir_path;
-#endif
 }
 
 /* Checks whether colorscheme is in unusable state and resets it to normal
@@ -947,7 +1057,7 @@ load_color_pairs(col_scheme_t *cs)
 	int i;
 	for(i = 0; i < MAXNUM_COLOR; ++i)
 	{
-		cs->pair[i] = colmgr_get_pair(cs->color[i].fg, cs->color[i].bg);
+		cs->pair[i] = cs_load_color(&cs->color[i]);
 	}
 }
 
@@ -980,12 +1090,20 @@ cs_complete(const char name[])
 }
 
 const char *
-cs_attrs_to_str(int attrs)
+cs_attrs_to_str(const col_attr_t *color, int gui_part)
 {
 	static char result[64];
+
+	const int attrs = (gui_part ? color->gui_attr : color->attr);
+	const int combine =
+		(gui_part ? color->combine_gui_attrs : color->combine_attrs);
+
+	if((attrs == 0 || attrs == -1) && !combine)
+	{
+		return strcpy(result, "none");
+	}
+
 	result[0] = '\0';
-	if(attrs == 0)
-		strcpy(result, "none,");
 	if((attrs & A_BOLD) == A_BOLD)
 		strcat(result, "bold,");
 	if((attrs & A_UNDERLINE) == A_UNDERLINE)
@@ -998,6 +1116,8 @@ cs_attrs_to_str(int attrs)
 	if((attrs & A_ITALIC) == A_ITALIC)
 		strcat(result, "italic,");
 #endif
+	if(combine)
+		strcat(result, "combine,");
 	if(result[0] != '\0')
 		result[strlen(result) - 1] = '\0';
 	return result;
@@ -1022,26 +1142,90 @@ ensure_dir_map_exists(void)
 {
 	if(dir_map == NULL)
 	{
-		dir_map = fsddata_create(1);
+		dir_map = fsddata_create(1, 1);
 	}
 }
 
 void
-cs_mix_colors(col_attr_t *base, const col_attr_t *mixup)
+cs_mix_colors(col_attr_t *color, const col_attr_t *admixture)
 {
-	if(mixup->fg != -1)
+	if(curr_stats.direct_color)
 	{
-		base->fg = mixup->fg;
+		*color = convert_to_gui(color);
 	}
 
-	if(mixup->bg != -1)
+	const int attr = color->attr;
+	const int gui_attr = color->gui_attr;
+
+	cs_overlap_colors(color, admixture);
+
+	if(attr != -1 && admixture->combine_attrs)
 	{
-		base->bg = mixup->bg;
+		color->attr |= attr;
 	}
 
-	if(mixup->attr != -1)
+	if(gui_attr != -1 && admixture->combine_gui_attrs)
 	{
-		base->attr = mixup->attr;
+		color->gui_attr |= gui_attr;
+	}
+}
+
+void
+cs_overlap_colors(col_attr_t *color, const col_attr_t *admixture)
+{
+	if(curr_stats.direct_color)
+	{
+		*color = convert_to_gui(color);
+		const col_attr_t gui_admixture = convert_to_gui(admixture);
+		overlap_gui_colors(color, &gui_admixture);
+	}
+	else
+	{
+		overlap_cterm_colors(color, admixture);
+	}
+}
+
+/* Overlaps cterm colors of *admixture into the *color.  Non-transparent
+ * properties of *admixture are transferred onto *color.  Attributes are never
+ * combined. */
+static void
+overlap_cterm_colors(col_attr_t *color, const col_attr_t *admixture)
+{
+	if(admixture->fg != -1)
+	{
+		color->fg = admixture->fg;
+	}
+
+	if(admixture->bg != -1)
+	{
+		color->bg = admixture->bg;
+	}
+
+	if(admixture->attr != -1)
+	{
+		color->attr = admixture->attr;
+	}
+}
+
+/* Overlaps gui colors of *admixture into the *color.  Non-transparent
+ * properties of *admixture are transferred onto *color.  Attributes are never
+ * combined. */
+static void
+overlap_gui_colors(col_attr_t *color, const col_attr_t *admixture)
+{
+	if(admixture->gui_fg != -1)
+	{
+		color->gui_fg = admixture->gui_fg;
+	}
+
+	if(admixture->gui_bg != -1)
+	{
+		color->gui_bg = admixture->gui_bg;
+	}
+
+	if(admixture->gui_attr != -1)
+	{
+		color->gui_attr = admixture->gui_attr;
 	}
 }
 
@@ -1086,8 +1270,10 @@ cs_add_file_hi(struct matchers_t *matchers, const col_attr_t *hi)
 const col_attr_t *
 cs_get_file_hi(const col_scheme_t *cs, const char fname[], int *hi_hint)
 {
-	int i;
-
+	if(*hi_hint == INT_MAX)
+	{
+		return NULL;
+	}
 	if(*hi_hint != -1)
 	{
 		assert(*hi_hint >= 0 && "Wrong index.");
@@ -1095,6 +1281,7 @@ cs_get_file_hi(const col_scheme_t *cs, const char fname[], int *hi_hint)
 		return &cs->file_hi[*hi_hint].hi;
 	}
 
+	int i;
 	for(i = 0; i < cs->file_hi_count; ++i)
 	{
 		const file_hi_t *const file_hi = &cs->file_hi[i];
@@ -1104,6 +1291,8 @@ cs_get_file_hi(const col_scheme_t *cs, const char fname[], int *hi_hint)
 			return &file_hi->hi;
 		}
 	}
+
+	*hi_hint = INT_MAX;
 	return NULL;
 }
 
@@ -1131,7 +1320,117 @@ cs_del_file_hi(const char matchers_expr[])
 int
 cs_is_color_set(const col_attr_t *color)
 {
-	return color->fg != -1 || color->bg != -1 || color->attr != -1;
+	return color->fg != -1 || color->bg != -1 || color->attr != -1
+	    || color->gui_set;
+}
+
+int
+cs_load_color(const col_attr_t *color)
+{
+	if(curr_stats.direct_color)
+	{
+		if(color->gui_set)
+		{
+			return colmgr_get_pair(color->gui_fg, color->gui_bg);
+		}
+
+		const col_attr_t gui = convert_to_gui(color);
+		return colmgr_get_pair(gui.gui_fg, gui.gui_bg);
+	}
+
+	return colmgr_get_pair(color->fg, color->bg);
+}
+
+/* Converts cterm colors to a corresponding gui direct colors if necessary.
+ * Returns direct colors with cterm part erased. */
+static col_attr_t
+convert_to_gui(const col_attr_t *color)
+{
+	if(!curr_stats.direct_color || color->gui_set)
+	{
+		return *color;
+	}
+
+	col_attr_t gui = {
+		.gui_fg = -1,
+		.gui_bg = -1,
+		.gui_attr = color->attr,
+		.combine_gui_attrs = color->combine_attrs,
+		.gui_set = 1,
+	};
+
+	if(color->fg >= -1 && 1 + color->fg < (int)ARRAY_LEN(XTERM256_PALETTE))
+	{
+		gui.gui_fg = XTERM256_PALETTE[1 + color->fg];
+	}
+	if(color->bg >= -1 && 1 + color->bg < (int)ARRAY_LEN(XTERM256_PALETTE))
+	{
+		gui.gui_bg = XTERM256_PALETTE[1 + color->bg];
+	}
+
+	return gui;
+}
+
+int
+cs_color_get_attr(const col_attr_t *color)
+{
+	return (curr_stats.direct_color && color->gui_set) ? color->gui_attr
+	                                                   : color->attr;
+}
+
+cchar_t
+cs_color_to_cchar(const col_attr_t *color, int pair)
+{
+	if(pair < 0)
+	{
+		pair = cs_load_color(color);
+	}
+
+	cchar_t cch;
+	setcchar(&cch, L" ", cs_color_get_attr(color), pair, NULL);
+	return cch;
+}
+
+void
+cs_color_enable_gui(col_attr_t *color)
+{
+	if(!color->gui_set)
+	{
+		color->gui_set = 1;
+		color->gui_fg = -1;
+		color->gui_bg = -1;
+		color->gui_attr = 0;
+		color->combine_gui_attrs = 0;
+	}
+}
+
+int
+cs_downscale_color(int direct_color)
+{
+	int i;
+	int closest = 0;
+	int diff = color_diff(XTERM256_PALETTE[1], direct_color);
+	int color_limit = MIN(COLORS, 256);
+	for(i = 1; i < color_limit; ++i)
+	{
+		int d = color_diff(XTERM256_PALETTE[1 + i], direct_color);
+		if(d < diff)
+		{
+			diff = d;
+			closest = i;
+		}
+	}
+	return closest;
+}
+
+/* Computes distance between two RGB colors.  Returns the distance. */
+static int
+color_diff(int a, int b)
+{
+	int rd = abs((a >> 16) - (b >> 16));
+	int gd = abs(((a >> 8)&0xff) - ((b >> 8)&0xff));
+	int bd = abs((a&0xff) - (b&0xff));
+	return rd + gd + bd;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
