@@ -20,10 +20,40 @@
 #ifndef VIFM__UI__FILEVIEW_H__
 #define VIFM__UI__FILEVIEW_H__
 
+#include <stddef.h> /* size_t */
+
 #include "../utils/test_helpers.h"
 
+struct dir_entry_t;
 struct view_t;
 
+/* Packet set of parameters to pass as user data for processing columns. */
+typedef struct
+{
+	struct view_t *view;       /* View on which cell is being drawn. */
+	struct dir_entry_t *entry; /* Entry that is being displayed. */
+
+	int line_pos;      /* File position in the file list (the view).  Can be -1
+	                    * for filler (entry should still be supplied though). */
+	int line_hi_group; /* Line highlight (to avoid per-column calculation). */
+	int current_pos;   /* Position of entry selected with the cursor. */
+	int total_width;   /* Total width available for drawing. */
+	int number_width;  /* Whether to draw line numbers. */
+
+	size_t current_line;  /* Line of the cell within the view window. */
+	size_t column_offset; /* Offset in characters of the column. */
+
+	size_t *prefix_len; /* Data prefix length (should be drawn in neutral color).
+	                     * A pointer to allow changing value in const struct.
+	                     * Should be zero first time, then auto reset. */
+	int is_main;        /* Whether this is main file list. */
+
+	int custom_match;   /* Whether the keys below have meaningful values. */
+	int match_from;     /* Start offset of the match. */
+	int match_to;       /* End offset of the match. */
+}
+column_data_t;
+
 /* Initialization/termination functions. */
 
 /* Initializes file view unit. */
@@ -106,6 +136,10 @@ void fview_set_lsview(struct view_t *view, int enabled);
 /* Checks whether view displays grid that's filled by column. */
 int fview_is_transposed(const struct view_t *view);
 
+/* Checks whether view currently displays the path within itself.  Returns
+ * non-zero if so and zero otherwise. */
+int fview_previews(struct view_t *view, const char path[]);
+
 /* Enables/disables cascading columns style of the view. */
 void fview_set_millerview(struct view_t *view, int enabled);
 
@@ -122,6 +156,10 @@ void fview_dir_updated(struct view_t *view);
  * of files changes. */
 void fview_list_updated(struct view_t *view);
 
+/* Callback-like function which triggers some view-specific updates after
+ * decorations of files change. */
+void fview_decors_updated(struct view_t *view);
+
 /* Callback-like function which triggers some view-specific updates after cursor
  * position in the list changed. */
 void fview_position_updated(struct view_t *view);
@@ -130,36 +168,10 @@ void fview_position_updated(struct view_t *view);
  * sorting changed. */
 void fview_sorting_updated(struct view_t *view);
 
-#ifdef TEST
-#include <stddef.h> /* size_t */
-
-#include "ui.h"
-#endif
-
 TSTATIC_DEFS(
-	/* Packet set of parameters to pass as user data for processing columns. */
-	typedef struct
-	{
-		struct view_t *view; /* View on which cell is being drawn. */
-		dir_entry_t *entry;  /* Entry that is being displayed. */
-		int line_pos;        /* File position in the file list (the view). */
-		int line_hi_group;   /* Line highlight (to avoid per-column calculation). */
-		int current_pos;     /* Position of entry selected with the cursor. */
-		int total_width;     /* Total width available for drawing. */
-		int number_width;    /* Width of line number column (0 when disabled). */
-
-		size_t current_line;  /* Line of the cell within the view window. */
-		size_t column_offset; /* Offset in characters of the column. */
-
-		size_t *prefix_len; /* Data prefix length (should be drawn in neutral
-		                       color).  A pointer to allow changing value in const
-		                       struct.  Should be zero first time, then auto
-		                       reset. */
-		int is_main;        /* Whether this is main file list. */
-	}
-	column_data_t;
-
-	void format_name(int id, const void *data, size_t buf_len, char buf[]);
+	struct format_info_t;
+	void format_name(void *data, size_t buf_len, char buf[],
+		const struct format_info_t *info);
 )
 
 #endif /* VIFM__UI__FILEVIEW_H__ */
