@@ -33,16 +33,15 @@
 #include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../engine/mode.h"
+#include "../lua/vlua.h"
 #include "../modes/dialogs/msg_dialog.h"
 #include "../modes/modes.h"
 #include "../modes/view.h"
-#include "../utils/file_streams.h"
 #include "../utils/filemon.h"
 #include "../utils/fs.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
-#include "../utils/test_helpers.h"
 #include "../utils/utf8.h"
 #include "../utils/utils.h"
 #include "../filelist.h"
@@ -50,8 +49,8 @@
 #include "../macros.h"
 #include "../status.h"
 #include "../types.h"
+#include "../vcache.h"
 #include "cancellation.h"
-#include "color_manager.h"
 #include "color_scheme.h"
 #include "colors.h"
 #include "escape.h"
@@ -68,11 +67,12 @@ typedef struct
 	char *path;        /* Full path to the file. */
 	char *viewer;      /* Viewer of the file. */
 	filemon_t filemon; /* Timestamp for the file. */
-	strlist_t lines;   /* Top MAX_PREVIEW_LINES of preview contents. */
 	preview_area_t pa; /* Where preview is being drawn. */
+	strlist_t lines;   /* Lines of graphics (not pass through. */
 	int beg_x;         /* Original x coordinate of host window. */
 	int beg_y;         /* Original y coordinate of host window. */
 	ViewerKind kind;   /* Kind of preview. */
+	int max_lines;     /* Maximum number of lines requested last time. */
 	int graphics_lost; /* Whether graphics was invalidated on the screen. */
 }
 quickview_cache_t;
@@ -85,21 +85,24 @@ typedef struct
 	int ndirs;         /* Number of seen directories. */
 	int nfiles;        /* Number of seen files. */
 	int max;           /* Maximum line number. */
+	int full_stats;    /* Collect statistics for the whole tree. */
 	char prefix[4096]; /* Prefix character for each tree level. */
 }
 tree_print_state_t;
 
-static void view_entry(const dir_entry_t *entry, const preview_area_t *parea,
-		quickview_cache_t *cache);
-static void view_file(const char path[], const preview_area_t *parea,
+static const char * view_entry(const dir_entry_t *entry,
+		const preview_area_t *parea, quickview_cache_t *cache);
+static const char * view_file(const char path[], const preview_area_t *parea,
 		quickview_cache_t *cache);
 static int is_cache_valid(const quickview_cache_t *cache, const char path[],
 		const char viewer[], const preview_area_t *parea);
-static void fill_cache(quickview_cache_t *cache, FILE *fp, const char path[],
-		const char viewer[], ViewerKind kind, const preview_area_t *parea);
-TSTATIC strlist_t read_lines(FILE *fp, int max_lines);
-static FILE * view_dir(const char path[], int max_lines);
+static void update_cache(quickview_cache_t *cache, const char path[],
+		const char viewer[], ViewerKind kind, const preview_area_t *parea,
+		int max_lines);
+static strlist_t get_lines(const quickview_cache_t *cache);
+static void print_tree_stats(tree_print_state_t *s);
 static int print_dir_tree(tree_print_state_t *s, const char path[], int last);
+static void collect_subtree_stats(tree_print_state_t *s, const char path[]);
 static int enter_dir(tree_print_state_t *s, const char path[], int last);
 static int visit_file(tree_print_state_t *s, const char path[], int last);
 static int visit_link(tree_print_state_t *s, const char path[], int last,
@@ -115,9 +118,8 @@ static void draw_lines(const strlist_t *lines, int wrapped,
 		const preview_area_t *parea, ViewerKind kind);
 static void write_message(const char msg[], const preview_area_t *parea);
 static void cleanup_for_text(const preview_area_t *parea);
-static char * expand_viewer_command(const char viewer[]);
-static void cleanup_area(const preview_area_t *parea, const char cmd[]);
 static void wipe_area(const preview_area_t *parea);
+static void fill_area(const preview_area_t *parea);
 
 /* Cached preview data for a single file entry. */
 static quickview_cache_t qv_cache;
@@ -155,7 +157,7 @@ qv_toggle(void)
 	if(!curr_stats.preview.on)
 	{
 		stats_set_quickview(1);
-		stats_redraw_schedule();
+		stats_redraw_later();
 		return;
 	}
 
@@ -196,13 +198,13 @@ qv_draw(view_t *view)
 	 * location. */
 	(void)vifm_chdir(flist_get_dir(view));
 
-	if(view_detached_draw())
+	if(modview_detached_draw())
 	{
 		/* View mode handled the drawing. */
 		return;
 	}
 
-	ui_view_erase(other_view);
+	ui_view_erase(other_view, 1);
 
 	curr = get_current_entry(view);
 	if(!fentry_is_fake(curr))
@@ -210,56 +212,58 @@ qv_draw(view_t *view)
 		const preview_area_t parea = {
 			.source = view,
 			.view = other_view,
-			.def_col = ui_view_get_cs(other_view)->color[WIN_COLOR],
+			.def_col = cfg.cs.color[WIN_COLOR],
 			.x = ui_qv_left(other_view),
 			.y = ui_qv_top(other_view),
 			.w = ui_qv_width(other_view),
 			.h = ui_qv_height(other_view),
 		};
-		view_entry(curr, &parea, &qv_cache);
+		(void)view_entry(curr, &parea, &qv_cache);
 	}
 
 	refresh_view_win(other_view);
 	ui_view_title_update(other_view);
 }
 
-void
+const char *
 qv_draw_on(const dir_entry_t *entry, const preview_area_t *parea)
 {
 	static quickview_cache_t lwin_cache, rwin_cache;
 
-	char filler[parea->w + 1];
-	memset(filler, ' ', sizeof(filler) - 1U);
-	filler[sizeof(filler) - 1U] = '\0';
-
 	ui_set_attr(parea->view->win, &parea->def_col, -1);
-
-	int line;
-	for(line = parea->y; line < parea->y + parea->h; ++line)
-	{
-		mvwaddstr(parea->view->win, line, parea->x, filler);
-	}
+	fill_area(parea);
 
 	quickview_cache_t *cache = (parea->view == &lwin ? &lwin_cache : &rwin_cache);
 
-	view_entry(entry, parea, cache);
+	const char *clear_cmd = view_entry(entry, parea, cache);
 
 	parea->view->displays_graphics = (cache->kind != VK_TEXTUAL);
 
 	/* Unconditionally invalidate graphics cache, since we don't keep track of its
 	 * validity in any way. */
 	cache->graphics_lost = 1;
+
+	return clear_cmd;
 }
 
-/* Draws preview of the entry in the other view. */
-static void
+/* Draws preview of the entry in the other view.  Returns preview clear command
+ * or NULL. */
+static const char *
 view_entry(const dir_entry_t *entry, const preview_area_t *parea,
 		quickview_cache_t *cache)
 {
 	char path[PATH_MAX + 1];
 	qv_get_path_to_explore(entry, path, sizeof(path));
 
-	switch(entry->type)
+	FileType type = entry->type;
+
+	struct stat st;
+	if(os_stat(path, &st) == 0)
+	{
+		type = get_type_from_mode(st.st_mode);
+	}
+
+	switch(type)
 	{
 		case FT_CHAR_DEV:
 			write_message("File is a Character Device", parea);
@@ -281,6 +285,15 @@ view_entry(const dir_entry_t *entry, const preview_area_t *parea,
 				write_message("Cannot resolve Link", parea);
 				break;
 			}
+
+			if(!path_exists(path, DEREF))
+			{
+				char *msg = format_str("Cannot access link's target: %s", path);
+				write_message(msg, parea);
+				free(msg);
+				break;
+			}
+
 			if(!ends_with_slash(path) && is_dir(path))
 			{
 				strncat(path, "/", sizeof(path) - strlen(path) - 1);
@@ -288,99 +301,65 @@ view_entry(const dir_entry_t *entry, const preview_area_t *parea,
 			/* break is omitted intentionally. */
 		case FT_UNK:
 		default:
-			view_file(path, parea, cache);
-			break;
+			return view_file(path, parea, cache);
 	}
+
+	return NULL;
 }
 
 /* Displays contents of file or output of its viewer in the other pane
- * starting from the second line and second column. */
-static void
+ * starting from the second line and second column.  Returns preview clear
+ * command or NULL. */
+static const char *
 view_file(const char path[], const preview_area_t *parea,
 		quickview_cache_t *cache)
 {
 	const char *viewer = qv_get_viewer(path);
+	const char *clear_cmd = (viewer != NULL ? ma_get_clear_cmd(viewer) : NULL);
 
 	if(is_cache_valid(cache, path, viewer, parea))
 	{
 		/* Update area as we might draw preview at a different location. */
 		cache->pa = *parea;
+
 		draw_lines(&cache->lines, cfg.wrap_quick_view, &cache->pa, cache->kind);
-		return;
+		return clear_cmd;
 	}
 
-	ViewerKind kind = VK_TEXTUAL;
+	ViewerKind kind = ft_viewer_kind(viewer);
+	int max_lines = is_dir(path) ? ui_qv_height(parea->view)
+	                             : MAX_PREVIEW_LINES;
 
-	FILE *fp;
-	if(viewer == NULL && is_dir(path))
+	/* If graphics will be displayed, clear the window and wait a bit to let
+	 * terminal emulator do actual refresh (at least some of them need this). */
+	if(kind != VK_TEXTUAL)
 	{
-		ui_cancellation_reset();
-		ui_cancellation_enable();
-		fp = view_dir(path, ui_qv_height(other_view));
-		ui_cancellation_disable();
-
-		if(fp == NULL)
-		{
-			write_message("Failed to view directory", parea);
-			return;
-		}
-	}
-	else if(is_null_or_empty(viewer))
-	{
-		fp = os_fopen(path, "rb");
-		if(fp == NULL)
-		{
-			write_message("Cannot open file", parea);
-			return;
-		}
+		qv_cleanup_area(parea, curr_stats.preview.cleanup_cmd);
+		usleep(cfg.graphics_delay);
 	}
 	else
 	{
-		kind = ft_viewer_kind(viewer);
-
-		/* If graphics will be displayed, clear the window and wait a bit to let
-		 * terminal emulator do actual refresh (at least some of them need this). */
-		if(kind != VK_TEXTUAL)
-		{
-			cleanup_area(parea, curr_stats.preview.cleanup_cmd);
-			usleep(50000);
-		}
-
-		view_t *const curr = curr_view;
-		curr_view = parea->source;
-		curr_stats.preview_hint = parea;
-		fp = qv_execute_viewer(viewer);
-		curr_stats.preview_hint = NULL;
-		curr_view = curr;
-
-		if(fp == NULL)
-		{
-			write_message("Cannot read viewer output", parea);
-			return;
-		}
-	}
-
-	ui_cancellation_reset();
-	ui_cancellation_enable();
-
-	/* We want to wipe the view if it was displaying graphics, but won't anymore.
-	 * Do this only if we didn't already cleared the window. */
-	if(kind == VK_TEXTUAL)
-	{
+		/* We want to wipe the view if it was displaying graphics, but won't
+		 * anymore.  Do this only if we didn't already cleared the window. */
 		cleanup_for_text(parea);
 	}
+
 	curr_stats.preview.kind = kind;
 
-	const char *clear_cmd = (viewer != NULL) ? ma_get_clear_cmd(viewer) : NULL;
 	update_string(&curr_stats.preview.cleanup_cmd, clear_cmd);
 
-	fill_cache(cache, fp, path, viewer, kind, parea);
+	update_cache(cache, path, viewer, kind, parea, max_lines);
+	strlist_t lines = get_lines(cache);
+	draw_lines(&lines, cfg.wrap_quick_view, &cache->pa, cache->kind);
 
-	fclose(fp);
+	if(cache->kind != VK_TEXTUAL)
+	{
+		free_string_array(cache->lines.items, cache->lines.nitems);
+		cache->lines.items = copy_string_array(lines.items, lines.nitems);
+		cache->lines.nitems = lines.nitems;
+	}
 
-	ui_cancellation_disable();
-
-	draw_lines(&cache->lines, cfg.wrap_quick_view, &cache->pa, cache->kind);
+	return clear_cmd;
 }
 
 /* Checks whether data in the cache is up to date with the file on disk.
@@ -389,6 +368,12 @@ static int
 is_cache_valid(const quickview_cache_t *cache, const char path[],
 		const char viewer[], const preview_area_t *parea)
 {
+	if(cache->kind == VK_TEXTUAL)
+	{
+		/* This level of cache deals only with graphics and ignores text. */
+		return 0;
+	}
+
 	int same_viewer = (cache->viewer == NULL && viewer == NULL)
 	               || (cache->viewer != NULL && viewer != NULL &&
 	                   strcmp(cache->viewer, viewer) == 0);
@@ -400,11 +385,6 @@ is_cache_valid(const quickview_cache_t *cache, const char path[],
 			paths_are_equal(cache->path, path) &&
 			filemon_equal(&cache->filemon, &filemon))
 	{
-		if(cache->kind == VK_TEXTUAL)
-		{
-			return 1;
-		}
-
 		return !cache->graphics_lost
 		    && cache->pa.h == parea->h
 		    && cache->pa.w == parea->w
@@ -415,96 +395,113 @@ is_cache_valid(const quickview_cache_t *cache, const char path[],
 	return 0;
 }
 
-/* Fills the cache data with file's contents. */
+/* Updates cache data. */
 static void
-fill_cache(quickview_cache_t *cache, FILE *fp, const char path[],
-		const char viewer[], ViewerKind kind, const preview_area_t *parea)
+update_cache(quickview_cache_t *cache, const char path[], const char viewer[],
+		ViewerKind kind, const preview_area_t *parea, int max_lines)
 {
-	/* File monitor must always be initialized, because it's used below. */
-	filemon_t filemon = {};
-	(void)filemon_from_file(path, FMT_MODIFIED, &filemon);
-	filemon_assign(&cache->filemon, &filemon);
+	(void)filemon_from_file(path, FMT_MODIFIED, &cache->filemon);
 
 	replace_string(&cache->path, path);
 	update_string(&cache->viewer, viewer);
 
-	free_string_array(cache->lines.items, cache->lines.nitems);
-	cache->lines = read_lines(fp, MAX_PREVIEW_LINES);
-
 	cache->pa = *parea;
 	cache->beg_x = getbegx(parea->view->win);
 	cache->beg_y = getbegy(parea->view->win);
 	cache->kind = kind;
+	cache->max_lines = max_lines;
 	cache->graphics_lost = 0;
 }
 
-/* Reads at most max_lines from the stream ignoring BOM.  Returns the lines
- * read. */
-TSTATIC strlist_t
-read_lines(FILE *fp, int max_lines)
+/* Retrieves lines to be printed.  Returns the lines. */
+static strlist_t
+get_lines(const quickview_cache_t *cache)
 {
-	strlist_t lines = {};
-	skip_bom(fp);
+	view_t *curr = curr_view;
+	curr_view = cache->pa.source;
+	curr_stats.preview_hint = &cache->pa;
 
-	char *next_line;
-	while(lines.nitems < max_lines && (next_line = read_line(fp, NULL)) != NULL)
+	const char *error;
+
+	char *expanded = (cache->viewer == NULL) ? NULL
+	                                         : qv_expand_viewer(cache->viewer);
+	strlist_t lines = vcache_lookup(cache->path, expanded, MF_NONE, cache->kind,
+			cache->max_lines, VC_ASYNC, &error);
+	free(expanded);
+
+	if(error != NULL)
 	{
-		const int old_len = lines.nitems;
-		lines.nitems = put_into_string_array(&lines.items, lines.nitems, next_line);
-		if(lines.nitems == old_len)
-		{
-			free(next_line);
-			break;
-		}
+		lines.nitems = add_to_string_array(&lines.items, lines.nitems, error);
 	}
 
+	curr_stats.preview_hint = NULL;
+	curr_view = curr;
+
 	return lines;
 }
 
 FILE *
-qv_view_dir(const char path[])
+qv_view_dir(const char path[], int max_lines)
 {
-	return view_dir(path, INT_MAX);
+	enum { NSPACES = 64 };
+
+	FILE *fp = os_tmpfile();
+	if(fp == NULL)
+	{
+		return NULL;
+	}
+
+	tree_print_state_t s = {
+		.fp = fp,
+		/* Increase by one to cause cached data to be recognized as incomplete
+		 * when max_lines isn't enough. */
+		.max = (max_lines == INT_MAX ? max_lines : max_lines + 1),
+		.full_stats = cfg.top_tree_stats,
+		.n = (cfg.top_tree_stats ? 2 : 0),
+	};
+
+	/* Spare blank line on the top of the view to put the (files, directories)
+	 * count in case "toptreestats" option is set. */
+	fprintf(fp, "%*s\n", NSPACES, "");
+
+	const int whole_tree = (print_dir_tree(&s, path, 0) == 0 && s.n != 0);
+	if(!whole_tree && ui_cancellation_requested())
+	{
+		fputs("(cancelled)", fp);
+	}
+
+	if(s.n == 0)
+	{
+		fclose(fp);
+		return NULL;
+	}
+
+	if(cfg.top_tree_stats)
+	{
+		/* Print count on the top, spare line. */
+		fseek(fp, 0, SEEK_SET);
+		print_tree_stats(&s);
+		fseek(fp, 0, SEEK_SET);
+	}
+	else
+	{
+		/* Print count at the bottom and "hide" the spare line away. */
+		fputs("\n", fp);
+		print_tree_stats(&s);
+		fseek(fp, NSPACES + 1, SEEK_SET);
+	}
+
+	return fp;
 }
 
-/* Previews directory, actual preview is to be read from returned stream.
- * Returns the stream or NULL on error. */
-static FILE *
-view_dir(const char path[], int max_lines)
+/* Prints one-line tree statistics. */
+static void
+print_tree_stats(tree_print_state_t *s)
 {
-	FILE *fp = os_tmpfile();
-
-	if(fp != NULL)
-	{
-		tree_print_state_t s = {
-			.fp = fp,
-			.max = max_lines,
-		};
-
-		if(print_dir_tree(&s, path, 0) == 0 && s.n != 0)
-		{
-			/* Print summary only if we visited the whole subtree. */
-			fprintf(fp, "%s\n%d director%s, %d file%s",
-					ui_cancellation_requested() ? "(cancelled)\n" : "",
-					s.ndirs, (s.ndirs == 1) ? "y" : "ies",
-					s.nfiles, (s.nfiles == 1) ? "" : "s");
-		}
-		else if(ui_cancellation_requested())
-		{
-			fputs("(cancelled)", fp);
-		}
-
-		if(s.n == 0)
-		{
-			fclose(fp);
-			fp = NULL;
-		}
-		else
-		{
-			fseek(fp, 0, SEEK_SET);
-		}
-	}
-	return fp;
+	fprintf(s->fp, "%s%d director%s, %d file%s\n",
+			ui_cancellation_requested() ? "(cancelled)\n" : "",
+			s->ndirs, (s->ndirs == 1) ? "y" : "ies",
+			s->nfiles, (s->nfiles == 1) ? "" : "s");
 }
 
 /* Produces tree preview of the path.  Returns non-zero to request stopping of
@@ -512,28 +509,24 @@ view_dir(const char path[], int max_lines)
 static int
 print_dir_tree(tree_print_state_t *s, const char path[], int last)
 {
-	int i;
-	int reached_limit;
-
 	int len;
 	char **lst = list_sorted_files(path, &len);
-
 	if(len < 0)
 	{
-		free_string_array(lst, len);
 		return 1;
 	}
 
 	if(enter_dir(s, path, last) != 0)
 	{
 		free_string_array(lst, len);
+		collect_subtree_stats(s, path);
 		return 1;
 	}
 
-	reached_limit = 0;
+	int i;
+	int reached_limit = 0;
 	for(i = 0; i < len && !reached_limit && !ui_cancellation_requested(); ++i)
 	{
-		char link_target[PATH_MAX + 1];
 		const int last_entry = (i == len - 1);
 		char *const full_path = format_str("%s/%s", path, lst[i]);
 
@@ -546,6 +539,7 @@ print_dir_tree(tree_print_state_t *s, const char path[], int last)
 			++s->nfiles;
 		}
 
+		char link_target[PATH_MAX + 1];
 		if(get_link_target(full_path, link_target, sizeof(link_target)) == 0)
 		{
 			if(visit_link(s, full_path, last_entry, link_target) != 0)
@@ -553,8 +547,8 @@ print_dir_tree(tree_print_state_t *s, const char path[], int last)
 				reached_limit = 1;
 			}
 		}
-		/* If is_dir_empty() returns non-zero than we know that it's directory and
-		 * no additional checks are needed. */
+		/* If is_dir_empty() returns non-zero then we know that it's a directory
+		 * and no additional checks are needed. */
 		else if(!is_dir_empty(full_path))
 		{
 			if(last_entry)
@@ -566,20 +560,83 @@ print_dir_tree(tree_print_state_t *s, const char path[], int last)
 				reached_limit = 1;
 			}
 		}
-		else if(visit_file(s, full_path, last_entry) != 0)
+		else
 		{
-			reached_limit = 1;
+			if(visit_file(s, full_path, last_entry) != 0)
+			{
+				reached_limit = 1;
+			}
 		}
 
 		free(full_path);
 	}
-	free_string_array(lst, len);
 
+	if(reached_limit && s->full_stats)
+	{
+		for(; i < len && !ui_cancellation_requested(); ++i)
+		{
+			char *const full_path = format_str("%s/%s", path, lst[i]);
+			if(is_symlink(full_path))
+			{
+				++s->nfiles;
+			}
+			else if(is_dir(full_path))
+			{
+				++s->ndirs;
+				collect_subtree_stats(s, full_path);
+			}
+			else
+			{
+				++s->nfiles;
+			}
+			free(full_path);
+		}
+	}
+
+	free_string_array(lst, len);
 	leave_dir(s);
 
 	return reached_limit;
 }
 
+/* Collects stats for items of a directory in a much faster way than traversal
+ * for printing. */
+static void
+collect_subtree_stats(tree_print_state_t *s, const char path[])
+{
+	DIR *dir = os_opendir(path);
+	if(dir == NULL)
+	{
+		return;
+	}
+
+	struct dirent *d;
+	while((d = os_readdir(dir)) != NULL)
+	{
+		if(is_builtin_dir(d->d_name))
+		{
+			continue;
+		}
+
+		char *const full_path = format_str("%s/%s", path, d->d_name);
+		if(entry_is_dir(full_path, d))
+		{
+			++s->ndirs;
+			collect_subtree_stats(s, full_path);
+		}
+		else if(is_dirent_targets_dir(full_path, d))
+		{
+			++s->ndirs;
+		}
+		else
+		{
+			++s->nfiles;
+		}
+		free(full_path);
+	}
+	os_closedir(dir);
+}
+
 /* Handles entering directory on directory tree traversal.  Returns non-zero to
  * request stopping of the traversal, otherwise zero is returned. */
 static int
@@ -595,7 +652,7 @@ enter_dir(tree_print_state_t *s, const char path[], int last)
 	indent_prefix(s);
 	set_prefix_char(s, '|');
 
-	return ++s->n >= s->max;
+	return (++s->n >= s->max);
 }
 
 /* Handles visiting file on directory tree traversal.  Returns non-zero to
@@ -606,7 +663,7 @@ visit_file(tree_print_state_t *s, const char path[], int last)
 	set_prefix_char(s, last ? '`' : '|');
 	print_tree_entry(s, path, 1);
 
-	return ++s->n >= s->max;
+	return (++s->n >= s->max);
 }
 
 /* Handles visiting symbolic link on directory tree traversal.  Returns non-zero
@@ -621,7 +678,7 @@ visit_link(tree_print_state_t *s, const char path[], int last,
 	fputs(target, s->fp);
 	fputc('\n', s->fp);
 
-	return ++s->n >= s->max;
+	return (++s->n >= s->max);
 }
 
 /* Handles leaving directory on directory tree traversal. */
@@ -753,7 +810,7 @@ cleanup_for_text(const preview_area_t *parea)
 	if(curr_stats.preview.cleanup_cmd != NULL ||
 			curr_stats.preview.kind != VK_TEXTUAL)
 	{
-		cleanup_area(parea, curr_stats.preview.cleanup_cmd);
+		qv_cleanup_area(parea, curr_stats.preview.cleanup_cmd);
 	}
 	update_string(&curr_stats.preview.cleanup_cmd, NULL);
 	curr_stats.preview.kind = VK_TEXTUAL;
@@ -768,31 +825,18 @@ qv_hide(void)
 	}
 	if(lwin.explore_mode)
 	{
-		view_quit_explore_mode(&lwin);
+		modview_quit_exploring(&lwin);
 	}
 	if(rwin.explore_mode)
 	{
-		view_quit_explore_mode(&rwin);
+		modview_quit_exploring(&rwin);
 	}
 }
 
-FILE *
-qv_execute_viewer(const char viewer[])
-{
-	FILE *fp;
-	char *expanded;
-
-	expanded = expand_viewer_command(viewer);
-	fp = read_cmd_output(expanded, 0);
-	free(expanded);
-
-	return fp;
-}
-
 /* Returns a pointer to newly allocated memory, which should be released by the
  * caller. */
-static char *
-expand_viewer_command(const char viewer[])
+char *
+qv_expand_viewer(const char viewer[])
 {
 	char *result;
 	if(strchr(viewer, '%') == NULL)
@@ -820,12 +864,11 @@ qv_cleanup(view_t *view, const char cmd[])
 		.w = view->window_cols,
 		.h = view->window_rows,
 	};
-	cleanup_area(&parea, cmd);
+	qv_cleanup_area(&parea, cmd);
 }
 
-/* Erases area using external command if available. */
-static void
-cleanup_area(const preview_area_t *parea, const char cmd[])
+void
+qv_cleanup_area(const preview_area_t *parea, const char cmd[])
 {
 	if(cmd == NULL)
 	{
@@ -838,15 +881,30 @@ cleanup_area(const preview_area_t *parea, const char cmd[])
 
 	curr_stats.preview.clearing = 1;
 	curr_stats.preview_hint = parea;
-	FILE *fp = qv_execute_viewer(cmd);
+
+	char *expanded = qv_expand_viewer(cmd);
+	if(vlua_handler_cmd(curr_stats.vlua, expanded))
+	{
+		char path[PATH_MAX + 1];
+		dir_entry_t *entry = get_current_entry(parea->source);
+		qv_get_path_to_explore(entry, path, sizeof(path));
+
+		strlist_t lines = vlua_view_file(curr_stats.vlua, expanded, path, parea);
+		free_string_array(lines.items, lines.nitems);
+	}
+	else
+	{
+		FILE *fp = read_cmd_output(expanded, /*preserve_stdin=*/0);
+		while(fgetc(fp) != EOF);
+		fclose(fp);
+	}
+	free(expanded);
+
 	curr_stats.preview_hint = NULL;
 	curr_stats.preview.clearing = 0;
 
 	curr_view = curr;
 
-	while(fgetc(fp) != EOF);
-	fclose(fp);
-
 	wipe_area(parea);
 }
 
@@ -855,11 +913,34 @@ cleanup_area(const preview_area_t *parea, const char cmd[])
 static void
 wipe_area(const preview_area_t *parea)
 {
+	if(cfg.hard_graphics_clear)
+	{
+		wclear(parea->view->win);
+		return;
+	}
+
 	/* User doesn't need to see fake filling so draw it with the color of
 	 * background. */
-	col_attr_t col = { .fg = parea->def_col.fg, .bg = parea->def_col.bg };
+	col_attr_t col = parea->def_col;
+	col.attr = 0;
+	col.gui_attr = 0;
 	ui_set_attr(parea->view->win, &col, -1);
 
+	fill_area(parea);
+	/* The check is for tests, which work otherwise. */
+	if(parea->view->win != NULL)
+	{
+		redrawwin(parea->view->win);
+	}
+	ui_refresh_win(parea->view->win);
+
+	ui_set_attr(parea->view->win, &parea->def_col, -1);
+}
+
+/* Fills preview area with spaces just to clear background. */
+static void
+fill_area(const preview_area_t *parea)
+{
 	char filler[parea->w + 1];
 	memset(filler, ' ', sizeof(filler) - 1U);
 	filler[sizeof(filler) - 1U] = '\0';
@@ -869,10 +950,6 @@ wipe_area(const preview_area_t *parea)
 	{
 		mvwaddstr(parea->view->win, line, parea->x, filler);
 	}
-	redrawwin(parea->view->win);
-	ui_refresh_win(parea->view->win);
-
-	ui_set_attr(parea->view->win, &parea->def_col, -1);
 }
 
 const char *
