@@ -23,14 +23,14 @@
 #include <stddef.h> /* size_t */
 #include <stdio.h> /* FILE */
 
-#include "../utils/test_helpers.h"
 #include "colors.h"
 
 struct dir_entry_t;
 struct view_t;
 
 /* Description of area used for preview. */
-typedef struct
+typedef struct preview_area_t preview_area_t;
+struct preview_area_t
 {
 	struct view_t *source; /* View which does the preview. */
 	struct view_t *view;   /* View which displays the preview. */
@@ -39,8 +39,7 @@ typedef struct
 	int y;                 /* Relative y coordinate of the top-left corner. */
 	int w;                 /* Width of the area. */
 	int h;                 /* Height of the area. */
-}
-preview_area_t;
+};
 
 /* Enables quick view (just enables, no drawing) if possible.  Returns zero on
  * success, otherwise non-zero is returned and error message is printed on the
@@ -55,8 +54,9 @@ int qv_can_show(void);
  * doesn't make sense (e.g. only one pane is visible). */
 void qv_draw(struct view_t *view);
 
-/* Draws file entry on an area. */
-void qv_draw_on(const struct dir_entry_t *entry, const preview_area_t *parea);
+/* Draws file entry on an area.  Returns preview clear command or NULL. */
+const char * qv_draw_on(const struct dir_entry_t *entry,
+		const preview_area_t *parea);
 
 /* Toggles state of the quick view. */
 void qv_toggle(void);
@@ -64,14 +64,15 @@ void qv_toggle(void);
 /* Quits preview pane or view modes. */
 void qv_hide(void);
 
-/* Expands and executes viewer command.  Returns file containing results of the
- * viewer. */
-FILE * qv_execute_viewer(const char viewer[]);
+char * qv_expand_viewer(const char viewer[]);
 
 /* Performs view clearing with the given command, which can be NULL in which
  * case only internal clearing is done. */
 void qv_cleanup(struct view_t *view, const char cmd[]);
 
+/* Erases area using external command if cmd parameter isn't NULL. */
+void qv_cleanup_area(const preview_area_t *parea, const char cmd[]);
+
 /* Gets viewer command for a file considering its type (directory vs. file).
  * Returns NULL if no suitable viewer available, otherwise returns pointer to
  * string stored internally. */
@@ -79,7 +80,7 @@ const char * qv_get_viewer(const char path[]);
 
 /* Previews directory, actual preview is to be read from returned stream.
  * Returns the stream or NULL on error. */
-FILE * qv_view_dir(const char path[]);
+FILE * qv_view_dir(const char path[], int max_lines);
 
 /* Decides on path that should be explored when cursor points to the given
  * entry. */
@@ -89,11 +90,6 @@ void qv_get_path_to_explore(const struct dir_entry_t *entry, char buf[],
 /* Informs this unit that it's data was probably erased from the screen. */
 void qv_ui_updated(void);
 
-TSTATIC_DEFS(
-	struct strlist_t;
-	struct strlist_t read_lines(FILE *fp, int max_lines);
-)
-
 #endif /* VIFM__UI__QUICKVIEW_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
