@@ -26,6 +26,7 @@
 #include <string.h> /* strcat() strchr() strcmp() strcpy() strdup() strncpy() */
 
 #include "../cfg/config.h"
+#include "../engine/mode.h"
 #include "../modes/modes.h"
 #include "../modes/more.h"
 #include "../utils/macros.h"
@@ -63,7 +64,7 @@ ui_sb_clear(void)
 	if(curr_stats.load_stage <= 2)
 	{
 		multiline_status_bar = 0;
-		curr_stats.need_update = UT_FULL;
+		stats_refresh_later();
 		return;
 	}
 
@@ -89,7 +90,7 @@ ui_sb_quick_msgf(const char format[], ...)
 
 	checked_wmove(status_bar, 0, 0);
 	werase(status_bar);
-	vwprintw(status_bar, format, ap);
+	vw_printw(status_bar, format, ap);
 	wnoutrefresh(status_bar);
 	doupdate();
 
@@ -107,6 +108,12 @@ ui_sb_quick_msg_clear(void)
 	{
 		ui_sb_quick_msgf("%s", "");
 	}
+
+	if(vle_mode_is(CMDLINE_MODE))
+	{
+		/* Restore previous contents of the statusbar. */
+		stats_redraw_later();
+	}
 }
 
 void
