@@ -35,6 +35,7 @@
 #include "../engine/mode.h"
 #include "../engine/parsing.h"
 #include "../engine/var.h"
+#include "../lua/vlua.h"
 #include "../modes/modes.h"
 #include "../utils/fs.h"
 #include "../utils/log.h"
@@ -47,38 +48,31 @@
 #include "../utils/utils.h"
 #include "../background.h"
 #include "../filelist.h"
-#include "color_manager.h"
 #include "color_scheme.h"
+#include "colored_line.h"
 #include "ui.h"
 
-/* Line paired with parallel character of arrays that specify user colors. */
-typedef struct
-{
-	char *line;       /* Text of the line. */
-	size_t line_len;  /* Length of line field. */
-	char *attrs;      /* Specifies when to enable which user highlight group. */
-	size_t attrs_len; /* Length of attrs field. */
-}
-LineWithAttrs;
-
-static void print_with_attrs(WINDOW *win, const char line[], const char attrs[],
-		const cchar_t *default_attr);
+static void split_and_print_status_line(view_t *view, int width);
 static void update_stat_window_old(view_t *view, int lazy_redraw);
 static void refresh_window(WINDOW *win, int lazily);
-TSTATIC LineWithAttrs expand_status_line_macros(view_t *view,
-		const char format[]);
-static LineWithAttrs parse_view_macros(view_t *view, const char **format,
+TSTATIC cline_t expand_status_line_macros(view_t *view, const char format[]);
+static cline_t parse_view_macros(view_t *view, const char **format,
 		const char macros[], int opt);
-static int sync_attrs(LineWithAttrs *result, int extra_width);
 static int expand_num(char buf[], size_t buf_len, int val);
 static const char * get_tip(void);
 static void check_expanded_str(const char buf[], int skip, int *nexpansions);
+static char * fetch_status_line(view_t *view, int width);
+TSTATIC char * find_view_macro(const char **format, const char macros[],
+		char macro, int opt);
 static pthread_spinlock_t * get_job_bar_changed_lock(void);
 static void init_job_bar_changed_lock(void);
 static int is_job_bar_visible(void);
 static const char * format_job_bar(void);
 static char ** take_job_descr_snapshot(void);
 
+/* List of macros that are expanded in the status line. */
+static const char STATUS_LINE_MACROS[] = "tTfacAugsEdD-xlLPSz%[]{*";
+
 /* Number of background jobs. */
 static size_t nbar_jobs;
 /* Array of jobs. */
@@ -91,7 +85,8 @@ static pthread_spinlock_t job_bar_changed_lock;
 void
 ui_stat_update(view_t *view, int lazy_redraw)
 {
-	if(!cfg.display_statusline || view != curr_view)
+	if(!cfg.display_statusline || curr_stats.reusing_statusline ||
+			view != curr_view)
 	{
 		return;
 	}
@@ -112,56 +107,44 @@ ui_stat_update(view_t *view, int lazy_redraw)
 
 	const int width = getmaxx(stdscr);
 
-	wresize(stat_win, 1, width);
+	wresize(stat_win, ui_stat_height(), width);
 	ui_set_bg(stat_win, &cfg.cs.color[STATUS_LINE_COLOR],
 			cfg.cs.pair[STATUS_LINE_COLOR]);
 	werase(stat_win);
 	checked_wmove(stat_win, 0, 0);
 
-	cchar_t default_attr;
-	setcchar(&default_attr, L" ", cfg.cs.color[STATUS_LINE_COLOR].attr,
-			cfg.cs.pair[STATUS_LINE_COLOR], NULL);
-
-	LineWithAttrs result = expand_status_line_macros(view, cfg.status_line);
-	assert(strlen(result.attrs) == utf8_strsw(result.line) && "Broken attrs!");
-	result.line = break_in_two(result.line, width, "%=");
-	result.attrs = break_in_two(result.attrs, width, "=");
-	print_with_attrs(stat_win, result.line, result.attrs, &default_attr);
-	free(result.line);
-	free(result.attrs);
-
+	split_and_print_status_line(view, width);
 	refresh_window(stat_win, lazy_redraw);
 }
 
-/* Prints line onto a window highlighting it according to attrs, which should
- * specify 0-9 color groups for every character in line. */
+/* Splits status line format by %N macros and prints each on a separate line
+ * respecting %=. */
 static void
-print_with_attrs(WINDOW *win, const char line[], const char attrs[],
-		const cchar_t *default_attr)
+split_and_print_status_line(view_t *view, int width)
 {
-	cchar_t attr = *default_attr;
-	while(*line != '\0')
+	char *status_line = fetch_status_line(view, width);
+
+	const char *part = status_line;
+	int line = 0;
+	while(*part != '\0')
 	{
-		if(*attrs == '0')
+		const char *current = part;
+		char *next = find_view_macro(&part, STATUS_LINE_MACROS, 'N', 0);
+		if(next != NULL)
 		{
-			attr = *default_attr;
-		}
-		else if(*attrs != ' ')
-		{
-			const int color = (USER1_COLOR + (*attrs - '1'));
-			col_attr_t col = cfg.cs.color[STATUS_LINE_COLOR];
-			cs_mix_colors(&col, &cfg.cs.color[color]);
-			setcchar(&attr, L" ", col.attr, colmgr_get_pair(col.fg, col.bg), NULL);
+			*next = '\0';
 		}
 
-		const size_t len = utf8_chrw(line);
-		char char_buf[len + 1];
-		copy_str(char_buf, sizeof(char_buf), line);
-		wprinta(win, char_buf, &attr, 0);
-
-		line += len;
-		attrs += utf8_chrsw(char_buf);
+		cline_t result = expand_status_line_macros(view, current);
+		assert(strlen(result.attrs) == utf8_strsw(result.line) && "Broken attrs!");
+		result.line = break_in_two(result.line, width, "%=");
+		result.attrs = break_in_two(result.attrs, width, "=");
+		checked_wmove(stat_win, line++, 0);
+		cline_print(&result, stat_win, &cfg.cs.color[STATUS_LINE_COLOR]);
+		cline_dispose(&result);
 	}
+
+	free(status_line);
 }
 
 /* Formats status line in the "old way" (before introduction of 'statusline'
@@ -250,22 +233,18 @@ refresh_window(WINDOW *win, int lazily)
 }
 
 /* Expands view macros to be displayed on the status line according to the
- * format string.  Returns expanded line and attribute line, the latter one
- * contains a character in the set [0-9 ] (space included) per utf-8 character
- * of the former that specifies which user highlight group should be used
- * starting with that character. */
-TSTATIC LineWithAttrs
+ * format string.  Returns colored line. */
+TSTATIC cline_t
 expand_status_line_macros(view_t *view, const char format[])
 {
 	const dir_entry_t *const curr = get_current_entry(view);
 	if(curr == NULL || fentry_is_fake(curr))
 	{
 		/* Fake entries don't have valid information. */
-		LineWithAttrs result = { .line = strdup(""), .attrs = strdup("") };
-		return result;
+		return cline_make();
 	}
 
-	return parse_view_macros(view, &format, "tTfaAugsEdD-xlLSz%[]{*", 0);
+	return parse_view_macros(view, &format, STATUS_LINE_MACROS, 0);
 }
 
 /* Expands possibly limited set of view macros.  Returns newly allocated string,
@@ -273,21 +252,22 @@ expand_status_line_macros(view_t *view, const char format[])
 char *
 expand_view_macros(view_t *view, const char format[], const char macros[])
 {
-	LineWithAttrs result = parse_view_macros(view, &format, macros, 0);
+	cline_t result = parse_view_macros(view, &format, macros, 0);
 	free(result.attrs);
 	return result.line;
 }
 
 /* Expands macros in the *format string advancing the pointer as it goes.  The
  * opt represents conditional expression state, should be zero for non-recursive
- * calls.  Returns newly allocated string, which should be freed by the
- * caller. */
-static LineWithAttrs
+ * calls.  Returns colored line. */
+static cline_t
 parse_view_macros(view_t *view, const char **format, const char macros[],
 		int opt)
 {
+	/* Mind that find_view_macro() needs to be in sync with this function. */
+
 	const dir_entry_t *const curr = get_current_entry(view);
-	LineWithAttrs result = { .line = strdup(""), .attrs = strdup("") };
+	cline_t result = cline_make();
 	char c;
 	int nexpansions = 0;
 	int has_expander = 0;
@@ -318,7 +298,7 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 
 		if(*next == '=')
 		{
-			(void)sync_attrs(&result, 0);
+			(void)cline_sync(&result, 0);
 
 			if(strappend(&result.line, &result.line_len, "%=") != 0 ||
 					strappendch(&result.attrs, &result.attrs_len, '=') != 0)
@@ -342,17 +322,48 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 		}
 		c = *(*format)++;
 
+		/* Query drive information at most once. */
+		uint64_t free_space;
+		uint64_t total_space;
+		int have_drive_info = 0;
+
 		skip = 0;
 		ok = 1;
 		buf[0] = '\0';
 		switch(c)
 		{
+			char path[PATH_MAX + 1];
+			char *escaped;
+
 			case 'a':
-				friendly_size_notation(get_free_space(curr_view->curr_dir), sizeof(buf),
-						buf);
+				have_drive_info = have_drive_info ||
+					(get_drive_info(curr_view->curr_dir, &total_space, &free_space) == 0);
+				if(have_drive_info)
+				{
+					friendly_size_notation(free_space, sizeof(buf), buf);
+				}
+				break;
+			case 'c':
+				have_drive_info = have_drive_info ||
+					(get_drive_info(curr_view->curr_dir, &total_space, &free_space) == 0);
+				if(have_drive_info)
+				{
+					friendly_size_notation(total_space, sizeof(buf), buf);
+				}
 				break;
 			case 't':
-				format_entry_name(curr, NF_FULL, sizeof(buf), buf);
+			case 'f':
+				if(c == 't')
+				{
+					format_entry_name(curr, NF_FULL, sizeof(path), path);
+				}
+				else
+				{
+					get_short_path_of(view, curr, NF_FULL, 0, sizeof(path), path);
+				}
+				escaped = escape_unreadable(path);
+				copy_str(buf, sizeof(buf), escaped);
+				free(escaped);
 				break;
 			case 'T':
 				if(curr->type == FT_LINK)
@@ -365,9 +376,6 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 					}
 				}
 				break;
-			case 'f':
-				get_short_path_of(view, curr, NF_FULL, 0, sizeof(buf), buf);
-				break;
 			case 'A':
 #ifndef _WIN32
 				get_perm_string(buf, sizeof(buf), curr->mode);
@@ -419,6 +427,10 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 			case 'L':
 				skip = expand_num(buf, sizeof(buf), view->list_rows + view->filtered);
 				break;
+			case 'P':
+				format_position(buf, sizeof(buf), view->top_line, view->list_rows,
+						view->window_cells);
+				break;
 			case 'S':
 				skip = expand_num(buf, sizeof(buf), view->list_rows);
 				break;
@@ -437,21 +449,11 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 				break;
 			case '[':
 				{
-					LineWithAttrs opt = parse_view_macros(view, format, macros, 1);
+					cline_t opt = parse_view_macros(view, format, macros, 1);
 					copy_str(buf, sizeof(buf), opt.line);
 					free(opt.line);
 
-					char *attrs = opt.attrs;
-					if(sync_attrs(&result, 0) && opt.attrs_len > 0U)
-					{
-						if(*attrs != ' ')
-						{
-							result.attrs[result.attrs_len - 1U] = *attrs;
-						}
-						++attrs;
-					}
-					strappend(&result.attrs, &result.attrs_len, attrs);
-					free(opt.attrs);
+					cline_splice_attrs(&result, &opt);
 					break;
 				}
 			case ']':
@@ -459,15 +461,9 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 				{
 					if(nexpansions == 0)
 					{
-						replace_string(&result.line, "");
-						replace_string(&result.attrs, "");
-						result.line_len = 0U;
-						result.attrs_len = 0U;
-					}
-					if(sync_attrs(&result, 0))
-					{
-						result.attrs[--result.attrs_len] = '\0';
+						cline_clear(&result);
 					}
+					cline_finish(&result);
 					return result;
 				}
 
@@ -528,8 +524,7 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 					width = 0;
 					break;
 				}
-				(void)sync_attrs(&result, 1);
-				result.attrs[result.attrs_len - 1] = '0' + width;
+				cline_set_attr(&result, '0' + width);
 				width = 0;
 				break;
 
@@ -567,33 +562,13 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 	if(opt)
 	{
 		(void)strprepend(&result.line, &result.line_len, "%[");
+		(void)strprepend(&result.attrs, &result.attrs_len, "  ");
 	}
 
-	if(sync_attrs(&result, 0))
-	{
-		result.attrs[--result.attrs_len] = '\0';
-	}
+	cline_finish(&result);
 	return result;
 }
 
-/* Makes sure that result->attrs has at least as many elements as result->line
- * contains characters + extra_width.  Returns non-zero if result->attrs has
- * extra characters compared to result->line. */
-static int
-sync_attrs(LineWithAttrs *result, int extra_width)
-{
-	const size_t nchars = utf8_strsw(result->line) + extra_width;
-	if(result->attrs_len < nchars)
-	{
-		char *const new_attrs = format_str("%s%*s", result->attrs,
-				(int)(nchars - result->attrs_len), "");
-		free(result->attrs);
-		result->attrs = new_attrs;
-		result->attrs_len = nchars;
-	}
-	return (result->attrs_len > nchars);
-}
-
 /* Prints number into the buffer.  Returns non-zero if numeric value is
  * "empty" (zero). */
 static int
@@ -627,10 +602,11 @@ get_tip(void)
 	  "Trailing slash in :file[x]type, :filter and = matches directories only",
 	  ":edit forwards arguments to editor, e.g.: :edit +set\\ filetype=sh script",
 	  "Use :nmap/:vmap/etc. for short information on available mappings",
+	  "c key in :file menu puts command into command-line to edit before running",
+	  ":copen reopens the last list of files found by a :grep-like command",
+	  "Up/down arrows on command-line load history entries with identical prefix",
 	};
 
-	time_t now;
-
 	if(need_to_shuffle)
 	{
 		/* Fisher-Yates ("Knuth") shuffle algorithm implementation that mildly
@@ -647,7 +623,7 @@ get_tip(void)
 		need_to_shuffle = 0;
 	}
 
-	now = time(NULL);
+	time_t now = time(NULL);
 	if(now > last_time + SECS_PER_MIN)
 	{
 		last_time = now;
@@ -668,11 +644,14 @@ check_expanded_str(const char buf[], int skip, int *nexpansions)
 }
 
 int
-ui_stat_reposition(int statusbar_height, int force_stat_win)
+ui_stat_reposition(int statusbar_height, int stat_height)
 {
-	const int stat_line_height = (force_stat_win || cfg.display_statusline)
-	                           ? getmaxy(stat_win)
-	                           : 0;
+	int stat_line_height = stat_height;
+	if(stat_line_height == 0 && cfg.display_statusline)
+	{
+		stat_line_height = getmaxy(stat_win);
+	}
+
 	const int job_bar_height = ui_stat_job_bar_height();
 	const int y = getmaxy(stdscr)
 	            - statusbar_height
@@ -685,7 +664,8 @@ ui_stat_reposition(int statusbar_height, int force_stat_win)
 		wresize(job_bar, job_bar_height, getmaxx(job_bar));
 	}
 
-	if(force_stat_win || cfg.display_statusline)
+	if(stat_height > 0 ||
+			(cfg.display_statusline && !curr_stats.reusing_statusline))
 	{
 		mvwin(stat_win, y + job_bar_height, 0);
 		return 1;
@@ -693,6 +673,133 @@ ui_stat_reposition(int statusbar_height, int force_stat_win)
 	return 0;
 }
 
+int
+ui_stat_height(void)
+{
+	if(!cfg.display_statusline)
+	{
+		return 0;
+	}
+
+	char *status_line = fetch_status_line(curr_view, getmaxx(stdscr));
+	const char *part = status_line;
+
+	int height = 0;
+	do
+	{
+		++height;
+	}
+	while(find_view_macro(&part, STATUS_LINE_MACROS, 'N', 0) != NULL);
+
+	free(status_line);
+
+	if(curr_stats.reusing_statusline)
+	{
+		return MIN(getmaxy(stat_win), height);
+	}
+	return height;
+}
+
+/* Retrieves status line format.  Returns newly allocated string. */
+static char *
+fetch_status_line(view_t *view, int width)
+{
+	if(vlua_handler_cmd(curr_stats.vlua, cfg.status_line))
+	{
+		return vlua_make_status_line(curr_stats.vlua, cfg.status_line, view, width);
+	}
+
+	return strdup(cfg.status_line);
+}
+
+/* strstr() for format line.  Basically parse_view_macros() in dry mode.
+ * Returns position of a particular macro or NULL.  *format is updated to keep
+ * the state between successive calls. */
+TSTATIC char *
+find_view_macro(const char **format, const char macros[], char macro, int opt)
+{
+	/* Mind that parse_view_macros() needs to be in sync with this function. */
+
+	char c;
+	while((c = **format) != '\0')
+	{
+		const char *const next = ++*format;
+
+		if(c != '%' ||
+				(!char_is_one_of(macros, *next) && !isdigit(*next) &&
+				 *next != '=' && *next != 'N'))
+		{
+			continue;
+		}
+
+		if(*next == macro)
+		{
+			++*format;
+			return (char *)next - 1;
+		}
+
+		if(*next == '=' || *next == 'N')
+		{
+			++*format;
+			continue;
+		}
+
+		if(*next == '-')
+		{
+			++*format;
+		}
+
+		while(isdigit(**format))
+		{
+			++*format;
+		}
+		c = *(*format)++;
+
+		int ok = 1;
+
+		if(c == '[')
+		{
+			char *ptr = find_view_macro(format, macros, macro, 1);
+			if(ptr != NULL)
+			{
+				return ptr;
+			}
+		}
+		else if(c == ']')
+		{
+			if(opt)
+			{
+				return NULL;
+			}
+
+			/* Unmatched %]. */
+			ok = 0;
+		}
+		else if(c == '{')
+		{
+			const char *e = strchr(*format, '}');
+			if(e == NULL)
+			{
+				ok = 0;
+				break;
+			}
+
+			*format = e + 1;
+		}
+		else if(!char_is_one_of(macros, c))
+		{
+			ok = 0;
+		}
+
+		if(!ok)
+		{
+			*format = next;
+		}
+	}
+
+	return NULL;
+}
+
 void
 ui_stat_refresh(void)
 {
@@ -732,7 +839,7 @@ ui_stat_job_bar_add(bg_op_t *bg_op)
 
 	if(ui_stat_job_bar_height() != prev_height)
 	{
-		stats_redraw_schedule();
+		stats_redraw_later();
 	}
 }
 
@@ -748,19 +855,18 @@ ui_stat_job_bar_remove(bg_op_t *bg_op)
 		{
 			memmove(&bar_jobs[i], &bar_jobs[i + 1],
 					sizeof(*bar_jobs)*(nbar_jobs - 1 - i));
+			--nbar_jobs;
 			break;
 		}
 	}
 
-	--nbar_jobs;
-
 	if(ui_stat_job_bar_height() != 0)
 	{
 		ui_stat_job_bar_redraw();
 	}
 	else if(prev_height != 0)
 	{
-		stats_redraw_schedule();
+		stats_redraw_later();
 	}
 }
 
