@@ -64,6 +64,7 @@
 #include "../event_loop.h"
 #include "../filelist.h"
 #include "../flist_sel.h"
+#include "../macros.h"
 #include "../opt_handlers.h"
 #include "../status.h"
 #include "../vifm.h"
@@ -71,6 +72,7 @@
 #include "cancellation.h"
 #include "color_manager.h"
 #include "color_scheme.h"
+#include "colored_line.h"
 #include "colors.h"
 #include "fileview.h"
 #include "quickview.h"
@@ -78,6 +80,19 @@
 #include "statusline.h"
 #include "tabs.h"
 
+/* Information for formatting tab title. */
+typedef struct
+{
+	char *name;               /* Tab name. */
+	char *num;                /* Tab number. */
+	char *escaped_view_title; /* Auto-formatted title. */
+	char *escaped_path;       /* Current path (could be a file path). */
+	char *cv_title;           /* Prefix of custom view. */
+	int tree;                 /* Whether in tree mode. */
+	int current;              /* Whether it's a current tab. */
+}
+tab_title_info_t;
+
 /* Type of path transformation function for format_view_title(). */
 typedef char * (*path_func)(const char[]);
 
@@ -93,20 +108,41 @@ static WINDOW *ltop_line2;
 static WINDOW *rtop_line1;
 static WINDOW *rtop_line2;
 
-/* Mutexes for views, located out of view_t so that they are never moved nor
- * copied, which would yield undefined behaviour. */
-static pthread_mutex_t lwin_timestamps_mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_mutex_t rwin_timestamps_mutex = PTHREAD_MUTEX_INITIALIZER;
+unsigned int ui_next_view_id = 3;
 
-view_t lwin = { .timestamps_mutex = &lwin_timestamps_mutex };
-view_t rwin = { .timestamps_mutex = &rwin_timestamps_mutex };
+view_t lwin = { .id = 1 };
+view_t rwin = { .id = 2 };
 
-static int pair_in_use(short int pair);
-static void move_pair(short int from, short int to);
+view_t *other_view;
+view_t *curr_view;
+
+WINDOW *status_bar;
+WINDOW *stat_win;
+WINDOW *job_bar;
+WINDOW *ruler_win;
+WINDOW *input_win;
+WINDOW *menu_win;
+WINDOW *sort_win;
+WINDOW *change_win;
+WINDOW *error_win;
+
+static WINDOW *top_line;
+static WINDOW *tab_line;
+
+static WINDOW *lborder;
+static WINDOW *mborder;
+static WINDOW *rborder;
+
+static int init_pair_wrapper(int pair, int fg, int bg);
+static int pair_content_wrapper(int pair, int *fg, int *bg);
+static int pair_in_use(int pair);
+static void move_pair(int from, int to);
 static void create_windows(void);
 static void update_geometry(void);
+static int update_start(UpdateType update_kind);
+static void update_finish(void);
+static void adjust_splitter(int screen_w, int screen_h);
 static int get_working_area_height(void);
-static void resize_all(void);
 static void clear_border(WINDOW *border);
 static int middle_border_is_visible(void);
 static void update_views(int reload);
@@ -126,17 +162,25 @@ static void refresh_bottom_lines(void);
 static char * path_identity(const char path[]);
 static int view_shows_tabline(const view_t *view);
 static int get_tabline_height(void);
-static void print_tab_title(WINDOW *win, view_t *view, col_attr_t base_col,
+static void print_tabline(WINDOW *win, view_t *view, col_attr_t base_col,
 		path_func pf);
-static void compute_avg_width(int *avg_width, int *spare_width, int max_width,
-		view_t *view, path_func pf);
-static char * make_tab_title(const tab_info_t *tab_info, path_func pf);
+static void compute_avg_width(int *avg_width, int *spare_width,
+		int min_widths[], int max_width, view_t *view, path_func pf);
+TSTATIC cline_t make_tab_title(const tab_title_info_t *title_info);
+TSTATIC tab_title_info_t make_tab_title_info(const tab_info_t *tab_info,
+		path_func pf, int tab_num, int current_tab);
+TSTATIC void dispose_tab_title_info(tab_title_info_t *title_info);
+static cline_t format_tab_part(const char fmt[],
+		const tab_title_info_t *title_info);
 static char * format_view_title(const view_t *view, path_func pf);
 static void print_view_title(const view_t *view, int active_view, char title[]);
 static col_attr_t fixup_titles_attributes(const view_t *view, int active_view);
 static int is_in_miller_view(const view_t *view);
 static int is_forced_list_mode(const view_t *view);
 
+/* List of macros that are expanded in the ruler. */
+static const char RULER_MACROS[] = "-xlLPS%[]";
+
 void
 ui_ruler_update(view_t *view, int lazy_redraw)
 {
@@ -203,8 +247,8 @@ setup_ncurses_interface(void)
 	const colmgr_conf_t colmgr_conf = {
 		.max_color_pairs = COLOR_PAIRS,
 		.max_colors = COLORS,
-		.init_pair = &init_pair,
-		.pair_content = &pair_content,
+		.init_pair = &init_pair_wrapper,
+		.pair_content = &pair_content_wrapper,
 		.pair_in_use = &pair_in_use,
 		.move_pair = &move_pair,
 	};
@@ -230,15 +274,64 @@ setup_ncurses_interface(void)
 #endif
 #endif
 
-	resize_all();
+#if defined(NCURSES_VERSION) && defined(ENABLE_EXTENDED_KEYS)
+	/* Disable 'unsupported' extended keys so that esc-codes will be
+	 * received instead of unnamed extended keycode values (> KEY_MAX).
+	 * These keys can then be mapped with esc-codes in vifmrc.
+	 * An example could be <c-down>:
+	 *     qnoremap <esc>[1;5B j
+	 * as without this change <c-down> can return a keycode value of 531
+	 * (terminfo name kDN5), which is larger than KEY_MAX and has no
+	 * pre-defined curses key name. */
+	int i;
+	for(i = KEY_MAX; i < 2000; ++i)
+	{
+		keyok(i, FALSE);
+	}
+#endif
+
+#ifdef HAVE_EXTENDED_COLORS
+	curr_stats.direct_color = (COLORS == 0x1000000);
+#endif
+
+	ui_resize_all();
 
 	return 1;
 }
 
+/* Calls init_pair() from libcurses. */
+static int
+init_pair_wrapper(int pair, int fg, int bg)
+{
+#ifdef HAVE_EXTENDED_COLORS
+	return init_extended_pair(pair, fg, bg);
+#else
+	return init_pair(pair, fg, bg);
+#endif
+}
+
+/* Calls pair_content() from libcurses. */
+static int
+pair_content_wrapper(int pair, int *fg, int *bg)
+{
+#ifdef HAVE_EXTENDED_COLORS
+	return extended_pair_content(pair, fg, bg);
+#else
+	short fg_short, bg_short;
+
+	const int result = pair_content(pair, &fg_short, &bg_short);
+
+	*fg = fg_short;
+	*bg = bg_short;
+
+	return result;
+#endif
+}
+
 /* Checks whether pair is being used at the moment.  Returns non-zero if so and
  * zero otherwise. */
 static int
-pair_in_use(short int pair)
+pair_in_use(int pair)
 {
 	int i;
 
@@ -256,7 +349,7 @@ pair_in_use(short int pair)
 
 /* Substitutes old pair number with the new one. */
 static void
-move_pair(short int from, short int to)
+move_pair(int from, int to)
 {
 	int i;
 	for(i = 0; i < MAXNUM_COLOR; ++i)
@@ -368,7 +461,7 @@ ui_char_pressed(wint_t c)
 		return 0;
 	}
 
-	const int cancellation_state = ui_cancellation_pause();
+	ui_cancellation_push_off();
 	wint_t pressed;
 
 	while(1)
@@ -391,7 +484,7 @@ ui_char_pressed(wint_t c)
 		}
 	}
 
-	ui_cancellation_resume(cancellation_state);
+	ui_cancellation_pop();
 
 	return (pressed == c);
 }
@@ -418,12 +511,12 @@ correct_size(view_t *view)
 static void
 only_layout(view_t *view, int screen_x)
 {
-	const int vborder_pos_correction = cfg.side_borders_visible ? 1 : 0;
-	const int vborder_size_correction = cfg.side_borders_visible ? -2 : 0;
+	const int pos_correction = cfg.side_borders_visible ? 1 : 0;
+	const int size_correction = cfg.side_borders_visible ? -2 : 0;
 	const int y = get_tabline_height();
 
-	wresize(view->title, 1, screen_x - 2);
-	mvwin(view->title, y, 1);
+	wresize(view->title, 1, screen_x + size_correction);
+	mvwin(view->title, y, pos_correction);
 
 	mvwin(ltop_line1, y, 0);
 	mvwin(ltop_line2, y, 0);
@@ -434,9 +527,8 @@ only_layout(view_t *view, int screen_x)
 	wresize(tab_line, 1, screen_x);
 	mvwin(tab_line, 0, 0);
 
-	wresize(view->win, get_working_area_height(),
-			screen_x + vborder_size_correction);
-	mvwin(view->win, y + 1, vborder_pos_correction);
+	wresize(view->win, get_working_area_height(), screen_x + size_correction);
+	mvwin(view->win, y + 1, pos_correction);
 }
 
 /* Updates TUI elements sizes and coordinates for vertical configuration of
@@ -444,32 +536,30 @@ only_layout(view_t *view, int screen_x)
 static void
 vertical_layout(int screen_x)
 {
-	const int vborder_pos_correction = cfg.side_borders_visible ? 1 : 0;
-	const int vborder_size_correction = cfg.side_borders_visible ? -1 : 0;
+	const int pos_correction = cfg.side_borders_visible ? 1 : 0;
+	const int size_correction = cfg.side_borders_visible ? -1 : 0;
 	const int border_height = get_working_area_height();
 	const int y = get_tabline_height();
 
 	int splitter_pos;
-	int splitter_width;
-
 	if(curr_stats.splitter_pos < 0)
 		splitter_pos = screen_x/2 - 1 + screen_x%2;
 	else
 		splitter_pos = curr_stats.splitter_pos;
 
-	splitter_width = 2 - screen_x%2;
+	int splitter_width = (cfg.flexible_splitter ? 2 - screen_x%2 : 1);
 	if(splitter_pos < 4)
 		splitter_pos = 4;
 	if(splitter_pos > screen_x - 4 - splitter_width)
 		splitter_pos = screen_x - 4 - splitter_width;
 	if(curr_stats.splitter_pos >= 0)
-		curr_stats.splitter_pos = splitter_pos;
+		stats_set_splitter_pos(splitter_pos);
 
-	wresize(lwin.title, 1, splitter_pos - 1);
-	mvwin(lwin.title, y, 1);
+	wresize(lwin.title, 1, splitter_pos + size_correction);
+	mvwin(lwin.title, y, pos_correction);
 
-	wresize(lwin.win, border_height, splitter_pos + vborder_size_correction);
-	mvwin(lwin.win, y + 1, vborder_pos_correction);
+	wresize(lwin.win, border_height, splitter_pos + size_correction);
+	mvwin(lwin.win, y + 1, pos_correction);
 
 	ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 	wresize(mborder, border_height, splitter_width);
@@ -487,11 +577,12 @@ vertical_layout(int screen_x)
 	mvwin(rtop_line1, y, screen_x - 1);
 	mvwin(rtop_line2, y, screen_x - 1);
 
-	wresize(rwin.title, 1, screen_x - (splitter_pos + splitter_width + 1));
+	wresize(rwin.title, 1,
+			screen_x - (splitter_pos + splitter_width) + size_correction);
 	mvwin(rwin.title, y, splitter_pos + splitter_width);
 
 	wresize(rwin.win, border_height,
-			screen_x - (splitter_pos + splitter_width) + vborder_size_correction);
+			screen_x - (splitter_pos + splitter_width) + size_correction);
 	mvwin(rwin.win, y + 1, splitter_pos + splitter_width);
 }
 
@@ -500,8 +591,8 @@ vertical_layout(int screen_x)
 static void
 horizontal_layout(int screen_x, int screen_y)
 {
-	const int vborder_pos_correction = cfg.side_borders_visible ? 1 : 0;
-	const int vborder_size_correction = cfg.side_borders_visible ? -2 : 0;
+	const int pos_correction = cfg.side_borders_visible ? 1 : 0;
+	const int size_correction = cfg.side_borders_visible ? -2 : 0;
 	const int y = get_tabline_height();
 
 	int splitter_pos;
@@ -515,20 +606,20 @@ horizontal_layout(int screen_x, int screen_y)
 	if(splitter_pos > get_working_area_height() - 1)
 		splitter_pos = get_working_area_height() - 1;
 	if(curr_stats.splitter_pos >= 0)
-		curr_stats.splitter_pos = splitter_pos;
+		stats_set_splitter_pos(splitter_pos);
 
-	wresize(lwin.title, 1, screen_x - 2);
-	mvwin(lwin.title, y, 1);
+	wresize(lwin.title, 1, screen_x + size_correction);
+	mvwin(lwin.title, y, pos_correction);
 
-	wresize(rwin.title, 1, screen_x - 2);
-	mvwin(rwin.title, splitter_pos, 1);
+	wresize(rwin.title, 1, screen_x + size_correction);
+	mvwin(rwin.title, splitter_pos, pos_correction);
 
-	wresize(lwin.win, splitter_pos - (y + 1), screen_x + vborder_size_correction);
-	mvwin(lwin.win, y + 1, vborder_pos_correction);
+	wresize(lwin.win, splitter_pos - (y + 1), screen_x + size_correction);
+	mvwin(lwin.win, y + 1, pos_correction);
 
 	wresize(rwin.win, get_working_area_height() - splitter_pos + y,
-			screen_x + vborder_size_correction);
-	mvwin(rwin.win, splitter_pos + 1, vborder_pos_correction);
+			screen_x + size_correction);
+	mvwin(rwin.win, splitter_pos + 1, pos_correction);
 
 	ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 	wresize(mborder, 1, screen_x);
@@ -558,12 +649,12 @@ horizontal_layout(int screen_x, int screen_y)
 static int
 get_working_area_height(void)
 {
-	return getmaxy(stdscr)                  /* Total available height. */
-	     - 1                                /* Top line. */
-	     - (cfg.display_statusline ? 1 : 0) /* Status line. */
-	     - ui_stat_job_bar_height()         /* Job bar. */
-	     - 1                                /* Status bar line. */
-	     - get_tabline_height();            /* Tab line. */
+	return getmaxy(stdscr)          /* Total available height. */
+	     - 1                        /* Top line. */
+	     - ui_stat_height()         /* Status line. */
+	     - ui_stat_job_bar_height() /* Job bar. */
+	     - 1                        /* Status bar line. */
+	     - get_tabline_height();    /* Tab line. */
 }
 
 /* Updates internal data structures to reflect actual terminal geometry. */
@@ -623,17 +714,38 @@ cv_tree(CVType type)
 void
 update_screen(UpdateType update_kind)
 {
-	if(curr_stats.load_stage < 2)
-		return;
+	if(update_start(update_kind))
+	{
+		update_all_windows();
+		update_finish();
+	}
+}
 
-	if(update_kind == UT_NONE)
-		return;
+void
+ui_redraw_as_background(void)
+{
+	if(update_start(UT_REDRAW))
+	{
+		update_finish();
+	}
+}
 
-	resize_all();
+/* Most of the update logic.  Everything that's done before updating windows.
+ * Returns non-zero if update was carried out until the end, otherwise zero is
+ * returned. */
+static int
+update_start(UpdateType update_kind)
+{
+	if(curr_stats.load_stage < 2 || update_kind == UT_NONE)
+	{
+		return 0;
+	}
+
+	ui_resize_all();
 
 	if(curr_stats.restart_in_progress)
 	{
-		return;
+		return 0;
 	}
 
 	update_attributes();
@@ -650,10 +762,9 @@ update_screen(UpdateType update_kind)
 
 	if(curr_stats.term_state != TS_NORMAL)
 	{
-		return;
+		return 0;
 	}
 
-	curr_stats.need_update = UT_NONE;
 	qv_ui_updated();
 
 	update_views(update_kind == UT_FULL);
@@ -680,7 +791,7 @@ update_screen(UpdateType update_kind)
 
 		if(vle_mode_is(VIEW_MODE))
 		{
-			view_ruler_update();
+			modview_ruler_update();
 		}
 		else
 		{
@@ -690,17 +801,22 @@ update_screen(UpdateType update_kind)
 
 	if(curr_stats.save_msg == 0)
 	{
-		ui_sb_msg("");
+		ui_sb_clear();
 	}
 
 	if(vle_mode_is(VIEW_MODE) ||
 			(curr_stats.number_of_windows == 2 && other_view->explore_mode))
 	{
-		view_redraw();
+		modview_redraw();
 	}
 
-	update_all_windows();
+	return 1;
+}
 
+/* Post-windows-update part of an update. */
+static void
+update_finish(void)
+{
 	if(!curr_view->explore_mode)
 	{
 		fview_cursor_redraw(curr_view);
@@ -708,17 +824,11 @@ update_screen(UpdateType update_kind)
 
 	update_input_buf();
 	ui_stat_job_bar_redraw();
-
-	curr_stats.need_update = UT_NONE;
 }
 
-/* Resizes all windows according to current screen size and TUI
- * configuration. */
-static void
-resize_all(void)
+void
+ui_resize_all(void)
 {
-	static float prev_w = -1.f, prev_h = -1.f;
-
 	update_geometry();
 
 	int screen_w, screen_h;
@@ -730,22 +840,7 @@ resize_all(void)
 		return;
 	}
 
-	if(prev_w < 0)
-	{
-		prev_w = screen_w;
-		prev_h = screen_h;
-	}
-
-	if(curr_stats.splitter_pos >= 0)
-	{
-		if(curr_stats.split == HSPLIT)
-			curr_stats.splitter_pos *= screen_h/prev_h;
-		else
-			curr_stats.splitter_pos *= screen_w/prev_w;
-	}
-
-	prev_w = screen_w;
-	prev_h = screen_h;
+	adjust_splitter(screen_w, screen_h);
 
 	wresize(stdscr, screen_h, screen_w);
 	wresize(menu_win, screen_h - 1, screen_w);
@@ -785,7 +880,7 @@ resize_all(void)
 	correct_size(&lwin);
 	correct_size(&rwin);
 
-	wresize(stat_win, 1, screen_w);
+	wresize(stat_win, ui_stat_height(), screen_w);
 	(void)ui_stat_reposition(1, 0);
 
 	wresize(job_bar, 1, screen_w);
@@ -795,6 +890,27 @@ resize_all(void)
 	curs_set(0);
 }
 
+/* Adjusts splitter position after screen resize. */
+static void
+adjust_splitter(int screen_w, int screen_h)
+{
+	static int prev_w = -1, prev_h = -1;
+
+	if(curr_stats.splitter_pos < 0)
+	{
+		prev_w = -1;
+		prev_h = -1;
+		return;
+	}
+
+	if(prev_w != screen_w || prev_h != screen_h)
+	{
+		stats_set_splitter_ratio(curr_stats.splitter_ratio);
+		prev_w = screen_w;
+		prev_h = screen_h;
+	}
+}
+
 /* Clears border, possibly by filling it with a pattern (depends on
  * configuration). */
 static void
@@ -913,7 +1029,7 @@ change_window(void)
 	}
 	else
 	{
-		curr_stats.need_update = UT_REDRAW;
+		stats_redraw_later();
 	}
 }
 
@@ -938,16 +1054,12 @@ update_all_windows(void)
 void
 touch_all_windows(void)
 {
-	int in_menu;
-
 	if(curr_stats.load_stage < 2)
 	{
 		return;
 	}
 
-	in_menu = is_in_menu_like_mode();
-
-	if(!in_menu)
+	if(!is_in_menu_like_mode())
 	{
 		update_window_lazy(tab_line);
 
@@ -968,6 +1080,13 @@ touch_all_windows(void)
 
 		if(cfg.side_borders_visible)
 		{
+			/* This needs to be updated before things like status bar and status line
+			 * to account for cases when they hide the top line. */
+			update_window_lazy(ltop_line1);
+			update_window_lazy(ltop_line2);
+			update_window_lazy(rtop_line1);
+			update_window_lazy(rtop_line2);
+
 			update_window_lazy(lborder);
 			update_window_lazy(rborder);
 		}
@@ -987,14 +1106,6 @@ touch_all_windows(void)
 	update_window_lazy(input_win);
 	update_window_lazy(status_bar);
 
-	if(!in_menu)
-	{
-		update_window_lazy(ltop_line1);
-		update_window_lazy(ltop_line2);
-		update_window_lazy(rtop_line1);
-		update_window_lazy(rtop_line2);
-	}
-
 	if(vle_mode_is(MSG_MODE))
 	{
 		/* Location of message dialog can change, so it's better to let it position
@@ -1023,10 +1134,8 @@ static void
 update_window_lazy(WINDOW *win)
 {
 	touchwin(win);
-	/*
-	 * redrawwin() shouldn't be needed.  But without it there is a
-	 * lot of flickering when redrawing the windows?
-	 */
+	/* Tell curses that it shouldn't assume that screen isn't messed up in any
+	 * way. */
 	redrawwin(win);
 	wnoutrefresh(win);
 }
@@ -1108,6 +1217,11 @@ show_progress(const char msg[], int period)
 void
 redraw_lists(void)
 {
+	if(curr_stats.load_stage == 0)
+	{
+		return;
+	}
+
 	redraw_current_view();
 	if(curr_stats.number_of_windows == 2)
 	{
@@ -1137,30 +1251,31 @@ update_attributes(void)
 		werase(lborder);
 		ui_set_bg(rborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 		werase(rborder);
+
+		ui_set_bg(ltop_line1, &cfg.cs.color[TOP_LINE_COLOR],
+				cfg.cs.pair[TOP_LINE_COLOR]);
+		werase(ltop_line1);
+
+		ui_set_bg(ltop_line2, &cfg.cs.color[TOP_LINE_COLOR],
+				cfg.cs.pair[TOP_LINE_COLOR]);
+		werase(ltop_line2);
+
+		ui_set_bg(top_line, &cfg.cs.color[TOP_LINE_COLOR],
+				cfg.cs.pair[TOP_LINE_COLOR]);
+		werase(top_line);
+
+		ui_set_bg(rtop_line1, &cfg.cs.color[TOP_LINE_COLOR],
+				cfg.cs.pair[TOP_LINE_COLOR]);
+		werase(rtop_line1);
+
+		ui_set_bg(rtop_line2, &cfg.cs.color[TOP_LINE_COLOR],
+				cfg.cs.pair[TOP_LINE_COLOR]);
+		werase(rtop_line2);
 	}
+
 	ui_set_bg(mborder, &cfg.cs.color[BORDER_COLOR], cfg.cs.pair[BORDER_COLOR]);
 	werase(mborder);
 
-	ui_set_bg(ltop_line1, &cfg.cs.color[TOP_LINE_COLOR],
-			cfg.cs.pair[TOP_LINE_COLOR]);
-	werase(ltop_line1);
-
-	ui_set_bg(ltop_line2, &cfg.cs.color[TOP_LINE_COLOR],
-			cfg.cs.pair[TOP_LINE_COLOR]);
-	werase(ltop_line2);
-
-	ui_set_bg(top_line, &cfg.cs.color[TOP_LINE_COLOR],
-			cfg.cs.pair[TOP_LINE_COLOR]);
-	werase(top_line);
-
-	ui_set_bg(rtop_line1, &cfg.cs.color[TOP_LINE_COLOR],
-			cfg.cs.pair[TOP_LINE_COLOR]);
-	werase(rtop_line1);
-
-	ui_set_bg(rtop_line2, &cfg.cs.color[TOP_LINE_COLOR],
-			cfg.cs.pair[TOP_LINE_COLOR]);
-	werase(rtop_line2);
-
 	ui_set_bg(tab_line, &cfg.cs.color[TAB_LINE_COLOR],
 			cfg.cs.pair[TAB_LINE_COLOR]);
 	werase(tab_line);
@@ -1220,7 +1335,10 @@ wprinta(WINDOW *win, const char str[], const cchar_t *line_attrs,
 	wchar_t wch[getcchar(line_attrs, NULL, &attrs, &color_pair, NULL)];
 	getcchar(line_attrs, wch, &attrs, &color_pair, NULL);
 
-	col_attr_t col = { .attr = attrs ^ attrs_xors };
+	col_attr_t col = {
+		.attr = attrs ^ attrs_xors,
+		.gui_attr = attrs ^ attrs_xors
+	};
 	ui_set_attr(win, &col, color_pair);
 	wprint(win, str);
 	wnoutrefresh(win);
@@ -1378,7 +1496,7 @@ get_ruler_width(view_t *view)
 static char *
 expand_ruler_macros(view_t *view, const char format[])
 {
-	return expand_view_macros(view, format, "-xlLS%[]");
+	return expand_view_macros(view, format, RULER_MACROS);
 }
 
 void
@@ -1413,7 +1531,7 @@ void
 switch_panes(void)
 {
 	switch_panes_content();
-	view_try_activate_mode();
+	modview_try_activate();
 }
 
 void
@@ -1453,12 +1571,12 @@ switch_panes_content(void)
 {
 	ui_swap_view_data(&lwin, &rwin);
 
-	flist_update_origins(&lwin, &rwin.curr_dir[0], &lwin.curr_dir[0]);
-	flist_update_origins(&rwin, &lwin.curr_dir[0], &rwin.curr_dir[0]);
+	flist_update_origins(&lwin);
+	flist_update_origins(&rwin);
 
-	view_panes_swapped();
+	modview_panes_swapped();
 
-	curr_stats.need_update = UT_REDRAW;
+	stats_redraw_later();
 }
 
 void
@@ -1493,7 +1611,7 @@ void
 go_to_other_pane(void)
 {
 	change_window();
-	view_try_activate_mode();
+	modview_try_activate();
 }
 
 void
@@ -1502,17 +1620,15 @@ split_view(SPLIT orientation)
 	if(curr_stats.number_of_windows == 2 && curr_stats.split == orientation)
 		return;
 
-	if(curr_stats.number_of_windows == 2 && curr_stats.splitter_pos > 0)
-	{
-		if(orientation == VSPLIT)
-			curr_stats.splitter_pos *= (float)getmaxx(stdscr)/getmaxy(stdscr);
-		else
-			curr_stats.splitter_pos *= (float)getmaxy(stdscr)/getmaxx(stdscr);
-	}
-
 	curr_stats.split = orientation;
 	curr_stats.number_of_windows = 2;
-	curr_stats.need_update = UT_REDRAW;
+
+	if(curr_stats.number_of_windows == 2 && curr_stats.splitter_pos > 0)
+	{
+		stats_set_splitter_ratio(curr_stats.splitter_ratio);
+	}
+
+	stats_redraw_later();
 }
 
 void
@@ -1528,20 +1644,22 @@ only(void)
 void
 move_splitter(int by, int fact)
 {
+	int pos = curr_stats.splitter_pos;
+
 	/* Determine exact splitter position if it's centered at the moment. */
 	if(curr_stats.splitter_pos < 0)
 	{
-		if(curr_stats.split == VSPLIT)
+		if(curr_stats.split == HSPLIT)
 		{
-			curr_stats.splitter_pos = getmaxx(stdscr)/2 - 1 + getmaxx(stdscr)%2;
+			pos = getmaxy(stdscr)/2 - 1;
 		}
 		else
 		{
-			curr_stats.splitter_pos = getmaxy(stdscr)/2 - 1;
+			pos = getmaxx(stdscr)/2 - 1 + getmaxx(stdscr)%2;
 		}
 	}
 
-	set_splitter(curr_stats.splitter_pos + fact*by);
+	set_splitter(pos + fact*by);
 }
 
 void
@@ -1568,8 +1686,7 @@ ui_view_resize(view_t *view, int to)
 static void
 set_splitter(int pos)
 {
-	curr_stats.splitter_pos = (pos < 0) ? 0 : pos;
-	update_screen(UT_REDRAW);
+	stats_set_splitter_pos(pos < 0 ? 0 : pos);
 }
 
 void
@@ -1796,7 +1913,7 @@ ui_view_title_update(view_t *view)
 		return;
 	}
 
-	if(view == selected && cfg.set_title && !stats_silenced_ui())
+	if(view == selected && !stats_silenced_ui())
 	{
 		char *const term_title = format_view_title(view, pf);
 		term_title_update(term_title);
@@ -1807,7 +1924,7 @@ ui_view_title_update(view_t *view)
 
 	if(view_shows_tabline(view))
 	{
-		print_tab_title(view->title, view, title_col, pf);
+		print_tabline(view->title, view, title_col, pf);
 	}
 	else
 	{
@@ -1819,7 +1936,7 @@ ui_view_title_update(view_t *view)
 
 	if(view == curr_view && get_tabline_height() > 0)
 	{
-		print_tab_title(tab_line, view, cfg.cs.color[TAB_LINE_COLOR], pf);
+		print_tabline(tab_line, view, cfg.cs.color[TAB_LINE_COLOR], pf);
 	}
 }
 
@@ -1857,36 +1974,64 @@ get_tabline_height(void)
 
 /* Prints title of the tab on specified curses window. */
 static void
-print_tab_title(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
+print_tabline(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
 {
 	int i;
 	tab_info_t tab_info;
 
-	const int max_width = getmaxx(win);
+	const int max_width = (vifm_testing() ? cfg.columns : getmaxx(win));
 	int width_used = 0;
 	int avg_width, spare_width;
 
+	int min_widths[tabs_count(view)];
+
 	ui_set_bg(win, &base_col, -1);
 	werase(win);
 	checked_wmove(win, 0, 0);
 
-	compute_avg_width(&avg_width, &spare_width, max_width, view, pf);
+	compute_avg_width(&avg_width, &spare_width, min_widths, max_width, view, pf);
 
-	for(i = 0; tabs_get(view, i, &tab_info); ++i)
+	int tab_count = tabs_count(view);
+	int min_width = 0;
+	for(i = 0; i < tab_count; ++i)
 	{
-		char *title = make_tab_title(&tab_info, pf);
-		const int width_needed = utf8_strsw(title);
-		char buffer_dummy[1];
-		const int extra_width = snprintf(buffer_dummy, 0U, "[%d:]", i + 1);
+		min_width += min_widths[i];
+	}
+
+	int before_current = 1;
+
+	for(i = 0; tabs_get(view, i, &tab_info) && width_used < max_width; ++i)
+	{
+		int current = (tab_info.view == view);
+		if(current)
+		{
+			before_current = 0;
+		}
+		else if(before_current && min_width > max_width)
+		{
+			min_width -= min_widths[i];
+			spare_width = max_width - min_width;
+			continue;
+		}
+
+		tab_title_info_t title_info = make_tab_title_info(&tab_info, pf, i,
+				current);
+		cline_t prefix = format_tab_part(cfg.tab_prefix, &title_info);
+		cline_t title = make_tab_title(&title_info);
+		cline_t suffix = format_tab_part(cfg.tab_suffix, &title_info);
+		dispose_tab_title_info(&title_info);
+
+		const int width_needed = title.attrs_len;
+		const int extra_width = prefix.attrs_len + suffix.attrs_len;
 		int width = max_width;
 
 		col_attr_t col = base_col;
-		if(tab_info.view == view)
+		if(current)
 		{
 			cs_mix_colors(&col, &cfg.cs.color[TAB_LINE_SEL_COLOR]);
 		}
 
-		if(tab_info.view != view)
+		if(!current)
 		{
 			width = tab_info.last ? (max_width - width_used)
 			                      : MIN(avg_width, extra_width + width_needed);
@@ -1894,32 +2039,36 @@ print_tab_title(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
 
 		if(width < width_needed + extra_width)
 		{
-			char *ellipsed;
-
 			if(spare_width > 0)
 			{
 				width += 1;
 				--spare_width;
 			}
 
-			ellipsed = left_ellipsis(title, MAX(0, width - extra_width),
+			cline_left_ellipsis(&title, MAX(0, width - extra_width),
 					curr_stats.ellipsis);
-			free(title);
-			title = ellipsed;
 		}
 
 		ui_set_attr(win, &col, -1);
 
-		if(width > extra_width)
+		int real_width = prefix.attrs_len + title.attrs_len + suffix.attrs_len;
+
+		if(width < real_width && max_width - width_used >= real_width)
 		{
-			wprintw(win, "[%d:%s]", i + 1, title);
+			width = real_width;
+		}
+		if(width >= real_width)
+		{
+			cline_print(&prefix, win, &col);
+			cline_print(&title, win, &col);
+			cline_print(&suffix, win, &col);
 		}
 
-		/* Here result of `utf8_strsw(title)` might be different from one computed
-		 * above. */
-		width_used += extra_width + utf8_strsw(title);
+		width_used += real_width;
 
-		free(title);
+		cline_dispose(&prefix);
+		cline_dispose(&title);
+		cline_dispose(&suffix);
 	}
 
 	wnoutrefresh(win);
@@ -1928,8 +2077,8 @@ print_tab_title(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
 /* Computes average width of tab tips as well as number of spare character
  * positions. */
 static void
-compute_avg_width(int *avg_width, int *spare_width, int max_width, view_t *view,
-		path_func pf)
+compute_avg_width(int *avg_width, int *spare_width, int min_widths[],
+		int max_width, view_t *view, path_func pf)
 {
 	int left = max_width;
 	int widths[tabs_count(view)];
@@ -1941,15 +2090,34 @@ compute_avg_width(int *avg_width, int *spare_width, int max_width, view_t *view,
 
 	for(i = 0; tabs_get(view, i, &tab_info); ++i)
 	{
-		char *const title = make_tab_title(&tab_info, pf);
-		widths[i] = utf8_strsw(title) + snprintf(title, 0U, "[%d:]", i + 1);
-		free(title);
+		int current = (tab_info.view == view);
 
-		if(tab_info.view == view && tabs_count(view) != 1)
+		tab_title_info_t title_info = make_tab_title_info(&tab_info, pf, i,
+				current);
+		cline_t prefix = format_tab_part(cfg.tab_prefix, &title_info);
+		cline_t title = make_tab_title(&title_info);
+		cline_t suffix = format_tab_part(cfg.tab_suffix, &title_info);
+		dispose_tab_title_info(&title_info);
+
+		widths[i] = prefix.attrs_len + title.attrs_len + suffix.attrs_len;
+		min_widths[i] = prefix.attrs_len + suffix.attrs_len;
+
+		cline_dispose(&prefix);
+		cline_dispose(&title);
+		cline_dispose(&suffix);
+
+		if(current)
 		{
-			left = MAX(max_width - widths[i], 0);
-			*avg_width = left/(tabs_count(view) - 1);
-			*spare_width = left%(tabs_count(view) - 1);
+			min_widths[i] = MIN(max_width, widths[i]);
+
+			if(tabs_count(view) != 1)
+			{
+				int tab_count = tabs_count(view);
+
+				left = MAX(max_width - widths[i], 0);
+				*avg_width = left/(tab_count - 1);
+				*spare_width = left%(tab_count - 1);
+			}
 		}
 	}
 
@@ -1983,15 +2151,39 @@ compute_avg_width(int *avg_width, int *spare_width, int max_width, view_t *view,
 	while(new_avg_width != *avg_width);
 }
 
-/* Gets title of the tab.  Returns newly allocated string. */
-static char *
-make_tab_title(const tab_info_t *tab_info, path_func pf)
+/* Gets title of the tab.  Returns colored line. */
+TSTATIC cline_t
+make_tab_title(const tab_title_info_t *title_info)
 {
-	if(tab_info->name != NULL)
+	if(!is_null_or_empty(cfg.tab_label))
 	{
-		return strdup(tab_info->name);
+		return format_tab_part(cfg.tab_label, title_info);
 	}
 
+	cline_t result = cline_make();
+
+	if(is_null_or_empty(title_info->name))
+	{
+		replace_string(&result.line, title_info->escaped_view_title);
+	}
+	else
+	{
+		replace_string(&result.line, title_info->name);
+	}
+
+	result.line_len = strlen(result.line);
+	cline_finish(&result);
+	return result;
+}
+
+/* Produces and stores information needed to format tab title.  Returns the
+ * information. */
+TSTATIC tab_title_info_t
+make_tab_title_info(const tab_info_t *tab_info, path_func pf, int tab_num,
+		int current_tab)
+{
+	view_t *view = tab_info->view;
+
 	if(cfg.tail_tab_line_paths)
 	{
 		/* We can just do the replacement, because shortening home part doesn't make
@@ -1999,7 +2191,73 @@ make_tab_title(const tab_info_t *tab_info, path_func pf)
 		pf = &get_last_path_component;
 	}
 
-	return format_view_title(tab_info->view, pf);
+	char path[PATH_MAX + 1];
+	if(view->explore_mode)
+	{
+		get_current_full_path(view, sizeof(path), path);
+	}
+	else
+	{
+		copy_str(path, sizeof(path), flist_get_dir(view));
+	}
+
+	tab_title_info_t result = {
+		.current = current_tab,
+	};
+
+	const char *custom_title = "";
+	if(flist_custom_active(view))
+	{
+		custom_title = view->custom.title;
+		result.tree = cv_tree(view->custom.type);
+	}
+
+	result.name = escape_unreadable(tab_info->name == NULL ? "" : tab_info->name);
+	result.escaped_path = escape_unreadable(path);
+	result.cv_title = escape_unreadable(custom_title);
+
+	char *view_title = format_view_title(view, pf);
+	result.escaped_view_title = escape_unreadable(view_title);
+	free(view_title);
+
+	result.num = format_str("%d", tab_num + 1);
+
+	return result;
+}
+
+/* Frees information used to format tab title. */
+TSTATIC void
+dispose_tab_title_info(tab_title_info_t *title_info)
+{
+	free(title_info->num);
+	free(title_info->escaped_view_title);
+	free(title_info->cv_title);
+	free(title_info->escaped_path);
+	free(title_info->name);
+}
+
+/* Formats part of a tab label.  Returns colored line. */
+static cline_t
+format_tab_part(const char fmt[], const tab_title_info_t *title_info)
+{
+	custom_macro_t macros[] = {
+		{ .letter = 'n', .value = title_info->name },
+		{ .letter = 'N', .value = title_info->num },
+		{ .letter = 't', .value = title_info->escaped_view_title },
+		{ .letter = 'p', .value = title_info->escaped_path,
+			.expand_mods = 1, .parent = "/" },
+		{ .letter = 'c', .value = title_info->cv_title },
+
+		{ .letter = 'C', .value = (title_info->current ? "*" : ""),
+		  .flag = 1},
+		{ .letter = 'T', .value = (title_info->tree ? "*" : ""),
+			.flag = 1 },
+	};
+
+	cline_t title = ma_expand_colored_custom(fmt, ARRAY_LEN(macros), macros,
+			MA_OPT);
+
+	return title;
 }
 
 /* Formats title for the view.  The pf function will be applied to full paths.
@@ -2008,30 +2266,49 @@ make_tab_title(const tab_info_t *tab_info, path_func pf)
 static char *
 format_view_title(const view_t *view, path_func pf)
 {
+	char *unescaped_title;
+
 	if(view->explore_mode)
 	{
 		char full_path[PATH_MAX + 1];
 		get_current_full_path(view, sizeof(full_path), full_path);
-		return strdup(pf(full_path));
+		unescaped_title = strdup(pf(full_path));
 	}
 	else if(curr_stats.preview.on && view == other_view)
 	{
-		const char *const viewer = view_detached_get_viewer();
+		const char *const viewer = modview_detached_get_viewer();
 		if(viewer != NULL)
 		{
-			return format_str("Command: %s", viewer);
+			unescaped_title = format_str("Command: %s", viewer);
+		}
+		else
+		{
+			unescaped_title = format_str("File: %s",
+					get_current_file_name(curr_view));
 		}
-		return format_str("File: %s", get_current_file_name(curr_view));
 	}
 	else if(flist_custom_active(view))
 	{
-		return format_str("[%s] @ %s", view->custom.title,
-				pf(view->custom.orig_dir));
+		const char *tree_mark = (cv_tree(view->custom.type) ? "[tree]" : "");
+		const char *path = pf(view->custom.orig_dir);
+		if(view->custom.title[0] == '\0')
+		{
+			unescaped_title = format_str("%s @ %s", tree_mark, path);
+		}
+		else
+		{
+			unescaped_title = format_str("%s[%s] @ %s", tree_mark, view->custom.title,
+					path);
+		}
 	}
 	else
 	{
-		return strdup(pf(view->curr_dir));
+		unescaped_title = strdup(pf(view->curr_dir));
 	}
+
+	char *escaped_title = escape_unreadable(unescaped_title);
+	free(unescaped_title);
+	return escaped_title;
 }
 
 /* Prints view title (which can be changed for printing).  Takes care of setting
@@ -2049,9 +2326,18 @@ print_view_title(const view_t *view, int active_view, char title[])
 
 	werase(view->title);
 
-	ellipsis = active_view
-	         ? left_ellipsis(title, title_width, curr_stats.ellipsis)
-	         : right_ellipsis(title, title_width, curr_stats.ellipsis);
+	if (cfg.ellipsis_position != 0)
+	{
+		ellipsis = cfg.ellipsis_position < 0
+		         ? left_ellipsis(title, title_width, curr_stats.ellipsis)
+		         : right_ellipsis(title, title_width, curr_stats.ellipsis);
+	}
+	else
+	{
+		ellipsis = active_view
+		         ? left_ellipsis(title, title_width, curr_stats.ellipsis)
+		         : right_ellipsis(title, title_width, curr_stats.ellipsis);
+	}
 
 	wprint(view->title, ellipsis);
 	free(ellipsis);
@@ -2090,7 +2376,7 @@ fixup_titles_attributes(const view_t *view, int active_view)
 }
 
 int
-ui_view_sort_list_contains(const char sort[SK_COUNT], char key)
+ui_view_sort_list_contains(const signed char sort[SK_COUNT], char key)
 {
 	int i = -1;
 	while(++i < SK_COUNT)
@@ -2109,7 +2395,7 @@ ui_view_sort_list_contains(const char sort[SK_COUNT], char key)
 }
 
 void
-ui_view_sort_list_ensure_well_formed(view_t *view, char sort_keys[])
+ui_view_sort_list_ensure_well_formed(view_t *view, signed char sort_keys[])
 {
 	int found_name_key = 0;
 	int i = -1;
@@ -2140,12 +2426,12 @@ ui_view_sort_list_ensure_well_formed(view_t *view, char sort_keys[])
 	}
 }
 
-char *
-ui_view_sort_list_get(const view_t *view, const char sort[])
+signed char *
+ui_view_sort_list_get(const view_t *view, const signed char sort[])
 {
 	return (flist_custom_active(view) && ui_view_unsorted(view))
-	     ? (char *)view->custom.sort
-	     : (char *)sort;
+	     ? (signed char *)view->custom.sort
+	     : (signed char *)sort;
 }
 
 int
@@ -2244,21 +2530,21 @@ ui_qv_width(const view_t *view)
 	return with_margin ? view->window_cols - 2 : view->window_cols;
 }
 
+void
+ui_qv_cleanup_if_needed(void)
+{
+	if(curr_stats.preview.on && curr_stats.preview.cleanup_cmd != NULL)
+	{
+		qv_cleanup(other_view, curr_stats.preview.cleanup_cmd);
+	}
+}
+
 void
 ui_invalidate_cs(const col_scheme_t *cs)
 {
 	int i;
 	tab_info_t tab_info;
-
-	for(i = 0; tabs_enum(curr_view, i, &tab_info); ++i)
-	{
-		if(ui_view_get_cs(tab_info.view) == cs)
-		{
-			fview_reset_cs(tab_info.view);
-		}
-	}
-
-	for(i = 0; tabs_enum(other_view, i, &tab_info); ++i)
+	for(i = 0; tabs_enum_all(i, &tab_info); ++i)
 	{
 		if(ui_view_get_cs(tab_info.view) == cs)
 		{
@@ -2274,9 +2560,9 @@ ui_view_get_cs(const view_t *view)
 }
 
 void
-ui_view_erase(view_t *view)
+ui_view_erase(view_t *view, int use_global_cs)
 {
-	const col_scheme_t *cs = ui_view_get_cs(view);
+	const col_scheme_t *cs = (use_global_cs ? &cfg.cs : ui_view_get_cs(view));
 	col_attr_t col = ui_get_win_color(view, cs);
 	ui_set_bg(view->win, &col, -1);
 	werase(view->win);
@@ -2302,8 +2588,10 @@ ui_view_unsorted(const view_t *view)
 void
 ui_shutdown(void)
 {
-	if(curr_stats.load_stage > 0 && !isendwin())
+	if(curr_stats.load_stage >= 0 && !vifm_testing() && !isendwin())
 	{
+		ui_qv_cleanup_if_needed();
+		modview_hide_graphics();
 		def_prog_mode();
 		endwin();
 	}
@@ -2330,25 +2618,17 @@ ui_pause(void)
 			pressed == KEY_RESIZE);
 
 	/* Redraw UI to account for all things including graphical preview. */
-	curr_stats.need_update = UT_REDRAW;
+	stats_redraw_later();
 }
 
 void
 ui_set_bg(WINDOW *win, const col_attr_t *col, int pair)
 {
-	if(curr_stats.load_stage < 1)
+	if(curr_stats.load_stage >= 1)
 	{
-		return;
+		const cchar_t bg = cs_color_to_cchar(col, pair);
+		wbkgrndset(win, &bg);
 	}
-
-	if(pair < 0)
-	{
-		pair = colmgr_get_pair(col->fg, col->bg);
-	}
-
-	cchar_t bg;
-	setcchar(&bg, L" ", col->attr, pair, NULL);
-	wbkgrndset(win, &bg);
 }
 
 void
@@ -2362,13 +2642,13 @@ ui_set_attr(WINDOW *win, const col_attr_t *col, int pair)
 
 	if(pair < 0)
 	{
-		pair = colmgr_get_pair(col->fg, col->bg);
+		pair = cs_load_color(col);
 	}
 
 	/* Compiler complains about unused result of comma operator, because
 	 * wattr_set() is a macro and it uses comma to evaluate multiple expresions.
-	 * So cast result to void.*/
-	(void)wattr_set(win, col->attr, pair, NULL);
+	 * So cast result to void. */
+	(void)wattr_set(win, cs_color_get_attr(col), pair, NULL);
 }
 
 void
