@@ -82,7 +82,7 @@ is_dir(const char path[])
 static int
 is_dir_fast(const char path[])
 {
-#if !defined(_WIN32) && !defined(__APPLE__) && !defined(BSD)
+#ifdef __linux__
 	/* Optimization idea: is_dir() ends up using stat() call, which in turn has
 	 * to:
 	 *  1) resolve path to an inode number;
@@ -101,7 +101,8 @@ is_dir_fast(const char path[])
 
 	return os_access(path_to_selfref, F_OK) == 0;
 #else
-	/* Some systems report that "/path/to/file/." is directory... */
+	/* Other systems report that "/path/to/file/." is a directory either for all
+	 * files or for executable ones, always or sometimes... */
 	return 0;
 #endif
 }
@@ -581,8 +582,12 @@ remove_dir_content(const char path[])
 				 * descending into. */
 				(void)os_chmod(full_path, 0777);
 				remove_dir_content(full_path);
+				(void)os_rmdir(full_path);
+			}
+			else
+			{
+				(void)remove(full_path);
 			}
-			(void)remove(full_path);
 			free(full_path);
 		}
 	}
