@@ -188,6 +188,19 @@ ends_with(const char str[], const char suffix[])
 	return strcmp(suffix, str + str_len - suffix_len) == 0;
 }
 
+int
+ends_with_case(const char str[], const char suffix[])
+{
+	size_t str_len = strlen(str);
+	size_t suffix_len = strlen(suffix);
+	if(str_len < suffix_len)
+	{
+		return 0;
+	}
+
+	return (strcasecmp(suffix, str + str_len - suffix_len) == 0);
+}
+
 int
 surrounded_with(const char str[], char left, char right)
 {
@@ -558,25 +571,23 @@ right_ellipsis(const char str[], size_t max_width, const char ell[])
 static char *
 ellipsis(const char str[], size_t max_width, const char ell[], int right)
 {
-	size_t ell_width, width;
-
 	if(max_width == 0U)
 	{
 		/* No room to print anything. */
 		return strdup("");
 	}
 
-	width = utf8_strsw(str);
+	size_t width = utf8_strsw(str);
 	if(width <= max_width)
 	{
 		/* No need to change the string. */
 		return strdup(str);
 	}
 
-	ell_width = utf8_strsw(ell);
+	size_t ell_width = utf8_strsw(ell);
 	if(max_width <= ell_width)
 	{
-		/* Insert as many characters as we can. */
+		/* Insert as many characters of ellipsis as we can. */
 		const int prefix = (int)utf8_nstrsnlen(ell, max_width);
 		return format_str("%.*s", prefix, ell);
 	}
@@ -599,19 +610,15 @@ ellipsis(const char str[], size_t max_width, const char ell[], int right)
 char *
 break_in_two(char str[], size_t max, const char separator[])
 {
-	int i;
-	size_t len, size;
-	char *result;
 	char *break_point = strstr(str, separator);
 	if(break_point == NULL)
 		return str;
 
 	const size_t separator_len = strlen(separator);
-	len = utf8_strsw(str) - separator_len;
-	size = strlen(str);
-	size = MAX(size, max);
-	result = malloc(size*4 + 2);
+	const size_t len = utf8_strsw(str) - separator_len;
+	const size_t size = MAX(strlen(str), max);
 
+	char *result = malloc(size*4 + 2);
 	copy_str(result, break_point - str + 1, str);
 
 	if(len > max)
@@ -621,7 +628,7 @@ break_in_two(char str[], size_t max, const char separator[])
 	}
 
 	copy_str(result, break_point - str + 1, str);
-	i = break_point - str;
+	int i = break_point - str;
 	while(max > len)
 	{
 		result[i++] = ' ';
@@ -1090,7 +1097,24 @@ chars_in_str(const char s[], char c)
 	return char_count;
 }
 
-#ifdef _WIN32
+char *
+double_char(const char str[], char c)
+{
+	char *doubled = malloc(strlen(str) + chars_in_str(str, c) + 1);
+	char *p = doubled;
+	while(*str != '\0')
+	{
+		if(*str == c)
+		{
+			*p++ = c;
+		}
+		*p++ = *str++;
+	}
+	*p = '\0';
+	return doubled;
+}
+
+#ifndef HAVE_STRCASESTR
 
 char *
 strcasestr(const char haystack[], const char needle[])
