@@ -19,7 +19,6 @@
 #include "string_array.h"
 
 #include <assert.h> /* assert() */
-#include <stdarg.h>
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* FILE SEEK_END SEEK_SET fclose() fprintf() fread()
                       ftell() fseek() */
@@ -32,32 +31,30 @@
 
 static char * read_whole_file(const char filepath[], size_t *read);
 static char * read_seekable_stream(FILE *fp, size_t *read);
+static char * read_stream(FILE *fp, size_t *read, int drop_bom, progress_cb cb,
+		const void *arg);
 static size_t get_remaining_stream_size(FILE *fp);
 static char ** text_to_lines(char text[], size_t text_len, int *nlines,
 		int null_sep);
 
 int
-add_to_string_array(char ***array, int len, int count, ...)
+add_to_string_array(char ***array, int len, const char item[])
 {
-	char **p;
-	va_list va;
-
-	p = reallocarray(*array, len + count, sizeof(char *));
+	char **p = reallocarray(*array, len + 1, sizeof(*p));
 	if(p == NULL)
+	{
 		return len;
+	}
 	*array = p;
 
-	va_start(va, count);
-	while(count-- > 0)
+	if(item == NULL)
 	{
-		char *arg = va_arg(va, char *);
-		if(arg == NULL)
-			p[len] = NULL;
-		else if((p[len] = strdup(arg)) == NULL)
-			break;
-		len++;
+		p[len++] = NULL;
+	}
+	else if((p[len] = strdup(item)) != NULL)
+	{
+		++len;
 	}
-	va_end(va);
 
 	return len;
 }
@@ -151,6 +148,27 @@ string_array_pos_case(char *array[], size_t len, const char item[])
 	return (i < len) ? (int)i : -1;
 }
 
+int
+string_array_equal(char *first[], size_t first_len, char *second[],
+		size_t second_len)
+{
+	if(first_len != second_len)
+	{
+		return 0;
+	}
+
+	size_t i;
+	for(i = 0U; i < first_len; ++i)
+	{
+		if(strcmp(first[i], second[i]) != 0)
+		{
+			return 0;
+		}
+	}
+
+	return 1;
+}
+
 void
 free_string_array(char *array[], size_t len)
 {
@@ -165,7 +183,7 @@ void
 free_strings(char *array[], size_t len)
 {
 	size_t i;
-	for(i = 0; i < len; i++)
+	for(i = 0; i < len; ++i)
 	{
 		free(array[i]);
 	}
@@ -243,6 +261,58 @@ read_stream_lines(FILE *f, int *nlines, int null_sep_heuristic, progress_cb cb,
 
 char *
 read_nonseekable_stream(FILE *fp, size_t *read, progress_cb cb, const void *arg)
+{
+	return read_stream(fp, read, 1, cb, arg);
+}
+
+/* Reads content of the fp stream that supports seek operation (points to a
+ * file) until end-of-file into null terminated string.  Returns string of
+ * length *read to be freed by caller on success, otherwise NULL is returned and
+ * *read is set to 0UL. */
+static char *
+read_seekable_stream(FILE *fp, size_t *read)
+{
+	char *content;
+	size_t len;
+
+	skip_bom(fp);
+	len = get_remaining_stream_size(fp);
+
+	if(len == 0)
+	{
+		/* Give this file another shot in case it's a virtual file that doesn't
+		 * report its size ahead of time. */
+		return read_stream(fp, read, 0, NULL, NULL);
+	}
+
+	*read = 0UL;
+
+	if((content = malloc(len + 1U)) == NULL)
+	{
+		return NULL;
+	}
+
+	if(fread(content, len, 1U, fp) == 1U)
+	{
+		content[len] = '\0';
+		*read = len;
+	}
+	else
+	{
+		free(content);
+		content = NULL;
+	}
+
+	return content;
+}
+
+/* Reads content of the fp stream that doesn't support seek operation (e.g. it
+ * points to a pipe) until end-of-file into null terminated string.  cb can be
+ * NULL.  Returns string of length *read to be freed by caller on success,
+ * otherwise NULL is returned. */
+static char *
+read_stream(FILE *fp, size_t *read, int drop_bom, progress_cb cb,
+		const void *arg)
 {
 	enum { PIECE_LEN = 4096 };
 	char *content = malloc(PIECE_LEN + 1);
@@ -251,7 +321,10 @@ read_nonseekable_stream(FILE *fp, size_t *read, progress_cb cb, const void *arg)
 	{
 		char *last_allocated_block = content;
 		size_t len = 0U, piece_len;
-		skip_bom(fp);
+		if(drop_bom)
+		{
+			skip_bom(fp);
+		}
 		while((piece_len = fread(content + len, 1, PIECE_LEN, fp)) != 0U)
 		{
 			const size_t new_size = len + piece_len + PIECE_LEN + 1U;
@@ -290,40 +363,6 @@ read_nonseekable_stream(FILE *fp, size_t *read, progress_cb cb, const void *arg)
 	return content;
 }
 
-/* Reads content of the fp stream that supports seek operation (points to a
- * file) until end-of-file into null terminated string.  Returns string of
- * length *read to be freed by caller on success, otherwise NULL is returned and
- * *read is set to 0UL. */
-static char *
-read_seekable_stream(FILE *fp, size_t *read)
-{
-	char *content;
-	size_t len;
-
-	skip_bom(fp);
-	len = get_remaining_stream_size(fp);
-
-	*read = 0UL;
-
-	if((content = malloc(len + 1U)) == NULL)
-	{
-		return NULL;
-	}
-
-	if(fread(content, len, 1U, fp) == 1U)
-	{
-		content[len] = '\0';
-		*read = len;
-	}
-	else
-	{
-		free(content);
-		content = NULL;
-	}
-
-	return content;
-}
-
 /* Calculates remaining size of the stream.  Assumes that the fp stream supports
  * seek operation.  Returns the size. */
 static size_t
@@ -388,7 +427,7 @@ break_into_lines(char text[], size_t text_len, int *nlines, int null_sep)
 		}
 
 		text[line_len] = '\0';
-		*nlines = add_to_string_array(&list, *nlines, 1, text);
+		*nlines = add_to_string_array(&list, *nlines, text);
 
 		text = after_line;
 	}
