@@ -30,7 +30,7 @@
 #include <sys/types.h> /* pid_t */
 #include <unistd.h>
 
-#include <ctype.h> /* isalnum() isalpha() */
+#include <ctype.h> /* isalnum() isalpha() iscntrl() */
 #include <errno.h> /* errno */
 #include <math.h> /* modf() pow() */
 #include <stddef.h> /* size_t */
@@ -49,6 +49,7 @@
 #include "../ui/cancellation.h"
 #include "../ui/ui.h"
 #include "../background.h"
+#include "../filelist.h"
 #include "../registers.h"
 #include "../status.h"
 #include "env.h"
@@ -59,6 +60,7 @@
 #include "path.h"
 #include "str.h"
 #include "string_array.h"
+#include "utf8.h"
 
 static void show_progress_cb(const void *descr);
 static const char ** get_size_suffixes(void);
@@ -97,8 +99,22 @@ vifm_system(char command[], ShellRequester by)
 }
 
 int
-process_cmd_output(const char descr[], const char cmd[], int user_sh,
-		int interactive, cmd_output_handler handler, void *arg)
+vifm_system_input(char command[], FILE *input, ShellRequester by)
+{
+#ifdef _WIN32
+	/* The check is primarily for tests, otherwise screen is reset. */
+	if(curr_stats.load_stage != 0)
+	{
+		system("cls");
+	}
+#endif
+	LOG_INFO_MSG("Shell command with custom input: %s", command);
+	return run_with_input(command, input, by);
+}
+
+int
+process_cmd_output(const char descr[], const char cmd[], FILE *input,
+		int user_sh, int interactive, cmd_output_handler handler, void *arg)
 {
 	FILE *file, *err;
 	pid_t pid;
@@ -108,14 +124,13 @@ process_cmd_output(const char descr[], const char cmd[], int user_sh,
 
 	LOG_INFO_MSG("Capturing output of the command: %s", cmd);
 
-	pid = bg_run_and_capture((char *)cmd, user_sh, &file, &err);
+	pid = bg_run_and_capture((char *)cmd, user_sh, input, &file, &err);
 	if(pid == (pid_t)-1)
 	{
 		return 1;
 	}
 
-	ui_cancellation_reset();
-	ui_cancellation_enable();
+	ui_cancellation_push_on();
 
 	if(!interactive)
 	{
@@ -126,7 +141,7 @@ process_cmd_output(const char descr[], const char cmd[], int user_sh,
 	lines = read_stream_lines(file, &nlines, 1,
 			interactive ? NULL : &show_progress_cb, descr);
 
-	ui_cancellation_disable();
+	ui_cancellation_pop();
 	fclose(file);
 
 	for(i = 0; i < nlines; ++i)
@@ -326,7 +341,7 @@ enclose_in_dquotes(const char str[])
 	*p++ = '"';
 	while(*str != '\0')
 	{
-		if(*str == '\\' || *str == '"' ||
+		if((curr_stats.shell_type != ST_PS && *str == '\\') || *str == '"' ||
 				(curr_stats.shell_type == ST_NORMAL && (*str == '$' || *str == '`')))
 		{
 			*p++ = '\\';
@@ -521,6 +536,36 @@ escape_for_dquotes(const char string[], size_t offset)
 	return escaped;
 }
 
+char *
+escape_unreadable(const char str[])
+{
+	int str_len = strlen(str);
+	char *escaped = malloc(str_len*2 + 1);
+
+	char *out = escaped;
+	while(str_len > 0)
+	{
+		const size_t char_len = utf8_chrw(str);
+
+		if(char_len == 1 && iscntrl((unsigned char)*str))
+		{
+			*out++ = '^';
+			*out++ = *str ^ 64;
+		}
+		else
+		{
+			memcpy(out, str, char_len);
+			out += char_len;
+		}
+
+		str += char_len;
+		str_len -= char_len;
+	}
+	*out = '\0';
+
+	return escaped;
+}
+
 void
 expand_percent_escaping(char s[])
 {
@@ -730,5 +775,53 @@ safe_qsort(void *base, size_t nmemb, size_t size,
 	}
 }
 
+void
+format_position(char buf[], size_t buf_len, int top, int total, int visible)
+{
+	if(total <= visible)
+	{
+		copy_str(buf, buf_len, "All");
+	}
+	else if(top == 0)
+	{
+		copy_str(buf, buf_len, "Top");
+	}
+	else if(total - top <= visible)
+	{
+		copy_str(buf, buf_len, "Bot");
+	}
+	else
+	{
+		snprintf(buf, buf_len, "%2d%%", (top*100)/(total - visible));
+	}
+}
+
+FILE *
+make_in_file(view_t *view, MacroFlags flags)
+{
+	if(ma_flags_missing(flags, MF_PIPE_FILE_LIST) &&
+			ma_flags_missing(flags, MF_PIPE_FILE_LIST_Z))
+	{
+		return NULL;
+	}
+
+	FILE *input_tmp = os_tmpfile();
+	const int null_sep = ma_flags_present(flags, MF_PIPE_FILE_LIST_Z);
+	write_marked_paths(input_tmp, view, null_sep);
+	return input_tmp;
+}
+
+void
+write_marked_paths(FILE *file, view_t *view, int null_sep)
+{
+	const char separator = (null_sep ? '\0' : '\n');
+	dir_entry_t *entry = NULL;
+	while(iter_marked_entries(view, &entry))
+	{
+		const char *const sep = (ends_with_slash(entry->origin) ? "" : "/");
+		fprintf(file, "%s%s%s%c", entry->origin, sep, entry->name, separator);
+	}
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
