@@ -27,6 +27,7 @@
 #include <stdint.h> /* uint64_t */
 #include <stdio.h> /* FILE */
 
+#include "../macros.h"
 #include "../status.h"
 
 /* Type of operating environment in which the application is running. */
@@ -45,8 +46,9 @@ typedef enum
 }
 ShellRequester;
 
-/* Forward declaration. */
+/* Forward declarations. */
 struct dir_entry_t;
+struct view_t;
 
 /* Callback for process_cmd_output() function. */
 typedef void (*cmd_output_handler)(const char line[], void *arg);
@@ -57,6 +59,10 @@ typedef void (*cmd_output_handler)(const char line[], void *arg);
  * the command.  Returns error code, which is zero on success. */
 int vifm_system(char command[], ShellRequester by);
 
+/* Same as vifm_system(), but provides the command with custom input.  Don't
+ * pass pipe for input, it can cause deadlock. */
+int vifm_system_input(char command[], FILE *input, ShellRequester by);
+
 /* Pauses shell.  Assumes that curses interface is off. */
 void pause_shell(void);
 
@@ -64,13 +70,15 @@ void pause_shell(void);
 void recover_after_shellout(void);
 
 /* Invokes handler for each line read from stdout of the command specified via
- * cmd.  Error stream is displayed separately.  Implements heuristic according
- * to which if command output includes null character, it's taken as a separator
- * instead of regular newline characters.  Supports cancellation.  Ignores exit
- * code of the command and succeeds even if it doesn't exist.  Returns zero on
- * success, otherwise non-zero is returned. */
-int process_cmd_output(const char descr[], const char cmd[], int user_sh,
-		int interactive, cmd_output_handler handler, void *arg);
+ * cmd.  Input is redirected only if in parameter isn't NULL.  Don't pass pipe
+ * for input, it can cause deadlock.  Error stream is displayed separately.
+ * Implements heuristic according to which if command output includes null
+ * character, it's taken as a separator instead of regular newline characters.
+ * Supports cancellation.  Ignores exit code of the command and succeeds even if
+ * it doesn't exist.  Returns zero on success, otherwise non-zero is
+ * returned. */
+int process_cmd_output(const char descr[], const char cmd[], FILE *input,
+		int user_sh, int interactive, cmd_output_handler handler, void *arg);
 
 /* Other functions. */
 
@@ -139,6 +147,10 @@ char * escape_for_squotes(const char string[], size_t offset);
  * quoted string, prefix is not escaped.  Returns newly allocated string. */
 char * escape_for_dquotes(const char string[], size_t offset);
 
+/* Escapes characters aren't visible or don't looks nice to the user.  Returns
+ * newly allocated string. */
+char * escape_unreadable(const char str[]);
+
 /* Expands double percent sequences into single percent character in place. */
 void expand_percent_escaping(char s[]);
 
@@ -161,6 +173,19 @@ int def_count(int count);
 void safe_qsort(void *base, size_t nmemb, size_t size,
 		int (*compar)(const void *, const void *));
 
+/* Formats position within the viewport as one of the following: All, Top, xx%
+ * or Bot. */
+void format_position(char buf[], size_t buf_len, int top, int total,
+		int visible);
+
+/* Makes input file for a command if requested.  Returns the file or NULL if
+ * it's not necessary. */
+FILE * make_in_file(struct view_t *view, MacroFlags flags);
+
+/* Writes list of marked files to the file.  Files are separated by new line or
+ * null characters. */
+void write_marked_paths(FILE *file, struct view_t *view, int null_sep);
+
 /* Checks line for path in it.  Ignores empty lines and attempts to parse it as
  * location line (path followed by a colon and optional line and column
  * numbers).  Returns canonicalized path as a newly allocated string or NULL. */
@@ -187,10 +212,6 @@ struct cancellation_t;
 void wait_for_data_from(pid_t pid, FILE *f, int fd,
 		const struct cancellation_t *cancellation);
 
-/* Blocks/unblocks SIGCHLD signal.  Returns zero on success, otherwise non-zero
- * is returned. */
-int set_sigchld(int block);
-
 /* Blocks all signals of current thread that can be blocked. */
 void block_all_thread_signals(void);
 
@@ -259,13 +280,22 @@ FILE * read_cmd_output(const char cmd[], int preserve_stdin);
  * pointer to a statically allocated buffer. */
 const char * get_installed_data_dir(void);
 
+/* Gets path to directory where global configuration files of Vifm are stored.
+ * Returns pointer to a statically allocated buffer. */
+const char * get_sys_conf_dir(void);
+
 /* Clones attributes from file specified by from to file at path.  st is a hint
  * to omit extra file system requests if possible, can be NULL on Windows. */
 void clone_attribs(const char path[], const char from[], const struct stat *st);
 
-/* Retrives amount of free space at location specified by the path.  Returns the
- * amount in bytes. */
-uint64_t get_free_space(const char at[]);
+/* Retrieves drive information for location specified by the path.  Returns zero
+ * on success and non-zero otherwise. */
+int get_drive_info(const char at[], uint64_t *total_bytes,
+		uint64_t *free_bytes);
+
+/* Retrieves inode number that corresponds to the entry by resolving symbolic
+ * links if necessary.  Returns the inode number. */
+uint64_t get_true_inode(const struct dir_entry_t *entry);
 
 #ifdef _WIN32
 #include "utils_win.h"
