@@ -34,7 +34,7 @@
 #include <sys/stat.h> /* O_* S_* */
 #include <sys/statvfs.h> /* statvfs statvfs() */
 #include <sys/time.h> /* timeval futimens() utimes() */
-#include <sys/wait.h> /* waitpid */
+#include <sys/wait.h> /* WEXITSTATUS() WIFEXITED() WIFSIGNALED() waitpid() */
 #include <fcntl.h> /* open() close() */
 #include <grp.h> /* getgrnam() getgrgid_r() */
 #include <pthread.h> /* pthread_sigmask() */
@@ -45,8 +45,8 @@
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
 #include <errno.h> /* EINTR ENOTSUP errno */
-#include <signal.h> /* SIG* SIG_* sigset_t kill() sigaddset() sigemptyset()
-                       sigfillset() signal() sigprocmask() */
+#include <signal.h> /* SIG* SIG_* sigset_t kill() sigemptyset() sigfillset()
+                       signal() */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* FILE stderr fclose() fdopen() fprintf() snprintf() */
 #include <stdlib.h> /* atoi() free() */
@@ -59,6 +59,7 @@
 #include "../compat/reallocarray.h"
 #include "../ui/tabs.h"
 #include "../ui/ui.h"
+#include "../filelist.h"
 #include "../running.h"
 #include "../status.h"
 #include "cancellation.h"
@@ -111,6 +112,12 @@ pause_shell(void)
 
 int
 run_in_shell_no_cls(char command[], ShellRequester by)
+{
+	return run_with_input(command, /*input=*/NULL, by);
+}
+
+int
+run_with_input(char command[], FILE *input, ShellRequester by)
 {
 	int pid;
 	int result;
@@ -125,25 +132,36 @@ run_in_shell_no_cls(char command[], ShellRequester by)
 	new.sa_flags = SA_RESTART;
 	sigaction(SIGTSTP, &new, &old);
 
-	/* We need to block SIGCHLD signal.  One can't just set it to SIG_DFL, because
-	 * it will possibly cause missing of SIGCHLD from a background process
-	 * (job). */
-	(void)set_sigchld(1);
+	if(input != NULL)
+	{
+		fflush(input);
+	}
 
 	pid = fork();
 	if(pid == -1)
 	{
 		sigaction(SIGTSTP, &old, NULL);
-		(void)set_sigchld(0);
 		return -1;
 	}
 	if(pid == 0)
 	{
 		signal(SIGTSTP, SIG_DFL);
 		signal(SIGINT, SIG_DFL);
-		(void)set_sigchld(0);
 
 		prepare_for_exec();
+
+		if(input != NULL)
+		{
+			rewind(input);
+
+			if(dup2(fileno(input), STDIN_FILENO) == -1)
+			{
+				_Exit(127);
+			}
+
+			fclose(input);
+		}
+
 		char *sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "-c");
 		execve(get_execv_path(cfg.shell),
 				make_execv_array(cfg.shell, sh_flag, command), environ);
@@ -153,7 +171,6 @@ run_in_shell_no_cls(char command[], ShellRequester by)
 	result = get_proc_exit_status(pid);
 
 	sigaction(SIGTSTP, &old, NULL);
-	(void)set_sigchld(0);
 
 	return result;
 }
@@ -161,7 +178,11 @@ run_in_shell_no_cls(char command[], ShellRequester by)
 void
 recover_after_shellout(void)
 {
-	/* Do nothing.  No need to recover anything on this platform. */
+	if(curr_stats.load_stage > 0)
+	{
+		reset_prog_mode();
+		doupdate();
+	}
 }
 
 void
@@ -185,17 +206,7 @@ wait_for_data_from(pid_t pid, FILE *f, int fd,
 		select_result = select(fd + 1, &read_ready, NULL, NULL, &ts);
 	}
 	while(select_result == 0 || (select_result == -1 && errno == EINTR));
-}
-
-int
-set_sigchld(int block)
-{
-	const int action = block ? SIG_BLOCK : SIG_UNBLOCK;
-	sigset_t sigchld_mask;
-
-	return sigemptyset(&sigchld_mask) == -1
-	    || sigaddset(&sigchld_mask, SIGCHLD) == -1
-	    || sigprocmask(action, &sigchld_mask, NULL) == -1;
+	process_cancel_request(pid, cancellation);
 }
 
 void
@@ -222,23 +233,25 @@ process_cancel_request(pid_t pid, const struct cancellation_t *cancellation)
 int
 get_proc_exit_status(pid_t pid)
 {
-	do
+	while(1)
 	{
 		int status;
 		if(waitpid(pid, &status, 0) == -1)
 		{
-			if(errno != EINTR)
+			if(errno == EINTR)
 			{
-				LOG_SERROR_MSG(errno, "waitpid()");
-				return -1;
+				continue;
 			}
+			LOG_SERROR_MSG(errno, "waitpid()");
+			break;
 		}
-		else
+
+		if(WIFEXITED(status) || WIFSIGNALED(status))
 		{
 			return status;
 		}
 	}
-	while(1);
+	return -1;
 }
 
 void _gnuc_noreturn
@@ -302,14 +315,7 @@ prepare_for_exec(void)
 	 * freeing resources which will be replaced by exec(). */
 	tab_info_t tab_info;
 	int i;
-	for(i = 0; tabs_enum(&lwin, i, &tab_info); ++i)
-	{
-		view_t *view = tab_info.view;
-		fswatch_free(view->watch);
-		fswatch_free(view->left_column.watch);
-		fswatch_free(view->right_column.watch);
-	}
-	for(i = 0; tabs_enum(&rwin, i, &tab_info); ++i)
+	for(i = 0; tabs_enum_all(i, &tab_info); ++i)
 	{
 		view_t *view = tab_info.view;
 		fswatch_free(view->watch);
@@ -551,7 +557,7 @@ traverse_mount_points(mptraverser client, void *arg)
 	if(filemon_from_file("/etc/mtab", FMT_MODIFIED, &mon) != 0 ||
 			!filemon_equal(&mon, &mtab_mon))
 	{
-		filemon_assign(&mtab_mon, &mon);
+		mtab_mon = mon;
 		free_mnt_entries(entries, nentries);
 		entries = read_mnt_entries(&nentries);
 	}
@@ -742,6 +748,24 @@ get_gid(const char group[], gid_t *gid)
 	return 0;
 }
 
+int
+status_to_exit_code(int status)
+{
+	if(status == -1)
+	{
+		return -127;
+	}
+	if(WIFEXITED(status))
+	{
+		return WEXITSTATUS(status);
+	}
+	if(WIFSIGNALED(status))
+	{
+		return -WTERMSIG(status);
+	}
+	return -128;
+}
+
 int
 S_ISEXE(mode_t mode)
 {
@@ -777,8 +801,11 @@ get_exec_env_type(void)
 	}
 	else
 	{
-		const char *const display = env_get("DISPLAY");
-		return is_null_or_empty(display) ? EET_EMULATOR : EET_EMULATOR_WITH_X;
+		const char *display = env_get("DISPLAY");
+		const char *wayland_display = env_get("WAYLAND_DISPLAY");
+		return is_null_or_empty(display) && is_null_or_empty(wayland_display)
+		     ? EET_EMULATOR
+		     : EET_EMULATOR_WITH_X;
 	}
 }
 
@@ -801,7 +828,7 @@ format_help_cmd(char cmd[], size_t cmd_size)
 void
 display_help(const char cmd[])
 {
-	(void)shellout(cmd, PAUSE_ON_ERROR, 1, SHELL_BY_APP);
+	(void)rn_shell(cmd, PAUSE_ON_ERROR, 1, SHELL_BY_APP);
 }
 
 void
@@ -1047,13 +1074,34 @@ read_cmd_output(const char cmd[], int preserve_stdin)
 const char *
 get_installed_data_dir(void)
 {
-	return PACKAGE_DATA_DIR;
+	static char data_dir[PATH_MAX + 1];
+	if(data_dir[0] == '\0')
+	{
+		const char *prefix = env_get_def("VIFM_APPDIR_ROOT", "");
+		snprintf(data_dir, sizeof(data_dir), "%s%s", prefix, PACKAGE_DATA_DIR);
+	}
+	return data_dir;
+}
+
+const char *
+get_sys_conf_dir(void)
+{
+	static char conf_dir[PATH_MAX + 1];
+	if(conf_dir[0] == '\0')
+	{
+		const char *prefix = env_get_def("VIFM_APPDIR_ROOT", "");
+		snprintf(conf_dir, sizeof(conf_dir), "%s%s", prefix, PACKAGE_SYSCONF_DIR);
+	}
+	return conf_dir;
 }
 
 void
 clone_attribs(const char path[], const char from[], const struct stat *st)
 {
-	chown(path, st->st_uid, st->st_gid);
+	if(chown(path, st->st_uid, st->st_gid) != 0)
+	{
+		LOG_SERROR_MSG(errno, "Failed to chown `%s`", path);
+	}
 	clone_timestamps(path, from, st);
 	clone_xattrs(path, from);
 }
@@ -1062,7 +1110,8 @@ clone_attribs(const char path[], const char from[], const struct stat *st)
 static void
 clone_timestamps(const char path[], const char from[], const struct stat *st)
 {
-#if defined(HAVE_STRUCT_STAT_ST_MTIM) && defined(HAVE_FUTIMENS)
+#if defined(HAVE_STRUCT_STAT_ST_MTIM) && defined(HAVE_FUTIMENS) && \
+    defined(HAVE_CONSISTENT_TIMESPEC)
 	const int fd = open(path, O_WRONLY);
 	if(fd != -1)
 	{
@@ -1144,16 +1193,44 @@ clone_xattrs(const char path[], const char from[])
 #endif
 }
 
-uint64_t
-get_free_space(const char at[])
+int
+get_drive_info(const char at[], uint64_t *total_bytes, uint64_t *free_bytes)
 {
 	struct statvfs st;
 	if(statvfs(at, &st) != 0)
 	{
-		return 0;
+		return -1;
 	}
 
-	return (uint64_t)st.f_bsize*st.f_bavail;
+#ifdef __APPLE__
+	/* Apple is so fucking different... */
+	const uint64_t block_size = st.f_frsize;
+#else
+	const uint64_t block_size = st.f_bsize;
+#endif
+
+	*total_bytes = st.f_blocks*block_size;
+	*free_bytes = st.f_bavail*block_size;
+	return 0;
+}
+
+uint64_t
+get_true_inode(const struct dir_entry_t *entry)
+{
+	if(entry->type != FT_LINK)
+	{
+		return entry->inode;
+	}
+
+	char full_path[PATH_MAX + 1];
+	get_full_path_of(entry, sizeof(full_path), full_path);
+
+	struct stat s;
+	if(os_stat(full_path, &s) == 0)
+	{
+		return s.st_ino;
+	}
+	return entry->inode;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
