@@ -20,8 +20,11 @@
 #include "utils.h"
 #include "utils_int.h"
 
-#include <windows.h>
 #include <ntdef.h>
+#include <objidl.h>
+#include <shlguid.h>
+#include <shobjidl.h>
+#include <windows.h>
 
 #include <curses.h>
 
@@ -43,11 +46,13 @@
 #include "../ui/ui.h"
 #include "../cmd_completion.h"
 #include "../status.h"
+#include "cancellation.h"
 #include "env.h"
 #include "fs.h"
 #include "log.h"
 #include "macros.h"
 #include "path.h"
+#include "selector.h"
 #include "str.h"
 #include "test_helpers.h"
 #include "utf8.h"
@@ -59,8 +64,11 @@
 
 static const char PATHEXT_EXT_DEF[] = ".bat;.exe;.com";
 
+static void process_cancel_request(pid_t pid,
+		const cancellation_t *cancellation);
 TSTATIC int should_wait_for_program(const char cmd[]);
 static DWORD handle_process(const char cmd[], HANDLE proc, int *got_exit_code);
+static char * base64_encode(const char str[]);
 static int get_subsystem(const char filename[]);
 static int get_stream_subsystem(FILE *fp);
 static FILE * read_cmd_output_internal(const char cmd[], int out_pipe[2],
@@ -76,6 +84,10 @@ pause_shell(void)
 	{
 		run_in_shell_no_cls("pause", SHELL_BY_APP);
 	}
+	else if(curr_stats.shell_type == ST_PS)
+	{
+		/* TODO: make pausing work. */
+	}
 	else
 	{
 		run_in_shell_no_cls(PAUSE_CMD, SHELL_BY_APP);
@@ -89,7 +101,7 @@ run_in_shell_no_cls(char command[], ShellRequester by)
 	char *const sh_cmd = win_make_sh_cmd(command, by);
 
 	/* XXX: why do we use different functions for different cases? */
-	if(curr_stats.shell_type == ST_CMD)
+	if(curr_stats.shell_type == ST_CMD || curr_stats.shell_type == ST_PS)
 	{
 		ret = os_system(sh_cmd);
 	}
@@ -97,6 +109,10 @@ run_in_shell_no_cls(char command[], ShellRequester by)
 	{
 		int returned_exit_code;
 		ret = win_exec_cmd(sh_cmd, &returned_exit_code);
+		if (!returned_exit_code)
+		{
+			ret = -1;
+		}
 	}
 
 	free(sh_cmd);
@@ -104,6 +120,50 @@ run_in_shell_no_cls(char command[], ShellRequester by)
 	return ret;
 }
 
+int
+run_with_input(char command[], FILE *input, ShellRequester by)
+{
+	rewind(input);
+
+	PROCESS_INFORMATION pinfo;
+
+	STARTUPINFOW startup = {
+		.dwFlags = STARTF_USESTDHANDLES,
+		.hStdInput = (HANDLE)_get_osfhandle(_fileno(input)),
+		.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE),
+		.hStdError = GetStdHandle(STD_ERROR_HANDLE),
+	};
+	SetHandleInformation(startup.hStdInput, HANDLE_FLAG_INHERIT, 1);
+	SetHandleInformation(startup.hStdOutput, HANDLE_FLAG_INHERIT, 1);
+	SetHandleInformation(startup.hStdError, HANDLE_FLAG_INHERIT, 1);
+
+	char *sh_cmd = win_make_sh_cmd(command, by);
+	wchar_t *wide_cmd = to_wide(sh_cmd);
+	free(sh_cmd);
+
+	int started = CreateProcessW(NULL, wide_cmd, NULL, NULL, 1, 0,
+			NULL, NULL, &startup, &pinfo);
+	free(wide_cmd);
+
+	if(!started)
+	{
+		return -1;
+	}
+
+	CloseHandle(pinfo.hThread);
+
+	int is_exit_code;
+	DWORD code = handle_process(NULL, pinfo.hProcess, &is_exit_code);
+	if(!is_exit_code && code != NO_ERROR)
+	{
+		LOG_WERROR(code);
+		code = -1;
+	}
+
+	CloseHandle(pinfo.hProcess);
+	return code;
+}
+
 void
 recover_after_shellout(void)
 {
@@ -116,15 +176,45 @@ recover_after_shellout(void)
 
 void
 wait_for_data_from(pid_t pid, FILE *f, int fd,
-		const struct cancellation_t *cancellation)
+		const cancellation_t *cancellation)
 {
-	/* Do nothing.  No need to wait for anything on this platform. */
+	enum { DELAY_MS = 250 };
+
+	selector_t *selector = selector_alloc();
+	if(selector == NULL)
+	{
+		return;
+	}
+
+	fd = (f != NULL ? fileno(f) : fd);
+	HANDLE h = (HANDLE)_get_osfhandle(fd);
+	selector_add(selector, h);
+
+	int has_data;
+	do
+	{
+		process_cancel_request(pid, cancellation);
+		has_data = selector_wait(selector, DELAY_MS);
+	}
+	while(!has_data);
+	process_cancel_request(pid, cancellation);
+
+	selector_free(selector);
 }
 
-int
-set_sigchld(int block)
+/* Checks whether cancelling of current operation is requested and tries to
+ * cancel the process specified by its id. */
+static void
+process_cancel_request(pid_t pid, const cancellation_t *cancellation)
 {
-	return 0;
+	if(cancellation_requested(cancellation))
+	{
+		if(win_cancel_process(pid) != 0)
+		{
+			LOG_SERROR_MSG(errno, "Failed to cancel process with PID %" PRINTF_ULL,
+					(unsigned long long)pid);
+		}
+	}
 }
 
 void
@@ -195,22 +285,36 @@ win_exec_cmd(char cmd[], int *returned_exit_code)
 char *
 win_make_sh_cmd(const char cmd[], ShellRequester by)
 {
-	char buf[strlen(cfg.shell) + 5 + strlen(cmd)*4 + 1 + 1];
+	const char *sh_flag;
+	char *free_me = NULL;
 
+	const char *fmt;
 	if(curr_stats.shell_type == ST_CMD)
 	{
+		sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "/C");
 		/* Documentation in `cmd /?` seems to LIE, can't make both spaces and
 		 * special characters work at the same time. */
-		const char *const fmt = (cmd[0] == '"') ? "%s %s \"%s\"" : "%s %s %s";
-		const char *sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "/C");
-		snprintf(buf, sizeof(buf), fmt, cfg.shell, sh_flag, cmd);
+		fmt = (cmd[0] == '"') ? "%s %s \"%s\"" : "%s %s %s";
+	}
+	else if(curr_stats.shell_type == ST_PS)
+	{
+		fmt = "%s %s %s";
+		sh_flag = "-encodedCommand";
+
+		char *actual_cmd = format_str("& %s", cmd);
+
+		free_me = base64_encode(actual_cmd);
+		cmd = free_me;
+
+		free(actual_cmd);
 	}
 	else
 	{
-		const char *sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "/C");
-		snprintf(buf, sizeof(buf), "%s %s '", cfg.shell, sh_flag);
+		sh_flag = (by == SHELL_BY_USER ? cfg.shell_cmd_flag : "-c");
+		fmt = "%s %s '%s'";
 
-		char *p = buf + strlen(buf);
+		free_me = malloc(strlen(cmd)*2 + 1);
+		char *p = free_me;
 		while(*cmd != '\0')
 		{
 			if(*cmd == '\\')
@@ -221,20 +325,76 @@ win_make_sh_cmd(const char cmd[], ShellRequester by)
 		}
 		*p = '\0';
 
-		strcat(buf, "'");
+		cmd = free_me;
 	}
 
+	/* Size of format minus the size of the %s-s. */
+	int buf_size = strlen(fmt) - 3*2
+	             + strlen(cfg.shell)
+	             + strlen(sh_flag)
+	             + strlen(cmd)
+	             + 1; /* Trailing '\0'. */
+	char buf[buf_size];
+	snprintf(buf, sizeof(buf), fmt, cfg.shell, sh_flag, cmd);
+	free(free_me);
 	return strdup(buf);
 }
 
-/* Handles process execution.  Returns system error code when sets
- * *got_exit_code to 0 and exit code of the process otherwise. */
+/* Base64-encodes the string after converting it to Unicode.  Returns the
+ * encoded version. */
+static char *
+base64_encode(const char str[])
+{
+	const char *dict = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+	                   "abcdefghijklmnopqrstuvwxyz"
+	                   "0123456789+/";
+
+	wchar_t *wstr = to_wide(str);
+	int bytes = wcslen(wstr)*sizeof(wchar_t);
+
+	str = (const char *)wstr;
+
+	char *out = malloc(DIV_ROUND_UP(bytes, 3)*4 + 1);
+	char *p = out;
+
+	while(bytes > 0)
+	{
+		unsigned int x = 0;
+
+		int i;
+		for(i = 0; i < 3 && bytes > 0; ++i, --bytes)
+		{
+			x = (x << 8) | (unsigned char)*str++;
+		}
+
+		x <<= 8*(3 - i);
+
+		while(i-- > -1)
+		{
+			*p++ = dict[(x >> 18) & 0x3f];
+			x <<= 6;
+		}
+	}
+
+	while((p - out)%4 != 0)
+	{
+		*p++ = '=';
+	}
+
+	*p = '\0';
+
+	free(wstr);
+	return out;
+}
+
+/* Handles process execution.  The cmd can be NULL.  Returns system error code
+ * when sets *got_exit_code to 0 and exit code of the process otherwise. */
 static DWORD
 handle_process(const char cmd[], HANDLE proc, int *got_exit_code)
 {
 	DWORD exit_code;
 
-	if(!should_wait_for_program(cmd))
+	if(cmd != NULL && !should_wait_for_program(cmd))
 	{
 		return 0;
 	}
@@ -534,10 +694,14 @@ get_shell_type(const char shell_cmd[])
 	{
 		return ST_CMD;
 	}
-	else
+	if(stroscmp(shell_name, "powershell") == 0 ||
+			stroscmp(shell_name, "powershell.exe") == 0 ||
+			stroscmp(shell_name, "pwsh") == 0 ||
+			stroscmp(shell_name, "pwsh.exe") == 0)
 	{
-		return ST_NORMAL;
+		return ST_PS;
 	}
+	return ST_NORMAL;
 }
 
 int
@@ -766,8 +930,14 @@ get_installed_data_dir(void)
 	return data_dir;
 }
 
+const char *
+get_sys_conf_dir(void)
+{
+	return get_installed_data_dir();
+}
+
 FILE *
-win_tmpfile()
+win_tmpfile(void)
 {
 	char dir[PATH_MAX + 1];
 	char file[PATH_MAX + 1];
@@ -902,21 +1072,24 @@ get_set_owner_privilege(void)
 	return granted;
 }
 
-uint64_t
-get_free_space(const char at[])
+int
+get_drive_info(const char at[], uint64_t *total_bytes, uint64_t *free_bytes)
 {
 	char *const root_path = get_root_path(at);
-	DWORD sectors_per_cluster, bytes_per_sector, number_of_free_clusters;
+	DWORD sectors_in_cluster, bytes_in_sector, free_cluster_count;
 	DWORD total_number_of_clusters;
-	if(!GetDiskFreeSpaceA(root_path, &sectors_per_cluster,
-				&bytes_per_sector, &number_of_free_clusters, &total_number_of_clusters))
+	if(!GetDiskFreeSpaceA(root_path, &sectors_in_cluster, &bytes_in_sector,
+				&free_cluster_count, &total_number_of_clusters))
 	{
 		free(root_path);
-		return 0;
+		return -1;
 	}
 	free(root_path);
 
-	return (uint64_t)bytes_per_sector*sectors_per_cluster*number_of_free_clusters;
+	*total_bytes =
+		(uint64_t)bytes_in_sector*sectors_in_cluster*total_number_of_clusters;
+	*free_bytes = (uint64_t)bytes_in_sector*sectors_in_cluster*free_cluster_count;
+	return 0;
 }
 
 /* Extracts root part of the path (drive name or UNC share name).  Returns newly
@@ -941,7 +1114,7 @@ get_root_path(const char path[])
 }
 
 int
-win_cancel_process(DWORD pid, HANDLE hprocess)
+win_cancel_process(DWORD pid)
 {
 	return EnumWindows((WNDENUMPROC)&close_app_enum, (LPARAM)pid) == FALSE;
 }
@@ -975,5 +1148,210 @@ win_to_unix_time(FILETIME ft)
 	return win_time/WINDOWS_TICK - SEC_TO_UNIX_EPOCH;
 }
 
+DWORD
+win_get_file_attrs(const char path[])
+{
+	DWORD attr;
+	wchar_t *utf16_path;
+
+	if(is_path_absolute(path) && !is_unc_path(path))
+	{
+		if(isalpha(path[0]) && !drive_exists(path[0]))
+		{
+			return INVALID_FILE_ATTRIBUTES;
+		}
+	}
+
+	utf16_path = utf8_to_utf16(path);
+	attr = GetFileAttributesW(utf16_path);
+	free(utf16_path);
+
+	return attr;
+}
+
+DWORD
+win_get_reparse_point_type(const char path[])
+{
+	char path_copy[PATH_MAX + 1];
+	DWORD attr;
+	wchar_t *utf16_filename;
+	HANDLE hfind;
+	WIN32_FIND_DATAW ffd;
+
+	attr = win_get_file_attrs(path);
+	if(attr == INVALID_FILE_ATTRIBUTES)
+	{
+		LOG_WERROR(GetLastError());
+		return 0;
+	}
+
+	if(!(attr & FILE_ATTRIBUTE_REPARSE_POINT))
+	{
+		return 0;
+	}
+
+	copy_str(path_copy, sizeof(path_copy), path);
+	chosp(path_copy);
+
+	utf16_filename = utf8_to_utf16(path_copy);
+	hfind = FindFirstFileW(utf16_filename, &ffd);
+	free(utf16_filename);
+
+	if(hfind == INVALID_HANDLE_VALUE)
+	{
+		LOG_WERROR(GetLastError());
+		return 0;
+	}
+
+	if(!FindClose(hfind))
+	{
+		LOG_WERROR(GetLastError());
+	}
+
+	return ffd.dwReserved0;
+}
+
+int
+win_reparse_point_read(const char path[], char buf[], size_t buf_len)
+{
+	char path_copy[PATH_MAX + 1];
+	copy_str(path_copy, sizeof(path_copy), path);
+	chosp(path_copy);
+
+	wchar_t *utf16_filename = to_wide(path_copy);
+	HANDLE hfile = CreateFileW(utf16_filename, 0, FILE_SHARE_READ | FILE_SHARE_WRITE,
+			NULL, OPEN_EXISTING,
+			FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT, NULL);
+	free(utf16_filename);
+
+	if(hfile == INVALID_HANDLE_VALUE)
+	{
+		LOG_WERROR(GetLastError());
+		return -1;
+	}
+
+	DWORD attr;
+	if(!DeviceIoControl(hfile, FSCTL_GET_REPARSE_POINT, NULL, 0, buf,
+			buf_len, &attr, NULL))
+	{
+		LOG_WERROR(GetLastError());
+		CloseHandle(hfile);
+		return -1;
+	}
+	CloseHandle(hfile);
+
+	return 0;
+}
+
+int
+win_symlink_read(const char link[], char buf[], int buf_len)
+{
+	if(!is_symlink(link))
+	{
+		return -1;
+	}
+
+	char rdb[2048];
+	if(win_reparse_point_read(link, rdb, sizeof(rdb)) != 0)
+	{
+		return -1;
+	}
+
+	REPARSE_DATA_BUFFER *sbuf = (REPARSE_DATA_BUFFER *)rdb;
+	WCHAR *path = sbuf->SymbolicLinkReparseBuffer.PathBuffer;
+	path[sbuf->SymbolicLinkReparseBuffer.PrintNameOffset/sizeof(WCHAR) +
+			sbuf->SymbolicLinkReparseBuffer.PrintNameLength/sizeof(WCHAR)] = L'\0';
+	char *mb = to_multibyte(path +
+			sbuf->SymbolicLinkReparseBuffer.PrintNameOffset/sizeof(WCHAR));
+	if(strncmp(mb, "\\??\\", 4) == 0)
+		strncpy(buf, mb + 4, buf_len);
+	else
+		strncpy(buf, mb, buf_len);
+	buf[buf_len - 1] = '\0';
+	free(mb);
+	system_to_internal_slashes(buf);
+	return 0;
+}
+
+int
+win_shortcut_read(const char shortcut[], char buf[], int buf_len)
+{
+	HRESULT hres;
+
+	static int initialized;
+	if(!initialized)
+	{
+		hres = CoInitializeEx(NULL, COINIT_MULTITHREADED);
+		if(!SUCCEEDED(hres))
+		{
+			return 1;
+		}
+		initialized = 1;
+	}
+
+	static IShellLinkW *shlink;
+	if(shlink == NULL)
+	{
+		hres = CoCreateInstance(&CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
+				&IID_IShellLinkW, (void **)&shlink);
+		if(!SUCCEEDED(hres))
+		{
+			return 1;
+		}
+	}
+
+	static IPersistFile *pfile;
+	if(pfile == NULL)
+	{
+		hres = shlink->lpVtbl->QueryInterface(shlink, &IID_IPersistFile,
+				(void **)&pfile);
+		if(!SUCCEEDED(hres))
+		{
+			return 1;
+		}
+	}
+
+	wchar_t *wpath = to_wide(shortcut);
+	if(wpath == NULL)
+	{
+		return 1;
+	}
+
+	hres = pfile->lpVtbl->Load(pfile, wpath, STGM_READ);
+	free(wpath);
+
+	if(SUCCEEDED(hres))
+	{
+		hres = shlink->lpVtbl->Resolve(shlink, INVALID_HANDLE_VALUE, SLR_NO_UI);
+
+		if(SUCCEEDED(hres))
+		{
+			wchar_t target[PATH_MAX + 1];
+			hres = shlink->lpVtbl->GetPath(shlink, target, sizeof(target), NULL, 0);
+
+			if(hres == S_OK)
+			{
+				char *mb = to_multibyte(target);
+				if(mb == NULL)
+				{
+					return 1;
+				}
+				copy_str(buf, buf_len, mb);
+				system_to_internal_slashes(buf);
+				free(mb);
+				return 0;
+			}
+		}
+	}
+
+	return 1;
+}
+
+uint64_t
+get_true_inode(const struct dir_entry_t *entry)
+{
+	return 0;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
