@@ -74,15 +74,36 @@ const char * escape_for_cd(const char str[]);
  * handler opened for read and write that is automatically removed on
  * application close.  Don't use tmpfile(), they utterly failed to implement
  * it. */
-FILE * win_tmpfile();
+FILE * win_tmpfile(void);
 
 /* Tries to cancel process gracefully.  Returns zero if cancellation was
  * requested, otherwise non-zero is returned. */
-int win_cancel_process(DWORD pid, HANDLE hprocess);
+int win_cancel_process(DWORD pid);
 
 /* Converts FILETIME to time_t.  Returns converted time. */
 time_t win_to_unix_time(FILETIME ft);
 
+/* Obtains attributes of a file.  Skips check for unmounted disks.  Doesn't
+ * dereference symbolic links.  Returns the attributes, which is
+ * INVALID_FILE_ATTRIBUTES on error. */
+DWORD win_get_file_attrs(const char path[]);
+
+/* Obtains type of reparse point at the location.  Returns the type or 0 on
+ * error (including when path doesn't point at a reparse point). */
+DWORD win_get_reparse_point_type(const char path[]);
+
+/* Fills buffer with REPARSE_DATA_BUFFER structure characterizing the path.
+ * Returns zero on success, otherwise non-zero is returned. */
+int win_reparse_point_read(const char path[], char buf[], size_t buf_len);
+
+/* Fills the buf of size buf_len with the path symbolic link specified by link
+ * parameter.  Returns zero on success, otherwise non-zero is returned. */
+int win_symlink_read(const char link[], char buf[], int buf_len);
+
+/* Resolves target of a .lnk-file.  Returns zero on success and non-zero
+ * otherwise. */
+int win_shortcut_read(const char shortcut[], char buf[], int buf_len);
+
 TSTATIC_DEFS(
 	int should_wait_for_program(const char cmd[]);
 )
