@@ -18,19 +18,24 @@
 
 #include "version.h"
 
+#include <curses.h> /* COLORS COLOR_PAIRS */
+
 #include <assert.h> /* assert() */
 #include <stddef.h> /* NULL */
 #include <string.h> /* strdup() */
 
 #include "utils/str.h"
+#include "utils/utils.h"
+#include "status.h"
+#include "vcache.h"
 
 /* This variable is automatically updated during build. */
 extern const char GIT_INFO[];
 
 int
-fill_version_info(char **list)
+fill_version_info(char **list, int include_stats)
 {
-	const int LEN = 13;
+	const int LEN = 22;
 	int x = 0;
 
 	if(list == NULL)
@@ -101,6 +106,32 @@ fill_version_info(char **list)
 	list[x++] = strdup("Without remote command execution");
 #endif
 
+	if(include_stats)
+	{
+		char size[64];
+		(void)friendly_size_notation(vcache_size(), sizeof(size), size);
+
+		list[x++] = strdup("");
+#ifndef _WIN32
+		list[x++] = format_str("Terminal name: %s", curr_stats.term_name);
+#endif
+		list[x++] = format_str("Max colors: %d", COLORS);
+		list[x++] = format_str("Max color pairs: %d", COLOR_PAIRS);
+#ifdef HAVE_EXTENDED_COLORS
+		list[x++] = strdup("Extended colors: present");
+#else
+		list[x++] = strdup("Extended colors: missing");
+#endif
+#ifndef _WIN32
+		list[x++] = format_str("RGB: %d", tigetflag("RGB"));
+#endif
+		list[x++] = format_str("Direct color: %s",
+				curr_stats.direct_color ? "yes" : "no");
+
+		list[x++] = strdup("");
+		list[x++] = format_str("Preview cache size: %s", size);
+	}
+
 	assert(x <= LEN);
 
 	return x;
