@@ -1,7 +1,7 @@
 #ifndef VIFM_TESTS__COLUMN_VIEW__TEST_H__
 #define VIFM_TESTS__COLUMN_VIEW__TEST_H__
 
-#include "../../src/column_view.h"
+#include "../../src/ui/column_view.h"
 
 #define COL1_ID 1
 #define COL2_ID 2
@@ -10,7 +10,7 @@
  * Omitted those which are provide additional information and do not very
  * valuable in tests. */
 typedef void (*print_func)(const void *data, int column_id, const char buf[],
-		size_t offset);
+		size_t offset, AlignType align);
 
 print_func print_next;
 
@@ -20,4 +20,4 @@ column_func col2_next;
 #endif /* VIFM_TESTS__COLUMN_VIEW__TEST_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
