@@ -2,7 +2,7 @@
 
 #include <locale.h> /* setlocale() */
 #include <stddef.h> /* NULL size_t */
-#include <string.h>
+#include <string.h> /* memcpy() */
 
 #include "../../src/utils/utf8.h"
 #include "../../src/utils/utils.h"
@@ -12,8 +12,10 @@
 
 static void column_line_print(const void *data, int column_id, const char buf[],
 		size_t offset, AlignType align);
-static void column1_func(int id, const void *data, size_t buf_len, char buf[]);
-static void column2_func(int id, const void *data, size_t buf_len, char buf[]);
+static void column1_func(void *data, size_t buf_len, char buf[],
+		const format_info_t *info);
+static void column2_func(void *data, size_t buf_len, char buf[],
+		const format_info_t *info);
 static int locale_works(void);
 
 static const size_t MAX_WIDTH = 20;
@@ -50,18 +52,17 @@ static void
 column_line_print(const void *data, int column_id, const char buf[],
 		size_t offset, AlignType align)
 {
-	strncpy(print_buffer +
-			utf8_nstrsnlen(print_buffer, offset), buf, strlen(buf));
+	memcpy(print_buffer + utf8_nstrsnlen(print_buffer, offset), buf, strlen(buf));
 }
 
 static void
-column1_func(int id, const void *data, size_t buf_len, char buf[])
+column1_func(void *data, size_t buf_len, char buf[], const format_info_t *info)
 {
 	snprintf(buf, buf_len + 1, "%s", col1_str);
 }
 
 static void
-column2_func(int id, const void *data, size_t buf_len, char buf[])
+column2_func(void *data, size_t buf_len, char buf[], const format_info_t *info)
 {
 	snprintf(buf, buf_len + 1, "%s", "яюэъыьщшчцхфутсрпонмлкйизжёедгв推");
 }
