@@ -3,10 +3,13 @@
 #include <sys/stat.h> /* chmod() */
 #include <unistd.h> /* chdir() unlink() */
 
+#include <limits.h> /* INT_MAX */
 #include <stddef.h> /* NULL */
 #include <stdlib.h> /* free() */
 #include <string.h> /* strcpy() strdup() */
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/compat/fs_limits.h"
 #include "../../src/utils/dynarray.h"
@@ -17,8 +20,6 @@
 #include "../../src/fops_cpmv.h"
 #include "../../src/trash.h"
 
-#include "utils.h"
-
 static void check_directory_clash(int parent_to_child, CopyMoveLikeOp op);
 
 static char *saved_cwd;
@@ -26,14 +27,12 @@ static char *saved_cwd;
 SETUP()
 {
 	saved_cwd = save_cwd();
-
-	set_to_sandbox_path(lwin.curr_dir, sizeof(lwin.curr_dir));
-	set_to_sandbox_path(rwin.curr_dir, sizeof(rwin.curr_dir));
-
 	assert_success(chdir(SANDBOX_PATH));
 
 	/* lwin */
 	view_setup(&lwin);
+	make_abs_path(lwin.curr_dir, sizeof(lwin.curr_dir), SANDBOX_PATH, "",
+			saved_cwd);
 	lwin.list_rows = 1;
 	lwin.list_pos = 0;
 	lwin.dir_entry = dynarray_cextend(NULL,
@@ -43,6 +42,8 @@ SETUP()
 
 	/* rwin */
 	view_setup(&rwin);
+	make_abs_path(rwin.curr_dir, sizeof(rwin.curr_dir), SANDBOX_PATH, "",
+			saved_cwd);
 	rwin.filtered = 0;
 	rwin.list_pos = 0;
 
@@ -63,7 +64,7 @@ TEST(move_file)
 	char new_fname[] = "new_name";
 	char *list[] = { &new_fname[0] };
 
-	create_empty_file(lwin.dir_entry[0].name);
+	create_file(lwin.dir_entry[0].name);
 
 	lwin.dir_entry[0].marked = 1;
 	(void)fops_cpmv(&lwin, list, ARRAY_LEN(list), CMLO_MOVE, 0);
@@ -132,7 +133,7 @@ TEST(refuse_to_copy_or_move_to_source_files_with_the_same_name)
 
 	rwin.dir_entry[0].marked = 1;
 	rwin.dir_entry[1].marked = 1;
-	rwin.selected_files = 2;
+	rwin.pending_marking = 1;
 
 	check_marking(curr_view, 0, NULL);
 
@@ -144,11 +145,49 @@ TEST(refuse_to_copy_or_move_to_source_files_with_the_same_name)
 	assert_false(path_exists("a", NODEREF));
 }
 
+TEST(operation_with_rename_of_identically_named_files_is_allowed)
+{
+	assert_false(flist_custom_active(&rwin));
+
+	restore_cwd(saved_cwd);
+	saved_cwd = save_cwd();
+
+	strcpy(rwin.curr_dir, saved_cwd);
+	flist_custom_start(&rwin, "test");
+	flist_custom_add(&rwin, TEST_DATA_PATH "/existing-files/a");
+	flist_custom_add(&rwin, TEST_DATA_PATH "/rename/a");
+	assert_true(flist_custom_finish(&rwin, CV_REGULAR, 0) == 0);
+	assert_int_equal(2, rwin.list_rows);
+
+	curr_view = &rwin;
+	other_view = &lwin;
+
+	rwin.dir_entry[0].marked = 1;
+	rwin.dir_entry[1].marked = 1;
+	rwin.pending_marking = 1;
+
+	check_marking(curr_view, 0, NULL);
+
+	char *list[] = { "a", "b" };
+	char a_path[PATH_MAX + 1], b_path[PATH_MAX + 1];
+	make_abs_path(a_path, sizeof(a_path), SANDBOX_PATH, "a", saved_cwd);
+	make_abs_path(b_path, sizeof(b_path), SANDBOX_PATH, "b", saved_cwd);
+
+	/* Not testing moving for simplicity. */
+	(void)fops_cpmv(&rwin, list, ARRAY_LEN(list), CMLO_COPY, 0);
+	remove_file(a_path);
+	remove_file(b_path);
+	(void)fops_cpmv(&rwin, list, ARRAY_LEN(list), CMLO_COPY, 1);
+	remove_file(a_path);
+	remove_file(b_path);
+}
+
 TEST(cpmv_crash_on_wrong_list_access)
 {
 	char *list[] = { "." };
 
 	view_teardown(&lwin);
+	view_setup(&lwin);
 
 	restore_cwd(saved_cwd);
 	saved_cwd = save_cwd();
@@ -196,11 +235,12 @@ TEST(cpmv_considers_tree_structure)
 	char new_fname[] = "new_name";
 	char *list[] = { &new_fname[0] };
 
-	create_empty_dir("dir");
+	cfg.dot_dirs = DD_TREE_LEAFS_PARENT;
+	create_dir("dir");
 
 	/* Move from tree root to nested dir. */
-	create_empty_file("file");
-	flist_load_tree(&rwin, rwin.curr_dir);
+	create_file("file");
+	flist_load_tree(&rwin, rwin.curr_dir, INT_MAX);
 	rwin.list_pos = 1;
 	lwin.dir_entry[0].marked = 1;
 	(void)fops_cpmv(&lwin, list, 1, CMLO_MOVE, 0);
@@ -209,9 +249,9 @@ TEST(cpmv_considers_tree_structure)
 	/* Move back. */
 	curr_view = &rwin;
 	other_view = &lwin;
-	create_empty_file("dir/file");
-	flist_load_tree(&lwin, flist_get_dir(&lwin));
-	flist_load_tree(&rwin, flist_get_dir(&rwin));
+	create_file("dir/file");
+	flist_load_tree(&lwin, flist_get_dir(&lwin), INT_MAX);
+	flist_load_tree(&rwin, flist_get_dir(&rwin), INT_MAX);
 	lwin.list_pos = 0;
 	rwin.dir_entry[1].marked = 1;
 	(void)fops_cpmv(&rwin, NULL, 0, CMLO_MOVE, 0);
@@ -225,7 +265,8 @@ TEST(cpmv_can_move_files_from_and_out_of_trash_at_the_same_time)
 	int bg;
 
 	strcat(lwin.curr_dir, "/trash");
-	set_trash_dir(lwin.curr_dir);
+	trash_set_specs(lwin.curr_dir);
+	assert_success(rmdir("trash"));
 	remove_last_path_component(lwin.curr_dir);
 
 	strcat(lwin.curr_dir, "/dir");
@@ -235,12 +276,12 @@ TEST(cpmv_can_move_files_from_and_out_of_trash_at_the_same_time)
 
 	for(bg = 0; bg < 2; ++bg)
 	{
-		create_empty_dir("trash");
-		create_empty_file("trash/000_a");
-		create_empty_dir("trash/nested");
-		create_empty_file("trash/nested/000_file");
-		create_empty_dir("dir");
-		create_empty_file("000_b");
+		create_dir("trash");
+		create_file("trash/000_a");
+		create_dir("trash/nested");
+		create_file("trash/nested/000_file");
+		create_dir("dir");
+		create_file("000_b");
 
 		flist_custom_start(&rwin, "test");
 		flist_custom_add(&rwin, "trash/000_a");
@@ -312,10 +353,10 @@ TEST(parent_overwrite_is_prevented_on_file_move)
 	check_directory_clash(0, CMLO_MOVE);
 }
 
-TEST(copying_is_aborted_if_we_can_not_read_a_file, IF(not_windows))
+TEST(copying_is_aborted_if_we_can_not_read_a_file, IF(regular_unix_user))
 {
-	create_empty_file("can-read");
-	create_empty_file("can-not-read");
+	create_file("can-read");
+	create_file("can-not-read");
 	assert_success(chmod("can-not-read", 0000));
 	populate_dir_list(&lwin, 0);
 
@@ -327,12 +368,98 @@ TEST(copying_is_aborted_if_we_can_not_read_a_file, IF(not_windows))
 	assert_success(unlink("can-not-read"));
 }
 
+TEST(bail_out_if_source_matches_destination)
+{
+	create_file("file");
+
+	populate_dir_list(&lwin, 0);
+	populate_dir_list(&rwin, 0);
+
+	lwin.dir_entry[0].marked = 1;
+
+	(void)fops_cpmv(&lwin, NULL, 0, CMLO_COPY, /*force=*/0);
+	(void)fops_cpmv(&lwin, NULL, 0, CMLO_COPY, /*force=*/1);
+	(void)fops_cpmv(&lwin, NULL, 0, CMLO_MOVE, /*force=*/0);
+	(void)fops_cpmv(&lwin, NULL, 0, CMLO_MOVE, /*force=*/1);
+	(void)fops_cpmv(&lwin, NULL, 0, CMLO_LINK_REL, /*force=*/0);
+	(void)fops_cpmv(&lwin, NULL, 0, CMLO_LINK_REL, /*force=*/1);
+	(void)fops_cpmv(&lwin, NULL, 0, CMLO_LINK_ABS, /*force=*/0);
+	(void)fops_cpmv(&lwin, NULL, 0, CMLO_LINK_ABS, /*force=*/1);
+
+	(void)fops_cpmv_bg(&lwin, NULL, 0, /*move=*/0, /*force=*/0);
+	wait_for_bg();
+	(void)fops_cpmv_bg(&lwin, NULL, 0, /*move=*/0, /*force=*/1);
+	wait_for_bg();
+	(void)fops_cpmv_bg(&lwin, NULL, 0, /*move=*/1, /*force=*/0);
+	wait_for_bg();
+	(void)fops_cpmv_bg(&lwin, NULL, 0, /*move=*/1, /*force=*/1);
+	wait_for_bg();
+
+	remove_file("file");
+}
+
+TEST(cpmv_can_copy_or_move_files_to_a_subdirectory)
+{
+	char dir[] = "dir";
+	char *list[] = { dir };
+
+	int bg;
+	for(bg = 0; bg < 2; ++bg)
+	{
+		create_dir("dir");
+		create_file("file1");
+		create_file("file2");
+
+		populate_dir_list(&lwin, 0);
+		assert_int_equal(3, lwin.list_rows);
+		assert_string_equal("file1", lwin.dir_entry[1].name);
+		assert_string_equal("file2", lwin.dir_entry[2].name);
+
+		/* Copy. */
+
+		lwin.dir_entry[1].marked = 1;
+		lwin.dir_entry[2].marked = 1;
+
+		if(!bg)
+		{
+			(void)fops_cpmv(&lwin, list, ARRAY_LEN(list), CMLO_COPY, 0);
+		}
+		else
+		{
+			(void)fops_cpmv_bg(&lwin, list, ARRAY_LEN(list), CMLO_COPY, 0);
+			wait_for_bg();
+		}
+
+		assert_success(unlink("dir/file1"));
+		assert_success(unlink("dir/file2"));
+
+		/* Move. */
+
+		lwin.dir_entry[1].marked = 1;
+		lwin.dir_entry[2].marked = 1;
+
+		if(!bg)
+		{
+			(void)fops_cpmv(&lwin, list, ARRAY_LEN(list), CMLO_MOVE, 0);
+		}
+		else
+		{
+			(void)fops_cpmv_bg(&lwin, list, ARRAY_LEN(list), CMLO_MOVE, 0);
+			wait_for_bg();
+		}
+
+		assert_success(unlink("dir/file1"));
+		assert_success(unlink("dir/file2"));
+		assert_success(rmdir("dir"));
+	}
+}
+
 static void
 check_directory_clash(int parent_to_child, CopyMoveLikeOp op)
 {
-	create_empty_dir("dir");
-	create_empty_dir("dir/dir");
-	create_empty_file("dir/dir/file");
+	create_dir("dir");
+	create_dir("dir/dir");
+	create_file("dir/dir/file");
 
 	strcat(parent_to_child ? rwin.curr_dir : lwin.curr_dir, "/dir");
 
