@@ -2,20 +2,22 @@
 
 #include <unistd.h> /* chdir() rmdir() */
 
+#include <limits.h> /* INT_MAX */
 #include <stdlib.h> /* free() */
 #include <string.h> /* strcpy() */
 
+#include <test-utils.h>
+
 #include "../../src/compat/fs_limits.h"
 #include "../../src/cfg/config.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/dynarray.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/path.h"
+#include "../../src/utils/str.h"
 #include "../../src/filelist.h"
 #include "../../src/fops_misc.h"
 
-#include "utils.h"
-
 static char *saved_cwd;
 
 SETUP()
@@ -23,24 +25,30 @@ SETUP()
 	saved_cwd = save_cwd();
 	assert_success(chdir(SANDBOX_PATH));
 
+	view_setup(&lwin);
 	make_abs_path(lwin.curr_dir, sizeof(lwin.curr_dir), SANDBOX_PATH, "",
 			saved_cwd);
+
+	update_string(&cfg.fuse_home, "");
 }
 
 TEARDOWN()
 {
+	view_teardown(&lwin);
+	update_string(&cfg.fuse_home, NULL);
+
 	restore_cwd(saved_cwd);
 }
 
 TEST(make_dirs_does_nothing_for_custom_view)
 {
-	int i;
 	char path[] = "dir";
 	char *paths[] = {path};
 
 	make_abs_path(lwin.curr_dir, sizeof(lwin.curr_dir), TEST_DATA_PATH, "",
 			saved_cwd);
 
+	filter_dispose(&lwin.local_filter.filter);
 	assert_int_equal(0, filter_init(&lwin.local_filter.filter, 0));
 
 	flist_custom_start(&lwin, "test");
@@ -49,14 +57,6 @@ TEST(make_dirs_does_nothing_for_custom_view)
 
 	fops_mkdirs(&lwin, -1, paths, 1, 0);
 	assert_false(path_exists("dir", NODEREF));
-
-	for(i = 0; i < lwin.list_rows; ++i)
-	{
-		fentry_free(&lwin, &lwin.dir_entry[i]);
-	}
-	dynarray_free(lwin.dir_entry);
-
-	filter_dispose(&lwin.local_filter.filter);
 }
 
 TEST(make_dirs_does_nothing_for_duplicated_names)
@@ -143,11 +143,9 @@ TEST(make_dirs_considers_tree_structure)
 	char path[] = "new-dir";
 	char *paths[] = { path };
 
-	view_setup(&lwin);
+	create_dir("dir");
 
-	create_empty_dir("dir");
-
-	flist_load_tree(&lwin, lwin.curr_dir);
+	flist_load_tree(&lwin, lwin.curr_dir, INT_MAX);
 
 	/* Set at to -1. */
 	lwin.list_pos = 0;
@@ -162,8 +160,6 @@ TEST(make_dirs_considers_tree_structure)
 	assert_success(rmdir("dir/new-dir"));
 
 	assert_success(rmdir("dir"));
-
-	view_teardown(&lwin);
 }
 
 TEST(check_by_absolute_path_is_performed_beforehand)
@@ -173,7 +169,7 @@ TEST(check_by_absolute_path_is_performed_beforehand)
 	char *names[] = { name_a, name_b };
 
 	snprintf(name_b, sizeof(name_b), "%s/b", lwin.curr_dir);
-	create_empty_dir(name_b);
+	create_dir(name_b);
 
 	(void)fops_mkdirs(&lwin, -1, names, 2, 0);
 
