@@ -6,12 +6,12 @@
 TEST(remap_builtin_cmds)
 {
 	assert_success(vle_keys_user_add(L"ZZ", L":strange movement", NORMAL_MODE,
-				0));
+				KEYS_FLAG_NONE));
 }
 
 TEST(remap_builtin_waitpoint)
 {
-	assert_success(vle_keys_user_add(L"q", L"m", NORMAL_MODE, 0));
+	assert_success(vle_keys_user_add(L"q", L"m", NORMAL_MODE, KEYS_FLAG_NONE));
 	assert_int_equal(KEYS_WAIT, vle_keys_exec(L"q"));
 	assert_success(vle_keys_exec(L"qa"));
 }
