@@ -9,8 +9,8 @@ TEST(unmap_users)
 	assert_int_equal(KEYS_UNKNOWN, vle_keys_exec(L"q"));
 	assert_int_equal(KEYS_UNKNOWN, vle_keys_exec(L"s"));
 
-	assert_success(vle_keys_user_add(L",q", L"k", NORMAL_MODE, 0));
-	assert_success(vle_keys_user_add(L",s", L"j", NORMAL_MODE, 0));
+	assert_success(vle_keys_user_add(L",q", L"k", NORMAL_MODE, KEYS_FLAG_NONE));
+	assert_success(vle_keys_user_add(L",s", L"j", NORMAL_MODE, KEYS_FLAG_NONE));
 
 	assert_int_equal(KEYS_WAIT, vle_keys_exec(L","));
 
@@ -29,7 +29,7 @@ TEST(unmap_remapped)
 {
 	assert_success(vle_keys_exec(L"j"));
 
-	assert_success(vle_keys_user_add(L"j", L"k", NORMAL_MODE, 0));
+	assert_success(vle_keys_user_add(L"j", L"k", NORMAL_MODE, KEYS_FLAG_NONE));
 
 	assert_success(vle_keys_exec(L"j"));
 
