@@ -20,6 +20,7 @@
 #include "../../src/engine/options.h"
 #include "../../src/engine/variables.h"
 #include "../../src/int/path_env.h"
+#include "../../src/lua/vlua.h"
 #include "../../src/modes/cmdline.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/env.h"
@@ -29,6 +30,7 @@
 #include "../../src/bmarks.h"
 #include "../../src/builtin_functions.h"
 #include "../../src/cmd_core.h"
+#include "../../src/plugins.h"
 
 #define ASSERT_COMPLETION(initial, expected) \
 	do \
@@ -52,6 +54,7 @@
 
 static void dummy_handler(OPT_OP op, optval_t val);
 static int dquotes_allowed_in_paths(void);
+static void prepare_for_line_completion(const wchar_t str[]);
 
 static line_stats_t stats;
 static char *saved_cwd;
@@ -115,11 +118,6 @@ TEARDOWN()
 	function_reset_all();
 }
 
-static void
-dummy_handler(OPT_OP op, optval_t val)
-{
-}
-
 TEST(vim_like_completion)
 {
 	vle_compl_reset();
@@ -168,18 +166,6 @@ TEST(only_user)
 	ASSERT_NEXT_MATCH("bar");
 }
 
-static void
-prepare_for_line_completion(const wchar_t str[])
-{
-	free(stats.line);
-	stats.line = wcsdup(str);
-	stats.len = wcslen(stats.line);
-	stats.index = stats.len;
-	stats.complete_continue = 0;
-
-	vle_compl_reset();
-}
-
 TEST(test_set_completion)
 {
 	ASSERT_COMPLETION(L"set ", L"set all");
@@ -593,16 +579,6 @@ TEST(autocmd_name_completion_is_case_insensitive)
 	ASSERT_COMPLETION(L"autocmd dir", L"autocmd DirEnter");
 }
 
-TEST(highlight_is_completed)
-{
-	ASSERT_COMPLETION(L"hi ", L"hi AuxWin");
-	ASSERT_COMPLETION(L"hi wi", L"hi WildMenu");
-	ASSERT_COMPLETION(L"hi WildMenu cter", L"hi WildMenu cterm");
-
-	assert_success(exec_commands("hi {*.jpg} cterm=none", &lwin, CIT_COMMAND));
-	ASSERT_COMPLETION(L"hi clear ", L"hi clear {*.jpg}");
-}
-
 TEST(case_override_of_paths)
 {
 	make_abs_path(curr_view->curr_dir, sizeof(curr_view->curr_dir),
@@ -770,6 +746,81 @@ TEST(delsession_is_completed)
 	cfg.config_dir[0] = '\0';
 }
 
+TEST(plugin_is_completed)
+{
+	ASSERT_COMPLETION(L"plugin ", L"plugin blacklist");
+	ASSERT_NEXT_MATCH("whitelist");
+	ASSERT_NEXT_MATCH("");
+
+	ASSERT_COMPLETION(L"plugin w", L"plugin whitelist");
+	ASSERT_NEXT_MATCH("whitelist");
+
+	restore_cwd(saved_cwd);
+	saved_cwd = save_cwd();
+
+	make_abs_path(cfg.config_dir, sizeof(cfg.config_dir), SANDBOX_PATH, "", NULL);
+	create_dir(SANDBOX_PATH "/plugins");
+	create_dir(SANDBOX_PATH "/plugins/plug1");
+	create_dir(SANDBOX_PATH "/plugins/plug2");
+	make_file(SANDBOX_PATH "/plugins/plug1/init.lua", "return {}");
+	make_file(SANDBOX_PATH "/plugins/plug2/init.lua", "return");
+
+	curr_stats.vlua = vlua_init();
+	curr_stats.plugs = plugs_create(curr_stats.vlua);
+	plugs_load(curr_stats.plugs, cfg.config_dir);
+
+	ASSERT_COMPLETION(L"plugin whitelist ", L"plugin whitelist plug1");
+	ASSERT_NEXT_MATCH("plug2");
+	ASSERT_NEXT_MATCH("");
+
+	plugs_free(curr_stats.plugs);
+	vlua_finish(curr_stats.vlua);
+	curr_stats.plugs = NULL;
+	curr_stats.vlua = NULL;
+
+	remove_file(SANDBOX_PATH "/plugins/plug1/init.lua");
+	remove_file(SANDBOX_PATH "/plugins/plug2/init.lua");
+	remove_dir(SANDBOX_PATH "/plugins/plug1");
+	remove_dir(SANDBOX_PATH "/plugins/plug2");
+	remove_dir(SANDBOX_PATH "/plugins");
+}
+
+TEST(tree_is_completed)
+{
+	prepare_for_line_completion(L"tree ");
+
+	assert_success(line_completion(&stats));
+	assert_wstring_equal(L"tree depth=", stats.line);
+}
+
+TEST(highlight_is_completed)
+{
+	ASSERT_COMPLETION(L"hi ", L"hi AuxWin");
+	ASSERT_COMPLETION(L"hi wi", L"hi WildMenu");
+	ASSERT_COMPLETION(L"hi WildMenu cter", L"hi WildMenu cterm");
+
+	ASSERT_COMPLETION(L"hi WildMenu ctermfg=def", L"hi WildMenu ctermfg=default");
+	ASSERT_COMPLETION(L"hi WildMenu ctermfg=no", L"hi WildMenu ctermfg=none");
+	ASSERT_COMPLETION(L"hi WildMenu ctermfg=r", L"hi WildMenu ctermfg=Red1");
+	ASSERT_COMPLETION(L"hi WildMenu ctermfg=lightb",
+			L"hi WildMenu ctermfg=lightblack");
+	ASSERT_COMPLETION(L"hi WildMenu cterm=re", L"hi WildMenu cterm=reverse");
+	ASSERT_COMPLETION(L"hi WildMenu cterm=bold,re",
+			L"hi WildMenu cterm=bold,reverse");
+
+	ASSERT_COMPLETION(L"hi WildMenu guibg=r", L"hi WildMenu guibg=red");
+	ASSERT_COMPLETION(L"hi WildMenu guibg=l", L"hi WildMenu guibg=l");
+
+	assert_success(exec_commands("hi {*.jpg} cterm=none", &lwin, CIT_COMMAND));
+	ASSERT_COMPLETION(L"hi clear ", L"hi clear {*.jpg}");
+}
+
+static void
+dummy_handler(OPT_OP op, optval_t val)
+{
+	ASSERT_NO_COMPLETION(L"nosuchcommand a");
+}
+
 static int
 dquotes_allowed_in_paths(void)
 {
@@ -781,5 +832,17 @@ dquotes_allowed_in_paths(void)
 	return 0;
 }
 
+static void
+prepare_for_line_completion(const wchar_t str[])
+{
+	free(stats.line);
+	stats.line = wcsdup(str);
+	stats.len = wcslen(stats.line);
+	stats.index = stats.len;
+	stats.complete_continue = 0;
+
+	vle_compl_reset();
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
