@@ -2,6 +2,8 @@
 
 #include <string.h> /* strdup() */
 
+#include <test-utils.h>
+
 #include "../../src/engine/keys.h"
 #include "../../src/engine/variables.h"
 #include "../../src/modes/modes.h"
@@ -9,14 +11,14 @@
 #include "../../src/ui/ui.h"
 #include "../../src/utils/dynarray.h"
 #include "../../src/utils/fs.h"
-
-#include "utils.h"
+#include "../../src/cmd_core.h"
 
 SETUP()
 {
 	curr_view = &lwin;
 
 	view_setup(&lwin);
+	init_commands();
 	init_modes();
 	opt_handlers_setup();
 }
@@ -37,12 +39,10 @@ TEST(count_vars_are_set_to_defaults_normal)
 	var = getvar("v:count");
 	assert_int_equal(VTYPE_INT, var.type);
 	assert_int_equal(0, var_to_int(var));
-	var_free(var);
 
 	var = getvar("v:count1");
 	assert_int_equal(VTYPE_INT, var.type);
 	assert_int_equal(1, var_to_int(var));
-	var_free(var);
 
 	(void)vle_keys_exec_timed_out(WK_ESC);
 }
@@ -56,12 +56,10 @@ TEST(count_vars_are_set_to_passed_value_normal)
 	var = getvar("v:count");
 	assert_int_equal(VTYPE_INT, var.type);
 	assert_int_equal(3, var_to_int(var));
-	var_free(var);
 
 	var = getvar("v:count1");
 	assert_int_equal(VTYPE_INT, var.type);
 	assert_int_equal(3, var_to_int(var));
-	var_free(var);
 
 	(void)vle_keys_exec_timed_out(WK_ESC);
 }
@@ -82,12 +80,10 @@ TEST(count_vars_are_set_to_passed_value_visual)
 	var = getvar("v:count");
 	assert_int_equal(VTYPE_INT, var.type);
 	assert_int_equal(1, var_to_int(var));
-	var_free(var);
 
 	var = getvar("v:count1");
 	assert_int_equal(VTYPE_INT, var.type);
 	assert_int_equal(1, var_to_int(var));
-	var_free(var);
 
 	(void)vle_keys_exec_timed_out(WK_ESC);
 }
