@@ -7,6 +7,8 @@
 #include <stdlib.h> /* free() */
 #include <string.h> /* strcpy() */
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/ui/column_view.h"
 #include "../../src/ui/ui.h"
@@ -15,10 +17,8 @@
 #include "../../src/event_loop.h"
 #include "../../src/ops.h"
 
-#include "utils.h"
-
-static void column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset, AlignType align, const char full_column[]);
+static void column_line_print(const char buf[], size_t offset, AlignType align,
+		const char full_column[], const format_info_t *info);
 static int files_are_identical(const char a[], const char b[]);
 
 static char *saved_cwd;
@@ -35,6 +35,7 @@ SETUP()
 
 	cfg.delete_prg = strdup("");
 	cfg.use_system_calls = 1;
+	cfg.sizefmt.base = 1;
 
 	undo_setup();
 
@@ -122,8 +123,8 @@ TEST(moving_to_fake_entry_creates_the_other_file_and_entry_is_updated)
 }
 
 static void
-column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset, AlignType align, const char full_column[])
+column_line_print(const char buf[], size_t offset, AlignType align,
+		const char full_column[], const format_info_t *info)
 {
 	/* Do nothing. */
 }
@@ -182,7 +183,7 @@ TEST(moving_equal_does_nothing)
 	assert_success(remove(SANDBOX_PATH "/same-name-same-content"));
 }
 
-TEST(file_id_is_not_updated_on_failed_move, IF(not_windows))
+TEST(file_id_is_not_updated_on_failed_move, IF(regular_unix_user))
 {
 	make_abs_path(rwin.curr_dir, sizeof(rwin.curr_dir), SANDBOX_PATH, "",
 			saved_cwd);
