@@ -1,17 +1,24 @@
 #include <stic.h>
 
+#include <stddef.h> /* NULL */
 #include <stdlib.h> /* free() */
 
 #include "../../src/engine/functions.h"
+#include "../../src/engine/parsing.h"
 #include "../../src/engine/var.h"
+#include "../../src/utils/env.h"
 #include "../../src/cmd_core.h"
 
 static var_t echo_builtin(const call_info_t *call_info);
 
 SETUP_ONCE()
 {
-	static const function_t echo_function = { "a", "descr", 1, &echo_builtin };
+	static const function_t echo_function = {
+		"a", "descr", {1,1}, &echo_builtin
+	};
 	assert_success(function_register(&echo_function));
+
+	init_parser(&env_get);
 }
 
 static var_t
@@ -20,6 +27,16 @@ echo_builtin(const call_info_t *call_info)
 	return var_clone(call_info->argv[0]);
 }
 
+TEST(broken_syntax)
+{
+	const char *args = "a('foo '; bar";
+	const char *stop_ptr;
+	char *result;
+
+	result = eval_arglist(args, &stop_ptr);
+	assert_string_equal(NULL, result);
+}
+
 TEST(one_arg)
 {
 	const char *args = "'a'";
