@@ -5,6 +5,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/ui/quickview.h"
 #include "../../src/ui/ui.h"
@@ -16,8 +18,6 @@
 #include "../../src/registers.h"
 #include "../../src/status.h"
 
-#include "utils.h"
-
 #ifdef _WIN32
 #define SL "\\\\"
 #else
@@ -27,6 +27,7 @@
 static void
 setup_lwin(void)
 {
+	view_setup(&lwin);
 	strcpy(lwin.curr_dir, "/lwin");
 
 	lwin.list_rows = 4;
@@ -50,6 +51,7 @@ setup_lwin(void)
 static void
 setup_rwin(void)
 {
+	view_setup(&rwin);
 	strcpy(rwin.curr_dir, "/rwin");
 
 	rwin.list_rows = 7;
@@ -78,6 +80,11 @@ setup_rwin(void)
 	rwin.selected_files = 3;
 }
 
+SETUP_ONCE()
+{
+	stats_update_shell_type("/bin/sh");
+}
+
 SETUP()
 {
 	setup_lwin();
@@ -89,15 +96,8 @@ SETUP()
 
 TEARDOWN()
 {
-	int i;
-
-	for(i = 0; i < lwin.list_rows; i++)
-		free(lwin.dir_entry[i].name);
-	dynarray_free(lwin.dir_entry);
-
-	for(i = 0; i < rwin.list_rows; i++)
-		free(rwin.dir_entry[i].name);
-	dynarray_free(rwin.dir_entry);
+	view_teardown(&lwin);
+	view_teardown(&rwin);
 }
 
 TEST(literal_percent)
@@ -279,15 +279,53 @@ TEST(good_flag_macros)
 	assert_string_equal(" echo log", expanded);
 	assert_int_equal(MF_VERYCUSTOMVIEW_OUTPUT, flags);
 	free(expanded);
+
+	expanded = ma_expand("%v echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_SPLIT_VERT, flags);
+	free(expanded);
+
+	expanded = ma_expand("%Pl echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_PIPE_FILE_LIST, flags);
+	free(expanded);
+
+	expanded = ma_expand("%Pz echo log", "", &flags, 0);
+	assert_string_equal(" echo log", expanded);
+	assert_int_equal(MF_PIPE_FILE_LIST_Z, flags);
+	free(expanded);
 }
 
 TEST(bad_flag_macros)
 {
 	MacroFlags flags;
-	char *expanded = ma_expand("%IX echo log", "", &flags, 0);
+	char *expanded;
+
+	expanded = ma_expand("%IX echo log", "", &flags, 0);
 	assert_string_equal("X echo log", expanded);
 	assert_int_equal(MF_NONE, flags);
 	free(expanded);
+
+	expanded = ma_expand("%PX echo log", "", &flags, 0);
+	assert_string_equal("X echo log", expanded);
+	assert_int_equal(MF_NONE, flags);
+	free(expanded);
+}
+
+TEST(flags_from_different_sets)
+{
+	MacroFlags flags;
+	char *expanded;
+
+	expanded = ma_expand("echo%Pl%S", "", &flags, 0);
+	assert_string_equal("echo", expanded);
+	assert_int_equal(MF_PIPE_FILE_LIST | MF_STATUSBAR_OUTPUT, flags);
+	free(expanded);
+
+	expanded = ma_expand("echo%q%Pz", "", &flags, 0);
+	assert_string_equal("echo", expanded);
+	assert_int_equal(MF_PIPE_FILE_LIST_Z | MF_PREVIEW_OUTPUT, flags);
+	free(expanded);
 }
 
 TEST(r_well_formed)
@@ -454,45 +492,100 @@ TEST(singly_expanded_single_macros)
 
 TEST(singly_expanded_multiple_macros_single_file)
 {
+	char *expanded;
+
 	lwin.dir_entry[2].selected = 0;
 	lwin.selected_files = 1;
-	char *expanded = ma_expand_single("%f");
+	expanded = ma_expand_single("%f");
 	assert_string_equal("lfi le0", expanded);
 	free(expanded);
+
+	clear_marking(&rwin);
+	rwin.dir_entry[2].marked = 1;
+	rwin.pending_marking = 1;
+	expanded = ma_expand_single("%F");
+	assert_string_equal("/rwin/rfile2", expanded);
+	free(expanded);
 }
 
 TEST(singly_not_expanded_multiple_macros_multiple_files)
 {
+	char *expanded;
+
 	lwin.list_pos = 0;
-	char *expanded = ma_expand_single("%f");
+	expanded = ma_expand_single("%f");
+	assert_string_equal("", expanded);
+	free(expanded);
+
+	clear_marking(&rwin);
+	rwin.dir_entry[1].selected = 0;
+	rwin.dir_entry[1].marked = 1;
+	rwin.dir_entry[3].selected = 0;
+	rwin.dir_entry[3].marked = 1;
+	rwin.dir_entry[5].selected = 0;
+	rwin.dir_entry[5].marked = 1;
+	rwin.selected_files = 0;
+	rwin.pending_marking = 1;
+	expanded = ma_expand_single("%F");
 	assert_string_equal("", expanded);
 	free(expanded);
 }
 
+TEST(marking_is_not_disturbed)
+{
+	int indexes[] = { 1, 3 };
+	mark_files_at(&lwin, 2, indexes);
+
+	assert_true(lwin.pending_marking);
+	assert_true(lwin.pending_marking);
+	assert_false(lwin.dir_entry[0].marked);
+	assert_true(lwin.dir_entry[1].marked);
+	assert_false(lwin.dir_entry[2].marked);
+	assert_true(lwin.dir_entry[3].marked);
+
+	char *expanded = ma_expand("%%", "", NULL, 0);
+	assert_string_equal("%", expanded);
+	free(expanded);
+
+	assert_true(lwin.pending_marking);
+	assert_false(lwin.dir_entry[0].marked);
+	assert_true(lwin.dir_entry[1].marked);
+	assert_false(lwin.dir_entry[2].marked);
+	assert_true(lwin.dir_entry[3].marked);
+}
+
 TEST(singly_no_crash_on_wrong_register_name)
 {
-	assert_success(chdir(TEST_DATA_PATH "/spaces-in-names"));
+	make_abs_path(curr_view->curr_dir, sizeof(curr_view->curr_dir), SANDBOX_PATH,
+			"", NULL);
+	create_file(SANDBOX_PATH "/spaces in the middle");
+
 	regs_init();
 
 	assert_success(regs_append('r', "spaces in the middle"));
 	free(ma_expand_single("%r "));
 
 	regs_reset();
+
+	remove_file(SANDBOX_PATH "/spaces in the middle");
 }
 
 TEST(singly_expanded_single_file_register)
 {
-	char *expanded;
+	make_abs_path(curr_view->curr_dir, sizeof(curr_view->curr_dir), SANDBOX_PATH,
+			"", NULL);
+	create_file(SANDBOX_PATH "/spaces in the middle");
 
-	assert_success(chdir(TEST_DATA_PATH "/spaces-in-names"));
 	regs_init();
 
 	assert_success(regs_append('r', "spaces in the middle"));
-	expanded = ma_expand_single("%rr");
+	char *expanded = ma_expand_single("%rr");
 	assert_string_equal("spaces in the middle", expanded);
 	free(expanded);
 
 	regs_reset();
+
+	remove_file(SANDBOX_PATH "/spaces in the middle");
 }
 
 TEST(singly_not_expanded_multiple_files_register)
@@ -574,8 +667,8 @@ TEST(preview_macros_use_hint)
 
 	curr_stats.preview_hint = &parea;
 
-	char *expanded = ma_expand("draw %pw %ph", "", NULL, 0);
-	assert_string_equal("draw 3 4", expanded);
+	char *expanded = ma_expand("draw %pw %ph %px %py", "", NULL, 0);
+	assert_string_equal("draw 3 4 0 1", expanded);
 	free(expanded);
 
 	curr_stats.preview_hint = NULL;
@@ -593,7 +686,7 @@ TEST(preview_clear_cmd_gets_cut_off)
 
 TEST(preview_clear_cmd_is_extracted)
 {
-	assert_string_equal(" clear", ma_get_clear_cmd("draw %pw %ph %pc clear"));
+	assert_string_equal("clear", ma_get_clear_cmd("draw %pw %ph %pc clear"));
 }
 
 TEST(preview_clear_cmd_is_optional)
@@ -624,6 +717,9 @@ TEST(flags_to_str)
 	assert_string_equal("%s", ma_flags_to_str(MF_SPLIT));
 	assert_string_equal("%i", ma_flags_to_str(MF_IGNORE));
 	assert_string_equal("%n", ma_flags_to_str(MF_NO_TERM_MUX));
+
+	assert_string_equal("%Pl", ma_flags_to_str(MF_PIPE_FILE_LIST));
+	assert_string_equal("%Pz", ma_flags_to_str(MF_PIPE_FILE_LIST_Z));
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
