@@ -3,6 +3,8 @@
 #include <stdlib.h> /* free() */
 #include <string.h> /* strchr() strcmp() */
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/engine/parsing.h"
 #include "../../src/ui/statusline.h"
@@ -12,13 +14,11 @@
 #include "../../src/utils/str.h"
 #include "../../src/status.h"
 
-#include "utils.h"
-
 /* Checks that expanded string isn't equal to format string. */
 #define ASSERT_EXPANDED(format) \
 	do \
 	{ \
-		LineWithAttrs result = expand_status_line_macros(&lwin, format); \
+		cline_t result = expand_status_line_macros(&lwin, format); \
 		free(result.attrs); \
 		char *const expanded = result.line; \
 		assert_false(strcmp(expanded, format) == 0); \
@@ -30,7 +30,7 @@
 #define ASSERT_EXPANDED_TO(format, expected) \
 	do \
 	{ \
-		LineWithAttrs result = expand_status_line_macros(&lwin, format); \
+		cline_t result = expand_status_line_macros(&lwin, format); \
 		free(result.attrs); \
 		char *const expanded = result.line; \
 		assert_string_equal(expected, expanded); \
@@ -43,7 +43,7 @@
 #define ASSERT_EXPANDED_TO_WITH_HI(format, expected, highlighting) \
 	do \
 	{ \
-		LineWithAttrs result = expand_status_line_macros(&lwin, format); \
+		cline_t result = expand_status_line_macros(&lwin, format); \
 		char *const expanded = result.line; \
 		assert_string_equal(expected, expanded); \
 		assert_string_equal(highlighting, result.attrs); \
@@ -124,6 +124,13 @@ TEST(a_macro_expanded)
 	ASSERT_EXPANDED("%a");
 }
 
+TEST(c_macro_expanded)
+{
+	ASSERT_EXPANDED("%c");
+	strcpy(lwin.curr_dir, SANDBOX_PATH);
+	ASSERT_EXPANDED("%c");
+}
+
 TEST(A_macro_expanded)
 {
 	ASSERT_EXPANDED("%A");
@@ -198,6 +205,11 @@ TEST(dash_macro_expanded)
 	ASSERT_EXPANDED("%-%");
 }
 
+TEST(P_macro_expanded)
+{
+	ASSERT_EXPANDED("%P");
+}
+
 TEST(S_macro_expanded)
 {
 	ASSERT_EXPANDED("%S");
@@ -210,7 +222,7 @@ TEST(percent_macro_expanded)
 
 TEST(wrong_macros_ignored)
 {
-	static const char STATUS_CHARS[] = "tTfaAugsEdD-xlLS%[]z{*";
+	static const char STATUS_CHARS[] = "tTfacAugsEdD-xlLPS%[]z{*";
 	int i;
 
 	for(i = 1; i <= 255; ++i)
@@ -225,7 +237,7 @@ TEST(wrong_macros_ignored)
 
 TEST(wrong_macros_with_width_field_ignored)
 {
-	static const char STATUS_CHARS[] = "tTfaAugsEdD-xlLS%[]z{*";
+	static const char STATUS_CHARS[] = "tTfacAugsEdD-xlLPS%[]z{*";
 	int i;
 
 	for(i = 1; i <= 255; ++i)
@@ -291,9 +303,16 @@ TEST(ignore_mismatched_opening_curly_bracket)
 
 TEST(highlighting_is_set_correctly)
 {
-	ASSERT_EXPANDED_TO_WITH_HI("%1*%t%*",
-	                           "file",
-	                           "1   ");
+	ASSERT_EXPANDED_TO_WITH_HI("[%1*%t%*]",
+	                           "[file]",
+	                           " 1   0");
+}
+
+TEST(highlighting_is_set_correctly_for_optional)
+{
+	ASSERT_EXPANDED_TO_WITH_HI("%[%1*a",
+	                           "%[a",
+	                           "  1");
 }
 
 TEST(highlighting_has_equals_macro_preserved)
