@@ -3,17 +3,20 @@
 #include <unistd.h> /* symlink() unlink() */
 
 #include <stdio.h> /* fopen() fclose() */
+#include <string.h> /* strcmp() */
+
+#include <test-utils.h>
 
 #include "../../src/compat/fs_limits.h"
 #include "../../src/compat/os.h"
 #include "../../src/int/file_magic.h"
+#include "../../src/utils/env.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/path.h"
 
-#include "utils.h"
-
 static void check_empty_file(const char fname[]);
 static int has_mime_type_detection_and_symlinks(void);
+static int has_mime_type_detection_and_can_test_cache(void);
 static int has_mime_type_detection(void);
 
 TEST(escaping_for_determining_mime_type, IF(has_mime_type_detection))
@@ -23,6 +26,20 @@ TEST(escaping_for_determining_mime_type, IF(has_mime_type_detection))
 	check_empty_file(SANDBOX_PATH "/start`end");
 }
 
+TEST(mimetype_cache_can_be_invalidated,
+		IF(has_mime_type_detection_and_can_test_cache))
+{
+	copy_file(TEST_DATA_PATH "/read/very-long-line", SANDBOX_PATH "/file");
+	assert_string_equal("text/plain", get_mimetype(SANDBOX_PATH "/file", 0));
+
+	copy_file(TEST_DATA_PATH "/read/binary-data", SANDBOX_PATH "/file");
+	reset_timestamp(SANDBOX_PATH "/file");
+	assert_false(strcmp("text/plain", get_mimetype(SANDBOX_PATH "/file", 0)) ==
+			0);
+
+	remove_file(SANDBOX_PATH "/file");
+}
+
 TEST(relatively_large_file_name_does_not_crash,
 		IF(has_mime_type_detection_and_symlinks))
 {
@@ -77,10 +94,29 @@ has_mime_type_detection_and_symlinks(void)
 	return not_windows() && has_mime_type_detection();
 }
 
+static int
+has_mime_type_detection_and_can_test_cache(void)
+{
+	/* There is something wrong with cache test specifically on AppVeyor. */
+	return (env_get("APPVEYOR") == NULL && has_mime_type_detection());
+}
+
 static int
 has_mime_type_detection(void)
 {
-	return get_mimetype(TEST_DATA_PATH "/read/dos-line-endings", 0) != NULL;
+	const char *text = get_mimetype(TEST_DATA_PATH "/read/two-lines", 0);
+	if(text == NULL || strcmp(text, "text/plain") != 0)
+	{
+		return 0;
+	}
+
+	const char *binary = get_mimetype(TEST_DATA_PATH "/read/binary_data", 0);
+	if(binary == NULL || strcmp(binary, "text/plain") == 0)
+	{
+		return 0;
+	}
+
+	return 1;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
