@@ -3,21 +3,22 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/utils/dynarray.h"
 #include "../../src/utils/str.h"
 #include "../../src/cmd_core.h"
 #include "../../src/flist_hist.h"
 
-#include "utils.h"
-
 #define INITIAL_SIZE 10
 
 /* This should be a macro to see what test have failed. */
 #define VALIDATE_HISTORY(i, str) \
-	assert_string_equal(str, curr_stats.cmd_hist.items[i]); \
-	assert_string_equal(str, curr_stats.search_hist.items[i]); \
-	assert_string_equal(str, curr_stats.prompt_hist.items[i]); \
+	assert_string_equal(str, curr_stats.cmd_hist.items[i].text); \
+	assert_string_equal(str, curr_stats.search_hist.items[i].text); \
+	assert_string_equal(str, curr_stats.prompt_hist.items[i].text); \
+	assert_string_equal(str, curr_stats.filter_hist.items[i].text); \
 	\
 	assert_string_equal(str, lwin.history[(i) + 1].dir); \
 	assert_string_equal((str) + 1, lwin.history[(i) + 1].file); \
@@ -84,9 +85,10 @@ save_to_history(const char str[])
 	hists_commands_save(str);
 	hists_search_save(str);
 	hists_prompt_save(str);
+	hists_filter_save(str);
 
-	flist_hist_save(&lwin, str, str + 1, 0);
-	flist_hist_save(&rwin, str, str + 1, 0);
+	flist_hist_setup(&lwin, str, str + 1, 0, 1);
+	flist_hist_setup(&rwin, str, str + 1, 0, 1);
 }
 
 TEST(view_history_after_reset_contains_valid_data)
@@ -103,8 +105,8 @@ TEST(view_history_avoids_duplicates)
 	assert_int_equal(1, lwin.history_num);
 	assert_int_equal(1, rwin.history_num);
 
-	flist_hist_save(&lwin, NULL, NULL, -1);
-	flist_hist_save(&rwin, NULL, NULL, -1);
+	flist_hist_save(&lwin);
+	flist_hist_save(&rwin);
 
 	assert_int_equal(1, lwin.history_num);
 	assert_int_equal(1, rwin.history_num);
@@ -168,7 +170,7 @@ TEST(navigating_within_history)
 TEST(specified_file_position_is_unaffected_by_top_line)
 {
 	lwin.top_line = 3;
-	flist_hist_save(&lwin, "/dir", "file", 0);
+	flist_hist_setup(&lwin, "/dir", "file", 0, 1);
 	assert_string_equal("/dir", lwin.history[1].dir);
 	assert_string_equal("file", lwin.history[1].file);
 	assert_int_equal(0, lwin.history[1].rel_pos);
@@ -177,8 +179,8 @@ TEST(specified_file_position_is_unaffected_by_top_line)
 TEST(history_size_reduction_leaves_correct_number_of_elements)
 {
 	assert_int_equal(1, lwin.history_num);
-	flist_hist_save(&lwin, "/dir1", "file1", 1);
-	flist_hist_save(&lwin, "/dir2", "file2", 2);
+	flist_hist_setup(&lwin, "/dir1", "file1", 1, 1);
+	flist_hist_setup(&lwin, "/dir2", "file2", 2, 1);
 	assert_int_equal(2, lwin.history_pos);
 	assert_int_equal(3, lwin.history_num);
 
@@ -212,7 +214,7 @@ TEST(defaults_are_returned_on_dir_lookup_failure)
 	entries_t entries = {};
 	int top, pos;
 
-	flist_hist_save(&lwin, "/bin", "sh", 0);
+	flist_hist_setup(&lwin, "/bin", "sh", 0, 1);
 
 	pos = flist_hist_find(&lwin, entries, "/etc", &top);
 
@@ -226,7 +228,7 @@ TEST(defaults_are_is_returned_on_file_lookup_failure)
 	entries_t entries = { entry_list, 2 };
 	int top, pos;
 
-	flist_hist_save(&lwin, "/bin", "x", 0);
+	flist_hist_setup(&lwin, "/bin", "x", 0, 1);
 
 	pos = flist_hist_find(&lwin, entries, "/bin", &top);
 
@@ -240,7 +242,7 @@ TEST(history_item_is_found)
 	entries_t entries = { entry_list, 2 };
 	int top, pos;
 
-	flist_hist_save(&lwin, "/bin", "b", 0);
+	flist_hist_setup(&lwin, "/bin", "b", 0, 1);
 
 	pos = flist_hist_find(&lwin, entries, "/bin", &top);
 
@@ -254,7 +256,7 @@ TEST(top_value_can_not_be_negative)
 	entries_t entries = { entry_list, 2 };
 	int top, pos;
 
-	flist_hist_save(&lwin, "/bin", "b", 2);
+	flist_hist_setup(&lwin, "/bin", "b", 2, 1);
 
 	pos = flist_hist_find(&lwin, entries, "/bin", &top);
 
@@ -264,8 +266,8 @@ TEST(top_value_can_not_be_negative)
 
 TEST(hist_update_performs_update_but_does_not_move_entry)
 {
-	flist_hist_save(&lwin, "/bin", "a", 2);
-	flist_hist_save(&lwin, "/etc", "b", 4);
+	flist_hist_setup(&lwin, "/bin", "a", 2, 1);
+	flist_hist_setup(&lwin, "/etc", "b", 4, 1);
 
 	flist_hist_update(&lwin, "/bin", "g", 8);
 
@@ -280,8 +282,8 @@ TEST(hist_update_performs_update_but_does_not_move_entry)
 TEST(history_without_suffix_is_cloned)
 {
 	assert_int_equal(1, lwin.history_num);
-	flist_hist_save(&lwin, "/dir1", "file1", 1);
-	flist_hist_save(&lwin, "/dir2", "file2", 2);
+	flist_hist_setup(&lwin, "/dir1", "file1", 1, 1);
+	flist_hist_setup(&lwin, "/dir2", "file2", 2, 1);
 
 	flist_hist_clone(&rwin, &lwin);
 
@@ -302,8 +304,8 @@ TEST(history_without_suffix_is_cloned)
 TEST(history_with_suffix_is_cloned)
 {
 	assert_int_equal(1, lwin.history_num);
-	flist_hist_save(&lwin, "/dir1", "file1", 1);
-	flist_hist_save(&lwin, "/dir2", "file2", 2);
+	flist_hist_setup(&lwin, "/dir1", "file1", 1, 1);
+	flist_hist_setup(&lwin, "/dir2", "file2", 2, 1);
 
 	lwin.history_pos = 1;
 
@@ -323,5 +325,18 @@ TEST(history_with_suffix_is_cloned)
 	assert_int_equal(2, rwin.history[2].rel_pos);
 }
 
+TEST(reducing_history_does_not_leak)
+{
+	flist_hist_setup(&lwin, "/dir1", "file1", 1, 1);
+	flist_hist_setup(&lwin, "/dir2", "file2", 2, 1);
+	flist_hist_setup(&lwin, "/dir3", "file3", 2, 1);
+	flist_hist_setup(&lwin, "/dir4", "file4", 2, 1);
+	flist_hist_setup(&lwin, "/dir5", "file5", 2, 1);
+
+	/* The leak happened when new_size < old_size - new_size.  old_size == 5,
+	 * old_size - new_size == 3. */
+	cfg_resize_histories(2);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
