@@ -11,6 +11,7 @@
 #include "../../src/filelist.h"
 #include "../../src/macros.h"
 #include "../../src/registers.h"
+#include "../../src/status.h"
 
 #ifdef _WIN32
 #define SL "\\\\"
@@ -82,6 +83,11 @@ setup_registers(void)
 	regs_append('z', "existing-files/c");
 }
 
+SETUP_ONCE()
+{
+	stats_update_shell_type("/bin/sh");
+}
+
 SETUP()
 {
 	assert_success(chdir(TEST_DATA_PATH));
