@@ -6,6 +6,8 @@
 #include <stdio.h> /* fclose() fmemopen() remove() */
 #include <string.h> /* strcpy() */
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/compat/fs_limits.h"
 #include "../../src/engine/keys.h"
@@ -22,8 +24,6 @@
 #include "../../src/filelist.h"
 #include "../../src/status.h"
 
-#include "utils.h"
-
 SETUP()
 {
 	update_string(&cfg.shell, "");
@@ -87,6 +87,7 @@ TEST(paths_in_root_do_not_have_extra_slash_on_choosing)
 	curr_stats.chosen_files_out = out_spec;
 	curr_stats.original_stdout = fmemopen(out_buf, sizeof(out_buf), "w");
 
+	flist_set_marking(&lwin, 1);
 	vim_write_file_list(&lwin, 1, files);
 
 	fclose(curr_stats.original_stdout);
@@ -108,6 +109,7 @@ TEST(abs_paths_are_not_touched_on_choosing)
 	curr_stats.chosen_files_out = out_spec;
 	curr_stats.original_stdout = fmemopen(out_buf, sizeof(out_buf), "w");
 
+	flist_set_marking(&lwin, 1);
 	vim_write_file_list(&lwin, 1, files);
 
 	fclose(curr_stats.original_stdout);
@@ -139,6 +141,7 @@ TEST(selection_can_be_chosen)
 	curr_stats.chosen_files_out = out_spec;
 	curr_stats.original_stdout = fmemopen(out_buf, sizeof(out_buf), "w");
 
+	flist_set_marking(&lwin, 1);
 	vim_write_file_list(&lwin, 0, NULL);
 
 	fclose(curr_stats.original_stdout);
@@ -173,6 +176,7 @@ TEST(path_are_formed_right_on_choosing_in_cv)
 	curr_stats.chosen_files_out = out_spec;
 	curr_stats.original_stdout = fmemopen(out_buf, sizeof(out_buf), "w");
 
+	flist_set_marking(&lwin, 1);
 	vim_write_file_list(&lwin, 1, files);
 
 	fclose(curr_stats.original_stdout);
@@ -273,26 +277,26 @@ TEST(title_support_is_detected_correctly)
 	unsigned int i;
 	for(i = 0; i < ARRAY_LEN(XTERM_LIKE); ++i)
 	{
-		assert_int_equal(TK_REGULAR, get_title_kind(XTERM_LIKE[i]));
+		assert_int_equal(TK_REGULAR, title_kind_for_termenv(XTERM_LIKE[i]));
 	}
 	for(i = 0; i < ARRAY_LEN(SCREEN_LIKE); ++i)
 	{
-		assert_int_equal(TK_REGULAR, get_title_kind(SCREEN_LIKE[i]));
+		assert_int_equal(TK_REGULAR, title_kind_for_termenv(SCREEN_LIKE[i]));
 	}
-	assert_int_equal(TK_REGULAR, get_title_kind("linux"));
-	assert_int_equal(TK_REGULAR, get_title_kind("unknown"));
+	assert_int_equal(TK_REGULAR, title_kind_for_termenv("linux"));
+	assert_int_equal(TK_REGULAR, title_kind_for_termenv("unknown"));
 #else
 	unsigned int i;
 	for(i = 0; i < ARRAY_LEN(XTERM_LIKE); ++i)
 	{
-		assert_int_equal(TK_REGULAR, get_title_kind(XTERM_LIKE[i]));
+		assert_int_equal(TK_REGULAR, title_kind_for_termenv(XTERM_LIKE[i]));
 	}
 	for(i = 0; i < ARRAY_LEN(SCREEN_LIKE); ++i)
 	{
-		assert_int_equal(TK_SCREEN, get_title_kind(SCREEN_LIKE[i]));
+		assert_int_equal(TK_SCREEN, title_kind_for_termenv(SCREEN_LIKE[i]));
 	}
-	assert_int_equal(TK_ABSENT, get_title_kind("linux"));
-	assert_int_equal(TK_ABSENT, get_title_kind("unknown"));
+	assert_int_equal(TK_ABSENT, title_kind_for_termenv("linux"));
+	assert_int_equal(TK_ABSENT, title_kind_for_termenv("unknown"));
 #endif
 }
 
