@@ -5,28 +5,25 @@
 #include <stdlib.h> /* remove() */
 #include <string.h> /* strcpy() strdup() */
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/compat/fs_limits.h"
-#include "../../src/engine/keys.h"
-#include "../../src/menus/map_menu.h"
 #include "../../src/menus/menus.h"
 #include "../../src/modes/modes.h"
-#include "../../src/ui/statusbar.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/string_array.h"
 
-#include "utils.h"
-
 static menu_data_t m;
 
 SETUP()
 {
 	menus_init_data(&m, &lwin, strdup("test"), strdup("No matches"));
 
-	m.len = add_to_string_array(&m.items, m.len, 1, "a");
-	m.len = add_to_string_array(&m.items, m.len, 1, "b");
-	m.len = add_to_string_array(&m.items, m.len, 1, "c");
+	m.len = add_to_string_array(&m.items, m.len, "a");
+	m.len = add_to_string_array(&m.items, m.len, "b");
+	m.len = add_to_string_array(&m.items, m.len, "c");
 }
 
 TEARDOWN()
@@ -82,13 +79,13 @@ TEST(search_via_menu_search)
 
 TEST(ok_to_print_message_if_there_is_no_pattern)
 {
-	menus_search_print_msg(m.state);
+	menus_search_print_msg(&m);
 }
 
 TEST(ok_to_print_message_for_wrong_pattern)
 {
 	assert_true(menus_search("*", &m, 1));
-	menus_search_print_msg(m.state);
+	menus_search_print_msg(&m);
 }
 
 TEST(forward_found_no_wrap)
@@ -210,21 +207,5 @@ TEST(null_pattern_causes_pattern_reuse)
 	assert_int_equal(2, m.pos);
 }
 
-TEST(empty_mappings_menu_is_not_displayed)
-{
-	init_modes();
-
-	ui_sb_msg("");
-	assert_failure(show_map_menu(&lwin, "normal", NORMAL_MODE, L"nonsense"));
-	assert_string_equal("No mappings found", ui_sb_last());
-
-	vle_keys_user_add(L"this", L"that", NORMAL_MODE, KEYS_FLAG_NONE);
-	ui_sb_msg("");
-	assert_failure(show_map_menu(&lwin, "normal", NORMAL_MODE, L"nonsense"));
-	assert_string_equal("No mappings found", ui_sb_last());
-
-	vle_keys_reset();
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
