@@ -17,6 +17,8 @@
 #include "../../src/filelist.h"
 #include "../../src/status.h"
 
+#include "utils.h"
+
 static char cwd[PATH_MAX + 1];
 
 SETUP_ONCE()
@@ -92,6 +94,26 @@ TEST(center_splitter)
 	assert_true(curr_stats.splitter_ratio == 0.5);
 }
 
+TEST(folding)
+{
+	assert_success(load_tree(&lwin, TEST_DATA_PATH "/tree", cwd));
+	assert_int_equal(12, lwin.list_rows);
+
+	(void)vle_keys_exec_timed_out(WK_z WK_x);
+	populate_dir_list(&lwin, /*reload=*/1);
+	assert_int_equal(5, lwin.list_rows);
+
+	(void)vle_keys_exec_timed_out(WK_z WK_x);
+	populate_dir_list(&lwin, /*reload=*/1);
+	assert_int_equal(12, lwin.list_rows);
+
+	lwin.list_pos = 4;
+	(void)vle_keys_exec_timed_out(WK_z WK_x);
+	populate_dir_list(&lwin, /*reload=*/1);
+	assert_int_equal(10, lwin.list_rows);
+	assert_int_equal(2, lwin.list_pos);
+}
+
 TEST(gf, IF(not_windows))
 {
 	char dir_path[PATH_MAX + 1];
