@@ -1,5 +1,11 @@
 #include <stic.h>
 
+#include <stdio.h> /* remove() */
+#include <stdlib.h> /* free() */
+#include <string.h> /* strcpy() strdup() */
+
+#include <test-utils.h>
+
 #include "../../src/compat/os.h"
 #include "../../src/cfg/config.h"
 #include "../../src/modes/normal.h"
@@ -8,28 +14,28 @@
 #include "../../src/utils/str.h"
 #include "../../src/status.h"
 
-#include <stdio.h> /* remove() */
-#include <stdlib.h> /* free() */
-#include <string.h> /* strcpy() strdup() */
-
-#include "utils.h"
-
 SETUP()
 {
 	curr_stats.save_msg = 0;
+	view_setup(&lwin);
+}
+
+TEARDOWN()
+{
+	view_teardown(&lwin);
 }
 
 TEST(incorrect_length_causes_error, IF(not_windows))
 {
 	key_info_t key_info = { .count = 77777 };
-	normal_cmd_cp(&lwin, key_info);
+	modnorm_cp(&lwin, key_info);
 	assert_true(curr_stats.save_msg);
 }
 
 TEST(incorrect_num_causes_error, IF(not_windows))
 {
 	key_info_t key_info = { .count = 9 };
-	normal_cmd_cp(&lwin, key_info);
+	modnorm_cp(&lwin, key_info);
 	assert_true(curr_stats.save_msg);
 }
 
@@ -57,13 +63,12 @@ TEST(file_mode_is_changed, IF(not_windows))
 
 	undo_setup();
 
-	normal_cmd_cp(&lwin, key_info);
+	modnorm_cp(&lwin, key_info);
 	assert_false(curr_stats.save_msg);
 
 	assert_success(os_stat(SANDBOX_PATH "/empty", &st));
 	assert_true((st.st_mode & 0777) != 0000);
 
-	view_teardown(&lwin);
 	assert_success(remove(SANDBOX_PATH "/empty"));
 
 	undo_teardown();
