@@ -3,6 +3,8 @@
 #include <stdlib.h> /* free() */
 #include <string.h> /* memcmp() strdup() */
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/engine/options.h"
 #include "../../src/engine/text_buffer.h"
@@ -10,16 +12,16 @@
 #include "../../src/ui/fileview.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/dynarray.h"
+#include "../../src/utils/gmux.h"
+#include "../../src/utils/shmem.h"
 #include "../../src/utils/str.h"
 #include "../../src/cmd_core.h"
 #include "../../src/filelist.h"
 #include "../../src/opt_handlers.h"
 #include "../../src/registers.h"
 
-#include "utils.h"
-
-static void print_func(const void *data, int column_id, const char buf[],
-		size_t offset, AlignType align, const char full_column[]);
+static void print_func(const char buf[], size_t offset, AlignType align,
+		const char full_column[], const format_info_t *info);
 
 static int ncols;
 
@@ -66,10 +68,10 @@ TEARDOWN()
 }
 
 static void
-print_func(const void *data, int column_id, const char buf[], size_t offset,
-		AlignType align, const char full_column[])
+print_func(const char buf[], size_t offset, AlignType align,
+		const char full_column[], const format_info_t *info)
 {
-	ncols += (column_id != FILL_COLUMN_ID);
+	ncols += (info->id != FILL_COLUMN_ID);
 }
 
 TEST(lsview_block_columns_update_on_sort_change)
@@ -557,19 +559,36 @@ TEST(previewprg_updates_state_of_view)
 TEST(tuioptions)
 {
 	assert_success(vle_opts_set("tuioptions=", OPT_GLOBAL));
+	assert_int_equal(0, cfg.ellipsis_position);
 	assert_false(cfg.extra_padding);
 	assert_false(cfg.side_borders_visible);
 	assert_false(cfg.use_unicode_characters);
+	assert_false(cfg.flexible_splitter);
 
-	assert_success(vle_opts_set("tuioptions=pu", OPT_GLOBAL));
+	assert_success(vle_opts_set("tuioptions=pul", OPT_GLOBAL));
+	assert_int_equal(-1, cfg.ellipsis_position);
 	assert_true(cfg.extra_padding);
 	assert_false(cfg.side_borders_visible);
 	assert_true(cfg.use_unicode_characters);
+	assert_false(cfg.flexible_splitter);
 
-	assert_success(vle_opts_set("tuioptions+=s", OPT_GLOBAL));
+	assert_success(vle_opts_set("tuioptions+=sr", OPT_GLOBAL));
+	assert_int_equal(1, cfg.ellipsis_position);
 	assert_true(cfg.extra_padding);
 	assert_true(cfg.side_borders_visible);
 	assert_true(cfg.use_unicode_characters);
+	assert_false(cfg.flexible_splitter);
+	/* "l" shouldn't be in the value along with "r". */
+	assert_string_equal("prsu", vle_opts_get("tuioptions", OPT_GLOBAL));
+
+	assert_success(vle_opts_set("tuioptions+=vl", OPT_GLOBAL));
+	assert_int_equal(-1, cfg.ellipsis_position);
+	assert_true(cfg.extra_padding);
+	assert_true(cfg.side_borders_visible);
+	assert_true(cfg.use_unicode_characters);
+	assert_true(cfg.flexible_splitter);
+	/* "r" shouldn't be in the value along with "l". */
+	assert_string_equal("lpsuv", vle_opts_get("tuioptions", OPT_GLOBAL));
 }
 
 TEST(setting_tabscope_works)
@@ -641,6 +660,12 @@ TEST(quickview)
 
 TEST(syncregs)
 {
+	/* Put all the temporary files into sandbox to avoid issues with running tests
+	 * from multiple accounts. */
+	char sandbox[PATH_MAX + 1];
+	make_abs_path(sandbox, sizeof(sandbox), SANDBOX_PATH, "", NULL);
+	char *tmpdir_value = mock_env("TMPDIR", sandbox);
+
 	assert_false(regs_sync_enabled());
 	assert_success(exec_commands("set syncregs=test1", &lwin, CIT_COMMAND));
 	assert_true(regs_sync_enabled());
@@ -650,6 +675,14 @@ TEST(syncregs)
 	assert_true(regs_sync_enabled());
 	assert_success(exec_commands("set syncregs=", &lwin, CIT_COMMAND));
 	assert_false(regs_sync_enabled());
+
+	/* Make sure nothing is retained from the tests. */
+	gmux_destroy(gmux_create("regs-test1"));
+	shmem_destroy(shmem_create("regs-test1", 10, 10));
+	gmux_destroy(gmux_create("regs-test2"));
+	shmem_destroy(shmem_create("regs-test2", 10, 10));
+
+	unmock_env("TMPDIR", tmpdir_value);
 }
 
 TEST(mediaprg, IF(not_windows))
@@ -676,5 +709,80 @@ TEST(shellcmdflag)
 	assert_string_equal("-c", cfg.shell_cmd_flag);
 }
 
+TEST(tablabel)
+{
+	assert_success(exec_commands("set tabprefix={", &lwin, CIT_COMMAND));
+	assert_success(exec_commands("set tablabel=%[(%n)%]%[%[%T{tree}%]{%c}@%]%p:t",
+				&lwin, CIT_COMMAND));
+	assert_success(exec_commands("set tabsuffix=}", &lwin, CIT_COMMAND));
+
+	assert_string_equal("{", cfg.tab_prefix);
+	assert_string_equal("%[(%n)%]%[%[%T{tree}%]{%c}@%]%p:t", cfg.tab_label);
+	assert_string_equal("}", cfg.tab_suffix);
+}
+
+TEST(sessionoptions)
+{
+	assert_success(exec_commands("set sessionoptions=dhistory", &lwin,
+				CIT_COMMAND));
+	assert_int_equal(VINFO_DHISTORY, cfg.session_options);
+
+	assert_success(exec_commands("set ssop=savedirs,tui", &lwin, CIT_COMMAND));
+	assert_int_equal(VINFO_SAVEDIRS | VINFO_TUI, cfg.session_options);
+
+	assert_success(exec_commands("set ssop=", &lwin, CIT_COMMAND));
+	assert_int_equal(0, cfg.session_options);
+}
+
+TEST(previewoptions)
+{
+	assert_success(exec_commands("set previewoptions=graphicsdelay:12345", &lwin,
+				CIT_COMMAND));
+	assert_int_equal(12345, cfg.graphics_delay);
+	assert_false(cfg.hard_graphics_clear);
+	assert_false(cfg.top_tree_stats);
+
+	assert_failure(exec_commands("set previewoptions=graphicsdelay:inf", &lwin,
+				CIT_COMMAND));
+	assert_int_equal(12345, cfg.graphics_delay);
+	assert_false(cfg.hard_graphics_clear);
+	assert_false(cfg.top_tree_stats);
+	assert_string_equal("Failed to parse \"graphicsdelay\" value: inf",
+			vle_tb_get_data(vle_err));
+
+	assert_failure(exec_commands("set previewoptions=graphicsdelay:-12345", &lwin,
+				CIT_COMMAND));
+	assert_int_equal(12345, cfg.graphics_delay);
+	assert_false(cfg.hard_graphics_clear);
+	assert_false(cfg.top_tree_stats);
+	assert_string_equal("\"graphicsdelay\" can't be negative, got: -12345",
+			vle_tb_get_data(vle_err));
+
+	assert_failure(exec_commands("set previewoptions=graphicsdelay:145,wtf",
+				&lwin, CIT_COMMAND));
+	assert_int_equal(12345, cfg.graphics_delay);
+	assert_false(cfg.hard_graphics_clear);
+	assert_false(cfg.top_tree_stats);
+	assert_string_equal("Unknown key for 'previewoptions' option: wtf",
+			vle_tb_get_data(vle_err));
+
+	assert_success(exec_commands("set previewoptions=hardgraphicsclear", &lwin,
+				CIT_COMMAND));
+	assert_int_equal(0, cfg.graphics_delay);
+	assert_true(cfg.hard_graphics_clear);
+	assert_false(cfg.top_tree_stats);
+
+	assert_success(exec_commands("set previewoptions=toptreestats", &lwin,
+				CIT_COMMAND));
+	assert_true(cfg.top_tree_stats);
+	assert_int_equal(0, cfg.graphics_delay);
+	assert_false(cfg.hard_graphics_clear);
+
+	assert_success(exec_commands("set previewoptions=", &lwin, CIT_COMMAND));
+	assert_int_equal(0, cfg.graphics_delay);
+	assert_false(cfg.hard_graphics_clear);
+	assert_false(cfg.top_tree_stats);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
