@@ -9,12 +9,12 @@
 #include <stdlib.h> /* exit() */
 #include <string.h> /* strerror() */
 
+#include <test-utils.h>
+
 #include "../../src/utils/gmux.h"
 #include "../../src/utils/shmem.h"
 #include "../../src/utils/utils.h"
 
-#include "utils.h"
-
 static void spawn_regcmd(int number);
 static void send_query(int instance, const char query[]);
 static void receive_ack(int instance);
@@ -38,8 +38,8 @@ static pid_t popen2(const char cmd[], FILE **in, FILE **out);
 #define TEST_REGISTERS_MINUS_DEF  "abcghijklmnopqrstuvwxyz"
 #define TEST_REGISTERS_MINUS_DEFG "abchijklmnopqrstuvwxyz"
 
-#define TEST_EXPECT_FOR_D "d,3,newd,nd1,nd2,"
-#define TEST_EXPECT_FOR_E "e,4,longerthanbeforee,le1,le2,le3,"
+#define TEST_EXPECT_FOR_D "d,3,nd1,nd2,newd,"
+#define TEST_EXPECT_FOR_E "e,4,le1,le2,le3,longerthanbeforee,"
 #define TEST_EXPECT_FOR_G "g,1,G,"
 
 static FILE *instance_stdin[NUM_INSTANCES];
@@ -47,13 +47,21 @@ static FILE *instance_stdout[NUM_INSTANCES];
 
 static char pat4kib[4096 + 8];
 
+static char *tmpdir_value;
+
 /* All of the tests need to run in exactly the order given for correct
  * results. */
 
 /* Tests basic transfer of values from 0 to 1 and spawns processes. */
 SETUP_ONCE()
 {
-	/* Make sure nothing retained from previous tests. */
+	/* Put all the temporary files into sandbox to avoid issues with running tests
+	 * from multiple accounts. */
+	char sandbox[PATH_MAX + 1];
+	make_abs_path(sandbox, sizeof(sandbox), SANDBOX_PATH, "", NULL);
+	tmpdir_value = mock_env("TMPDIR", sandbox);
+
+	/* Make sure nothing is retained from previous tests. */
 	gmux_destroy(gmux_create("regs-test-shmem"));
 	shmem_destroy(shmem_create("regs-test-shmem", 10, 10));
 
@@ -65,7 +73,7 @@ SETUP_ONCE()
 	const char *curletr;
 	for(curletr = TEST_REGISTERS; *curletr != '\0'; ++curletr)
 	{
-		fprintf(instance_stdin[0], "set,%c,initial%c,i%c1,i%c2,i%c3\n",
+		fprintf(instance_stdin[0], "set,%c,_initial%c,i%c1,i%c2,i%c3\n",
 			*curletr, *curletr, *curletr, *curletr, *curletr);
 		fprintf(instance_stdin[1], "set,%c,lossval%c,l%c1,l%c2,l%c3\n",
 			*curletr, *curletr, *curletr, *curletr, *curletr);
@@ -78,6 +86,15 @@ SETUP_ONCE()
 	receive_ack(1);
 }
 
+TEARDOWN_ONCE()
+{
+	/* Make sure nothing is retained from the tests. */
+	gmux_destroy(gmux_create("regs-test-shmem"));
+	shmem_destroy(shmem_create("regs-test-shmem", 10, 10));
+
+	unmock_env("TMPDIR", tmpdir_value);
+}
+
 static void
 spawn_regcmd(int number)
 {
@@ -131,8 +148,8 @@ check_is_initial(int instance, const char reglist[])
 	const char *curletr;
 	for(curletr = reglist; *curletr != '\0'; ++curletr)
 	{
-		char cmp[26];
-		snprintf(cmp, sizeof(cmp), "%c,4,initial%c,i%c1,i%c2,i%c3,",
+		char cmp[27];
+		snprintf(cmp, sizeof(cmp), "%c,4,_initial%c,i%c1,i%c2,i%c3,",
 			*curletr, *curletr, *curletr, *curletr, *curletr);
 		check_register_contents(instance, *curletr, cmp);
 	}
@@ -365,6 +382,7 @@ popen2(const char cmd[], FILE **in, FILE **out)
 	if(*in == NULL)
 	{
 		close(in_pipe[1]);
+		close(out_pipe[0]);
 		kill(pid, SIGTERM);
 		return (pid_t)-1;
 	}
@@ -372,7 +390,7 @@ popen2(const char cmd[], FILE **in, FILE **out)
 	if(*out == NULL)
 	{
 		fclose(*in);
-		close(out_pipe[1]);
+		close(out_pipe[0]);
 		kill(pid, SIGTERM);
 		return (pid_t)-1;
 	}
