@@ -4,12 +4,13 @@
 
 #include <string.h> /* memset() strcpy() */
 
+#include <test-utils.h>
+
 #include "../../src/cfg/config.h"
 #include "../../src/modes/normal.h"
 #include "../../src/utils/fs.h"
 #include "../../src/filelist.h"
 #include "../../src/search.h"
-#include "utils.h"
 
 static char *saved_cwd;
 
@@ -18,6 +19,7 @@ SETUP()
 	saved_cwd = save_cwd();
 
 	view_setup(&lwin);
+	view_setup(&rwin);
 
 	assert_success(chdir(TEST_DATA_PATH "/read"));
 	assert_non_null(get_cwd(lwin.curr_dir, sizeof(lwin.curr_dir)));
@@ -31,6 +33,7 @@ SETUP()
 TEARDOWN()
 {
 	view_teardown(&lwin);
+	view_teardown(&rwin);
 
 	restore_cwd(saved_cwd);
 }
@@ -176,7 +179,6 @@ TEST(view_patterns_are_synchronized)
 {
 	int found;
 
-	view_setup(&rwin);
 	strcpy(rwin.curr_dir, lwin.curr_dir);
 	populate_dir_list(&rwin, 0);
 
@@ -192,8 +194,6 @@ TEST(view_patterns_are_synchronized)
 	assert_int_equal(2, rwin.matches);
 	/* Same patterns don't cause reset. */
 	assert_int_equal(2, lwin.matches);
-
-	view_teardown(&rwin);
 }
 
 TEST(find_npattern_returns_zero_if_msg_is_not_printed)
@@ -201,7 +201,7 @@ TEST(find_npattern_returns_zero_if_msg_is_not_printed)
 	cfg.hl_search = 1;
 	cfg.inc_search = 0;
 
-	assert_int_equal(0, find_npattern(&lwin, "dos", 0, 1));
+	assert_int_equal(0, modnorm_find(&lwin, "dos", 0, 1));
 
 	cfg.hl_search = 0;
 }
