@@ -2,15 +2,18 @@
 
 #include <unistd.h> /* rmdir() symlink() */
 
+#include <limits.h> /* INT_MAX */
 #include <stdio.h> /* remove() */
 
+#include <test-utils.h>
+
+#include "../../src/cfg/config.h"
+#include "../../src/compat/fs_limits.h"
 #include "../../src/compat/os.h"
 #include "../../src/ui/quickview.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/string_array.h"
 
-#include "utils.h"
-
 static char *saved_cwd;
 
 SETUP()
@@ -27,7 +30,7 @@ TEARDOWN()
 
 TEST(file_can_not_be_viewed)
 {
-	assert_null(qv_view_dir(TEST_DATA_PATH "/existing-files/a"));
+	assert_null(qv_view_dir(TEST_DATA_PATH "/existing-files/a", INT_MAX));
 }
 
 TEST(empty_dir_produces_single_line_and_dirs_have_trailing_slash)
@@ -38,7 +41,7 @@ TEST(empty_dir_produces_single_line_and_dirs_have_trailing_slash)
 
 	assert_success(os_mkdir("empty-dir", 0777));
 
-	fp = qv_view_dir("empty-dir");
+	fp = qv_view_dir("empty-dir", INT_MAX);
 	lines = read_file_lines(fp, &nlines);
 
 	assert_int_equal(3, nlines);
@@ -61,7 +64,7 @@ TEST(single_file_is_displayed_correctly_file_without_slash)
 	assert_success(os_mkdir("dir", 0777));
 	create_file("dir/file");
 
-	fp = qv_view_dir("dir");
+	fp = qv_view_dir("dir", INT_MAX);
 	lines = read_file_lines(fp, &nlines);
 
 	assert_int_equal(4, nlines);
@@ -86,7 +89,7 @@ TEST(single_subdir_is_displayed_correctly)
 	assert_success(os_mkdir("dir", 0777));
 	assert_success(os_mkdir("dir/nested", 0777));
 
-	fp = qv_view_dir("dir");
+	fp = qv_view_dir("dir", INT_MAX);
 	lines = read_file_lines(fp, &nlines);
 
 	assert_int_equal(4, nlines);
@@ -112,7 +115,7 @@ TEST(multiple_nested_dirs_treated_correctly)
 	assert_success(os_mkdir("dir/nested1", 0777));
 	assert_success(os_mkdir("dir/nested1/nested2", 0777));
 
-	fp = qv_view_dir("dir");
+	fp = qv_view_dir("dir", INT_MAX);
 	lines = read_file_lines(fp, &nlines);
 
 	assert_int_equal(5, nlines);
@@ -140,7 +143,7 @@ TEST(multiple_files_treated_correctly)
 	create_file("dir/file1");
 	create_file("dir/file2");
 
-	fp = qv_view_dir("dir");
+	fp = qv_view_dir("dir", INT_MAX);
 	lines = read_file_lines(fp, &nlines);
 
 	assert_int_equal(5, nlines);
@@ -170,7 +173,7 @@ TEST(multiple_non_empty_dirs_have_correct_prefixes_plus_sorting)
 	create_file("dir/sub1/file");
 	create_file("dir/sub2/file");
 
-	fp = qv_view_dir("dir");
+	fp = qv_view_dir("dir", INT_MAX);
 	lines = read_file_lines(fp, &nlines);
 
 	assert_int_equal(7, nlines);
@@ -208,7 +211,7 @@ TEST(symlinks_are_not_resolved_in_tree_preview, IF(not_windows))
 	assert_success(symlink(".", SANDBOX_PATH "/dir/link"));
 #endif
 
-	fp = qv_view_dir(SANDBOX_PATH "/dir");
+	fp = qv_view_dir(SANDBOX_PATH "/dir", INT_MAX);
 	lines = read_file_lines(fp, &nlines);
 
 	assert_int_equal(4, nlines);
@@ -224,5 +227,72 @@ TEST(symlinks_are_not_resolved_in_tree_preview, IF(not_windows))
 	assert_success(rmdir(SANDBOX_PATH "/dir"));
 }
 
+TEST(top_tree_stats)
+{
+	int i;
+	int nlines;
+	FILE *fp;
+	char **lines;
+
+	cfg.top_tree_stats = 1;
+
+	assert_success(os_mkdir("dir", 0777));
+	assert_success(os_mkdir("dir/nested1", 0777));
+	assert_success(os_mkdir("dir/nested1/nested2", 0777));
+
+	fp = qv_view_dir("dir", INT_MAX);
+	lines = read_file_lines(fp, &nlines);
+
+	assert_int_equal(5, nlines);
+	assert_string_equal("2 directories, 0 files", lines[0]);
+	for(i = 0; lines[1][i] != '\0'; ++i)
+	{
+		assert_true(lines[1][i] == ' ');
+	}
+	assert_string_equal("dir/", lines[2]);
+	assert_string_equal("`-- nested1/", lines[3]);
+	assert_string_equal("    `-- nested2/", lines[4]);
+
+	free_string_array(lines, nlines);
+	fclose(fp);
+
+	assert_success(rmdir("dir/nested1/nested2"));
+	assert_success(rmdir("dir/nested1"));
+	assert_success(rmdir("dir"));
+
+	cfg.top_tree_stats = 0;
+}
+
+TEST(top_tree_stats_in_small_window)
+{
+	int i;
+	int nlines;
+	FILE *fp;
+	char **lines;
+	char dir_path[PATH_MAX + 1];
+
+	cfg.top_tree_stats = 1;
+
+	make_abs_path(dir_path, sizeof(dir_path), TEST_DATA_PATH, "tree", saved_cwd);
+	fp = qv_view_dir(dir_path, 5);
+	lines = read_file_lines(fp, &nlines);
+
+	assert_int_equal(6, nlines);
+	assert_string_equal("5 directories, 7 files", lines[0]);
+	for(i = 0; lines[1][i] != '\0'; ++i)
+	{
+		assert_true(lines[1][i] == ' ');
+	}
+	assert_string_equal("tree/", lines[2]);
+	assert_string_equal("|-- .hidden", lines[3]);
+	assert_string_equal("|-- dir1/", lines[4]);
+	assert_string_equal("|   |-- dir2/", lines[5]);
+
+	free_string_array(lines, nlines);
+	fclose(fp);
+
+	cfg.top_tree_stats = 0;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
