@@ -1,10 +1,10 @@
 #include <stic.h>
 
+#include <test-utils.h>
+
 #include "../../src/compat/fs_limits.h"
 #include "../../src/utils/path.h"
 
-#include "utils.h"
-
 #ifndef _WIN32
 #define ABS_PREFIX
 #else
