@@ -5,11 +5,11 @@
 #include <stdio.h> /* remove() */
 #include <stdlib.h> /* free() */
 
+#include <test-utils.h>
+
 #include "../../src/int/file_magic.h"
 #include "../../src/utils/matcher.h"
 
-#include "utils.h"
-
 static void check_glob(matcher_t *m);
 static void check_regexp(matcher_t *m);
 static int has_mime_type_detection(void);
@@ -322,6 +322,22 @@ TEST(globs_are_cloned)
 	matcher_free(clone);
 }
 
+TEST(globs_are_case_insensitive)
+{
+	char *error;
+	matcher_t *m, *clone;
+
+	assert_non_null(m = matcher_alloc("{*.ExT}", 0, 1, "", &error));
+	assert_null(error);
+	assert_non_null(clone = matcher_clone(m));
+
+	check_glob(m);
+	matcher_free(m);
+
+	check_glob(clone);
+	matcher_free(clone);
+}
+
 TEST(regexps_are_cloned)
 {
 	char *error;
@@ -338,6 +354,23 @@ TEST(regexps_are_cloned)
 	matcher_free(clone);
 }
 
+TEST(comma_escaping)
+{
+	char *error;
+	matcher_t *m;
+
+	assert_non_null(m = matcher_alloc("{a,,b,*.ext,c,,d}", 0, 1, "", &error));
+	assert_null(error);
+
+	check_glob(m);
+	assert_false(matcher_matches(m, "a"));
+	assert_false(matcher_matches(m, "b"));
+	assert_true(matcher_matches(m, "a,b"));
+	assert_true(matcher_matches(m, "c,d"));
+
+	matcher_free(m);
+}
+
 TEST(mime_type_pattern, IF(has_mime_type_detection))
 {
 	char *error;
@@ -386,14 +419,17 @@ TEST(mime_type_of_link_is_that_of_its_target,
 	/* symlink() is not available on Windows, but the rest of the code is fine. */
 #ifndef _WIN32
 	assert_success(symlink(".", "link"));
+	assert_success(symlink("link", "link2"));
 #endif
 
 	assert_non_null(m = matcher_alloc("<inode/directory>", 0, 1, "", &error));
 	assert_null(error);
 	assert_true(matcher_matches(m, "link"));
+	assert_true(matcher_matches(m, "link2"));
 	matcher_free(m);
 
 	assert_success(remove("link"));
+	assert_success(remove("link2"));
 }
 
 static void
