@@ -2,7 +2,9 @@
 
 #include <unistd.h> /* chdir() unlink() */
 
-#include <stdio.h> /* fclose() fopen() fprintf() */
+#include <stdio.h> /* fclose() fflush() fopen() fprintf() */
+
+#include <test-utils.h>
 
 #include "../../src/cfg/config.h"
 #include "../../src/utils/fs.h"
@@ -11,11 +13,10 @@
 #include "../../src/cmd_completion.h"
 
 static void line_handler(const char line[], void *arg);
-static int cat_is_available(void);
 
 static int nlines;
 
-TEST(check_null_separation, IF(cat_is_available))
+TEST(check_null_separation, IF(have_cat))
 {
 	char *saved_cwd;
 
@@ -37,8 +38,8 @@ TEST(check_null_separation, IF(cat_is_available))
 	stats_update_shell_type(cfg.shell);
 
 	nlines = 0;
-	assert_success(process_cmd_output("tests", "cat list", 1, 0, &line_handler,
-				NULL));
+	assert_success(process_cmd_output("tests", "cat list", NULL, 1, 0,
+				&line_handler, NULL));
 	assert_int_equal(1, nlines);
 
 	stats_update_shell_type("/bin/sh");
@@ -50,17 +51,47 @@ TEST(check_null_separation, IF(cat_is_available))
 	restore_cwd(saved_cwd);
 }
 
+TEST(input_redirection, IF(have_cat))
+{
+	char *saved_cwd;
+
+	FILE *const f = fopen(SANDBOX_PATH "/list", "w+");
+	fprintf(f, "%s", "/a\n/b");
+	fflush(f);
+
+	saved_cwd = save_cwd();
+
+	assert_success(chdir(SANDBOX_PATH));
+
+#ifndef _WIN32
+	replace_string(&cfg.shell, "/bin/sh");
+	update_string(&cfg.shell_cmd_flag, "-c");
+#else
+	replace_string(&cfg.shell, "cmd");
+	update_string(&cfg.shell_cmd_flag, "/C");
+#endif
+	stats_update_shell_type(cfg.shell);
+
+	nlines = 0;
+	assert_success(process_cmd_output("tests", "cat", f, 1, 0,
+				&line_handler, NULL));
+	assert_int_equal(2, nlines);
+
+	stats_update_shell_type("/bin/sh");
+	update_string(&cfg.shell, NULL);
+	update_string(&cfg.shell_cmd_flag, NULL);
+
+	fclose(f);
+	assert_success(unlink("list"));
+
+	restore_cwd(saved_cwd);
+}
+
 static void
 line_handler(const char line[], void *arg)
 {
 	++nlines;
 }
 
-static int
-cat_is_available(void)
-{
-	return external_command_exists("cat");
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
