@@ -2,7 +2,7 @@
 
 #include <string.h>
 
-#include "../../src/column_view.h"
+#include "../../src/ui/column_view.h"
 #include "../../src/viewcolumns_parser.h"
 
 #include "test.h"
@@ -66,4 +66,4 @@ TEST(percent_summ_greater_than_hundred_fail)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
